@@ -85,6 +85,7 @@ static int           *YCbCr_R_Cr_Tab = NULL;
 static int           *YCbCr_G_Cb_Tab = NULL;
 static int           *YCbCr_G_Cr_Tab = NULL;
 static int           *YCbCr_B_Cb_Tab = NULL;
+static GMutex         Tables_Mutex;
 
 
 #define SCALE_FACTOR   16
@@ -96,6 +97,8 @@ static int           *YCbCr_B_Cb_Tab = NULL;
 static void
 CMYK_table_init (void)
 {
+	g_mutex_lock (&Tables_Mutex);
+
 	if (CMYK_Tab == NULL) {
 		int    v, k, i;
 		double k1;
@@ -110,12 +113,16 @@ CMYK_table_init (void)
 				CMYK_Tab[i++] = (double) v * k1;
 		}
 	}
+
+	g_mutex_unlock (&Tables_Mutex);
 }
 
 
 static void
 YCbCr_tables_init (void)
 {
+	g_mutex_lock (&Tables_Mutex);
+
 	if (YCbCr_R_Cr_Tab == NULL) {
 		int i, v;
 
@@ -125,12 +132,14 @@ YCbCr_tables_init (void)
 		YCbCr_B_Cb_Tab = g_new (int, 256);
 
 		for (i = 0, v = -128; i <= 255; i++, v++) {
-			YCbCr_R_Cr_Tab[i] = SCALE_DOWN (SCALE_UP(1.402) * v + ONE_HALF);
+			YCbCr_R_Cr_Tab[i] = SCALE_DOWN (SCALE_UP (1.402) * v + ONE_HALF);
 			YCbCr_G_Cb_Tab[i] = - SCALE_UP (0.34414) * v;
 			YCbCr_G_Cr_Tab[i] = - SCALE_UP (0.71414) * v + ONE_HALF;
-			YCbCr_B_Cb_Tab[i] = SCALE_DOWN (SCALE_UP(1.77200) * v + ONE_HALF);
+			YCbCr_B_Cb_Tab[i] = SCALE_DOWN (SCALE_UP (1.77200) * v + ONE_HALF);
 		}
 	}
+
+	g_mutex_unlock (&Tables_Mutex);
 }
 
 
@@ -138,8 +147,9 @@ GthImage *
 _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 				       GthFileData   *file_data,
 				       int            requested_size,
-				       int           *original_width,
-				       int           *original_height,
+				       int           *original_width_p,
+				       int           *original_height_p,
+				       gboolean      *loaded_original_p,
 				       gpointer       user_data,
 				       GCancellable  *cancellable,
 				       GError       **error)
@@ -210,12 +220,17 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 			}
 		}
 
-		if (srcinfo.scale_denom == 0)
+		if (srcinfo.scale_denom <= 0) {
 			srcinfo.scale_denom = srcinfo.scale_num;
-
-		jpeg_calc_output_dimensions (&srcinfo);
+			load_scaled = FALSE;
+		}
 	}
 
+	jpeg_calc_output_dimensions (&srcinfo);
+
+	buffer_stride = srcinfo.output_width * srcinfo.output_components;
+	buffer = (*srcinfo.mem->alloc_sarray) ((j_common_ptr) &srcinfo, JPOOL_IMAGE, buffer_stride, srcinfo.rec_outbuf_height);
+
 	jpeg_start_decompress (&srcinfo);
 
 	orientation = _jpeg_exif_orientation (in_buffer, in_buffer_size);
@@ -238,23 +253,17 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 			destination_height);
 #endif
 
-	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, destination_width, destination_height);
-	if (cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS) {
-		/* g_warning ("%s", cairo_status_to_string (cairo_surface_status (surface))); */
-
-		jpeg_destroy ((j_common_ptr) &srcinfo);
-		cairo_surface_destroy (surface);
+	surface = _cairo_image_surface_create (CAIRO_FORMAT_ARGB32, destination_width, destination_height);
+	if (surface == NULL) {
+		jpeg_destroy_decompress (&srcinfo);
 		g_free (in_buffer);
 
 		return image;
 	}
+
 	metadata = _cairo_image_surface_get_metadata (surface);
 	metadata->has_alpha = FALSE;
-	cairo_surface_flush (surface);
-	surface_row = cairo_image_surface_get_data (surface) + line_start;
-
-	buffer_stride = srcinfo.output_width * srcinfo.output_components;
-	buffer = (*srcinfo.mem->alloc_sarray) ((j_common_ptr) &srcinfo, JPOOL_IMAGE, buffer_stride, srcinfo.rec_outbuf_height);
+	surface_row = _cairo_image_surface_flush_and_get_data (surface) + line_start;
 
 	switch (srcinfo.out_color_space) {
 	case JCS_CMYK:
@@ -267,7 +276,7 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 
 			while (srcinfo.output_scanline < srcinfo.output_height) {
 				if (g_cancellable_is_cancelled (cancellable))
-					break;
+					goto stop_loading;
 
 				n_lines = jpeg_read_scanlines (&srcinfo, buffer, srcinfo.rec_outbuf_height);
 
@@ -276,6 +285,9 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 					p_surface = surface_row;
 					p_buffer = buffer_row[l];
 
+					if (g_cancellable_is_cancelled (cancellable))
+						goto stop_loading;
+
 					for (x = 0; x < srcinfo.output_width; x++) {
 						if (srcinfo.saw_Adobe_marker) {
 							c = p_buffer[0];
@@ -312,7 +324,7 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 		{
 			while (srcinfo.output_scanline < srcinfo.output_height) {
 				if (g_cancellable_is_cancelled (cancellable))
-					break;
+					goto stop_loading;
 
 				n_lines = jpeg_read_scanlines (&srcinfo, buffer, srcinfo.rec_outbuf_height);
 
@@ -321,6 +333,9 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 					p_surface = surface_row;
 					p_buffer = buffer_row[l];
 
+					if (g_cancellable_is_cancelled (cancellable))
+						goto stop_loading;
+
 					for (x = 0; x < srcinfo.output_width; x++) {
 						r = g = b = p_buffer[0];
 						pixel = CAIRO_RGBA_TO_UINT32 (r, g, b, 0xff);
@@ -341,7 +356,7 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 		{
 			while (srcinfo.output_scanline < srcinfo.output_height) {
 				if (g_cancellable_is_cancelled (cancellable))
-					break;
+					goto stop_loading;
 
 				n_lines = jpeg_read_scanlines (&srcinfo, buffer, srcinfo.rec_outbuf_height);
 
@@ -350,6 +365,9 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 					p_surface = surface_row;
 					p_buffer = buffer_row[l];
 
+					if (g_cancellable_is_cancelled (cancellable))
+						goto stop_loading;
+
 					for (x = 0; x < srcinfo.output_width; x++) {
 						r = p_buffer[0];
 						g = p_buffer[1];
@@ -385,7 +403,7 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 
 			while (srcinfo.output_scanline < srcinfo.output_height) {
 				if (g_cancellable_is_cancelled (cancellable))
-					break;
+					goto stop_loading;
 
 				n_lines = jpeg_read_scanlines (&srcinfo, buffer, srcinfo.rec_outbuf_height);
 
@@ -394,6 +412,9 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 					p_surface = surface_row;
 					p_buffer = buffer_row[l];
 
+					if (g_cancellable_is_cancelled (cancellable))
+						goto stop_loading;
+
 					for (x = 0; x < srcinfo.output_width; x++) {
 						Y = p_buffer[0];
 						Cb = p_buffer[1];
@@ -437,7 +458,7 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 
 			while (srcinfo.output_scanline < srcinfo.output_height) {
 				if (g_cancellable_is_cancelled (cancellable))
-					break;
+					goto stop_loading;
 
 				n_lines = jpeg_read_scanlines (&srcinfo, buffer, srcinfo.rec_outbuf_height);
 
@@ -446,6 +467,9 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 					p_surface = surface_row;
 					p_buffer = buffer_row[l];
 
+					if (g_cancellable_is_cancelled (cancellable))
+						goto stop_loading;
+
 					for (x = 0; x < srcinfo.output_width; x++) {
 						Y = p_buffer[0];
 						Cb = p_buffer[1];
@@ -485,24 +509,13 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 		break;
 	}
 
+	stop_loading:
+
+	cairo_surface_mark_dirty (surface);
+
 	if (! g_cancellable_is_cancelled (cancellable)) {
-		cairo_surface_mark_dirty (surface);
-
-		/* Scale to the requested size */
-
-		if (load_scaled) {
-			cairo_surface_t *scaled;
-			int              width;
-			int              height;
-
-			width = destination_width;
-			height = destination_height;
-			scale_keeping_ratio (&width, &height, requested_size, requested_size, TRUE);
-			scaled = _cairo_image_surface_scale (surface, width, height, SCALE_FILTER_BEST, NULL);
-
-			cairo_surface_destroy (surface);
-			surface = scaled;
-		}
+		int original_width;
+		int original_height;
 
 		/* Set the original dimensions */
 
@@ -511,29 +524,34 @@ _cairo_image_surface_create_from_jpeg (GInputStream  *istream,
 		     ||	(orientation == GTH_TRANSFORM_TRANSPOSE)
 		     ||	(orientation == GTH_TRANSFORM_TRANSVERSE))
 		{
-			if (original_width != NULL)
-				*original_width = srcinfo.image_height;
-			if (original_height != NULL)
-				*original_height = srcinfo.image_width;
+			original_width = srcinfo.image_height;
+			original_height = srcinfo.image_width;
 		}
 		else {
-			if (original_width != NULL)
-				*original_width = srcinfo.image_width;
-			if (original_height != NULL)
-				*original_height = srcinfo.image_height;
+			original_width = srcinfo.image_width;
+			original_height = srcinfo.image_height;
 		}
 
-		/*_cairo_image_surface_set_attribute_int (surface, "Image::Rotation", rotation); FIXME*/
+		metadata->original_width = original_width;
+		metadata->original_height = original_height;
 
-		/* FIXME _cairo_image_surface_set_attribute (surface, "Jpeg::ColorSpace", jpeg_color_space_name (srcinfo.jpeg_color_space)); */
-
-		gth_image_set_cairo_surface (image, surface);
+		if (original_width_p != NULL)
+			*original_width_p = original_width;
+		if (original_height_p != NULL)
+			*original_height_p = original_height;
+		if (loaded_original_p != NULL)
+			*loaded_original_p = ! load_scaled;
 
 		jpeg_finish_decompress (&srcinfo);
 		jpeg_destroy_decompress (&srcinfo);
+
+		/*_cairo_image_surface_set_attribute_int (surface, "Image::Rotation", rotation); FIXME*/
+		/* FIXME _cairo_image_surface_set_attribute (surface, "Jpeg::ColorSpace", jpeg_color_space_name (srcinfo.jpeg_color_space)); */
+
+		gth_image_set_cairo_surface (image, surface);
 	}
 	else {
-		jpeg_destroy ((j_common_ptr) &srcinfo);
+		jpeg_destroy_decompress (&srcinfo);
 		g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_CANCELLED, "");
 	}
 
