@@ -55,8 +55,8 @@ gth_image_saver_webp_get_control (GthImageSaver *base)
 {
 	GthImageSaverWebp *self = GTH_IMAGE_SAVER_WEBP (base);
 
-	if (self->priv->builder == NULL)
-		self->priv->builder = _gtk_builder_new_from_file ("webp-options.ui", "cairo_io");
+	_g_object_unref (self->priv->builder);
+	self->priv->builder = _gtk_builder_new_from_file ("webp-options.ui", "cairo_io");
 
 	gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("quality_adjustment")),
 				  g_settings_get_int (self->priv->settings, PREF_WEBP_QUALITY));
@@ -138,7 +138,7 @@ _WebPPictureImportCairoSurface (WebPPicture     *const picture,
 		return 0;
 
 	stride = cairo_image_surface_get_stride (image);
-	src_row = cairo_image_surface_get_data (image);
+	src_row = _cairo_image_surface_flush_and_get_data (image);
 	dest_row = picture->argb;
 
 	for (y= 0; y < cairo_image_surface_get_height (image); y++) {
@@ -395,4 +395,5 @@ gth_image_saver_webp_init (GthImageSaverWebp *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_IMAGE_SAVER_WEBP, GthImageSaverWebpPrivate);
 	self->priv->settings = g_settings_new (GTHUMB_IMAGE_SAVERS_WEBP_SCHEMA);
+	self->priv->builder = NULL;
 }
