@@ -23,15 +23,18 @@
 #include <config.h>
 #include <gthumb.h>
 #include <gth-catalog.h>
+#include "actions.h"
 #include "dlg-add-to-catalog.h"
 #include "dlg-catalog-properties.h"
 #include "gth-file-source-catalogs.h"
 
 
 void
-gth_browser_activate_action_edit_add_to_catalog (GtkAction  *action,
-						 GthBrowser *browser)
+gth_browser_activate_add_to_catalog (GSimpleAction 	*action,
+				     GVariant		*parameter,
+				     gpointer		 user_data)
 {
+	GthBrowser *browser = GTH_BROWSER (user_data);
 	GList *items;
 	GList *file_list = NULL;
 	GList *files;
@@ -48,9 +51,11 @@ gth_browser_activate_action_edit_add_to_catalog (GtkAction  *action,
 
 
 void
-gth_browser_activate_action_edit_remove_from_catalog (GtkAction  *action,
-						      GthBrowser *browser)
+gth_browser_activate_remove_from_catalog (GSimpleAction	 *action,
+					  GVariant	 *parameter,
+					  gpointer	  user_data)
 {
+	GthBrowser *browser = GTH_BROWSER (user_data);
 	GList *items;
 	GList *file_data_list;
 
@@ -168,16 +173,18 @@ catalog_new_dialog_response_cb (GtkWidget *dialog,
 
 
 void
-gth_browser_activate_action_catalog_new (GtkAction  *action,
-					 GthBrowser *browser)
+gth_browser_activate_create_catalog (GSimpleAction	 *action,
+				  GVariant	 *parameter,
+				  gpointer	 user_data)
 {
+	GthBrowser *browser = GTH_BROWSER (user_data);
 	GtkWidget *dialog;
 
 	dialog = gth_request_dialog_new (GTK_WINDOW (browser),
 					 GTK_DIALOG_MODAL,
 					 _("New catalog"),
 					 _("Enter the catalog name:"),
-					 GTK_STOCK_CANCEL,
+					 _GTK_LABEL_CANCEL,
 					 _("C_reate"));
 	g_signal_connect (dialog,
 			  "response",
@@ -280,16 +287,18 @@ new_library_dialog_response_cb (GtkWidget *dialog,
 
 
 void
-gth_browser_activate_action_catalog_new_library (GtkAction  *action,
-						 GthBrowser *browser)
+gth_browser_activate_create_library (GSimpleAction	 *action,
+					  GVariant	 *parameter,
+					  gpointer	  user_data)
 {
+	GthBrowser *browser = GTH_BROWSER (user_data);
 	GtkWidget *dialog;
 
 	dialog = gth_request_dialog_new (GTK_WINDOW (browser),
 					 GTK_DIALOG_MODAL,
 					 _("New library"),
 					 _("Enter the library name:"),
-					 GTK_STOCK_CANCEL,
+					 _GTK_LABEL_CANCEL,
 					 _("C_reate"));
 	g_signal_connect (dialog,
 			  "response",
@@ -347,25 +356,29 @@ remove_catalog_response_cb (GtkDialog *dialog,
 
 
 void
-gth_browser_activate_action_catalog_remove (GtkAction  *action,
-					    GthBrowser *browser)
+gth_browser_activate_remove_catalog (GSimpleAction	 *action,
+				     GVariant		 *parameter,
+				     gpointer		  user_data)
 {
+	GthBrowser *browser = GTH_BROWSER (user_data);
 	GthFileData *file_data;
+	GSettings   *settings;
 
 	file_data = gth_browser_get_folder_popup_file_data (browser);
 
-	if (eel_gconf_get_boolean (PREF_MSG_CONFIRM_DELETION, DEFAULT_MSG_CONFIRM_DELETION)) {
+	settings = g_settings_new (GTHUMB_MESSAGES_SCHEMA);
+	if (g_settings_get_boolean (settings, PREF_MSG_CONFIRM_DELETION)) {
 		char      *prompt;
 		GtkWidget *d;
 
 		prompt = g_strdup_printf (_("Are you sure you want to remove \"%s\"?"), g_file_info_get_display_name (file_data->info));
 		d = _gtk_message_dialog_new (GTK_WINDOW (browser),
 					     GTK_DIALOG_MODAL,
-					     GTK_STOCK_DIALOG_QUESTION,
+					     _GTK_ICON_NAME_DIALOG_QUESTION,
 					     prompt,
 					     NULL,
-					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					     GTK_STOCK_REMOVE, GTK_RESPONSE_YES,
+					     _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
+					     _GTK_LABEL_REMOVE, GTK_RESPONSE_YES,
 					     NULL);
 		g_signal_connect (d, "response", G_CALLBACK (remove_catalog_response_cb), file_data);
 		gtk_widget_show (d);
@@ -376,13 +389,17 @@ gth_browser_activate_action_catalog_remove (GtkAction  *action,
 		remove_catalog (GTK_WINDOW (browser), file_data);
 		g_object_unref (file_data);
 	}
+
+	g_object_unref (settings);
 }
 
 
 void
-gth_browser_activate_action_catalog_rename (GtkAction  *action,
-					    GthBrowser *browser)
+gth_browser_activate_rename_catalog (GSimpleAction	 *action,
+				     GVariant		 *parameter,
+				     gpointer		  user_data)
 {
+	GthBrowser *browser = GTH_BROWSER (user_data);
 	GthFileData *file_data;
 
 	file_data = gth_browser_get_folder_popup_file_data (browser);
@@ -393,9 +410,11 @@ gth_browser_activate_action_catalog_rename (GtkAction  *action,
 
 
 void
-gth_browser_activate_action_catalog_properties (GtkAction  *action,
-						GthBrowser *browser)
+gth_browser_activate_catalog_properties (GSimpleAction	 *action,
+					 GVariant	 *parameter,
+					 gpointer	  user_data)
 {
+	GthBrowser *browser = GTH_BROWSER (user_data);
 	GthFileData *file_data;
 
 	file_data = gth_browser_get_folder_popup_file_data (browser);
@@ -406,9 +425,11 @@ gth_browser_activate_action_catalog_properties (GtkAction  *action,
 
 
 void
-gth_browser_activate_action_go_to_container (GtkAction  *action,
-					     GthBrowser *browser)
+gth_browser_activate_go_to_container_from_catalog (GSimpleAction	 *action,
+						   GVariant		 *parameter,
+						   gpointer		  user_data)
 {
+	GthBrowser *browser = GTH_BROWSER (user_data);
 	GList *items;
 	GList *file_list = NULL;
 
