@@ -21,6 +21,7 @@
 
 #include <config.h>
 #include <gthumb.h>
+#include "dlg-add-to-catalog.h"
 #include "gth-catalog.h"
 #include "gth-file-source-catalogs.h"
 
@@ -144,14 +145,14 @@ catalog_ready_cb (GObject  *catalog,
 
 	buffer = gth_catalog_to_data (add_data->catalog, &length);
 	gio_file = gth_catalog_file_to_gio_file (add_data->catalog_file);
-	g_write_file_async (gio_file,
-			    buffer,
-			    length,
-			    TRUE,
-			    G_PRIORITY_DEFAULT,
-			    NULL,
-			    catalog_save_done_cb,
-			    add_data);
+	_g_file_write_async (gio_file,
+			     buffer,
+			     length,
+			     TRUE,
+			     G_PRIORITY_DEFAULT,
+			     NULL,
+			     catalog_save_done_cb,
+			     add_data);
 
 	g_object_unref (gio_file);
 }
@@ -358,7 +359,7 @@ new_catalog_button_clicked_cb (GtkWidget  *widget,
 					 GTK_DIALOG_MODAL,
 					 _("New catalog"),
 					 _("Enter the catalog name:"),
-					 GTK_STOCK_CANCEL,
+					 _GTK_LABEL_CANCEL,
 					 _("C_reate"));
 	g_signal_connect (dialog,
 			  "response",
@@ -470,7 +471,7 @@ new_library_button_clicked_cb (GtkWidget  *widget,
 					 GTK_DIALOG_MODAL,
 					 _("New library"),
 					 _("Enter the library name:"),
-					 GTK_STOCK_CANCEL,
+					 _GTK_LABEL_CANCEL,
 					 _("C_reate"));
 	g_signal_connect (dialog,
 			  "response",
