@@ -127,27 +127,19 @@ save_button_clicked_cb (GtkButton  *button,
 
 	gio_file = gth_catalog_file_to_gio_file (data->file_data->file);
 	buffer = gth_catalog_to_data (data->catalog, &size);
-	g_write_file_async (gio_file,
-			    buffer,
-			    size,
-			    TRUE,
-			    G_PRIORITY_DEFAULT,
-			    NULL,
-			    catalog_saved_cb,
-			    data);
+	_g_file_write_async (gio_file,
+			     buffer,
+			     size,
+			     TRUE,
+			     G_PRIORITY_DEFAULT,
+			     NULL,
+			     catalog_saved_cb,
+			     data);
 
 	g_object_unref (gio_file);
 }
 
 
-static void
-help_button_clicked_cb (GtkWidget  *widget,
-			DialogData *data)
-{
-	show_help_dialog (GTK_WINDOW (data->dialog), "catalog-properties");
-}
-
-
 static void
 catalog_ready_cb (GObject  *object,
 		  GError   *error,
@@ -225,10 +217,6 @@ dlg_catalog_properties (GthBrowser  *browser,
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
 				  data->dialog);
-	g_signal_connect (G_OBJECT (GET_WIDGET ("help_button")),
-			  "clicked",
-			  G_CALLBACK (help_button_clicked_cb),
-			  data);
 
 	/* run dialog. */
 
