@@ -79,8 +79,8 @@ update_file_info (GthFileSource *file_source,
 	if (g_str_has_suffix (uri, ".gqv") || g_str_has_suffix (uri, ".catalog")) {
 		g_file_info_set_file_type (info, G_FILE_TYPE_DIRECTORY);
 		g_file_info_set_content_type (info, "gthumb/catalog");
-		icon = g_themed_icon_new ("file-catalog");
-		g_file_info_set_icon (info, icon);
+		icon = g_themed_icon_new ("file-catalog-symbolic");
+		g_file_info_set_symbolic_icon (info, icon);
 		g_file_info_set_sort_order (info, 1);
 		g_file_info_set_attribute_boolean (info, "gthumb::no-child", TRUE);
 		gth_catalog_update_standard_attributes (catalog_file, info);
@@ -88,8 +88,8 @@ update_file_info (GthFileSource *file_source,
 	else if (g_str_has_suffix (uri, ".search")) {
 		g_file_info_set_file_type (info, G_FILE_TYPE_DIRECTORY);
 		g_file_info_set_content_type (info, "gthumb/search");
-		icon = g_themed_icon_new ("file-search");
-		g_file_info_set_icon (info, icon);
+		icon = g_themed_icon_new ("file-search-symbolic");
+		g_file_info_set_symbolic_icon (info, icon);
 		g_file_info_set_sort_order (info, 1);
 		g_file_info_set_attribute_boolean (info, "gthumb::no-child", TRUE);
 		gth_catalog_update_standard_attributes (catalog_file, info);
@@ -97,8 +97,8 @@ update_file_info (GthFileSource *file_source,
 	else {
 		g_file_info_set_file_type (info, G_FILE_TYPE_DIRECTORY);
 		g_file_info_set_content_type (info, "gthumb/library");
-		icon = g_themed_icon_new ("file-library");
-		g_file_info_set_icon (info, icon);
+		icon = g_themed_icon_new ("file-library-symbolic");
+		g_file_info_set_symbolic_icon (info, icon);
 		g_file_info_set_sort_order (info, 0);
 		g_file_info_set_attribute_boolean (info, "gthumb::no-child", FALSE);
 		gth_catalog_update_standard_attributes (catalog_file, info);
@@ -253,14 +253,14 @@ write_metadata_load_buffer_ready_cb (void     **buffer,
 
 	catalog_buffer = gth_catalog_to_data (metadata_op->catalog, &catalog_size);
 	gio_file = gth_catalog_file_to_gio_file (metadata_op->file_data->file);
-	g_write_file_async (gio_file,
-			    catalog_buffer,
-			    catalog_size,
-			    TRUE,
-			    G_PRIORITY_DEFAULT,
-			    gth_file_source_get_cancellable (GTH_FILE_SOURCE (metadata_op->catalogs)),
-			    write_metadata_write_buffer_ready_cb,
-			    metadata_op);
+	_g_file_write_async (gio_file,
+			     catalog_buffer,
+			     catalog_size,
+			     TRUE,
+			     G_PRIORITY_DEFAULT,
+			     gth_file_source_get_cancellable (GTH_FILE_SOURCE (metadata_op->catalogs)),
+			     write_metadata_write_buffer_ready_cb,
+			     metadata_op);
 
 	g_object_unref (gio_file);
 }
@@ -301,11 +301,11 @@ gth_file_source_catalogs_write_metadata (GthFileSource *file_source,
 	metadata_op->catalog = gth_catalog_new ();
 	gio_file = gth_file_source_to_gio_file (file_source, file_data->file);
 	gth_catalog_set_file (metadata_op->catalog, gio_file);
-	g_load_file_async (gio_file,
-			   G_PRIORITY_DEFAULT,
-			   gth_file_source_get_cancellable (file_source),
-			   write_metadata_load_buffer_ready_cb,
-			   metadata_op);
+	_g_file_load_async (gio_file,
+			    G_PRIORITY_DEFAULT,
+			    gth_file_source_get_cancellable (file_source),
+			    write_metadata_load_buffer_ready_cb,
+			    metadata_op);
 
 	g_object_unref (gio_file);
 	g_free (uri);
@@ -837,14 +837,14 @@ catalog_ready_cb (GObject  *catalog,
 
 	buffer = gth_catalog_to_data (cod->catalog, &size);
 	gio_file = gth_catalog_file_to_gio_file (cod->destination->file);
-	g_write_file_async (gio_file,
-			    buffer,
-			    size,
-			    TRUE,
-			    G_PRIORITY_DEFAULT,
-			    NULL,
-			    copy__catalog_save_done_cb,
-			    cod);
+	_g_file_write_async (gio_file,
+			     buffer,
+			     size,
+			     TRUE,
+			     G_PRIORITY_DEFAULT,
+			     NULL,
+			     copy__catalog_save_done_cb,
+			     cod);
 
 	g_object_unref (gio_file);
 }
@@ -956,10 +956,10 @@ copy_catalog_ready_cb (GError   *error,
 
 		d = _gtk_message_dialog_new (NULL,
 					     GTK_DIALOG_MODAL,
-					     GTK_STOCK_DIALOG_QUESTION,
+					     _GTK_ICON_NAME_DIALOG_QUESTION,
 					     msg,
 					     NULL,
-					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					     _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
 					     _("Over_write"), GTK_RESPONSE_OK,
 					     NULL);
 		g_signal_connect (d,
@@ -1083,10 +1083,10 @@ gth_file_source_catalogs_copy (GthFileSource    *file_source,
 				msg = _("Cannot copy the files");
 			d = _gtk_message_dialog_new (NULL,
 						     GTK_DIALOG_MODAL,
-						     GTK_STOCK_DIALOG_ERROR,
+						     _GTK_ICON_NAME_DIALOG_ERROR,
 						     msg,
 						     _("Invalid destination."),
-						     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+						     _GTK_LABEL_CLOSE, GTK_RESPONSE_CLOSE,
 						     NULL);
 			g_signal_connect (d,
 					  "response",
@@ -1255,14 +1255,14 @@ reorder_catalog_ready_cb (GObject  *object,
 
 	buffer = gth_catalog_to_data (catalog, &size);
 	gio_file = gth_file_source_to_gio_file (reorder_data->file_source, reorder_data->destination->file);
-	g_write_file_async (gio_file,
-			    buffer,
-			    size,
-			    TRUE,
-			    G_PRIORITY_DEFAULT,
-			    gth_file_source_get_cancellable (reorder_data->file_source),
-			    reorder_buffer_ready_cb,
-			    reorder_data);
+	_g_file_write_async (gio_file,
+			     buffer,
+			     size,
+			     TRUE,
+			     G_PRIORITY_DEFAULT,
+			     gth_file_source_get_cancellable (reorder_data->file_source),
+			     reorder_buffer_ready_cb,
+			     reorder_data);
 
 	g_object_unref (gio_file);
 }
@@ -1399,14 +1399,14 @@ catalog_buffer_ready_cb (void     **buffer,
 		return;
 	}
 
-	g_write_file_async (data->gio_file,
-			    catalog_buffer,
-			    catalog_size,
-			    TRUE,
-			    G_PRIORITY_DEFAULT,
-			    NULL,
-			    remove_files__catalog_save_done_cb,
-			    data);
+	_g_file_write_async (data->gio_file,
+			     catalog_buffer,
+			     catalog_size,
+			     TRUE,
+			     G_PRIORITY_DEFAULT,
+			     NULL,
+			     remove_files__catalog_save_done_cb,
+			     data);
 }
 
 
@@ -1422,11 +1422,11 @@ gth_catalog_manager_remove_files (GtkWindow   *parent,
 	data->file_data_list = gth_file_data_list_dup (file_list);
 	data->gio_file = gth_main_get_gio_file (location->file);
 
-	g_load_file_async (data->gio_file,
-			   G_PRIORITY_DEFAULT,
-			   NULL,
-			   catalog_buffer_ready_cb,
-			   data);
+	_g_file_load_async (data->gio_file,
+			    G_PRIORITY_DEFAULT,
+			    NULL,
+			    catalog_buffer_ready_cb,
+			    data);
 }
 
 
