@@ -22,6 +22,7 @@
 #include <config.h>
 #include <gtk/gtk.h>
 #include <gthumb.h>
+#include "dlg-change-date.h"
 #include "gth-change-date-task.h"
 #include "preferences.h"
 
@@ -70,8 +71,6 @@ ok_button_clicked (GtkWidget  *button,
 		change_fields |= GTH_CHANGE_LAST_MODIFIED_DATE;
 	if (IS_ACTIVE (GET_WIDGET ("change_comment_checkbutton")))
 		change_fields |= GTH_CHANGE_COMMENT_DATE;
-	if (IS_ACTIVE (GET_WIDGET ("change_datetimeoriginal_checkbutton")))
-		change_fields |= GTH_CHANGE_EXIF_DATETIMEORIGINAL_TAG;
 
 	change_type = 0;
 	time_adjustment = 0;
@@ -99,7 +98,6 @@ ok_button_clicked (GtkWidget  *button,
 
 	g_settings_set_boolean (data->settings, PREF_CHANGE_DATE_SET_LAST_MODIFIED_DATE, (change_fields & GTH_CHANGE_LAST_MODIFIED_DATE) == GTH_CHANGE_LAST_MODIFIED_DATE);
 	g_settings_set_boolean (data->settings, PREF_CHANGE_DATE_SET_COMMENT_DATE, (change_fields & GTH_CHANGE_COMMENT_DATE) == GTH_CHANGE_COMMENT_DATE);
-	g_settings_set_boolean (data->settings, PREF_CHANGE_DATE_SET_EXIF_DATETIMEORIGINAL_TAG, (change_fields & GTH_CHANGE_EXIF_DATETIMEORIGINAL_TAG) == GTH_CHANGE_EXIF_DATETIMEORIGINAL_TAG);
 
 	g_settings_set_boolean (data->settings, PREF_CHANGE_DATE_TO_FOLLOWING_DATE, change_type == GTH_CHANGE_TO_FOLLOWING_DATE);
 	if (change_type == GTH_CHANGE_TO_FOLLOWING_DATE) {
@@ -131,21 +129,12 @@ ok_button_clicked (GtkWidget  *button,
 }
 
 
-static void
-help_button_cb (GtkWidget  *widget,
-		DialogData *data)
-{
-	show_help_dialog (GTK_WINDOW (data->dialog), "gthumb-batch-change-date");
-}
-
-
 static void
 update_sensitivity (DialogData *data)
 {
 	gtk_widget_set_sensitive (GET_WIDGET ("ok_button"),
 				  (IS_ACTIVE (GET_WIDGET ("change_last_modified_checkbutton"))
-				   || IS_ACTIVE (GET_WIDGET ("change_comment_checkbutton"))
-				   || IS_ACTIVE (GET_WIDGET ("change_datetimeoriginal_checkbutton"))));
+				   || IS_ACTIVE (GET_WIDGET ("change_comment_checkbutton"))));
 	gtk_widget_set_sensitive (data->date_selector, IS_ACTIVE (GET_WIDGET ("to_following_date_radiobutton")));
 	gtk_widget_set_sensitive (GET_WIDGET ("time_box"), IS_ACTIVE (GET_WIDGET ("adjust_time_radiobutton")));
 
@@ -195,8 +184,6 @@ dlg_change_date (GthBrowser *browser,
 				      g_settings_get_boolean (data->settings, PREF_CHANGE_DATE_SET_LAST_MODIFIED_DATE));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("change_comment_checkbutton")),
 				      g_settings_get_boolean (data->settings, PREF_CHANGE_DATE_SET_COMMENT_DATE));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("change_datetimeoriginal_checkbutton")),
-				      g_settings_get_boolean (data->settings, PREF_CHANGE_DATE_SET_EXIF_DATETIMEORIGINAL_TAG));
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("to_following_date_radiobutton")),
 				      g_settings_get_boolean (data->settings, PREF_CHANGE_DATE_TO_FOLLOWING_DATE));
@@ -270,10 +257,6 @@ dlg_change_date (GthBrowser *browser,
 			  "clicked",
 			  G_CALLBACK (ok_button_clicked),
 			  data);
-	g_signal_connect (GET_WIDGET ("help_button"),
-			  "clicked",
-			  G_CALLBACK (help_button_cb),
-			  data);
 	g_signal_connect (GET_WIDGET ("change_last_modified_checkbutton"),
 			  "clicked",
 			  G_CALLBACK (radio_button_clicked),
@@ -282,10 +265,6 @@ dlg_change_date (GthBrowser *browser,
 			  "clicked",
 			  G_CALLBACK (radio_button_clicked),
 			  data);
-	g_signal_connect (GET_WIDGET ("change_datetimeoriginal_checkbutton"),
-			  "clicked",
-			  G_CALLBACK (radio_button_clicked),
-			  data);
 	g_signal_connect (GET_WIDGET ("to_following_date_radiobutton"),
 			  "clicked",
 			  G_CALLBACK (radio_button_clicked),
