@@ -59,6 +59,7 @@ typedef struct {
 	GtkBuilder *builder;
 	GtkWidget  *dialog;
 	GtkWidget  *thumbnail_caption_chooser;
+	gulong      theme_selection_changed_event;
 } DialogData;
 
 
@@ -91,10 +92,22 @@ destroy_cb (GtkWidget  *widget,
 
 
 static void
-help_clicked_cb (GtkWidget  *widget,
-		 DialogData *data)
+close_dialog (DialogData *data)
 {
-	show_help_dialog (GTK_WINDOW (data->dialog), "contact-sheet");
+	if (data->theme_selection_changed_event != 0) {
+		g_signal_handler_disconnect (GET_WIDGET ("theme_iconview"), data->theme_selection_changed_event);
+		data->theme_selection_changed_event = 0;
+	}
+	gtk_widget_destroy (data->dialog);
+}
+
+
+static void
+dialog_delete_event_cb (GtkWidget *widget,
+			GdkEvent  *event,
+			gpointer   user_data)
+{
+	close_dialog (user_data);
 }
 
 
@@ -236,8 +249,9 @@ ok_clicked_cb (GtkWidget  *widget,
 	gth_contact_sheet_creator_set_thumbnail_caption (GTH_CONTACT_SHEET_CREATOR (task), thumbnail_caption);
 
 	gth_browser_exec_task (data->browser, task, FALSE);
-	gtk_widget_destroy (data->dialog);
+	close_dialog (data);
 
+	g_object_unref (task);
 	g_free (thumbnail_caption);
 	g_free (theme_name);
 	g_free (file_extension);
@@ -720,7 +734,7 @@ dlg_contact_sheet (GthBrowser *browser,
 		else
 			s_value = g_strdup (get_home_uri ());
 	}
-	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")), s_value);
+	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")), s_value);
 	g_free (s_value);
 
 	s_value = _g_settings_get_uri (data->settings, PREF_CONTACT_SHEET_TEMPLATE);
@@ -814,18 +828,18 @@ dlg_contact_sheet (GthBrowser *browser,
 			  "destroy",
 			  G_CALLBACK (destroy_cb),
 			  data);
+	g_signal_connect (G_OBJECT (data->dialog),
+			  "delete-event",
+			  G_CALLBACK (dialog_delete_event_cb),
+			  data);
 	g_signal_connect (GET_WIDGET ("ok_button"),
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
-        g_signal_connect (GET_WIDGET ("help_button"),
-                          "clicked",
-                          G_CALLBACK (help_clicked_cb),
-                          data);
 	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
 				  "clicked",
-				  G_CALLBACK (gtk_widget_destroy),
-				  data->dialog);
+				  G_CALLBACK (close_dialog),
+				  data);
 	g_signal_connect (GET_WIDGET ("footer_entry"),
 			  "icon-press",
 			  G_CALLBACK (entry_help_icon_press_cb),
@@ -850,10 +864,11 @@ dlg_contact_sheet (GthBrowser *browser,
 			  "clicked",
 			  G_CALLBACK (delete_theme_button_clicked_cb),
 			  data);
-	g_signal_connect (GET_WIDGET ("theme_iconview"),
-			  "selection-changed",
-			  G_CALLBACK (theme_iconview_selection_changed_cb),
-			  data);
+	data->theme_selection_changed_event =
+			g_signal_connect (GET_WIDGET ("theme_iconview"),
+					  "selection-changed",
+					  G_CALLBACK (theme_iconview_selection_changed_cb),
+					  data);
 
 	/* Run dialog. */
 
