@@ -82,14 +82,6 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
-static void
-help_clicked_cb (GtkWidget  *widget,
-		 DialogData *data)
-{
-	show_help_dialog (GTK_WINDOW (data->dialog), "image-wall");
-}
-
-
 static void
 ok_clicked_cb (GtkWidget  *widget,
 	       DialogData *data)
@@ -184,6 +176,7 @@ ok_clicked_cb (GtkWidget  *widget,
 	gth_browser_exec_task (data->browser, task, FALSE);
 	gtk_widget_destroy (data->dialog);
 
+	g_object_unref (task);
 	gth_contact_sheet_theme_unref (theme);
 	g_free (file_extension);
 	g_free (mime_type);
@@ -259,7 +252,7 @@ dlg_image_wall (GthBrowser *browser,
 		else
 			s_value = g_strdup (get_home_uri ());
 	}
-	gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")), s_value);
+	gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")), s_value);
 	g_free (s_value);
 
 	s_value = _g_settings_get_uri (data->settings, PREF_IMAGE_WALL_TEMPLATE);
@@ -349,10 +342,6 @@ dlg_image_wall (GthBrowser *browser,
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
-        g_signal_connect (GET_WIDGET ("help_button"),
-                          "clicked",
-                          G_CALLBACK (help_clicked_cb),
-                          data);
 	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
