@@ -176,6 +176,27 @@ const char *_RATING_TAG_NAMES[] = {
 	NULL
 };
 
+const char *_AUTHOR_TAG_NAMES[] = {
+	"Exif::Image::Artist",
+	"Iptc::Application2::Byline",
+	"Xmp::dc::creator",
+	"Xmp::xmpDM::artist",
+	"Xmp::tiff::Artist",
+	"Xmp::plus::ImageCreator",
+	"Xmp::plus::ImageCreatorName",
+	NULL
+};
+
+const char *_COPYRIGHT_TAG_NAMES[] = {
+	"Exif::Image::Copyright",
+	"Iptc::Application2::Copyright",
+	"Xmp::dc::rights",
+	"Xmp::xmpDM::copyright",
+	"Xmp::tiff::Copyright",
+	"Xmp::plus::CopyrightOwner",
+	"Xmp::plus::CopyrightOwnerName",
+	NULL
+};
 
 /* Some evil camera fill in the ImageDescription or UserComment fields
    with useless fluff. Try to filter these out, so they do not show up
@@ -457,9 +478,8 @@ set_attribute_from_metadata (GFileInfo  *info,
 }
 
 
-static void
-set_attribute_from_tagset (GFileInfo  *info,
-			   const char *attribute,
+static GObject *
+get_attribute_from_tagset (GFileInfo  *info,
 			   const char *tagset[])
 {
 	GObject *metadata;
@@ -469,9 +489,21 @@ set_attribute_from_tagset (GFileInfo  *info,
 	for (i = 0; tagset[i] != NULL; i++) {
 		metadata = g_file_info_get_attribute_object (info, tagset[i]);
 		if (metadata != NULL)
-			break;
+			return metadata;
 	}
 
+	return NULL;
+}
+
+
+static void
+set_attribute_from_tagset (GFileInfo  *info,
+			   const char *attribute,
+			   const char *tagset[])
+{
+	GObject *metadata;
+
+	metadata = get_attribute_from_tagset (info, tagset);
 	if (metadata != NULL)
 		set_attribute_from_metadata (info, attribute, metadata);
 }
@@ -575,6 +607,9 @@ exiv2_update_general_attributes (GFileInfo *info)
 }
 
 
+#define EXPOSURE_SEPARATOR " · "
+
+
 static void
 set_attributes_from_tagsets (GFileInfo *info,
 			     gboolean   update_general_attributes)
@@ -587,6 +622,87 @@ set_attributes_from_tagsets (GFileInfo *info,
 
 	set_attribute_from_tagset (info, "Embedded::Photo::DateTimeOriginal", _ORIGINAL_DATE_TAG_NAMES);
 	set_attribute_from_tagset (info, "Embedded::Image::Orientation", _ORIENTATION_TAG_NAMES);
+
+	set_attribute_from_tagset (info, "Embedded::Photo::Aperture", _APERTURE_TAG_NAMES);
+	set_attribute_from_tagset (info, "Embedded::Photo::ISOSpeed", _ISOSPEED_TAG_NAMES);
+	set_attribute_from_tagset (info, "Embedded::Photo::ExposureTime", _EXPOSURE_TIME_TAG_NAMES);
+	set_attribute_from_tagset (info, "Embedded::Photo::ShutterSpeed", _SHUTTER_SPEED_TAG_NAMES);
+	set_attribute_from_tagset (info, "Embedded::Photo::FocalLength", _FOCAL_LENGTH_TAG_NAMES);
+	set_attribute_from_tagset (info, "Embedded::Photo::Flash", _FLASH_TAG_NAMES);
+	set_attribute_from_tagset (info, "Embedded::Photo::CameraModel", _MODEL_TAG_NAMES);
+	set_attribute_from_tagset (info, "Embedded::Photo::Author", _AUTHOR_TAG_NAMES);
+	set_attribute_from_tagset (info, "Embedded::Photo::Copyright", _COPYRIGHT_TAG_NAMES);
+
+	/* Embedded::Photo::Exposure */
+
+	GObject *aperture;
+	GObject *iso_speed;
+	GObject *shutter_speed;
+	GObject *exposure_time;
+	GString *exposure;
+
+	aperture = get_attribute_from_tagset (info, _APERTURE_TAG_NAMES);
+	iso_speed = get_attribute_from_tagset (info, _ISOSPEED_TAG_NAMES);
+	shutter_speed = get_attribute_from_tagset (info, _SHUTTER_SPEED_TAG_NAMES);
+	exposure_time = get_attribute_from_tagset (info, _EXPOSURE_TIME_TAG_NAMES);
+
+	exposure = g_string_new ("");
+
+	if (aperture != NULL) {
+		char *formatted_value;
+
+		g_object_get (aperture, "formatted", &formatted_value, NULL);
+		if (formatted_value != NULL) {
+			g_string_append (exposure, formatted_value);
+			g_free (formatted_value);
+		}
+	}
+
+	if (iso_speed != NULL) {
+		char *formatted_value;
+
+		g_object_get (iso_speed, "formatted", &formatted_value, NULL);
+		if (formatted_value != NULL) {
+			if (exposure->len > 0)
+				g_string_append (exposure, EXPOSURE_SEPARATOR);
+			g_string_append (exposure, "ISO ");
+			g_string_append (exposure, formatted_value);
+			g_free (formatted_value);
+		}
+	}
+
+	if (shutter_speed != NULL) {
+		char *formatted_value;
+
+		g_object_get (shutter_speed, "formatted", &formatted_value, NULL);
+		if (formatted_value != NULL) {
+			if (exposure->len > 0)
+				g_string_append (exposure, EXPOSURE_SEPARATOR);
+			g_string_append (exposure, formatted_value);
+			g_free (formatted_value);
+		}
+	}
+	else if (exposure_time != NULL) {
+		char *formatted_value;
+
+		g_object_get (exposure_time, "formatted", &formatted_value, NULL);
+		if (formatted_value != NULL) {
+			if (exposure->len > 0)
+				g_string_append (exposure, EXPOSURE_SEPARATOR);
+			g_string_append (exposure, formatted_value);
+			g_free (formatted_value);
+		}
+	}
+
+	set_file_info (info,
+		       "Embedded::Photo::Exposure",
+		       _("Exposure"),
+		       exposure->str,
+		       NULL,
+		       NULL,
+		       NULL);
+
+	g_string_free (exposure, TRUE);
 }
 
 
@@ -1116,6 +1232,7 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 		ed["Exif.Image.DateTime"] = date_time;
 		g_free (date_time);
 	}
+	ed.sortByKey();
 
 	// IPTC Data
 
@@ -1164,6 +1281,7 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 
 		g_free (key);
 	}
+	id.sortByKey();
 	g_strfreev (attributes);
 
 	// XMP Data
@@ -1174,13 +1292,6 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 		gpointer  metadatum = (GthMetadata *) g_file_info_get_attribute_object (info, attributes[i]);
 		char     *key = exiv2_key_from_attribute (attributes[i]);
 
-		// Remove existing tags of the same type.
-		// Seems to be needed for storing category keywords.
-		// Not exactly sure why!
-		/*Exiv2::XmpData::iterator iter = xd.findKey (Exiv2::XmpKey (key));
-		if (iter != xd.end ())
-			xd.erase (iter);*/
-
 		try {
 			const char *value_type;
 
@@ -1220,6 +1331,7 @@ exiv2_write_metadata_private (Exiv2::Image::AutoPtr  image,
 
 		g_free (key);
 	}
+	xd.sortByKey();
 	g_strfreev (attributes);
 
 	try {
