@@ -22,12 +22,11 @@
 #include <config.h>
 #include <gtk/gtk.h>
 #include <gthumb.h>
+#include <extensions/oauth/oauth.h>
 #include "dlg-export-to-facebook.h"
-#include "facebook-authentication.h"
 #include "facebook-album.h"
 #include "facebook-album-properties-dialog.h"
 #include "facebook-service.h"
-#include "facebook-user.h"
 #include "preferences.h"
 
 
@@ -64,10 +63,7 @@ typedef struct {
 	GtkWidget              *dialog;
 	GtkWidget              *list_view;
 	GtkWidget              *progress_dialog;
-	FacebookConnection     *conn;
-	FacebookAuthentication *auth;
 	FacebookService        *service;
-	FacebookUser           *user;
 	GList                  *albums;
 	FacebookAlbum          *album;
 	GList                  *photos_ids;
@@ -80,17 +76,15 @@ destroy_dialog (DialogData *data)
 {
 	if (data->dialog != NULL)
 		gtk_widget_destroy (data->dialog);
-	if (data->conn != NULL)
-		gth_task_completed (GTH_TASK (data->conn), NULL);
+	if (data->service != NULL)
+		gth_task_completed (GTH_TASK (data->service), NULL);
 	_g_object_unref (data->cancellable);
 	_g_string_list_free (data->photos_ids);
 	_g_object_unref (data->album);
 	_g_object_list_unref (data->albums);
-	_g_object_unref (data->user);
 	_g_object_unref (data->service);
-	_g_object_unref (data->auth);
-	_g_object_unref (data->conn);
 	_g_object_unref (data->settings);
+	gtk_widget_destroy (data->progress_dialog);
 	_g_object_unref (data->builder);
 	_g_object_list_unref (data->file_list);
 	_g_object_unref (data->location);
@@ -125,8 +119,7 @@ completed_messagedialog_response_cb (GtkDialog *dialog,
 				url = g_strdup (data->album->link);
 
 			if ((url != NULL) && ! gtk_show_uri (screen, url, 0, &error)) {
-				if (data->conn != NULL)
-					gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
+				gth_task_dialog (GTH_TASK (data->service), TRUE, NULL);
 				_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->browser), _("Could not connect to the server"), error);
 				g_clear_error (&error);
 			}
@@ -149,7 +142,7 @@ export_completed_with_success (DialogData *data)
 	GtkBuilder *builder;
 	GtkWidget  *dialog;
 
-	gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
+	gth_task_dialog (GTH_TASK (data->service), TRUE, NULL);
 
 	builder = _gtk_builder_new_from_file ("facebook-export-completed.ui", "facebook");
 	dialog = _gtk_builder_get_widget (builder, "completed_messagedialog");
@@ -177,7 +170,7 @@ upload_photos_ready_cb (GObject      *source_object,
 	DialogData *data = user_data;
 	GError     *error = NULL;
 
-	data->photos_ids = facebook_service_upload_photos_finish (FACEBOOK_SERVICE (source_object), result, &error);
+	data->photos_ids = facebook_service_upload_photos_finish (data->service, result, &error);
 	if (error != NULL) {
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not upload the files"), error);
 		g_clear_error (&error);
@@ -197,10 +190,6 @@ export_dialog_response_cb (GtkDialog *dialog,
 	DialogData *data = user_data;
 
 	switch (response_id) {
-	case GTK_RESPONSE_HELP:
-		show_help_dialog (GTK_WINDOW (data->browser), "gthumb-export-social");
-		break;
-
 	case GTK_RESPONSE_DELETE_EVENT:
 	case GTK_RESPONSE_CANCEL:
 		gth_file_list_cancel (GTH_FILE_LIST (data->list_view), (DataFunc) destroy_dialog, data);
@@ -212,9 +201,6 @@ export_dialog_response_cb (GtkDialog *dialog,
 			GList       *file_list;
 			int          max_resolution;
 
-			gtk_widget_hide (data->dialog);
-			gth_task_dialog (GTH_TASK (data->conn), FALSE, NULL);
-
 			data->album = NULL;
 			if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (GET_WIDGET ("album_combobox")), &iter)) {
 				gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (GET_WIDGET ("album_combobox"))),
@@ -223,6 +209,12 @@ export_dialog_response_cb (GtkDialog *dialog,
 						    -1);
 			}
 
+			if (data->album == NULL)
+				return;
+
+			gtk_widget_hide (data->dialog);
+			gth_task_dialog (GTH_TASK (data->service), FALSE, NULL);
+
 			max_resolution = 0;
 			if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (GET_WIDGET ("resize_combobox")), &iter)) {
 				gtk_tree_model_get (gtk_combo_box_get_model (GTK_COMBO_BOX (GET_WIDGET ("resize_combobox"))),
@@ -255,18 +247,18 @@ export_dialog_response_cb (GtkDialog *dialog,
 static void
 update_account_list (DialogData *data)
 {
-	int              current_account_idx;
-	FacebookAccount *current_account;
-	int              idx;
-	GList           *scan;
-	GtkTreeIter      iter;
+	int           current_account_idx;
+	OAuthAccount *current_account;
+	int           idx;
+	GList        *scan;
+	GtkTreeIter   iter;
 
 	gtk_list_store_clear (GTK_LIST_STORE (GET_WIDGET ("account_liststore")));
 
 	current_account_idx = 0;
-	current_account = facebook_authentication_get_account (data->auth);
-	for (scan = facebook_authentication_get_accounts (data->auth), idx = 0; scan; scan = scan->next, idx++) {
-		FacebookAccount *account = scan->data;
+	current_account = web_service_get_current_account (WEB_SERVICE (data->service));
+	for (scan = web_service_get_accounts (WEB_SERVICE (data->service)), idx = 0; scan; scan = scan->next, idx++) {
+		OAuthAccount *account = scan->data;
 
 		if ((current_account != NULL) && (g_strcmp0 (current_account->username, account->username) == 0))
 			current_account_idx = idx;
@@ -274,7 +266,7 @@ update_account_list (DialogData *data)
 		gtk_list_store_append (GTK_LIST_STORE (GET_WIDGET ("account_liststore")), &iter);
 		gtk_list_store_set (GTK_LIST_STORE (GET_WIDGET ("account_liststore")), &iter,
 				    ACCOUNT_DATA_COLUMN, account,
-				    ACCOUNT_NAME_COLUMN, account->username,
+				    ACCOUNT_NAME_COLUMN, account->name,
 				    -1);
 	}
 	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("account_combobox")), current_account_idx);
@@ -282,8 +274,8 @@ update_account_list (DialogData *data)
 
 
 static void
-authentication_accounts_changed_cb (FacebookAuthentication *auth,
-				    gpointer              user_data)
+service_accounts_changed_cb (FacebookService *service,
+			     gpointer         user_data)
 {
 	update_account_list ((DialogData *) user_data);
 }
@@ -301,12 +293,12 @@ update_album_list (DialogData    *data,
 		char          *size;
 		GtkTreeIter    iter;
 
-		size = g_strdup_printf ("(%d)", album->size);
+		size = g_strdup_printf ("(%d)", album->count);
 
 		gtk_list_store_append (GTK_LIST_STORE (GET_WIDGET ("album_liststore")), &iter);
 		gtk_list_store_set (GTK_LIST_STORE (GET_WIDGET ("album_liststore")), &iter,
 				    ALBUM_DATA_COLUMN, album,
-				    ALBUM_ICON_COLUMN, "file-catalog",
+				    ALBUM_ICON_COLUMN, "file-catalog-symbolic",
 				    ALBUM_TITLE_COLUMN, album->name,
 				    ALBUM_SIZE_COLUMN, size,
 				    -1);
@@ -328,20 +320,18 @@ get_albums_ready_cb (GObject      *source_object,
 	GError     *error = NULL;
 
 	_g_object_list_unref (data->albums);
-	data->albums = facebook_service_get_albums_finish (FACEBOOK_SERVICE (source_object), res, &error);
+	data->albums = facebook_service_get_albums_finish (data->service, res, &error);
 	if (error != NULL) {
-		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
+		gth_task_dialog (GTH_TASK (data->service), TRUE, NULL);
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (data->browser), _("Could not connect to the server"), error);
 		g_clear_error (&error);
 		gtk_dialog_response (GTK_DIALOG (data->dialog), GTK_RESPONSE_DELETE_EVENT);
 		return;
 	}
 
-	gtk_widget_set_sensitive (GET_WIDGET ("upload_button"), TRUE);
 	update_album_list (data, NULL);
 
-	gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
+	gth_task_dialog (GTH_TASK (data->service), TRUE, NULL);
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), GTK_WINDOW (data->browser));
 	gtk_window_set_modal (GTK_WINDOW (data->dialog), FALSE);
 	gtk_window_present (GTK_WINDOW (data->dialog));
@@ -349,16 +339,11 @@ get_albums_ready_cb (GObject      *source_object,
 
 
 static void
-authentication_ready_cb (FacebookAuthentication *auth,
-			 FacebookUser           *user,
-			 DialogData             *data)
+service_account_ready_cb (FacebookService *service,
+			  DialogData      *data)
 {
-	_g_object_unref (data->user);
-	data->user = g_object_ref (user);
 	update_account_list (data);
-
 	facebook_service_get_albums (data->service,
-				     data->user->id,
 				     data->cancellable,
 				     get_albums_ready_cb,
 				     data);
@@ -369,7 +354,7 @@ static void
 edit_accounts_button_clicked_cb (GtkButton  *button,
 				 DialogData *data)
 {
-	facebook_authentication_edit_accounts (data->auth, GTK_WINDOW (data->dialog));
+	web_service_edit_accounts (WEB_SERVICE (data->service), GTK_WINDOW (data->dialog));
 }
 
 
@@ -377,9 +362,9 @@ static void
 account_combobox_changed_cb (GtkComboBox *widget,
 			     gpointer     user_data)
 {
-	DialogData      *data = user_data;
-	GtkTreeIter      iter;
-	FacebookAccount *account;
+	DialogData   *data = user_data;
+	GtkTreeIter   iter;
+	OAuthAccount *account;
 
 	if (! gtk_combo_box_get_active_iter (widget, &iter))
 		return;
@@ -389,8 +374,8 @@ account_combobox_changed_cb (GtkComboBox *widget,
 			    ACCOUNT_DATA_COLUMN, &account,
 			    -1);
 
-	if (facebook_account_cmp (account, facebook_authentication_get_account (data->auth)) != 0)
-		facebook_authentication_connect (data->auth, account);
+	if (oauth_account_cmp (account, web_service_get_current_account (WEB_SERVICE (data->service))) != 0)
+		web_service_connect (WEB_SERVICE (data->service), account);
 
 	g_object_unref (account);
 }
@@ -407,8 +392,8 @@ create_album_ready_cb (GObject      *source_object,
 
 	album = facebook_service_create_album_finish (data->service, result, &error);
 	if (error != NULL) {
-		if (data->conn != NULL)
-			gth_task_dialog (GTH_TASK (data->conn), TRUE, NULL);
+		if (data->service != NULL)
+			gth_task_dialog (GTH_TASK (data->service), TRUE, NULL);
 		_gtk_error_dialog_from_gerror_show (GTK_WINDOW (data->browser), _("Could not create the album"), error);
 		g_clear_error (&error);
 		return;
@@ -436,11 +421,11 @@ new_album_dialog_response_cb (GtkDialog *dialog,
 		{
 			FacebookAlbum *album;
 
-			album = facebook_album_new ();
-			facebook_album_set_name (album, facebook_album_properties_dialog_get_name (FACEBOOK_ALBUM_PROPERTIES_DIALOG (dialog)));
-			facebook_album_set_location (album, facebook_album_properties_dialog_get_location (FACEBOOK_ALBUM_PROPERTIES_DIALOG (dialog)));
-			facebook_album_set_description (album, facebook_album_properties_dialog_get_description (FACEBOOK_ALBUM_PROPERTIES_DIALOG (dialog)));
-			album->visibility = facebook_album_properties_dialog_get_visibility (FACEBOOK_ALBUM_PROPERTIES_DIALOG (dialog));
+			album = g_object_new (FACEBOOK_TYPE_ALBUM,
+					      "name", facebook_album_properties_dialog_get_name (FACEBOOK_ALBUM_PROPERTIES_DIALOG (dialog)),
+					      "description", facebook_album_properties_dialog_get_description (FACEBOOK_ALBUM_PROPERTIES_DIALOG (dialog)),
+					      "privacy", facebook_album_properties_dialog_get_visibility (FACEBOOK_ALBUM_PROPERTIES_DIALOG (dialog)),
+					      NULL);
 			facebook_service_create_album (data->service,
 						       album,
 						       data->cancellable,
@@ -466,7 +451,6 @@ add_album_button_clicked_cb (GtkButton *button,
 	GtkWidget  *dialog;
 
 	dialog = facebook_album_properties_dialog_new (g_file_info_get_edit_name (data->location->info),
-						       NULL,
 						       NULL,
 						       FACEBOOK_VISIBILITY_SELF);
 	g_signal_connect (dialog,
@@ -485,6 +469,18 @@ add_album_button_clicked_cb (GtkButton *button,
 }
 
 
+static void
+album_combobox_changed_cb (GtkComboBox *combo_box,
+			   gpointer     user_data)
+{
+	DialogData *data = user_data;
+
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (data->dialog),
+					   GTK_RESPONSE_OK,
+					   gtk_combo_box_get_active (combo_box) >= 0);
+}
+
+
 void
 dlg_export_to_facebook (GthBrowser *browser,
 		        GList      *file_list)
@@ -576,7 +572,7 @@ dlg_export_to_facebook (GthBrowser *browser,
 
 	/* Set the widget data */
 
-	data->list_view = gth_file_list_new (gth_grid_view_new (), GTH_FILE_LIST_TYPE_NO_SELECTION, FALSE);
+	data->list_view = gth_file_list_new (gth_grid_view_new (), GTH_FILE_LIST_MODE_NO_SELECTION, FALSE);
 	gth_file_list_set_thumb_size (GTH_FILE_LIST (data->list_view), 112);
 	gth_file_list_enable_thumbs (GTH_FILE_LIST (data->list_view), TRUE);
 	gth_file_list_set_ignore_hidden (GTH_FILE_LIST (data->list_view), TRUE);
@@ -586,7 +582,7 @@ dlg_export_to_facebook (GthBrowser *browser,
 	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("images_box")), data->list_view, TRUE, TRUE, 0);
 	gth_file_list_set_files (GTH_FILE_LIST (data->list_view), data->file_list);
 
-	gtk_widget_set_sensitive (GET_WIDGET ("upload_button"), FALSE);
+	gtk_dialog_set_response_sensitive (GTK_DIALOG (data->dialog), GTK_RESPONSE_OK, FALSE);
 
 	title = g_strdup_printf (_("Export to %s"), "Facebook");
 	gtk_window_set_title (GTK_WINDOW (data->dialog), title);
@@ -642,25 +638,25 @@ dlg_export_to_facebook (GthBrowser *browser,
 			  "clicked",
 			  G_CALLBACK (add_album_button_clicked_cb),
 			  data);
-
-	data->conn = facebook_connection_new ();
-	data->service = facebook_service_new (data->conn);
-	data->auth = facebook_authentication_new (data->conn,
-						  data->service,
-						  data->cancellable,
-						  GTK_WIDGET (data->browser),
-						  data->dialog);
-	g_signal_connect (data->auth,
-			  "ready",
-			  G_CALLBACK (authentication_ready_cb),
+	g_signal_connect (GET_WIDGET ("album_combobox"),
+			  "changed",
+			  G_CALLBACK (album_combobox_changed_cb),
 			  data);
-	g_signal_connect (data->auth,
-			  "accounts_changed",
-			  G_CALLBACK (authentication_accounts_changed_cb),
+
+	data->service = facebook_service_new (data->cancellable,
+					      GTK_WIDGET (data->browser),
+					      data->dialog);
+	g_signal_connect (data->service,
+			  "account-ready",
+			  G_CALLBACK (service_account_ready_cb),
+			  data);
+	g_signal_connect (data->service,
+			  "accounts-changed",
+			  G_CALLBACK (service_accounts_changed_cb),
 			  data);
 
 	data->progress_dialog = gth_progress_dialog_new (GTK_WINDOW (data->browser));
-	gth_progress_dialog_add_task (GTH_PROGRESS_DIALOG (data->progress_dialog), GTH_TASK (data->conn));
+	gth_progress_dialog_add_task (GTH_PROGRESS_DIALOG (data->progress_dialog), GTH_TASK (data->service));
 
-	facebook_authentication_auto_connect (data->auth);
+	web_service_autoconnect (WEB_SERVICE (data->service));
 }
