@@ -21,40 +21,40 @@
 
 #include <config.h>
 #include <math.h>
-#include <gthumb.h>
-#include <extensions/image_viewer/gth-image-viewer-page.h>
 #include "gth-file-tool-adjust-colors.h"
+#include "gth-preview-tool.h"
 
 
 #define GET_WIDGET(x) (_gtk_builder_get_widget (self->priv->builder, (x)))
 #define APPLY_DELAY 150
+#define PREVIEW_SIZE 0.9
 
-
-G_DEFINE_TYPE (GthFileToolAdjustColors, gth_file_tool_adjust_colors, GTH_TYPE_FILE_TOOL)
+G_DEFINE_TYPE (GthFileToolAdjustColors, gth_file_tool_adjust_colors, GTH_TYPE_IMAGE_VIEWER_PAGE_TOOL)
 
 
 struct _GthFileToolAdjustColorsPrivate {
-	cairo_surface_t *source;
-	cairo_surface_t *destination;
-	GtkBuilder      *builder;
-	GtkAdjustment   *gamma_adj;
-	GtkAdjustment   *brightness_adj;
-	GtkAdjustment   *contrast_adj;
-	GtkAdjustment   *saturation_adj;
-	GtkAdjustment   *cyan_red_adj;
-	GtkAdjustment   *magenta_green_adj;
-	GtkAdjustment   *yellow_blue_adj;
-	GtkWidget       *histogram_view;
-	GthHistogram    *histogram;
-	GthTask         *image_task;
-	guint            apply_event;
+	cairo_surface_t    *destination;
+	cairo_surface_t    *preview;
+	GtkBuilder         *builder;
+	GtkAdjustment      *gamma_adj;
+	GtkAdjustment      *brightness_adj;
+	GtkAdjustment      *contrast_adj;
+	GtkAdjustment      *saturation_adj;
+	GtkAdjustment      *cyan_red_adj;
+	GtkAdjustment      *magenta_green_adj;
+	GtkAdjustment      *yellow_blue_adj;
+	GtkWidget          *histogram_view;
+	GthHistogram       *histogram;
+	GthTask            *image_task;
+	guint               apply_event;
+	GthImageViewerTool *preview_tool;
+	gboolean            apply_to_original;
+	gboolean            closing;
 };
 
 
 typedef struct {
 	GthFileToolAdjustColors *self;
-	cairo_surface_t         *source;
-	cairo_surface_t         *destination;
 	GtkWidget               *viewer_page;
 	double                   gamma;
 	double                   brightness;
@@ -103,6 +103,7 @@ adjust_colors_exec (GthAsyncTask *task,
 		    gpointer      user_data)
 {
 	AdjustData      *adjust_data = user_data;
+	cairo_surface_t *source;
 	cairo_format_t   format;
 	int              width;
 	int              height;
@@ -114,22 +115,22 @@ adjust_colors_exec (GthAsyncTask *task,
 	unsigned char   *p_destination;
 	gboolean         cancelled;
 	double           progress;
-	gboolean         terminated;
 	int              x, y;
 	unsigned char    values[4];
 	int              channel;
 	unsigned char    value;
+	cairo_surface_t *destination;
 
-	format = cairo_image_surface_get_format (adjust_data->source);
-	width = cairo_image_surface_get_width (adjust_data->source);
-	height = cairo_image_surface_get_height (adjust_data->source);
-	source_stride = cairo_image_surface_get_stride (adjust_data->source);
+	source = gth_image_task_get_source_surface (GTH_IMAGE_TASK (task));
+	format = cairo_image_surface_get_format (source);
+	width = cairo_image_surface_get_width (source);
+	height = cairo_image_surface_get_height (source);
+	source_stride = cairo_image_surface_get_stride (source);
 
-	adjust_data->destination = cairo_image_surface_create (format, width, height);
-	cairo_surface_flush (adjust_data->destination);
-	destination_stride = cairo_image_surface_get_stride (adjust_data->destination);
-	p_source_line = cairo_image_surface_get_data (adjust_data->source);
-	p_destination_line = cairo_image_surface_get_data (adjust_data->destination);
+	destination = cairo_image_surface_create (format, width, height);
+	destination_stride = cairo_image_surface_get_stride (destination);
+	p_source_line = _cairo_image_surface_flush_and_get_data (source);
+	p_destination_line = _cairo_image_surface_flush_and_get_data (destination);
 	for (y = 0; y < height; y++) {
 		gth_async_task_get_data (task, NULL, &cancelled, NULL);
 		if (cancelled)
@@ -209,64 +210,28 @@ adjust_colors_exec (GthAsyncTask *task,
 		p_destination_line += destination_stride;
 	}
 
-	cairo_surface_mark_dirty (adjust_data->destination);
-	terminated = TRUE;
-	gth_async_task_set_data (task, &terminated, NULL, NULL);
+	cairo_surface_mark_dirty (destination);
+	gth_image_task_set_destination_surface (GTH_IMAGE_TASK (task), destination);
+
+	cairo_surface_destroy (destination);
+	cairo_surface_destroy (source);
 
 	return NULL;
 }
 
 
-static void
-adjust_colors_after (GthAsyncTask *task,
-		     GError       *error,
-		     gpointer      user_data)
-{
-	AdjustData              *adjust_data = user_data;
-	GthFileToolAdjustColors *self = adjust_data->self;
-
-	if (error == NULL) {
-		cairo_surface_destroy (self->priv->destination);
-		self->priv->destination = cairo_surface_reference (adjust_data->destination);
-
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("preview_checkbutton"))))
-			gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (adjust_data->viewer_page), self->priv->destination, FALSE);
-		gth_histogram_calculate_for_image (self->priv->histogram, self->priv->destination);
-	}
-
-	pixbuf_cache_free (adjust_data->cache);
-
-	if (self->priv->image_task == GTH_TASK (task))
-		self->priv->image_task = NULL;
-	g_object_unref (task);
-}
-
-
 static void
 adjust_data_free (gpointer user_data)
 {
 	AdjustData *adjust_data = user_data;
 
-	cairo_surface_destroy (adjust_data->destination);
-	cairo_surface_destroy (adjust_data->source);
+	pixbuf_cache_free (adjust_data->cache);
 	g_object_unref (adjust_data->viewer_page);
 	g_free (adjust_data);
 }
 
 
-static void
-gth_file_tool_adjust_colors_update_sensitivity (GthFileTool *base)
-{
-	GtkWidget *window;
-	GtkWidget *viewer_page;
-
-	window = gth_file_tool_get_window (base);
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	if (! GTH_IS_IMAGE_VIEWER_PAGE (viewer_page))
-		gtk_widget_set_sensitive (GTK_WIDGET (base), FALSE);
-	else
-		gtk_widget_set_sensitive (GTK_WIDGET (base), TRUE);
-}
+static void apply_changes (GthFileToolAdjustColors *self);
 
 
 static void
@@ -275,37 +240,8 @@ ok_button_clicked_cb (GtkButton *button,
 {
 	GthFileToolAdjustColors *self = user_data;
 
-	if (self->priv->destination != NULL) {
-		GtkWidget *window;
-		GtkWidget *viewer_page;
-
-		window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
-		viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-		gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), self->priv->destination, TRUE);
-	}
-
-	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
-}
-
-
-static void
-cancel_button_clicked_cb (GtkButton *button,
-			  gpointer   user_data)
-{
-	GthFileToolAdjustColors *self = user_data;
-	GtkWidget               *window;
-	GtkWidget               *viewer_page;
-
-	if (self->priv->apply_event != 0) {
-		g_source_remove (self->priv->apply_event);
-		self->priv->apply_event = 0;
-	}
-
-	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	gth_image_viewer_page_reset (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-
-	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
+	self->priv->apply_to_original = TRUE;
+	apply_changes (self);
 }
 
 
@@ -325,6 +261,58 @@ reset_button_clicked_cb (GtkButton *button,
 }
 
 
+static void
+image_task_completed_cb (GthTask  *task,
+			 GError   *error,
+			 gpointer  user_data)
+{
+	GthFileToolAdjustColors *self = user_data;
+	GthImage                *destination_image;
+
+	self->priv->image_task = NULL;
+
+	if (self->priv->closing) {
+		g_object_unref (task);
+		gth_image_viewer_page_tool_reset_image (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+		return;
+	}
+
+	if (error != NULL) {
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+			apply_changes (self);
+		g_object_unref (task);
+		return;
+	}
+
+	destination_image = gth_image_task_get_destination (GTH_IMAGE_TASK (task));
+	if (destination_image == NULL) {
+		g_object_unref (task);
+		return;
+	}
+
+	cairo_surface_destroy (self->priv->destination);
+	self->priv->destination = gth_image_get_cairo_surface (destination_image);
+
+	if (self->priv->apply_to_original) {
+		if (self->priv->destination != NULL) {
+			GtkWidget *viewer_page;
+
+			viewer_page = gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+			gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), self->priv->destination, TRUE);
+		}
+
+		gth_file_tool_hide_options (GTH_FILE_TOOL (self));
+	}
+	else {
+		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("preview_checkbutton"))))
+			gth_preview_tool_set_image (GTH_PREVIEW_TOOL (self->priv->preview_tool), self->priv->destination);
+		gth_histogram_calculate_for_image (self->priv->histogram, self->priv->destination);
+	}
+
+	g_object_unref (task);
+}
+
+
 static gboolean
 apply_cb (gpointer user_data)
 {
@@ -337,15 +325,16 @@ apply_cb (gpointer user_data)
 		self->priv->apply_event = 0;
 	}
 
-	if (self->priv->image_task != NULL)
+	if (self->priv->image_task != NULL) {
 		gth_task_cancel (self->priv->image_task);
+		return FALSE;
+	}
 
 	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
 
 	adjust_data = g_new0 (AdjustData, 1);
 	adjust_data->self = self;
 	adjust_data->viewer_page = g_object_ref (gth_browser_get_viewer_page (GTH_BROWSER (window)));
-	adjust_data->source = cairo_surface_reference (self->priv->source);
 	adjust_data->gamma = pow (10, - (gtk_adjustment_get_value (self->priv->gamma_adj) / 100.0));
 	adjust_data->brightness = gtk_adjustment_get_value (self->priv->brightness_adj) / 100.0 * -1.0;
 	adjust_data->contrast = gtk_adjustment_get_value (self->priv->contrast_adj) / 100.0 * -1.0;
@@ -354,11 +343,21 @@ apply_cb (gpointer user_data)
 	adjust_data->color_level[1] = gtk_adjustment_get_value (self->priv->magenta_green_adj);
 	adjust_data->color_level[2] = gtk_adjustment_get_value (self->priv->yellow_blue_adj);
 
-	self->priv->image_task = gth_async_task_new (adjust_colors_before,
+	self->priv->image_task = gth_image_task_new (_("Applying changes"),
+						     adjust_colors_before,
 						     adjust_colors_exec,
-						     adjust_colors_after,
+						     NULL,
 						     adjust_data,
 						     adjust_data_free);
+	if (self->priv->apply_to_original)
+		gth_image_task_set_source_surface (GTH_IMAGE_TASK (self->priv->image_task), gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self)));
+	else
+		gth_image_task_set_source_surface (GTH_IMAGE_TASK (self->priv->image_task), self->priv->preview);
+
+	g_signal_connect (self->priv->image_task,
+			  "completed",
+			  G_CALLBACK (image_task_completed_cb),
+			  self);
 	gth_browser_exec_task (GTH_BROWSER (window), self->priv->image_task, FALSE);
 
 	return FALSE;
@@ -366,8 +365,7 @@ apply_cb (gpointer user_data)
 
 
 static void
-value_changed_cb (GtkAdjustment           *adj,
-		  GthFileToolAdjustColors *self)
+apply_changes (GthFileToolAdjustColors *self)
 {
 	if (self->priv->apply_event != 0) {
 		g_source_remove (self->priv->apply_event);
@@ -377,20 +375,24 @@ value_changed_cb (GtkAdjustment           *adj,
 }
 
 
+static void
+value_changed_cb (GtkAdjustment *adj,
+		  gpointer       user_data)
+{
+	apply_changes (GTH_FILE_TOOL_ADJUST_COLORS (user_data));
+}
+
+
 static void
 preview_checkbutton_toggled_cb (GtkToggleButton *togglebutton,
 				gpointer         user_data)
 {
 	GthFileToolAdjustColors *self = user_data;
-	GtkWidget               *window;
-	GtkWidget               *viewer_page;
 
-	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
 	if (gtk_toggle_button_get_active (togglebutton))
-		gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), self->priv->destination, FALSE);
+		gth_preview_tool_set_image (GTH_PREVIEW_TOOL (self->priv->preview_tool), self->priv->destination);
 	else
-		gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), self->priv->source, FALSE);
+		gth_preview_tool_set_image (GTH_PREVIEW_TOOL (self->priv->preview_tool), self->priv->preview);
 }
 
 
@@ -398,27 +400,38 @@ static GtkWidget *
 gth_file_tool_adjust_colors_get_options (GthFileTool *base)
 {
 	GthFileToolAdjustColors *self;
-	GtkWidget               *window;
 	GtkWidget               *viewer_page;
 	GtkWidget               *viewer;
+	cairo_surface_t         *source;
 	GtkWidget               *options;
+	int                      width, height;
+	GtkAllocation            allocation;
 
 	self = (GthFileToolAdjustColors *) base;
 
-	window = gth_file_tool_get_window (base);
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	if (! GTH_IS_IMAGE_VIEWER_PAGE (viewer_page))
+	viewer_page = gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+	if (viewer_page == NULL)
 		return NULL;
 
-	cairo_surface_destroy (self->priv->source);
 	cairo_surface_destroy (self->priv->destination);
+	cairo_surface_destroy (self->priv->preview);
 
-	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	self->priv->source = cairo_surface_reference (gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (viewer)));
-	if (self->priv->source == NULL)
+	source = gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+	if (source == NULL)
 		return NULL;
 
-	self->priv->destination = NULL;
+	width = cairo_image_surface_get_width (source);
+	height = cairo_image_surface_get_height (source);
+	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
+	gtk_widget_get_allocation (GTK_WIDGET (viewer), &allocation);
+	if (scale_keeping_ratio (&width, &height, PREVIEW_SIZE * allocation.width, PREVIEW_SIZE * allocation.height, FALSE))
+		self->priv->preview = _cairo_image_surface_scale_bilinear (source, width, height);
+	else
+		self->priv->preview = cairo_surface_reference (source);
+
+	self->priv->destination = cairo_surface_reference (self->priv->preview);
+	self->priv->apply_to_original = FALSE;
+	self->priv->closing = FALSE;
 
 	self->priv->builder = _gtk_builder_new_from_file ("adjust-colors-options.ui", "file_tools");
 	options = _gtk_builder_get_widget (self->priv->builder, "options");
@@ -428,36 +441,43 @@ gth_file_tool_adjust_colors_get_options (GthFileTool *base)
 	gtk_widget_show (self->priv->histogram_view);
 	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("histogram_hbox")), self->priv->histogram_view, TRUE, TRUE, 0);
 
-	self->priv->brightness_adj    = gimp_scale_entry_new (GET_WIDGET ("brightness_hbox"),
-							      GTK_LABEL (GET_WIDGET ("brightness_label")),
-							      0.0, -100.0, 100.0, 1.0, 10.0, 0);
-	self->priv->contrast_adj      = gimp_scale_entry_new (GET_WIDGET ("contrast_hbox"),
-							      GTK_LABEL (GET_WIDGET ("contrast_label")),
-							      0.0, -100.0, 100.0, 1.0, 10.0, 0);
-	self->priv->gamma_adj         = gimp_scale_entry_new (GET_WIDGET ("gamma_hbox"),
-							      GTK_LABEL (GET_WIDGET ("gamma_label")),
-							      0.0, -100.0, 100.0, 1.0, 10.0, 0);
-	self->priv->saturation_adj    = gimp_scale_entry_new (GET_WIDGET ("saturation_hbox"),
-							      GTK_LABEL (GET_WIDGET ("saturation_label")),
-							      0.0, -100.0, 100.0, 1.0, 10.0, 0);
-	self->priv->cyan_red_adj      = gimp_scale_entry_new (GET_WIDGET ("cyan_red_hbox"),
-							      GTK_LABEL (GET_WIDGET ("cyan_red_label")),
-							      0.0, -100.0, 100.0, 1.0, 10.0, 0);
-	self->priv->magenta_green_adj = gimp_scale_entry_new (GET_WIDGET ("magenta_green_hbox"),
-							      GTK_LABEL (GET_WIDGET ("magenta_green_label")),
-							      0.0, -100.0, 100.0, 1.0, 10.0, 0);
-	self->priv->yellow_blue_adj   = gimp_scale_entry_new (GET_WIDGET ("yellow_blue_hbox"),
-							      GTK_LABEL (GET_WIDGET ("yellow_blue_label")),
-							      0.0, -100.0, 100.0, 1.0, 10.0, 0);
+	self->priv->brightness_adj    = gth_color_scale_label_new (GET_WIDGET ("brightness_hbox"),
+								   GTK_LABEL (GET_WIDGET ("brightness_label")),
+								   GTH_COLOR_SCALE_BLACK_WHITE,
+								   0.0, -99.0, 99.0, 1.0, 1.0, "%+.0f");
+	self->priv->contrast_adj      = gth_color_scale_label_new (GET_WIDGET ("contrast_hbox"),
+								   GTK_LABEL (GET_WIDGET ("contrast_label")),
+								   GTH_COLOR_SCALE_GRAY_BLACK,
+								   0.0, -99.0, 99.0, 1.0, 1.0, "%+.0f");
+	self->priv->gamma_adj         = gth_color_scale_label_new (GET_WIDGET ("gamma_hbox"),
+								   GTK_LABEL (GET_WIDGET ("gamma_label")),
+								   GTH_COLOR_SCALE_WHITE_BLACK,
+								   0.0, -99.0, 99.0, 1.0, 1.0, "%+.0f");
+	self->priv->saturation_adj    = gth_color_scale_label_new (GET_WIDGET ("saturation_hbox"),
+								   GTK_LABEL (GET_WIDGET ("saturation_label")),
+								   GTH_COLOR_SCALE_GRAY_WHITE,
+								   0.0, -99.0, 99.0, 1.0, 1.0, "%+.0f");
+	self->priv->cyan_red_adj      = gth_color_scale_label_new (GET_WIDGET ("cyan_red_hbox"),
+								   GTK_LABEL (GET_WIDGET ("cyan_red_label")),
+								   GTH_COLOR_SCALE_CYAN_RED,
+								   0.0, -99.0, 99.0, 1.0, 1.0, "%+.0f");
+	self->priv->magenta_green_adj = gth_color_scale_label_new (GET_WIDGET ("magenta_green_hbox"),
+								   GTK_LABEL (GET_WIDGET ("magenta_green_label")),
+								   GTH_COLOR_SCALE_MAGENTA_GREEN,
+								   0.0, -99.0, 99.0, 1.0, 1.0, "%+.0f");
+	self->priv->yellow_blue_adj   = gth_color_scale_label_new (GET_WIDGET ("yellow_blue_hbox"),
+								   GTK_LABEL (GET_WIDGET ("yellow_blue_label")),
+								   GTH_COLOR_SCALE_YELLOW_BLUE,
+								   0.0, -99.0, 99.0, 1.0, 1.0, "%+.0f");
 
 	g_signal_connect (GET_WIDGET ("ok_button"),
 			  "clicked",
 			  G_CALLBACK (ok_button_clicked_cb),
 			  self);
-	g_signal_connect (GET_WIDGET ("cancel_button"),
-			  "clicked",
-			  G_CALLBACK (cancel_button_clicked_cb),
-			  self);
+	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
+				  "clicked",
+				  G_CALLBACK (gth_file_tool_cancel),
+				  self);
 	g_signal_connect (GET_WIDGET ("reset_button"),
 			  "clicked",
 			  G_CALLBACK (reset_button_clicked_cb),
@@ -495,7 +515,10 @@ gth_file_tool_adjust_colors_get_options (GthFileTool *base)
 			  G_CALLBACK (preview_checkbutton_toggled_cb),
 			  self);
 
-	gth_histogram_calculate_for_image (self->priv->histogram, self->priv->source);
+	self->priv->preview_tool = gth_preview_tool_new ();
+	gth_preview_tool_set_image (GTH_PREVIEW_TOOL (self->priv->preview_tool), self->priv->preview);
+	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (viewer), self->priv->preview_tool);
+	gth_histogram_calculate_for_image (self->priv->histogram, self->priv->preview);
 
 	return options;
 }
@@ -505,6 +528,7 @@ static void
 gth_file_tool_adjust_colors_destroy_options (GthFileTool *base)
 {
 	GthFileToolAdjustColors *self;
+	GtkWidget               *viewer_page;
 
 	self = (GthFileToolAdjustColors *) base;
 
@@ -513,19 +537,34 @@ gth_file_tool_adjust_colors_destroy_options (GthFileTool *base)
 		self->priv->apply_event = 0;
 	}
 
-	cairo_surface_destroy (self->priv->source);
-	cairo_surface_destroy (self->priv->destination);
-	_g_object_unref (self->priv->builder);
-	self->priv->source = NULL;
-	self->priv->destination = NULL;
-	self->priv->builder = NULL;
+	viewer_page = gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+	gth_image_viewer_page_reset_viewer_tool (GTH_IMAGE_VIEWER_PAGE (viewer_page));
+	gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (viewer_page));
+
+	_cairo_clear_surface (&self->priv->preview);
+	_cairo_clear_surface (&self->priv->destination);
+	_g_clear_object (&self->priv->builder);
 }
 
 
 static void
-gth_file_tool_adjust_colors_activate (GthFileTool *base)
+gth_file_tool_sharpen_reset_image (GthImageViewerPageTool *base)
 {
-	gth_file_tool_show_options (base);
+	GthFileToolAdjustColors *self = (GthFileToolAdjustColors *) base;
+
+	if (self->priv->image_task != NULL) {
+		self->priv->closing = TRUE;
+		gth_task_cancel (self->priv->image_task);
+		return;
+	}
+
+	if (self->priv->apply_event != 0) {
+		g_source_remove (self->priv->apply_event);
+		self->priv->apply_event = 0;
+	}
+
+	gth_image_viewer_page_reset (GTH_IMAGE_VIEWER_PAGE (gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self))));
+	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
 }
 
 
@@ -534,8 +573,12 @@ gth_file_tool_adjust_colors_init (GthFileToolAdjustColors *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_FILE_TOOL_ADJUST_COLORS, GthFileToolAdjustColorsPrivate);
 	self->priv->histogram = gth_histogram_new ();
+	self->priv->preview = NULL;
+	self->priv->destination = NULL;
+	self->priv->builder = NULL;
+	self->priv->image_task = NULL;
 
-	gth_file_tool_construct (GTH_FILE_TOOL (self), "tool-adjust-colors", _("Adjust Colors..."), _("Adjust Colors"), FALSE);
+	gth_file_tool_construct (GTH_FILE_TOOL (self), "image-adjust-colors-symbolic", _("Adjust Colors"), GTH_TOOLBOX_SECTION_COLORS);
 	gtk_widget_set_tooltip_text (GTK_WIDGET (self), _("Change brightness, contrast, saturation and gamma level of the image"));
 }
 
@@ -550,7 +593,7 @@ gth_file_tool_adjust_colors_finalize (GObject *object)
 
 	self = (GthFileToolAdjustColors *) object;
 
-	cairo_surface_destroy (self->priv->source);
+	cairo_surface_destroy (self->priv->preview);
 	cairo_surface_destroy (self->priv->destination);
 	_g_object_unref (self->priv->builder);
 	_g_object_unref (self->priv->histogram);
@@ -560,19 +603,21 @@ gth_file_tool_adjust_colors_finalize (GObject *object)
 
 
 static void
-gth_file_tool_adjust_colors_class_init (GthFileToolAdjustColorsClass *class)
+gth_file_tool_adjust_colors_class_init (GthFileToolAdjustColorsClass *klass)
 {
-	GObjectClass     *gobject_class;
-	GthFileToolClass *file_tool_class;
+	GObjectClass                *gobject_class;
+	GthFileToolClass            *file_tool_class;
+	GthImageViewerPageToolClass *image_viewer_page_tool_class;
 
-	g_type_class_add_private (class, sizeof (GthFileToolAdjustColorsPrivate));
+	g_type_class_add_private (klass, sizeof (GthFileToolAdjustColorsPrivate));
 
-	gobject_class = (GObjectClass*) class;
+	gobject_class = (GObjectClass*) klass;
 	gobject_class->finalize = gth_file_tool_adjust_colors_finalize;
 
-	file_tool_class = (GthFileToolClass *) class;
-	file_tool_class->update_sensitivity = gth_file_tool_adjust_colors_update_sensitivity;
-	file_tool_class->activate = gth_file_tool_adjust_colors_activate;
+	file_tool_class = (GthFileToolClass *) klass;
 	file_tool_class->get_options = gth_file_tool_adjust_colors_get_options;
 	file_tool_class->destroy_options = gth_file_tool_adjust_colors_destroy_options;
+
+	image_viewer_page_tool_class = (GthImageViewerPageToolClass *) klass;
+	image_viewer_page_tool_class->reset_image = gth_file_tool_sharpen_reset_image;
 }
