@@ -21,8 +21,7 @@
 
 #include <config.h>
 #include <gthumb.h>
-#include <extensions/image_viewer/gth-image-viewer-page.h>
-#include <extensions/image_viewer/preferences.h>
+#include <extensions/image_viewer/image-viewer.h>
 #include "enum-types.h"
 #include "gth-file-tool-crop.h"
 #include "preferences.h"
@@ -31,14 +30,14 @@
 #define GET_WIDGET(x) (_gtk_builder_get_widget (self->priv->builder, (x)))
 
 
-G_DEFINE_TYPE (GthFileToolCrop, gth_file_tool_crop, GTH_TYPE_FILE_TOOL)
+G_DEFINE_TYPE (GthFileToolCrop, gth_file_tool_crop, GTH_TYPE_IMAGE_VIEWER_PAGE_TOOL)
 
 
 struct _GthFileToolCropPrivate {
 	GSettings        *settings;
 	GtkBuilder       *builder;
-	int               pixbuf_width;
-	int               pixbuf_height;
+	int               original_width;
+	int               original_height;
 	int               screen_width;
 	int               screen_height;
 	GthImageSelector *selector;
@@ -51,26 +50,60 @@ struct _GthFileToolCropPrivate {
 };
 
 
-static void
-gth_file_tool_crop_update_sensitivity (GthFileTool *base)
+static gpointer
+crop_exec (GthAsyncTask *task,
+	   gpointer      user_data)
 {
-	GtkWidget *window;
-	GtkWidget *viewer_page;
+	GthFileToolCrop       *self = user_data;
+	cairo_rectangle_int_t  selection;
+	cairo_surface_t       *source;
+	cairo_surface_t       *destination;
 
-	window = gth_file_tool_get_window (base);
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	if (! GTH_IS_IMAGE_VIEWER_PAGE (viewer_page))
-		gtk_widget_set_sensitive (GTK_WIDGET (base), FALSE);
-	else
-		gtk_widget_set_sensitive (GTK_WIDGET (base), TRUE);
+	gth_image_selector_get_selection (self->priv->selector, &selection);
+	if ((selection.width == 0) || (selection.height == 0))
+		return NULL;
+
+	source = gth_image_task_get_source_surface (GTH_IMAGE_TASK (task));
+	destination = _cairo_image_surface_copy_subsurface (source,
+					       	            selection.x,
+					       	            selection.y,
+					       	            selection.width,
+					       	            selection.height);
+	gth_image_task_set_destination_surface (GTH_IMAGE_TASK (task), destination);
+
+	cairo_surface_destroy (destination);
+	cairo_surface_destroy (source);
+
+	return NULL;
 }
 
 
 static void
-cancel_button_clicked_cb (GtkButton       *button,
-			  GthFileToolCrop *self)
+image_task_completed_cb (GthTask  *task,
+			 GError   *error,
+			 gpointer  user_data)
 {
+	GthFileToolCrop *self = user_data;
+	cairo_surface_t *destination;
+	GtkWidget       *viewer_page;
+
+	if (error != NULL) {
+		g_object_unref (task);
+		return;
+	}
+
+	destination = gth_image_task_get_destination_surface (GTH_IMAGE_TASK (task));
+	if (destination == NULL) {
+		g_object_unref (task);
+		return;
+	}
+
+	viewer_page = gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+	gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), destination, TRUE);
 	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
+
+	cairo_surface_destroy (destination);
+	g_object_unref (task);
 }
 
 
@@ -79,32 +112,24 @@ crop_button_clicked_cb (GtkButton       *button,
 			GthFileToolCrop *self)
 {
 	cairo_rectangle_int_t  selection;
-	GtkWidget             *window;
-	GtkWidget             *viewer_page;
-	GtkWidget             *viewer;
-	cairo_surface_t       *old_image;
-	cairo_surface_t       *new_image;
+	GthTask               *task;
 
 	gth_image_selector_get_selection (self->priv->selector, &selection);
 	if ((selection.width == 0) || (selection.height == 0))
 		return;
 
-	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	old_image = gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (viewer));
-
-	new_image = _cairo_image_surface_copy_subsurface (old_image,
-					       	          selection.x,
-					       	          selection.y,
-					       	          selection.width,
-					       	          selection.height);
-	if (new_image != NULL) {
-		gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), new_image, TRUE);
-		gth_file_tool_hide_options (GTH_FILE_TOOL (self));
-
-		cairo_surface_destroy (new_image);
-	}
+	task = gth_image_task_new (_("Applying changes"),
+				   NULL,
+				   crop_exec,
+				   NULL,
+				   self,
+				   NULL);
+	gth_image_task_set_source_surface (GTH_IMAGE_TASK (task), gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self)));
+	g_signal_connect (task,
+			  "completed",
+			  G_CALLBACK (image_task_completed_cb),
+			  self);
+	gth_browser_exec_task (GTH_BROWSER (gth_file_tool_get_window (GTH_FILE_TOOL (self))), task, FALSE);
 }
 
 
@@ -164,19 +189,19 @@ selector_selection_changed_cb (GthImageSelector *selector,
 	gth_image_selector_get_selection (selector, &selection);
 
 	min = 0;
-	max = self->priv->pixbuf_width - selection.width;
+	max = self->priv->original_width - selection.width;
 	set_spin_range_value (self, self->priv->crop_x_spinbutton, min, max, selection.x);
 
 	min = 0;
-	max = self->priv->pixbuf_height - selection.height;
+	max = self->priv->original_height - selection.height;
 	set_spin_range_value (self, self->priv->crop_y_spinbutton, min, max, selection.y);
 
 	min = 0;
-	max = self->priv->pixbuf_width - selection.x;
+	max = self->priv->original_width - selection.x;
 	set_spin_range_value (self, self->priv->crop_width_spinbutton, min, max, selection.width);
 
 	min = 0;
-	max = self->priv->pixbuf_height - selection.y;
+	max = self->priv->original_height - selection.y;
 	set_spin_range_value (self, self->priv->crop_height_spinbutton, min, max, selection.height);
 
 	gth_image_selector_set_mask_visible (selector, (selection.width != 0 || selection.height != 0));
@@ -219,8 +244,8 @@ ratio_combobox_changed_cb (GtkComboBox     *combobox,
 		w = h = 1;
 		break;
 	case GTH_ASPECT_RATIO_IMAGE:
-		w = self->priv->pixbuf_width;
-		h = self->priv->pixbuf_height;
+		w = self->priv->original_width;
+		h = self->priv->original_height;
 		break;
 	case GTH_ASPECT_RATIO_DISPLAY:
 		w = self->priv->screen_width;
@@ -270,7 +295,7 @@ ratio_combobox_changed_cb (GtkComboBox     *combobox,
 	else
 		ratio = (double) w / h;
 
-	gtk_widget_set_sensitive (GET_WIDGET ("custom_ratio_box"), idx == GTH_ASPECT_RATIO_CUSTOM);
+	gtk_widget_set_visible (GET_WIDGET ("custom_ratio_box"), idx == GTH_ASPECT_RATIO_CUSTOM);
 	gtk_widget_set_sensitive (GET_WIDGET ("invert_ratio_checkbutton"), use_ratio);
 	set_spin_value (self, ratio_w_spinbutton, w);
 	set_spin_value (self, ratio_h_spinbutton, h);
@@ -369,8 +394,8 @@ maximize_button_clicked_cb (GtkButton *button,
 	GthFileToolCrop *self = user_data;
 
 	gth_image_selector_set_selection_pos (self->priv->selector, 0, 0);
-	if (! gth_image_selector_set_selection_width (self->priv->selector, self->priv->pixbuf_width) || ! gth_image_selector_get_use_ratio (self->priv->selector))
-		gth_image_selector_set_selection_height (self->priv->selector, self->priv->pixbuf_height);
+	if (! gth_image_selector_set_selection_width (self->priv->selector, self->priv->original_width) || ! gth_image_selector_get_use_ratio (self->priv->selector))
+		gth_image_selector_set_selection_height (self->priv->selector, self->priv->original_height);
 	gth_image_selector_center (self->priv->selector);
 }
 
@@ -385,6 +410,18 @@ center_button_clicked_cb (GtkButton *button,
 }
 
 
+static void
+options_button_clicked_cb (GtkButton       *button,
+			   GthFileToolCrop *self)
+{
+	GtkWidget *dialog;
+
+	dialog = GET_WIDGET ("options_dialog");
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gth_file_tool_get_window (GTH_FILE_TOOL (self))));
+	gtk_widget_show (dialog);
+}
+
+
 static GtkWidget *
 gth_file_tool_crop_get_options (GthFileTool *base)
 {
@@ -392,7 +429,7 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 	GtkWidget       *window;
 	GtkWidget       *viewer_page;
 	GtkWidget       *viewer;
-	cairo_surface_t *image;
+	cairo_surface_t *source;
 	GtkWidget       *options;
 	char            *text;
 
@@ -404,12 +441,11 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 		return NULL;
 
 	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	image = gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (viewer));
-	if (image == NULL)
+	source = gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+	if (source == NULL)
 		return NULL;
 
-	self->priv->pixbuf_width = cairo_image_surface_get_width (image);
-	self->priv->pixbuf_height = cairo_image_surface_get_height (image);
+	gth_image_viewer_get_original_size (GTH_IMAGE_VIEWER (viewer), &self->priv->original_width, &self->priv->original_height);
 	_gtk_widget_get_screen_size (window, &self->priv->screen_width, &self->priv->screen_height);
 
 	self->priv->builder = _gtk_builder_new_from_file ("crop-options.ui", "file_tools");
@@ -422,7 +458,7 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 	self->priv->crop_height_spinbutton = _gtk_builder_get_widget (self->priv->builder, "crop_height_spinbutton");
 
 	self->priv->ratio_combobox = _gtk_combo_box_new_with_texts (_("None"), _("Square"), NULL);
-	text = g_strdup_printf (_("%d x %d (Image)"), self->priv->pixbuf_width, self->priv->pixbuf_height);
+	text = g_strdup_printf (_("%d x %d (Image)"), self->priv->original_width, self->priv->original_height);
 	gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (self->priv->ratio_combobox), text);
 	g_free (text);
 	text = g_strdup_printf (_("%d x %d (Screen)"), self->priv->screen_width, self->priv->screen_height);
@@ -440,7 +476,7 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 				     _("Custom"),
 				     NULL);
 	gtk_widget_show (self->priv->ratio_combobox);
-	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("ratio_combobox_box")), self->priv->ratio_combobox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("ratio_combobox_box")), self->priv->ratio_combobox, TRUE, TRUE, 0);
 
 	gtk_combo_box_set_active (GTK_COMBO_BOX (self->priv->ratio_combobox),
 				  g_settings_get_enum (self->priv->settings, PREF_CROP_ASPECT_RATIO));
@@ -461,7 +497,7 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 				  g_settings_get_enum (self->priv->settings, PREF_CROP_GRID_TYPE));
 	gtk_widget_show (self->priv->grid_type_combobox);
 	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("grid_type_combobox_box")),
-			    self->priv->grid_type_combobox, FALSE, FALSE, 0);
+			    self->priv->grid_type_combobox, TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("grid_label")),
 				       self->priv->grid_type_combobox);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("bind_dimensions_checkbutton")),
@@ -473,10 +509,22 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 			  "clicked",
 			  G_CALLBACK (crop_button_clicked_cb),
 			  self);
-	g_signal_connect (GET_WIDGET ("cancel_button"),
+	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
+				  "clicked",
+				  G_CALLBACK (gth_file_tool_cancel),
+				  self);
+	g_signal_connect (GET_WIDGET ("options_button"),
 			  "clicked",
-			  G_CALLBACK (cancel_button_clicked_cb),
+			  G_CALLBACK (options_button_clicked_cb),
 			  self);
+	g_signal_connect_swapped (GET_WIDGET ("options_close_button"),
+				  "clicked",
+				  G_CALLBACK (gtk_widget_hide),
+				  GET_WIDGET ("options_dialog"));
+	g_signal_connect (GET_WIDGET ("options_dialog"),
+			  "delete-event",
+			  G_CALLBACK (gtk_widget_hide_on_delete),
+			  NULL);
 	g_signal_connect (G_OBJECT (self->priv->crop_x_spinbutton),
 			  "value-changed",
 			  G_CALLBACK (selection_x_value_changed_cb),
@@ -540,13 +588,14 @@ gth_file_tool_crop_get_options (GthFileTool *base)
 			  G_CALLBACK (selector_selection_changed_cb),
 			  self);
 
-	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (viewer), (GthImageViewerTool *) self->priv->selector);
+	gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), source, FALSE);
+	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (viewer), GTH_IMAGE_VIEWER_TOOL (self->priv->selector));
 	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (viewer), GTH_ZOOM_QUALITY_LOW);
 	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (viewer), GTH_FIT_SIZE_IF_LARGER);
 	ratio_combobox_changed_cb (NULL, self);
 
-	if (! gth_image_selector_set_selection_width (self->priv->selector, self->priv->pixbuf_width * 2 / 3) || ! gth_image_selector_get_use_ratio (self->priv->selector))
-		gth_image_selector_set_selection_height (self->priv->selector, self->priv->pixbuf_height * 2 / 3);
+	if (! gth_image_selector_set_selection_width (self->priv->selector, self->priv->original_width * 2 / 3) || ! gth_image_selector_get_use_ratio (self->priv->selector))
+		gth_image_selector_set_selection_height (self->priv->selector, self->priv->original_height * 2 / 3);
 	gth_image_selector_center (self->priv->selector);
 
 	update_sensitivity (self);
@@ -587,10 +636,11 @@ gth_file_tool_crop_destroy_options (GthFileTool *base)
 
 	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
 	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (viewer), NULL);
+	gth_image_viewer_page_reset_viewer_tool (GTH_IMAGE_VIEWER_PAGE (viewer_page));
 
 	/* restore the zoom quality */
+
+	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
 	viewer_settings = g_settings_new (GTHUMB_IMAGE_VIEWER_SCHEMA);
 	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (viewer),
 					   g_settings_get_enum (viewer_settings, PREF_IMAGE_VIEWER_ZOOM_QUALITY));
@@ -600,9 +650,12 @@ gth_file_tool_crop_destroy_options (GthFileTool *base)
 
 
 static void
-gth_file_tool_crop_activate (GthFileTool *base)
+gth_file_tool_crop_reset_image (GthImageViewerPageTool *base)
 {
-	gth_file_tool_show_options (base);
+	GthFileToolCrop *self = (GthFileToolCrop *) base;
+
+	gth_image_viewer_page_reset (GTH_IMAGE_VIEWER_PAGE (gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self))));
+	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
 }
 
 
@@ -611,7 +664,7 @@ gth_file_tool_crop_init (GthFileToolCrop *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_FILE_TOOL_CROP, GthFileToolCropPrivate);
 	self->priv->settings = g_settings_new (GTHUMB_CROP_SCHEMA);
-	gth_file_tool_construct (GTH_FILE_TOOL (self), "tool-crop", _("Crop..."), _("Crop"), FALSE);
+	gth_file_tool_construct (GTH_FILE_TOOL (self), "image-crop-symbolic", _("Crop"), GTH_TOOLBOX_SECTION_FORMAT);
 }
 
 
@@ -635,19 +688,21 @@ gth_file_tool_crop_finalize (GObject *object)
 
 
 static void
-gth_file_tool_crop_class_init (GthFileToolCropClass *class)
+gth_file_tool_crop_class_init (GthFileToolCropClass *klass)
 {
-	GObjectClass     *gobject_class;
-	GthFileToolClass *file_tool_class;
+	GObjectClass                *gobject_class;
+	GthFileToolClass            *file_tool_class;
+	GthImageViewerPageToolClass *image_viewer_page_tool_class;
 
-	g_type_class_add_private (class, sizeof (GthFileToolCropPrivate));
+	g_type_class_add_private (klass, sizeof (GthFileToolCropPrivate));
 
-	gobject_class = (GObjectClass*) class;
+	gobject_class = (GObjectClass*) klass;
 	gobject_class->finalize = gth_file_tool_crop_finalize;
 
-	file_tool_class = (GthFileToolClass *) class;
-	file_tool_class->update_sensitivity = gth_file_tool_crop_update_sensitivity;
-	file_tool_class->activate = gth_file_tool_crop_activate;
+	file_tool_class = (GthFileToolClass *) klass;
 	file_tool_class->get_options = gth_file_tool_crop_get_options;
 	file_tool_class->destroy_options = gth_file_tool_crop_destroy_options;
+
+	image_viewer_page_tool_class = (GthImageViewerPageToolClass *) klass;
+	image_viewer_page_tool_class->reset_image = gth_file_tool_crop_reset_image;
 }
