@@ -22,7 +22,7 @@
 #include <config.h>
 #include <math.h>
 #include <gthumb.h>
-#include <extensions/image_viewer/gth-image-viewer-page.h>
+#include <extensions/image_viewer/image-viewer.h>
 #include "gth-file-tool-resize.h"
 #include "preferences.h"
 
@@ -33,12 +33,12 @@
 #define PIXELS_UNIT_POSITION 0
 
 
-G_DEFINE_TYPE (GthFileToolResize, gth_file_tool_resize, GTH_TYPE_FILE_TOOL)
+G_DEFINE_TYPE (GthFileToolResize, gth_file_tool_resize, GTH_TYPE_IMAGE_VIEWER_PAGE_TOOL)
 
 
 struct _GthFileToolResizePrivate {
 	GSettings       *settings;
-	cairo_surface_t *original_image;
+	cairo_surface_t *preview;
 	cairo_surface_t *new_image;
 	GtkBuilder      *builder;
 	GtkWidget       *ratio_combobox;
@@ -50,54 +50,24 @@ struct _GthFileToolResizePrivate {
 	double           aspect_ratio;
 	int              new_width;
 	int              new_height;
-	cairo_filter_t   filter;
+	gboolean         high_quality;
 	GthUnit          unit;
+	GthTask         *resize_task;
+	gboolean         closing;
+	gboolean         apply_to_original;
+	guint            update_size_id;
 };
 
 
-static void
-gth_file_tool_resize_update_sensitivity (GthFileTool *base)
-{
-	GtkWidget *window;
-	GtkWidget *viewer_page;
-
-	window = gth_file_tool_get_window (base);
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	if (! GTH_IS_IMAGE_VIEWER_PAGE (viewer_page))
-		gtk_widget_set_sensitive (GTK_WIDGET (base), FALSE);
-	else
-		gtk_widget_set_sensitive (GTK_WIDGET (base), TRUE);
-}
-
-
-static void
-cancel_button_clicked_cb (GtkButton         *button,
-			  GthFileToolResize *self)
-{
-	GtkWidget *window;
-	GtkWidget *viewer_page;
-
-	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	gth_image_viewer_page_reset (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
-}
+static void update_image_size (GthFileToolResize *self);
 
 
 static void
 resize_button_clicked_cb (GtkButton       *button,
 			GthFileToolResize *self)
 {
-	GtkWidget *window;
-	GtkWidget *viewer_page;
-
-	if (self->priv->new_image == NULL)
-		return;
-
-	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), self->priv->new_image, TRUE);
-	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
+	self->priv->apply_to_original = TRUE;
+	update_image_size (self);
 }
 
 
@@ -121,30 +91,130 @@ update_dimensione_info_label (GthFileToolResize *self,
 
 
 static void
-update_pixbuf_size (GthFileToolResize *self)
+resize_task_completed_cb (GthTask  *task,
+			  GError   *error,
+			  gpointer  user_data)
 {
-	GtkWidget *window;
-	GtkWidget *viewer_page;
+	GthFileToolResize *self = user_data;
+	GtkWidget         *window;
+	GtkWidget         *viewer_page;
+
+	self->priv->resize_task = NULL;
+
+	if (self->priv->closing) {
+		g_object_unref (task);
+		gth_image_viewer_page_tool_reset_image (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+		return;
+	}
+
+	if (error != NULL) {
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+			update_image_size (self);
+		g_object_unref (task);
+		return;
+	}
+
+	_cairo_clear_surface (&self->priv->new_image);
+	self->priv->new_image = gth_image_task_get_destination_surface (GTH_IMAGE_TASK (task));
+	if (self->priv->new_image == NULL) {
+		g_object_unref (task);
+		return;
+	}
 
-	cairo_surface_destroy (self->priv->new_image);
-	self->priv->new_image = _cairo_image_surface_scale_to (self->priv->original_image,
-							       self->priv->new_width,
-							       self->priv->new_height,
-							       self->priv->filter);
 	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
 	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
 	gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), self->priv->new_image, FALSE);
 
-	update_dimensione_info_label (self,
-				      "new_dimensions_label",
-				      self->priv->new_width,
-				      self->priv->new_height,
-				      TRUE);
-	update_dimensione_info_label (self,
-				      "scale_factor_label",
-				      (double) self->priv->new_width / self->priv->original_width,
-				      (double) self->priv->new_height / self->priv->original_height,
-				      FALSE);
+	if (self->priv->apply_to_original) {
+		gth_image_history_add_image (gth_image_viewer_page_get_history (GTH_IMAGE_VIEWER_PAGE (viewer_page)),
+					     self->priv->new_image,
+					     -1,
+					     TRUE);
+		gth_viewer_page_focus (GTH_VIEWER_PAGE (viewer_page));
+		gth_file_tool_hide_options (GTH_FILE_TOOL (self));
+	}
+	else {
+		update_dimensione_info_label (self,
+					      "new_dimensions_label",
+					      self->priv->new_width,
+					      self->priv->new_height,
+					      TRUE);
+		update_dimensione_info_label (self,
+					      "scale_factor_label",
+					      (double) self->priv->new_width / self->priv->original_width,
+					      (double) self->priv->new_height / self->priv->original_height,
+					      FALSE);
+	}
+
+	g_object_unref (task);
+}
+
+
+static gpointer
+resize_task_exec (GthAsyncTask *task,
+		  gpointer      user_data)
+{
+	GthFileToolResize *self = user_data;
+	cairo_surface_t   *source;
+	cairo_surface_t   *destination;
+
+	source = gth_image_task_get_source_surface (GTH_IMAGE_TASK (task));
+	destination = _cairo_image_surface_scale (source,
+						  self->priv->new_width,
+						  self->priv->new_height,
+						  (self->priv->high_quality ? SCALE_FILTER_BEST : SCALE_FILTER_FAST),
+						  task);
+	_cairo_image_surface_clear_metadata (destination);
+	gth_image_task_set_destination_surface (GTH_IMAGE_TASK (task), destination);
+
+	cairo_surface_destroy (destination);
+	cairo_surface_destroy (source);
+
+	return NULL;
+}
+
+
+static gboolean
+update_image_size_cb (gpointer user_data)
+{
+	GthFileToolResize *self = user_data;
+
+	self->priv->update_size_id = 0;
+
+	if (self->priv->resize_task != NULL) {
+		gth_task_cancel (self->priv->resize_task);
+		return FALSE;
+	}
+
+	self->priv->resize_task = gth_image_task_new (_("Resizing images"),
+						      NULL,
+						      resize_task_exec,
+						      NULL,
+						      self,
+						      NULL);
+	if (self->priv->apply_to_original)
+		gth_image_task_set_source_surface (GTH_IMAGE_TASK (self->priv->resize_task), gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self)));
+	else
+		gth_image_task_set_source_surface (GTH_IMAGE_TASK (self->priv->resize_task), self->priv->preview);
+
+	g_signal_connect (self->priv->resize_task,
+			  "completed",
+			  G_CALLBACK (resize_task_completed_cb),
+			  self);
+	gth_browser_exec_task (GTH_BROWSER (gth_file_tool_get_window (GTH_FILE_TOOL (self))),
+			       self->priv->resize_task,
+			       FALSE);
+
+	return FALSE;
+}
+
+
+static void
+update_image_size (GthFileToolResize *self)
+{
+	if (self->priv->update_size_id != 0)
+		g_source_remove (self->priv->update_size_id);
+	self->priv->update_size_id = g_timeout_add (100, update_image_size_cb, self);
 }
 
 
@@ -167,7 +237,7 @@ selection_width_value_changed_cb (GtkSpinButton     *spin,
 		g_signal_handlers_unblock_by_data (GET_WIDGET ("resize_height_spinbutton"), self);
 	}
 
-	update_pixbuf_size (self);
+	update_image_size (self);
 }
 
 
@@ -190,7 +260,7 @@ selection_height_value_changed_cb (GtkSpinButton     *spin,
 		g_signal_handlers_unblock_by_data (GET_WIDGET ("resize_width_spinbutton"), self);
 	}
 
-	update_pixbuf_size (self);
+	update_image_size (self);
 }
 
 
@@ -198,8 +268,8 @@ static void
 high_quality_checkbutton_toggled_cb (GtkToggleButton   *button,
 				     GthFileToolResize *self)
 {
-	self->priv->filter = gtk_toggle_button_get_active (button) ? CAIRO_FILTER_GAUSSIAN : CAIRO_FILTER_NEAREST;
-	update_pixbuf_size (self);
+	self->priv->high_quality = gtk_toggle_button_get_active (button);
+	update_image_size (self);
 }
 
 
@@ -324,7 +394,7 @@ ratio_combobox_changed_cb (GtkComboBox       *combobox,
 		break;
 	}
 
-	gtk_widget_set_sensitive (GET_WIDGET ("custom_ratio_box"), idx == GTH_ASPECT_RATIO_CUSTOM);
+	gtk_widget_set_visible (GET_WIDGET ("custom_ratio_box"), idx == GTH_ASPECT_RATIO_CUSTOM);
 	gtk_widget_set_sensitive (GET_WIDGET ("invert_ratio_checkbutton"), use_ratio);
 	set_spin_value (self, ratio_w_spinbutton, w);
 	set_spin_value (self, ratio_h_spinbutton, h);
@@ -408,7 +478,7 @@ set_image_size (GthFileToolResize *self,
 	g_signal_handlers_unblock_by_data (GET_WIDGET ("ratio_w_spinbutton"), self);
 	g_signal_handlers_unblock_by_data (GET_WIDGET ("ratio_h_spinbutton"), self);
 
-	update_pixbuf_size (self);
+	update_image_size (self);
 }
 
 
@@ -434,39 +504,59 @@ screen_size_button_clicked_cb (GtkButton         *button,
 }
 
 
+static void
+options_button_clicked_cb (GtkButton         *button,
+			   GthFileToolResize *self)
+{
+	GtkWidget *dialog;
+
+	dialog = GET_WIDGET ("options_dialog");
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gth_file_tool_get_window (GTH_FILE_TOOL (self))));
+	gtk_widget_show (dialog);
+}
+
+
 static GtkWidget *
 gth_file_tool_resize_get_options (GthFileTool *base)
 {
-	GthFileToolResize *self;
+	GthFileToolResize *self = (GthFileToolResize *) base;
+	cairo_surface_t   *source;
 	GtkWidget         *window;
 	GtkWidget         *viewer_page;
 	GtkWidget         *viewer;
+	GtkAllocation      allocation;
+	int                preview_width;
+	int                preview_height;
 	GtkWidget         *options;
 	char              *text;
 
-	self = (GthFileToolResize *) base;
+	source = gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
+	if (source == NULL)
+		return NULL;
+
+	self->priv->original_width = cairo_image_surface_get_width (source);
+	self->priv->original_height = cairo_image_surface_get_height (source);
 
 	window = gth_file_tool_get_window (base);
 	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	if (! GTH_IS_IMAGE_VIEWER_PAGE (viewer_page))
-		return NULL;
-
-	cairo_surface_destroy (self->priv->original_image);
-
 	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	self->priv->original_image = cairo_surface_reference (gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (viewer)));
-	if (self->priv->original_image == NULL)
-		return NULL;
 
-	self->priv->original_width = cairo_image_surface_get_width (self->priv->original_image);
-	self->priv->original_height = cairo_image_surface_get_height (self->priv->original_image);
+	gtk_widget_get_allocation (GTK_WIDGET (viewer), &allocation);
+	preview_width = self->priv->original_width;
+	preview_height = self->priv->original_height;
+	if (scale_keeping_ratio (&preview_width, &preview_height, allocation.width, allocation.height, FALSE))
+		self->priv->preview = _cairo_image_surface_scale_bilinear (source, preview_width, preview_height);
+	else
+		self->priv->preview = cairo_surface_reference (source);
+
 	_gtk_widget_get_screen_size (window, &self->priv->screen_width, &self->priv->screen_height);
 	self->priv->new_image = NULL;
 	self->priv->new_width = self->priv->original_width;
 	self->priv->new_height = self->priv->original_height;
-	self->priv->filter = g_settings_get_boolean (self->priv->settings, PREF_RESIZE_HIGH_QUALITY) ? CAIRO_FILTER_GAUSSIAN : CAIRO_FILTER_NEAREST;
+	self->priv->high_quality = g_settings_get_boolean (self->priv->settings, PREF_RESIZE_HIGH_QUALITY);
 	self->priv->unit = g_settings_get_enum (self->priv->settings, PREF_RESIZE_UNIT);
 	self->priv->builder = _gtk_builder_new_from_file ("resize-options.ui", "file_tools");
+	self->priv->apply_to_original = FALSE;
 
 	update_dimensione_info_label (self,
 				      "original_dimensions_label",
@@ -516,10 +606,10 @@ gth_file_tool_resize_get_options (GthFileTool *base)
 				     _("Custom"),
 				     NULL);
 	gtk_widget_show (self->priv->ratio_combobox);
-	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("ratio_combobox_box")), self->priv->ratio_combobox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("ratio_combobox_box")), self->priv->ratio_combobox, TRUE, TRUE, 0);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("high_quality_checkbutton")),
-				      self->priv->filter != CAIRO_FILTER_NEAREST);
+				      self->priv->high_quality);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("invert_ratio_checkbutton")),
 				      g_settings_get_boolean (self->priv->settings, PREF_RESIZE_ASPECT_RATIO_INVERT));
 
@@ -532,10 +622,22 @@ gth_file_tool_resize_get_options (GthFileTool *base)
 			  "clicked",
 			  G_CALLBACK (resize_button_clicked_cb),
 			  self);
-	g_signal_connect (GET_WIDGET ("cancel_button"),
+	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
+				  "clicked",
+				  G_CALLBACK (gth_file_tool_cancel),
+				  self);
+	g_signal_connect (GET_WIDGET ("options_button"),
 			  "clicked",
-			  G_CALLBACK (cancel_button_clicked_cb),
+			  G_CALLBACK (options_button_clicked_cb),
 			  self);
+	g_signal_connect_swapped (GET_WIDGET ("options_close_button"),
+				  "clicked",
+				  G_CALLBACK (gtk_widget_hide),
+				  GET_WIDGET ("options_dialog"));
+	g_signal_connect (GET_WIDGET ("options_dialog"),
+			  "delete-event",
+			  G_CALLBACK (gtk_widget_hide_on_delete),
+			  NULL);
 	g_signal_connect (GET_WIDGET ("resize_width_spinbutton"),
 			  "value-changed",
 			  G_CALLBACK (selection_width_value_changed_cb),
@@ -580,6 +682,8 @@ gth_file_tool_resize_get_options (GthFileTool *base)
 	gtk_combo_box_set_active (GTK_COMBO_BOX (self->priv->ratio_combobox),
 				  g_settings_get_enum (self->priv->settings, PREF_RESIZE_ASPECT_RATIO));
 
+	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (viewer), GTH_ZOOM_QUALITY_HIGH);
+
 	return options;
 }
 
@@ -591,9 +695,15 @@ gth_file_tool_resize_destroy_options (GthFileTool *base)
 	GtkWidget         *window;
 	GtkWidget         *viewer_page;
 	GtkWidget         *viewer;
+	GSettings         *viewer_settings;
 
 	self = (GthFileToolResize *) base;
 
+	if (self->priv->update_size_id != 0) {
+		g_source_remove (self->priv->update_size_id);
+		self->priv->update_size_id = 0;
+	}
+
 	if (self->priv->builder != NULL) {
 		int unit;
 
@@ -611,25 +721,44 @@ gth_file_tool_resize_destroy_options (GthFileTool *base)
 
 		/* destroy the options data */
 
-		cairo_surface_destroy (self->priv->new_image);
-		cairo_surface_destroy (self->priv->original_image);
-		_g_object_unref (self->priv->builder);
-		self->priv->new_image = NULL;
-		self->priv->original_image = NULL;
-		self->priv->builder = NULL;
+		_cairo_clear_surface (&self->priv->new_image);
+		_cairo_clear_surface (&self->priv->preview);
+		_g_clear_object (&self->priv->builder);
 	}
 
 	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
 	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
+	gth_image_viewer_page_reset_viewer_tool (GTH_IMAGE_VIEWER_PAGE (viewer_page));
+
+	/* restore the zoom quality */
+
 	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (viewer), NULL);
+	viewer_settings = g_settings_new (GTHUMB_IMAGE_VIEWER_SCHEMA);
+	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (viewer),
+					   g_settings_get_enum (viewer_settings, PREF_IMAGE_VIEWER_ZOOM_QUALITY));
+
+	g_object_unref (viewer_settings);
 }
 
 
 static void
-gth_file_tool_resize_activate (GthFileTool *base)
+gth_file_tool_resize_reset_image (GthImageViewerPageTool *base)
 {
-	gth_file_tool_show_options (base);
+	GthFileToolResize *self = (GthFileToolResize *) base;
+
+	if (self->priv->resize_task != NULL) {
+		self->priv->closing = TRUE;
+		gth_task_cancel (self->priv->resize_task);
+		return;
+	}
+
+	if (self->priv->update_size_id != 0) {
+		g_source_remove (self->priv->update_size_id);
+		self->priv->update_size_id = 0;
+	}
+
+	gth_image_viewer_page_reset (GTH_IMAGE_VIEWER_PAGE (gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self))));
+	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
 }
 
 
@@ -644,7 +773,7 @@ gth_file_tool_resize_finalize (GObject *object)
 	self = (GthFileToolResize *) object;
 
 	cairo_surface_destroy (self->priv->new_image);
-	cairo_surface_destroy (self->priv->original_image);
+	cairo_surface_destroy (self->priv->preview);
 	_g_object_unref (self->priv->builder);
 	_g_object_unref (self->priv->settings);
 
@@ -656,8 +785,9 @@ gth_file_tool_resize_finalize (GObject *object)
 static void
 gth_file_tool_resize_class_init (GthFileToolResizeClass *klass)
 {
-	GObjectClass     *gobject_class;
-	GthFileToolClass *file_tool_class;
+	GObjectClass                *gobject_class;
+	GthFileToolClass            *file_tool_class;
+	GthImageViewerPageToolClass *image_viewer_page_tool_class;
 
 	g_type_class_add_private (klass, sizeof (GthFileToolResizePrivate));
 
@@ -665,10 +795,11 @@ gth_file_tool_resize_class_init (GthFileToolResizeClass *klass)
 	gobject_class->finalize = gth_file_tool_resize_finalize;
 
 	file_tool_class = (GthFileToolClass *) klass;
-	file_tool_class->update_sensitivity = gth_file_tool_resize_update_sensitivity;
-	file_tool_class->activate = gth_file_tool_resize_activate;
 	file_tool_class->get_options = gth_file_tool_resize_get_options;
 	file_tool_class->destroy_options = gth_file_tool_resize_destroy_options;
+
+	image_viewer_page_tool_class = (GthImageViewerPageToolClass *) klass;
+	image_viewer_page_tool_class->reset_image = gth_file_tool_resize_reset_image;
 }
 
 
@@ -677,5 +808,8 @@ gth_file_tool_resize_init (GthFileToolResize *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_FILE_TOOL_RESIZE, GthFileToolResizePrivate);
 	self->priv->settings = g_settings_new (GTHUMB_RESIZE_SCHEMA);
-	gth_file_tool_construct (GTH_FILE_TOOL (self), "tool-resize", _("Resize..."), _("Resize"), FALSE);
+	self->priv->builder = NULL;
+	self->priv->preview = NULL;
+	self->priv->new_image = NULL;
+	gth_file_tool_construct (GTH_FILE_TOOL (self), "image-resize-symbolic", _("Resize"), GTH_TOOLBOX_SECTION_FORMAT);
 }
