@@ -22,7 +22,7 @@
 #include <math.h>
 #include <config.h>
 #include <gthumb.h>
-#include <extensions/image_viewer/gth-image-viewer-page.h>
+#include <extensions/image_viewer/image-viewer.h>
 #include "cairo-rotate.h"
 #include "enum-types.h"
 #include "gth-file-tool-rotate.h"
@@ -34,7 +34,7 @@
 #define GET_WIDGET(x) (_gtk_builder_get_widget (self->priv->builder, (x)))
 
 
-G_DEFINE_TYPE (GthFileToolRotate, gth_file_tool_rotate, GTH_TYPE_FILE_TOOL)
+G_DEFINE_TYPE (GthFileToolRotate, gth_file_tool_rotate, GTH_TYPE_IMAGE_VIEWER_PAGE_TOOL)
 
 
 struct _GthFileToolRotatePrivate {
@@ -54,21 +54,6 @@ struct _GthFileToolRotatePrivate {
 };
 
 
-static void
-gth_file_tool_rotate_update_sensitivity (GthFileTool *base)
-{
-	GtkWidget *window;
-	GtkWidget *viewer_page;
-
-	window = gth_file_tool_get_window (base);
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	if (! GTH_IS_IMAGE_VIEWER_PAGE (viewer_page))
-		gtk_widget_set_sensitive (GTK_WIDGET (base), FALSE);
-	else
-		gtk_widget_set_sensitive (GTK_WIDGET (base), TRUE);
-}
-
-
 static void
 update_crop_parameters (GthFileToolRotate *self)
 {
@@ -232,22 +217,64 @@ reset_button_clicked_cb (GtkButton         *button,
 }
 
 
+static void
+options_button_clicked_cb (GtkButton         *button,
+			       GthFileToolRotate *self)
+{
+	GtkWidget *dialog;
+
+	dialog = GET_WIDGET ("options_dialog");
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (gth_file_tool_get_window (GTH_FILE_TOOL (self))));
+	gtk_widget_show (dialog);
+}
+
+
+static gpointer
+rotate_exec (GthAsyncTask *task,
+	     gpointer      user_data)
+{
+	GthImageViewerTool *rotator = user_data;
+	cairo_surface_t    *source;
+	cairo_surface_t    *destination;
+
+	source = gth_image_task_get_source_surface (GTH_IMAGE_TASK (task));
+	destination = gth_image_rotator_get_result (GTH_IMAGE_ROTATOR (rotator), source, task);
+	gth_image_task_set_destination_surface (GTH_IMAGE_TASK (task), destination);
+
+	cairo_surface_destroy (destination);
+	cairo_surface_destroy (source);
+
+	return NULL;
+}
+
+
 static void
 apply_button_clicked_cb (GtkButton         *button,
 			 GthFileToolRotate *self)
 {
-	cairo_surface_t *image;
-	GtkWidget       *window;
-	GtkWidget       *viewer_page;
-
-	image = gth_image_rotator_get_result (GTH_IMAGE_ROTATOR (self->priv->rotator), FALSE);
+	GtkWidget   *window;
+	GtkWidget   *viewer_page;
+	GthTask     *task;
 
 	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
 	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), image, TRUE);
-	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
+	task = gth_image_viewer_task_new (GTH_IMAGE_VIEWER_PAGE (viewer_page),
+					  _("Applying changes"),
+					  NULL,
+					  rotate_exec,
+					  NULL,
+					  g_object_ref (self->priv->rotator),
+					  g_object_unref);
+	gth_image_viewer_task_set_load_original (GTH_IMAGE_VIEWER_TASK (task), FALSE);
+	gth_image_task_set_source_surface (GTH_IMAGE_TASK (task), gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self)));
 
-	cairo_surface_destroy (image);
+	g_signal_connect (task,
+			  "completed",
+			  G_CALLBACK (gth_image_viewer_task_set_destination),
+			  NULL);
+	gth_browser_exec_task (GTH_BROWSER (window), task, FALSE);
+
+	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
 }
 
 
@@ -292,16 +319,11 @@ static void
 background_colorbutton_color_set_cb (GtkColorButton    *color_button,
 		             	     GthFileToolRotate *self)
 {
-	GdkColor      color;
-	cairo_color_t background_color;
+	GdkRGBA  background_color;
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("background_transparent_checkbutton")), FALSE);
 
-	gtk_color_button_get_color (color_button, &color);
-	background_color.r = (double) color.red / 65535;
-	background_color.g = (double) color.green / 65535;
-	background_color.b = (double) color.blue / 65535;
-	background_color.a = 1.0;
+	gtk_color_chooser_get_rgba (GTK_COLOR_CHOOSER (color_button), &background_color);
 	gth_image_rotator_set_background (GTH_IMAGE_ROTATOR (self->priv->rotator), &background_color);
 
 	apply_changes (self);
@@ -313,12 +335,12 @@ background_transparent_toggled_cb (GtkToggleButton   *toggle_button,
 		             	   GthFileToolRotate *self)
 {
 	if (gtk_toggle_button_get_active (toggle_button)) {
-		cairo_color_t background_color;
+		GdkRGBA background_color;
 
-		background_color.r = 0.0;
-		background_color.g = 0.0;
-		background_color.b = 0.0;
-		background_color.a = 0.0;
+		background_color.red = 0.0;
+		background_color.green = 0.0;
+		background_color.blue = 0.0;
+		background_color.alpha = 0.0;
 		gth_image_rotator_set_background (GTH_IMAGE_ROTATOR (self->priv->rotator), &background_color);
 	}
 	else
@@ -364,8 +386,7 @@ gth_file_tool_rotate_get_options (GthFileTool *base)
 	GtkWidget         *viewer_page;
 	GtkWidget         *viewer;
 	char              *color_spec;
-	GdkColor           color;
-	cairo_color_t      background_color;
+	GdkRGBA            background_color;
 
 	self = (GthFileToolRotate *) base;
 
@@ -376,8 +397,7 @@ gth_file_tool_rotate_get_options (GthFileTool *base)
 
 	cairo_surface_destroy (self->priv->image);
 
-	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	self->priv->image = gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (viewer));
+	self->priv->image = gth_image_viewer_page_tool_get_source (GTH_IMAGE_VIEWER_PAGE_TOOL (self));
 	if (self->priv->image == NULL)
 		return NULL;
 
@@ -432,17 +452,19 @@ gth_file_tool_rotate_get_options (GthFileTool *base)
 	}
 
 	color_spec = g_settings_get_string (self->priv->settings, PREF_ROTATE_BACKGROUND_COLOR);
-	if (! self->priv->has_alpha && gdk_color_parse (color_spec, &color)) {
-		_gdk_color_to_cairo_color (&color, &background_color);
+	if (! self->priv->has_alpha) {
+		gdk_rgba_parse (&background_color, color_spec);
 	}
 	else {
-		background_color.r = 0.0;
-		background_color.g = 0.0;
-		background_color.b = 0.0;
-		background_color.a = self->priv->has_alpha ? 0.0 : 1.0;
+		background_color.red = 0.0;
+		background_color.green = 0.0;
+		background_color.blue = 0.0;
+		background_color.alpha = 1.0;
 	}
 	gth_image_rotator_set_background (GTH_IMAGE_ROTATOR (self->priv->rotator), &background_color);
 
+	gth_image_viewer_page_set_image (GTH_IMAGE_VIEWER_PAGE (viewer_page), self->priv->image, FALSE);
+	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
 	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (viewer), self->priv->rotator);
 	gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (viewer_page));
 
@@ -464,6 +486,18 @@ gth_file_tool_rotate_get_options (GthFileTool *base)
 			  "clicked",
 			  G_CALLBACK (reset_button_clicked_cb),
 			  self);
+	g_signal_connect (GET_WIDGET ("options_button"),
+			  "clicked",
+			  G_CALLBACK (options_button_clicked_cb),
+			  self);
+	g_signal_connect_swapped (GET_WIDGET ("options_close_button"),
+				  "clicked",
+				  G_CALLBACK (gtk_widget_hide),
+				  GET_WIDGET ("options_dialog"));
+	g_signal_connect (GET_WIDGET ("options_dialog"),
+			  "delete-event",
+			  G_CALLBACK (gtk_widget_hide_on_delete),
+			  NULL);
 	g_signal_connect (GET_WIDGET ("align_button"),
 			  "clicked",
 			  G_CALLBACK (align_button_clicked_cb),
@@ -531,14 +565,12 @@ gth_file_tool_rotate_destroy_options (GthFileTool *base)
 	GthFileToolRotate *self;
 	GtkWidget         *window;
 	GtkWidget         *viewer_page;
-	GtkWidget         *viewer;
 
 	self = (GthFileToolRotate *) base;
 
 	if (self->priv->builder != NULL) {
-		cairo_color_t  background_color;
-		GdkColor       color;
-		char          *color_spec;
+		GdkRGBA  background_color;
+		char    *color_spec;
 
 		/* save the dialog options */
 
@@ -547,18 +579,15 @@ gth_file_tool_rotate_destroy_options (GthFileTool *base)
 		g_settings_set_enum (self->priv->settings, PREF_ROTATE_GRID_TYPE, gth_image_rotator_get_grid_type (GTH_IMAGE_ROTATOR (self->priv->rotator)));
 
 		gth_image_rotator_get_background (GTH_IMAGE_ROTATOR (self->priv->rotator), &background_color);
-		color.red = background_color.r * 255.0;
-		color.green = background_color.g * 255.0;
-		color.blue = background_color.b * 255.0;
-		color_spec = gdk_color_to_string (&color);
+		color_spec = gdk_rgba_to_string (&background_color);
 		g_settings_set_string (self->priv->settings, PREF_ROTATE_BACKGROUND_COLOR, color_spec);
+
 		g_free (color_spec);
 	}
 
 	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
 	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-	gth_image_viewer_set_tool (GTH_IMAGE_VIEWER (viewer), NULL);
+	gth_image_viewer_page_reset_viewer_tool (GTH_IMAGE_VIEWER_PAGE (viewer_page));
 	gth_viewer_page_update_sensitivity (GTH_VIEWER_PAGE (viewer_page));
 
 	cairo_surface_destroy (self->priv->image);
@@ -570,22 +599,10 @@ gth_file_tool_rotate_destroy_options (GthFileTool *base)
 
 
 static void
-gth_file_tool_rotate_activate (GthFileTool *base)
+gth_file_tool_rotate_reset_image (GthImageViewerPageTool *self)
 {
-	gth_file_tool_show_options (base);
-}
-
-
-static void
-gth_file_tool_rotate_cancel (GthFileTool *base)
-{
-	GthFileToolRotate *self = (GthFileToolRotate *) base;
-	GtkWidget         *window;
-	GtkWidget         *viewer_page;
-
-	window = gth_file_tool_get_window (GTH_FILE_TOOL (self));
-	viewer_page = gth_browser_get_viewer_page (GTH_BROWSER (window));
-	gth_image_viewer_page_reset (GTH_IMAGE_VIEWER_PAGE (viewer_page));
+	gth_image_viewer_page_reset (GTH_IMAGE_VIEWER_PAGE (gth_image_viewer_page_tool_get_page (GTH_IMAGE_VIEWER_PAGE_TOOL (self))));
+	gth_file_tool_hide_options (GTH_FILE_TOOL (self));
 }
 
 
@@ -611,8 +628,9 @@ gth_file_tool_rotate_finalize (GObject *object)
 static void
 gth_file_tool_rotate_class_init (GthFileToolRotateClass *klass)
 {
-	GObjectClass     *gobject_class;
-	GthFileToolClass *file_tool_class;
+	GObjectClass                *gobject_class;
+	GthFileToolClass            *file_tool_class;
+	GthImageViewerPageToolClass *image_viewer_page_tool_class;
 
 	g_type_class_add_private (klass, sizeof (GthFileToolRotatePrivate));
 
@@ -620,11 +638,11 @@ gth_file_tool_rotate_class_init (GthFileToolRotateClass *klass)
 	gobject_class->finalize = gth_file_tool_rotate_finalize;
 
 	file_tool_class = (GthFileToolClass *) klass;
-	file_tool_class->update_sensitivity = gth_file_tool_rotate_update_sensitivity;
-	file_tool_class->activate = gth_file_tool_rotate_activate;
-	file_tool_class->cancel = gth_file_tool_rotate_cancel;
 	file_tool_class->get_options = gth_file_tool_rotate_get_options;
 	file_tool_class->destroy_options = gth_file_tool_rotate_destroy_options;
+
+	image_viewer_page_tool_class = (GthImageViewerPageToolClass *) klass;
+	image_viewer_page_tool_class->reset_image = gth_file_tool_rotate_reset_image;
 }
 
 
@@ -634,6 +652,5 @@ gth_file_tool_rotate_init (GthFileToolRotate *self)
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_FILE_TOOL_ROTATE, GthFileToolRotatePrivate);
 	self->priv->settings = g_settings_new (GTHUMB_ROTATE_SCHEMA);
 
-	gth_file_tool_construct (GTH_FILE_TOOL (self), "tool-rotate", _("Rotate..."), _("Rotate"), TRUE);
-	gtk_widget_set_tooltip_text (GTK_WIDGET (self), _("Freely rotate the image"));
+	gth_file_tool_construct (GTH_FILE_TOOL (self), "image-rotate-symbolic", _("Rotate"), GTH_TOOLBOX_SECTION_ROTATION);
 }
