@@ -49,21 +49,16 @@ destroy_cb (GtkWidget  *widget,
 }
 
 
-static void
-help_clicked_cb (GtkWidget  *widget,
-		 DialogData *data)
-{
-	show_help_dialog (GTK_WINDOW (data->dialog), "gthumb-find-duplicates");
-}
-
-
 static void
 ok_clicked_cb (GtkWidget  *widget,
 	       DialogData *data)
 {
 	GFile *folder;
 
-	folder = gtk_file_chooser_get_current_folder_file (GTK_FILE_CHOOSER ( _gtk_builder_get_widget (data->builder, "location_filechooserbutton")));
+	folder = gtk_file_chooser_get_file (GTK_FILE_CHOOSER ( _gtk_builder_get_widget (data->builder, "location_filechooserbutton")));
+	if (folder == NULL)
+		return;
+
 	gth_find_duplicates_exec (data->browser,
 				  folder,
 				  gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("include_subfolder_checkbutton"))),
@@ -156,10 +151,6 @@ dlg_find_duplicates (GthBrowser *browser)
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
-        g_signal_connect (GET_WIDGET ("help_button"),
-                          "clicked",
-                          G_CALLBACK (help_clicked_cb),
-                          data);
 	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
