@@ -64,7 +64,7 @@ typedef struct {
 SelectCommand select_commands[] = {
 	{ N_("leave the newest duplicates"), SELECT_LEAVE_NEWEST },
 	{ N_("leave the oldest duplicates"), SELECT_LEAVE_OLDEST },
-	{ N_("by folder..."), SELECT_BY_FOLDER },
+	{ N_("by folder…"), SELECT_BY_FOLDER },
 	{ N_("all files"), SELECT_ALL },
 	{ N_("no file"), SELECT_NONE }
 };
@@ -809,7 +809,7 @@ start_dir_func (GFile      *directory,
 }
 
 
-gboolean
+static gboolean
 pulse_progressbar_cb (gpointer user_data)
 {
 	GthFindDuplicates *self = user_data;
@@ -867,16 +867,6 @@ close_button_clicked_cb (GtkButton *button,
 }
 
 
-static void
-help_button_clicked_cb (GtkButton *button,
-			gpointer   user_data)
-{
-	GthFindDuplicates *self = user_data;
-
-	show_help_dialog (GTK_WINDOW (GET_WIDGET ("find_duplicates_dialog")), "gthumb-find-duplicates");
-}
-
-
 static void
 file_cellrenderertoggle_toggled_cb (GtkCellRendererToggle *cell_renderer,
                 		    char                  *path,
@@ -1270,6 +1260,8 @@ gth_find_duplicates_exec (GthBrowser *browser,
 	const char        *test_attributes;
 	int                i;
 
+	g_return_if_fail (location != NULL);
+
 	self = (GthFindDuplicates *) g_object_new (GTH_TYPE_FIND_DUPLICATES, NULL);
 
 	settings = g_settings_new (GTHUMB_BROWSER_SCHEMA);
@@ -1302,9 +1294,9 @@ gth_find_duplicates_exec (GthBrowser *browser,
 	gtk_widget_show (self->priv->duplicates_list);
 	gtk_container_add (GTK_CONTAINER (GET_WIDGET ("duplicates_list_box")), self->priv->duplicates_list);
 
-	self->priv->select_button = gth_menu_button_new ();
-	gth_menu_button_set_label (GTH_MENU_BUTTON (self->priv->select_button), _("Select"));
-	gtk_widget_show (self->priv->select_button);
+	self->priv->select_button = gtk_menu_button_new ();
+	gtk_container_add (GTK_CONTAINER (self->priv->select_button), gtk_label_new (_("Select")));
+	gtk_widget_show_all (self->priv->select_button);
 	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("select_button_box")), self->priv->select_button, FALSE, FALSE, 0);
 
 	self->priv->select_menu = gtk_menu_new ();
@@ -1322,7 +1314,7 @@ gth_find_duplicates_exec (GthBrowser *browser,
 
 		gtk_menu_shell_append (GTK_MENU_SHELL (self->priv->select_menu), menu_item);
 	}
-	gth_menu_button_set_menu (GTH_MENU_BUTTON (self->priv->select_button), self->priv->select_menu);
+	gtk_menu_button_set_popup (GTK_MENU_BUTTON (self->priv->select_button), self->priv->select_menu);
 
 	g_object_unref (settings);
 
@@ -1338,10 +1330,6 @@ gth_find_duplicates_exec (GthBrowser *browser,
 				  "clicked",
 				  G_CALLBACK (g_cancellable_cancel),
 				  self->priv->cancellable);
-	g_signal_connect (GET_WIDGET ("help_button"),
-			  "clicked",
-			  G_CALLBACK (help_button_clicked_cb),
-			  self);
 	g_signal_connect (gth_file_list_get_view (GTH_FILE_LIST (self->priv->duplicates_list)),
 			  "file-selection-changed",
 			  G_CALLBACK (duplicates_list_view_selection_changed_cb),
