@@ -25,13 +25,14 @@
 #include <gst/gst.h>
 #include <gthumb.h>
 #include <extensions/gstreamer_utils/gstreamer-utils.h>
+#include "actions.h"
 #include "gth-media-viewer-page.h"
 #include "preferences.h"
 
 #define MAX_ATTEMPTS 1024
 
 
-/* -- media_viewer_activate_action_screenshot -- */
+/* -- gth_browser_activate_video_screenshot -- */
 
 
 typedef struct {
@@ -68,6 +69,7 @@ save_screenshot_task_completed_cb (GthTask  *task,
 		gst_element_set_state (gth_media_viewer_page_get_playbin (page), GST_STATE_PLAYING);
 
 	save_date_free (save_data);
+	g_object_unref (task);
 }
 
 
@@ -76,11 +78,11 @@ save_as_response_cb (GtkDialog  *file_sel,
 		     int         response,
 		     SaveData   *save_data)
 {
-	GFile       *file;
-	GFile       *folder;
-	char        *folder_uri;
-	const char  *mime_type;
-	GthTask     *task;
+	GFile      *file;
+	const char *mime_type;
+	GFile      *folder;
+	char       *folder_uri;
+	GthTask    *task;
 
 	if (response != GTK_RESPONSE_OK) {
 		GthMediaViewerPage *page = save_data->page;
@@ -134,6 +136,7 @@ screenshot_ready_cb (GdkPixbuf *pixbuf,
 
 	save_data->image = gth_image_new_for_pixbuf (pixbuf);
 	file_sel = gth_file_chooser_dialog_new (_("Save Image"), GTK_WINDOW (save_data->browser), "image-saver");
+	gtk_window_set_modal (GTK_WINDOW (file_sel), TRUE);
 
 	{
 		char        *last_uri;
@@ -144,8 +147,15 @@ screenshot_ready_cb (GdkPixbuf *pixbuf,
 		int          attempt;
 
 		last_uri = g_settings_get_string (save_data->settings, PREF_GSTREAMER_TOOLS_SCREESHOT_LOCATION);
-		if ((last_uri == NULL) || (strcmp (last_uri, "~") == 0))
-			last_folder = g_file_new_for_path (g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
+		if ((last_uri == NULL) || (strcmp (last_uri, "~") == 0) || (strcmp (last_uri, "file://~") == 0)) {
+			const char *dir;
+
+			dir = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
+			if (dir != NULL)
+				last_folder = g_file_new_for_path (dir);
+			else
+				last_folder = g_file_new_for_uri (get_home_uri ());
+		}
 		else
 			last_folder = g_file_new_for_uri (last_uri);
 		gtk_file_chooser_set_current_folder_file (GTK_FILE_CHOOSER (file_sel), last_folder, NULL);
@@ -188,14 +198,18 @@ screenshot_ready_cb (GdkPixbuf *pixbuf,
 
 
 void
-media_viewer_activate_action_screenshot (GtkAction          *action,
-				         GthMediaViewerPage *page)
+gth_browser_activate_video_screenshot (GSimpleAction	*action,
+				       GVariant		*parameter,
+				       gpointer		 user_data)
 {
-	GstElement *playbin;
-	SaveData   *save_data;
-	int         video_fps_n;
-	int         video_fps_d;
+	GthBrowser		*browser = GTH_BROWSER (user_data);
+	GthMediaViewerPage	*page;
+	GstElement		*playbin;
+	SaveData		*save_data;
+	int			 video_fps_n;
+	int			 video_fps_d;
 
+	page = GTH_MEDIA_VIEWER_PAGE (gth_browser_get_viewer_page (browser));
 	playbin = gth_media_viewer_page_get_playbin (page);
 	if (playbin == NULL)
 		return;
