@@ -48,8 +48,6 @@ G_DEFINE_TYPE_WITH_CODE (GthMediaViewerPage,
 
 struct _GthMediaViewerPagePrivate {
 	GthBrowser     *browser;
-	GtkActionGroup *actions;
-	guint           merge_id;
 	GthFileData    *file_data;
 	GstElement     *playbin;
 	GtkBuilder     *builder;
@@ -68,42 +66,24 @@ struct _GthMediaViewerPagePrivate {
 	gulong          update_volume_id;
 	gdouble         rate;
 	GtkWidget      *mediabar;
-	GtkWidget      *fullscreen_toolbar;
+	GtkWidget      *mediabar_revealer;
 	gulong          video_window_xid;
 	gboolean        xwin_assigned;
 	GdkPixbuf      *icon;
 	PangoLayout    *caption_layout;
-	gboolean        block_next_jump;
 	GdkCursor      *cursor;
 	GdkCursor      *cursor_void;
 	gboolean        cursor_visible;
 	GthScreensaver *screensaver;
+	GtkWidget      *screenshot_button;
 };
 
 
 static double default_rates[] = { 0.03, 0.06, 0.12, 0.25, 0.33, 0.50, 0.66, 1.0, 1.50, 2.0, 3.0, 4.0, 8.0, 16.0, 32.0 };
 
 
-static const char *media_viewer_ui_info =
-"<ui>"
-"  <toolbar name='ViewerToolBar'>"
-"    <placeholder name='ViewerCommands'>"
-"      <toolitem action='MediaViewer_Screenshot'/>"
-"    </placeholder>"
-"  </toolbar>"
-"  <toolbar name='Fullscreen_ToolBar'>"
-"    <placeholder name='ViewerCommands'>"
-"      <toolitem action='MediaViewer_Screenshot'/>"
-"    </placeholder>"
-"  </toolbar>"
-"</ui>";
-
-
-static GtkActionEntry media_viewer_action_entries[] = {
-	{ "MediaViewer_Screenshot", "camera-photo",
-	  N_("Screenshot"), NULL,
-	  N_("Take a screenshot"),
-	  G_CALLBACK (media_viewer_activate_action_screenshot) },
+static const GActionEntry actions[] = {
+	{ "video-screenshot", gth_browser_activate_video_screenshot }
 };
 
 
@@ -226,11 +206,12 @@ video_area_draw_cb (GtkWidget *widget,
 	cairo_fill (cr);
 
 	if (self->priv->icon != NULL) {
-		int            icon_w, icon_h;
-		int            text_w;
-		int            icon_x, icon_y;
-		PangoRectangle logical_rect;
-		int            x, y;
+		int                   icon_w, icon_h;
+		int                   text_w;
+		int                   icon_x, icon_y;
+		PangoRectangle        logical_rect;
+		int                   x, y;
+		PangoFontDescription *font;
 
 		icon_w = gdk_pixbuf_get_width (self->priv->icon);
 		icon_h = gdk_pixbuf_get_height (self->priv->icon);
@@ -250,7 +231,8 @@ video_area_draw_cb (GtkWidget *widget,
 		cairo_fill (cr);
 
 		cairo_move_to (cr, x, y);
-		pango_layout_set_font_description (self->priv->caption_layout, gtk_style_context_get_font (style_context, gtk_widget_get_state (widget)));
+		gtk_style_context_get (style_context, gtk_widget_get_state_flags (widget), "font", &font, NULL);
+		pango_layout_set_font_description (self->priv->caption_layout, font);
 		pango_cairo_layout_path (cr, self->priv->caption_layout);
 		cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 		cairo_fill (cr);
@@ -306,6 +288,7 @@ volume_value_changed_cb (GtkAdjustment *adjustment,
 			 gpointer       user_data)
 {
 	GthMediaViewerPage *self = user_data;
+
 	if (self->priv->playbin != NULL)
 		g_object_set (self->priv->playbin,
 			      "volume",
@@ -383,76 +366,6 @@ position_value_changed_cb (GtkAdjustment *adjustment,
 }
 
 
-static char *
-volume_scale_format_value_cb (GtkScale *scale,
-			       double    value,
-			       gpointer  user_data)
-{
-	return g_strdup_printf ("%0.0f%%", value);
-}
-
-
-static gboolean
-position_scale_change_value_cb (GtkRange      *range,
-				 GtkScrollType  scroll,
-				 gdouble        value,
-				 gpointer       user_data)
-{
-	GthMediaViewerPage *self = user_data;
-
-	if (self->priv->block_next_jump && (scroll == GTK_SCROLL_JUMP)) {
-		self->priv->block_next_jump = FALSE;
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
-
-static gboolean
-position_scale_button_press_event_cb (GtkWidget      *widget,
-				      GdkEventButton *event,
-				      gpointer        user_data)
-{
-#if ! GTK_CHECK_VERSION(3,6,0)
-	/* In a video player when clicking on the progress bar the user expect
-	 * to jump to the specified position directly instead of scrolling one
-	 * page up or down as happens by default in Gtk+.  The button 2
-	 * behavior is what we want by default for button 1.  This is the
-	 * default behaviour in Gtk+ 3.6.0 or greater. */
-
-	if (event->button == 1)
-		event->button = 2;
-	else if (event->button == 2)
-		event->button = 1;
-#endif
-
-	return FALSE;
-}
-
-
-static gboolean
-position_scale_button_release_event_cb (GtkWidget      *widget,
-					 GdkEventButton *event,
-					 gpointer        user_data)
-{
-	GthMediaViewerPage *self = user_data;
-
-#if ! GTK_CHECK_VERSION(3,6,0)
-	/* Swap button 1 and 2 behaviors, see position_scale_button_press_event_cb */
-	if (event->button == 1)
-		event->button = 2;
-	else if (event->button == 2)
-		event->button = 1;
-#endif
-
-	if (self->priv->playing)
-		self->priv->block_next_jump = TRUE;
-
-	return FALSE;
-}
-
-
 static void
 update_playback_info (GthMediaViewerPage *self)
 {
@@ -538,24 +451,6 @@ play_button_clicked_cb (GtkButton *button,
 }
 
 
-static void
-volume_togglebutton_toggled_cb (GtkToggleButton *button,
-				gpointer         user_data)
-{
-	GthMediaViewerPage *self = user_data;
-
-	if (self->priv->playbin == NULL)
-		return;
-
-	if (gtk_toggle_button_get_active (button)) {
-		g_object_get (self->priv->playbin, "volume", &self->priv->last_volume, NULL);
-		g_object_set (self->priv->playbin, "volume", 0.0, NULL);
-	}
-	else
-		g_object_set (self->priv->playbin, "volume", self->priv->last_volume, NULL);
-}
-
-
 static int
 get_nearest_rate (double rate)
 {
@@ -626,23 +521,10 @@ update_volume_from_playbin (GthMediaViewerPage *self)
 
 	g_object_get (self->priv->playbin, "volume", &volume, NULL);
 
-	if (volume == 0.0)
-		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("volume_togglebutton_image")), "audio-volume-muted-symbolic", GTK_ICON_SIZE_BUTTON);
-	else if (volume < 0.33)
-		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("volume_togglebutton_image")), "audio-volume-low-symbolic", GTK_ICON_SIZE_BUTTON);
-	else if (volume < 0.66)
-		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("volume_togglebutton_image")), "audio-volume-medium-symbolic", GTK_ICON_SIZE_BUTTON);
-	else
-		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("volume_togglebutton_image")), "audio-volume-high-symbolic", GTK_ICON_SIZE_BUTTON);
-
 	g_signal_handlers_block_by_func (GET_WIDGET ("volume_adjustment"), volume_value_changed_cb, self);
 	gtk_adjustment_set_value (GTK_ADJUSTMENT (GET_WIDGET ("volume_adjustment")), volume * 100.0);
 	g_signal_handlers_unblock_by_func (GET_WIDGET ("volume_adjustment"), volume_value_changed_cb, self);
 
-	g_signal_handlers_block_by_func (GET_WIDGET ("volume_togglebutton"), volume_togglebutton_toggled_cb, self);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("volume_togglebutton")), volume == 0.0);
-	g_signal_handlers_unblock_by_func (GET_WIDGET ("volume_togglebutton"), volume_togglebutton_toggled_cb, self);
-
 	return FALSE;
 }
 
@@ -694,8 +576,15 @@ update_play_button (GthMediaViewerPage *self,
 		update_playback_info (self);
 	}
 	else if (self->priv->playing && (new_state != GST_STATE_PLAYING)) {
+		GtkWidget *play_button = GET_WIDGET ("play_button_image");
+		gboolean   rtl;
+
+		rtl = gtk_widget_get_direction (play_button) == GTK_TEXT_DIR_RTL;
+
 		set_playing_state (self, FALSE);
-		gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("play_button_image")), "media-playback-start-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
+		gtk_image_set_from_icon_name (GTK_IMAGE (play_button),
+					      rtl ? "media-playback-start-rtl-symbolic" : "media-playback-start-symbolic",
+					      GTK_ICON_SIZE_LARGE_TOOLBAR);
 		gtk_widget_set_tooltip_text (GET_WIDGET ("play_button_image"), _("Play"));
 
 		if (self->priv->update_progress_id != 0) {
@@ -715,6 +604,7 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 				     GthBrowser    *browser)
 {
 	GthMediaViewerPage *self;
+	gboolean            rtl;
 
 	if (! gstreamer_init ())
 		return;
@@ -722,18 +612,32 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 	self = (GthMediaViewerPage*) base;
 
 	self->priv->browser = browser;
+	g_action_map_add_action_entries (G_ACTION_MAP (browser),
+					 actions,
+					 G_N_ELEMENTS (actions),
+					 browser);
+	self->priv->screenshot_button =
+			gth_browser_add_header_bar_button (browser,
+							   GTH_BROWSER_HEADER_SECTION_VIEWER_VIEW,
+							   "camera-photo-symbolic",
+							   _("Take a screenshot"),
+							   "win.video-screenshot",
+							   NULL);
 
-	self->priv->actions = gtk_action_group_new ("Video Viewer Actions");
-	gtk_action_group_set_translation_domain (self->priv->actions, NULL);
-	gtk_action_group_add_actions (self->priv->actions,
-				      media_viewer_action_entries,
-				      G_N_ELEMENTS (media_viewer_action_entries),
-				      self);
-	gtk_ui_manager_insert_action_group (gth_browser_get_ui_manager (browser), self->priv->actions, 0);
+	rtl = gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL;
 
 	/* video area */
 
 	self->priv->area = gtk_drawing_area_new ();
+
+	/* do not use the rgba visual on the drawing area */
+	{
+	        GdkVisual *visual;
+	        visual = gdk_screen_get_system_visual (gtk_widget_get_screen (GTK_WIDGET (self->priv->area)));
+        	if (visual != NULL)
+                	gtk_widget_set_visual (GTK_WIDGET (self->priv->area), visual);
+	}
+
 	gtk_widget_set_double_buffered (self->priv->area, FALSE);
 	gtk_widget_add_events (self->priv->area, (gtk_widget_get_events (self->priv->area)
 						  | GDK_EXPOSURE_MASK
@@ -779,7 +683,15 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 
 	self->priv->builder = _gtk_builder_new_from_file ("mediabar.ui", "gstreamer_tools");
 	self->priv->mediabar = GET_WIDGET ("mediabar");
-	gtk_widget_show (self->priv->mediabar);
+	gtk_widget_set_halign (self->priv->mediabar, GTK_ALIGN_FILL);
+	gtk_widget_set_valign (self->priv->mediabar, GTK_ALIGN_END);
+
+	gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("play_slower_image")),
+				      rtl ? "media-seek-backward-rtl-symbolic" : "media-seek-backward-symbolic",
+				      GTK_ICON_SIZE_MENU);
+	gtk_image_set_from_icon_name (GTK_IMAGE (GET_WIDGET ("play_faster_image")),
+				      rtl ? "media-seek-forward-rtl-symbolic" : "media-seek-forward-symbolic",
+				      GTK_ICON_SIZE_MENU);
 
 	g_signal_connect (GET_WIDGET ("volume_adjustment"),
 			  "value-changed",
@@ -789,30 +701,10 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 			  "value-changed",
 			  G_CALLBACK (position_value_changed_cb),
 			  self);
-	g_signal_connect (GET_WIDGET ("volume_scale"),
-			  "format-value",
-			  G_CALLBACK (volume_scale_format_value_cb),
-			  self);
-	g_signal_connect (GET_WIDGET ("position_scale"),
-			  "change-value",
-			  G_CALLBACK (position_scale_change_value_cb),
-			  self);
-	g_signal_connect (GET_WIDGET ("position_scale"),
-			  "button-press-event",
-			  G_CALLBACK (position_scale_button_press_event_cb),
-			  self);
-	g_signal_connect (GET_WIDGET ("position_scale"),
-			  "button-release-event",
-			  G_CALLBACK (position_scale_button_release_event_cb),
-			  self);
 	g_signal_connect (GET_WIDGET ("play_button"),
 			  "clicked",
 			  G_CALLBACK (play_button_clicked_cb),
 			  self);
-	g_signal_connect (GET_WIDGET ("volume_togglebutton"),
-			  "toggled",
-			  G_CALLBACK (volume_togglebutton_toggled_cb),
-			  self);
 	g_signal_connect (GET_WIDGET ("play_slower_button"),
 			  "clicked",
 			  G_CALLBACK (play_slower_button_clicked_cb),
@@ -822,37 +714,26 @@ gth_media_viewer_page_real_activate (GthViewerPage *base,
 			  G_CALLBACK (play_faster_button_clicked_cb),
 			  self);
 
-	self->priv->area_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-	gtk_box_pack_start (GTK_BOX (self->priv->area_box), self->priv->area, TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (self->priv->area_box), self->priv->mediabar, FALSE, FALSE, 0);
+	self->priv->mediabar_revealer = gtk_revealer_new ();
+	gtk_revealer_set_transition_type (GTK_REVEALER (self->priv->mediabar_revealer), GTK_REVEALER_TRANSITION_TYPE_SLIDE_UP);
+	gtk_widget_set_halign (self->priv->mediabar_revealer, GTK_ALIGN_FILL);
+	gtk_widget_set_valign (self->priv->mediabar_revealer, GTK_ALIGN_END);
+	gtk_widget_show (self->priv->mediabar_revealer);
+	gtk_container_add (GTK_CONTAINER (self->priv->mediabar_revealer), self->priv->mediabar);
+
+	self->priv->area_box = gtk_overlay_new ();
+	gtk_container_add (GTK_CONTAINER (self->priv->area_box), self->priv->area);
+	gtk_overlay_add_overlay (GTK_OVERLAY (self->priv->area_box), self->priv->mediabar_revealer);
 	gtk_widget_show (self->priv->area_box);
 	gth_browser_set_viewer_widget (browser, self->priv->area_box);
 
 	gtk_widget_realize (self->priv->area);
+	gth_browser_register_viewer_control (self->priv->browser, self->priv->mediabar);
 
 	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 }
 
 
-static void
-remove_fullscreen_toolbar (GthMediaViewerPage *self)
-{
-	if (self->priv->fullscreen_toolbar == NULL)
-		return;
-
-	if (gtk_widget_get_parent (self->priv->mediabar) == self->priv->fullscreen_toolbar) {
-		g_object_ref (self->priv->mediabar);
-		gtk_container_remove (GTK_CONTAINER (self->priv->fullscreen_toolbar), self->priv->mediabar);
-		gtk_box_pack_start (GTK_BOX (self->priv->area_box), self->priv->mediabar, FALSE, FALSE, 0);
-		g_object_unref (self->priv->mediabar);
-	}
-
-	gth_browser_unregister_fullscreen_control (self->priv->browser, self->priv->fullscreen_toolbar);
-	gtk_widget_destroy (self->priv->fullscreen_toolbar);
-	self->priv->fullscreen_toolbar = NULL;
-}
-
-
 static void
 save_volume (GthMediaViewerPage *self)
 {
@@ -874,6 +755,8 @@ gth_media_viewer_page_real_deactivate (GthViewerPage *base)
 
 	self = (GthMediaViewerPage*) base;
 
+	gth_browser_unregister_viewer_control (self->priv->browser, self->priv->mediabar);
+
 	if (self->priv->builder != NULL) {
 		g_object_unref (self->priv->builder);
 		self->priv->builder = NULL;
@@ -896,11 +779,8 @@ gth_media_viewer_page_real_deactivate (GthViewerPage *base)
 		self->priv->playbin = NULL;
 	}
 
-	gtk_ui_manager_remove_action_group (gth_browser_get_ui_manager (self->priv->browser), self->priv->actions);
-	g_object_unref (self->priv->actions);
-	self->priv->actions = NULL;
-
-	remove_fullscreen_toolbar (self);
+	gtk_widget_destroy (self->priv->screenshot_button);
+	self->priv->screenshot_button = NULL;
 
 	gth_browser_set_viewer_widget (self->priv->browser, NULL);
 }
@@ -1044,7 +924,7 @@ bus_message_cb (GstBus     *bus,
 	case GST_MESSAGE_BUFFERING: {
 		int percent = 0;
 		gst_message_parse_buffering (message, &percent);
-		gst_element_set_state(self->priv->playbin, (percent == 100) ? GST_STATE_PLAYING : GST_STATE_PAUSED);
+		gst_element_set_state (self->priv->playbin, (percent == 100) ? GST_STATE_PLAYING : GST_STATE_PAUSED);
 		break;
 	}
 
@@ -1123,17 +1003,8 @@ static void
 gth_media_viewer_page_real_show (GthViewerPage *base)
 {
 	GthMediaViewerPage *self = GTH_MEDIA_VIEWER_PAGE (base);
-	GError             *error = NULL;
 
 	self->priv->visible = TRUE;
-
-	if (self->priv->merge_id == 0) {
-		self->priv->merge_id = gtk_ui_manager_add_ui_from_string (gth_browser_get_ui_manager (self->priv->browser), media_viewer_ui_info, -1, &error);
-		if (self->priv->merge_id == 0) {
-			g_warning ("ui building failed: %s", error->message);
-			g_error_free (error);
-		}
-	}
 	gth_viewer_page_focus (GTH_VIEWER_PAGE (self));
 
 	create_playbin (self);
@@ -1156,12 +1027,6 @@ gth_media_viewer_page_real_hide (GthViewerPage *base)
 	self = (GthMediaViewerPage*) base;
 
 	self->priv->visible = FALSE;
-
-	if (self->priv->merge_id != 0) {
-		gtk_ui_manager_remove_ui (gth_browser_get_ui_manager (self->priv->browser), self->priv->merge_id);
-		self->priv->merge_id = 0;
-	}
-
 	if ((self->priv->playbin != NULL) && self->priv->playing)
 		gst_element_set_state (self->priv->playbin, GST_STATE_PAUSED);
 }
@@ -1246,45 +1111,7 @@ static void
 gth_media_viewer_page_real_fullscreen (GthViewerPage *base,
 				       gboolean       active)
 {
-	GthMediaViewerPage *self = (GthMediaViewerPage*) base;
-	GdkScreen          *screen;
-	int                 n_monitor;
-	GdkRectangle        work_area;
-
-	if (! active) {
-		remove_fullscreen_toolbar (self);
-		return;
-	}
-
-	/* active == TRUE */
-
-	screen = gtk_widget_get_screen (GTK_WIDGET (self->priv->browser));
-	n_monitor = gdk_screen_get_monitor_at_window (screen, gtk_widget_get_window (GTK_WIDGET (self->priv->browser)));
-	gdk_screen_get_monitor_geometry (screen, n_monitor, &work_area);
-
-	if (self->priv->fullscreen_toolbar == NULL) {
-		self->priv->fullscreen_toolbar = gtk_window_new (GTK_WINDOW_POPUP);
-		gtk_container_set_border_width (GTK_CONTAINER (self->priv->fullscreen_toolbar), 0);
-	}
-
-	if (gtk_widget_get_parent (self->priv->mediabar) == self->priv->area_box) {
-		g_object_ref (self->priv->mediabar);
-		gtk_container_remove (GTK_CONTAINER (self->priv->area_box), self->priv->mediabar);
-		gtk_container_add (GTK_CONTAINER (self->priv->fullscreen_toolbar), self->priv->mediabar);
-		g_object_unref (self->priv->mediabar);
-	}
-
-	gtk_widget_realize (self->priv->fullscreen_toolbar);
-
-	gtk_window_set_screen (GTK_WINDOW (self->priv->fullscreen_toolbar), screen);
-	gtk_window_resize (GTK_WINDOW (self->priv->fullscreen_toolbar),
-			   work_area.width,
-			   gtk_widget_get_allocated_height (self->priv->fullscreen_toolbar));
-	gtk_window_move (GTK_WINDOW (self->priv->fullscreen_toolbar),
-			 work_area.x,
-			 work_area.height - gtk_widget_get_allocated_height (self->priv->fullscreen_toolbar));
-
-	gth_browser_register_fullscreen_control (self->priv->browser, self->priv->fullscreen_toolbar);
+	/* void */
 }
 
 
@@ -1299,27 +1126,10 @@ gth_media_viewer_page_real_show_pointer (GthViewerPage *base,
 	if (show && (self->priv->cursor != NULL))
 		gdk_window_set_cursor (gtk_widget_get_window (self->priv->area), self->priv->cursor);
 
-	if (! show && (self->priv->cursor_void != NULL))
+	if (! show && gth_browser_get_is_fullscreen (self->priv->browser) && (self->priv->cursor_void != NULL))
 		gdk_window_set_cursor (gtk_widget_get_window (self->priv->area), self->priv->cursor_void);
 
-	if (self->priv->fullscreen_toolbar != NULL) {
-		if (show)
-			gtk_widget_show (self->priv->fullscreen_toolbar);
-		else
-			gtk_widget_hide (self->priv->fullscreen_toolbar);
-	}
-}
-
-
-static void
-set_action_sensitive (GthMediaViewerPage *self,
-		      const char         *action_name,
-		      gboolean            sensitive)
-{
-	GtkAction *action;
-
-	action = gtk_action_group_get_action (self->priv->actions, action_name);
-	g_object_set (action, "sensitive", sensitive, NULL);
+	gtk_revealer_set_reveal_child (GTK_REVEALER (self->priv->mediabar_revealer), show);
 }
 
 
@@ -1329,7 +1139,7 @@ gth_media_viewer_page_real_update_sensitivity (GthViewerPage *base)
 	GthMediaViewerPage *self = (GthMediaViewerPage *) base;
 
 	gtk_widget_set_sensitive (GET_WIDGET ("volume_box"), self->priv->has_audio);
-	set_action_sensitive (self, "MediaViewer_Screenshot", self->priv->has_video);
+	gth_window_enable_action (GTH_WINDOW (self->priv->browser), "video-screenshot", self->priv->has_video);
 }
 
 
@@ -1379,20 +1189,6 @@ gth_media_viewer_page_real_update_info (GthViewerPage *base,
 }
 
 
-static void
-gth_media_viewer_page_real_shrink_wrap (GthViewerPage *base,
-					gboolean       value,
-					int           *other_width,
-					int           *other_height)
-{
-	GthMediaViewerPage *self = GTH_MEDIA_VIEWER_PAGE (base);
-
-	if (! value || (other_width == NULL) || (other_height == NULL))
-		return;
-	*other_height += _gtk_widget_get_allocated_height (self->priv->mediabar);
-}
-
-
 static void
 gth_media_viewer_page_finalize (GObject *obj)
 {
@@ -1454,7 +1250,6 @@ gth_viewer_page_interface_init (GthViewerPageInterface *iface)
 	iface->save_as = gth_media_viewer_page_real_save_as;
 	iface->revert = gth_media_viewer_page_real_revert;
 	iface->update_info = gth_media_viewer_page_real_update_info;
-	iface->shrink_wrap = gth_media_viewer_page_real_shrink_wrap;
 }
 
 
@@ -1474,6 +1269,7 @@ gth_media_viewer_page_init (GthMediaViewerPage *self)
 	self->priv->cursor_visible = TRUE;
 	self->priv->screensaver = gth_screensaver_new (NULL);
 	self->priv->visible = FALSE;
+	self->priv->screenshot_button = NULL;
 }
 
 
