@@ -24,15 +24,18 @@
 #include <glib/gi18n.h>
 #include <gthumb.h>
 #include <extensions/image_viewer/gth-image-viewer-page.h>
+#include "actions.h"
 #include "gth-image-print-job.h"
 
 
 void
-gth_browser_activate_action_file_print (GtkAction  *action,
-					GthBrowser *browser)
+gth_browser_activate_print (GSimpleAction *action,
+			    GVariant      *parameter,
+			    gpointer       user_data)
 {
-	GList *items;
-	GList *file_list;
+	GthBrowser *browser = GTH_BROWSER (user_data);
+	GList      *items;
+	GList      *file_list;
 
 	items = gth_file_selection_get_selected (GTH_FILE_SELECTION (gth_browser_get_file_list_view (browser)));
 	file_list = gth_file_list_get_files (GTH_FILE_LIST (gth_browser_get_file_list (browser)), items);
@@ -46,19 +49,16 @@ gth_browser_activate_action_file_print (GtkAction  *action,
 		viewer_page = gth_browser_get_viewer_page (browser);
 		if ((gth_main_extension_is_active ("image_viewer"))
 		    && (viewer_page != NULL)
-		    && GTH_IS_IMAGE_VIEWER_PAGE (viewer_page))
+		    && GTH_IS_IMAGE_VIEWER_PAGE (viewer_page)
+		    && gth_image_viewer_page_get_is_modified (GTH_IMAGE_VIEWER_PAGE (viewer_page)))
 		{
-			GtkWidget *viewer;
-
-			viewer = gth_image_viewer_page_get_image_viewer (GTH_IMAGE_VIEWER_PAGE (viewer_page));
-			current_image = gth_image_viewer_get_current_image (GTH_IMAGE_VIEWER (viewer));
+			current_image = gth_image_viewer_page_get_modified_image (GTH_IMAGE_VIEWER_PAGE (viewer_page));
 		}
 		print_job = gth_image_print_job_new (file_list,
 						     gth_browser_get_current_file (browser),
 						     current_image,
 						     g_file_info_get_display_name (gth_browser_get_location_data (browser)->info),
 						     &error);
-		cairo_surface_destroy (current_image);
 
 		if (print_job != NULL) {
 			gth_image_print_job_run (print_job,
