@@ -457,11 +457,11 @@ delete_script_cb (GtkButton  *button,
 
 	d = _gtk_message_dialog_new (GTK_WINDOW (data->dialog),
 				     GTK_DIALOG_MODAL,
-				     GTK_STOCK_DIALOG_QUESTION,
+				     _GTK_ICON_NAME_DIALOG_QUESTION,
 				     _("Are you sure you want to delete the selected command?"),
 				     NULL,
-				     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				     GTK_STOCK_DELETE, GTK_RESPONSE_OK,
+				     _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
+				     _GTK_LABEL_DELETE, GTK_RESPONSE_OK,
 				     NULL);
 	result = gtk_dialog_run (GTK_DIALOG (d));
 	gtk_widget_destroy (d);
@@ -541,7 +541,7 @@ dlg_personalize_scripts (GthBrowser *browser)
 
 	data = g_new0 (DialogData, 1);
 	data->browser = browser;
-	data->builder = _gtk_builder_new_from_file ("personalize-scripts.ui", "list_tools");
+	data->builder = gtk_builder_new_from_resource ("/org/gnome/gThumb/list_tools/data/ui/personalize-scripts.ui");
 
 	/* Get the widgets. */
 
