@@ -25,78 +25,52 @@
 #include <glib-object.h>
 #include <gthumb.h>
 #include "actions.h"
+#include "callbacks.h"
 #include "dlg-photo-importer.h"
+#include "photo-importer.h"
 #include "preferences.h"
 
 
-#define BROWSER_DATA_KEY "photo-importer-browser-data"
-
-
-static const char *ui_info =
-"<ui>"
-"  <menubar name='MenuBar'>"
-"    <menu name='File' action='FileMenu'>"
-"      <menu name='Import' action='ImportMenu'>"
-"        <placeholder name='Misc_Actions'>"
-"          <menuitem action='File_ImportFromDevice'/>"
-"          <menuitem action='File_ImportFromFolder'/>"
-"        </placeholder>"
-"      </menu>"
-"    </menu>"
-"  </menubar>"
-"</ui>";
-
-
-static GtkActionEntry action_entries[] = {
-	{ "File_ImportFromDevice", "camera-photo",
-	  N_("_Removable Device..."), NULL,
-	  N_("Import photos and other files from a removable device"),
-	  G_CALLBACK (gth_browser_activate_action_import_from_device) },
-	{ "File_ImportFromFolder", "folder",
-	  N_("F_older..."), NULL,
-	  N_("Import photos and other files from a folder"),
-	  G_CALLBACK (gth_browser_activate_action_import_from_folder) }
+static const GActionEntry actions[] = {
+	{ "import-device", gth_browser_activate_import_device },
+	{ "import-folder", gth_browser_activate_import_folder }
 };
 
 
-typedef struct {
-	GtkActionGroup *action_group;
-} BrowserData;
-
-
-static void
-browser_data_free (BrowserData *data)
-{
-	g_free (data);
-}
+static const GthMenuEntry action_entries[] = {
+	{ N_("_Removable Device…"), "win.import-device", NULL, "camera-photo-symbolic" },
+	{ N_("F_older…"), "win.import-folder", NULL, "folder-symbolic" }
+};
 
 
 void
 pi__gth_browser_construct_cb (GthBrowser *browser)
 {
-	BrowserData *data;
-	GError      *error = NULL;
-	guint        merge_id;
+	GtkBuilder	*builder;
+	GMenuModel	*import_menu;
+	GMenu		*other_actions;
 
 	g_return_if_fail (GTH_IS_BROWSER (browser));
 
-	data = g_new0 (BrowserData, 1);
+	g_action_map_add_action_entries (G_ACTION_MAP (browser),
+					 actions,
+					 G_N_ELEMENTS (actions),
+					 browser);
 
-	data->action_group = gtk_action_group_new ("Photo Importer Actions");
-	gtk_action_group_set_translation_domain (data->action_group, NULL);
-	gtk_action_group_add_actions (data->action_group,
-				      action_entries,
-				      G_N_ELEMENTS (action_entries),
-				      browser);
-	gtk_ui_manager_insert_action_group (gth_browser_get_ui_manager (browser), data->action_group, 0);
+	builder = gtk_builder_new_from_resource ("/org/gnome/gThumb/photo_importer/data/ui/import-menu.ui");
+	import_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "import-menu"));
+	other_actions = gth_menu_manager_get_menu (gth_browser_get_menu_manager (browser, GTH_BROWSER_MENU_MANAGER_GEARS_OTHER_ACTIONS));
+	g_menu_append_submenu (other_actions, _("I_mport From"), import_menu);
 
-	merge_id = gtk_ui_manager_add_ui_from_string (gth_browser_get_ui_manager (browser), ui_info, -1, &error);
-	if (merge_id == 0) {
-		g_warning ("building ui failed: %s", error->message);
-		g_clear_error (&error);
-	}
+	gth_browser_add_menu_manager_for_menu (browser, GTH_BROWSER_MENU_MANAGER_WEB_IMPORTERS, G_MENU (gtk_builder_get_object (builder, "web-importers")));
+	gth_browser_add_menu_manager_for_menu (browser, GTH_BROWSER_MENU_MANAGER_OTHER_IMPORTERS, G_MENU (gtk_builder_get_object (builder, "other-importers")));
+
+	gth_menu_manager_append_entries (gth_browser_get_menu_manager (browser, GTH_BROWSER_MENU_MANAGER_OTHER_IMPORTERS),
+				       	 action_entries,
+				       	 G_N_ELEMENTS (action_entries));
+
+	g_object_unref (builder);
 
-	g_object_set_data_full (G_OBJECT (browser), BROWSER_DATA_KEY, data, (GDestroyNotify) browser_data_free);
 }
 
 
@@ -187,7 +161,7 @@ pi__dlg_preferences_construct_cb (GtkWidget  *dialog,
 	GtkWidget       *widget;
 
 	data = g_new0 (PreferencesData, 1);
-	data->builder = _gtk_builder_new_from_file("photo-importer-options.ui", "photo_importer");
+	data->builder = gtk_builder_new_from_resource ("/org/gnome/gThumb/photo_importer/data/ui/photo-importer-options.ui");
 	data->settings = g_settings_new (GTHUMB_PHOTO_IMPORTER_SCHEMA);
 
 	general_vbox = _gtk_builder_get_widget (dialog_builder, "general_vbox");
