@@ -159,7 +159,7 @@ destroy_dialog (gpointer user_data)
 	_g_object_list_unref (data->files);
 	_g_string_list_free (data->general_tests);
 
-	if (! data->import && ImportPhotos)
+	if (! data->import && gth_browser_get_close_with_task (data->browser))
 		gth_window_close (GTH_WINDOW (data->browser));
 
 	g_free (data);
@@ -243,14 +243,6 @@ ok_clicked_cb (GtkWidget  *widget,
 }
 
 
-static void
-help_clicked_cb (GtkWidget  *widget,
-                 DialogData *data)
-{
-        show_help_dialog (GTK_WINDOW (data->dialog), "gthumb-importing");
-}
-
-
 static void
 update_sensitivity (DialogData *data)
 {
@@ -366,7 +358,7 @@ list_source_files (gpointer user_data)
 		return;
 	}
 
-	gth_file_list_clear (GTH_FILE_LIST (data->file_list), _("Getting folder listing..."));
+	gth_file_list_clear (GTH_FILE_LIST (data->file_list), _("Getting the folder content…"));
 
 	data->loading_list = TRUE;
 	list = g_list_prepend (NULL, data->source);
@@ -567,7 +559,7 @@ dlg_photo_importer (GthBrowser            *browser,
 
 	data = g_new0 (DialogData, 1);
 	data->browser = browser;
-	data->builder = _gtk_builder_new_from_file ("photo-importer.ui", "photo_importer");
+	data->builder = gtk_builder_new_from_resource ("/org/gnome/gThumb/photo_importer/data/ui/photo-importer.ui");
 	data->settings = g_settings_new (GTHUMB_PHOTO_IMPORTER_SCHEMA);
 	data->selector_type = selector_type;
 	data->source = _g_object_ref (source);
@@ -625,7 +617,7 @@ dlg_photo_importer (GthBrowser            *browser,
 		gtk_box_pack_start (GTK_BOX (GET_WIDGET ("source_box")), data->folder_chooser, TRUE, TRUE, 0);
 	}
 
-	data->file_list = gth_file_list_new (gth_grid_view_new (), GTH_FILE_LIST_TYPE_NORMAL, FALSE);
+	data->file_list = gth_file_list_new (gth_grid_view_new (), GTH_FILE_LIST_MODE_NORMAL, FALSE);
 	sort_type = gth_main_get_sort_type ("file::mtime");
 	gth_file_list_set_sort_func (GTH_FILE_LIST (data->file_list), sort_type->cmp_func, FALSE);
 	gth_file_list_enable_thumbs (GTH_FILE_LIST (data->file_list), TRUE);
@@ -671,7 +663,7 @@ dlg_photo_importer (GthBrowser            *browser,
 
 	_g_string_list_free (tests);
 
-	data->tags_entry = gth_tags_entry_new ();
+	data->tags_entry = gth_tags_entry_new (GTH_TAGS_ENTRY_MODE_POPUP);
 	gtk_widget_show (data->tags_entry);
 	gtk_box_pack_start (GTK_BOX (GET_WIDGET ("tags_entry_box")), data->tags_entry, TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("tags_label")), data->tags_entry);
@@ -703,10 +695,6 @@ dlg_photo_importer (GthBrowser            *browser,
 			  "clicked",
 			  G_CALLBACK (close_dialog),
 			  data);
-        g_signal_connect (GET_WIDGET ("help_button"),
-                          "clicked",
-                          G_CALLBACK (help_clicked_cb),
-                          data);
         if (data->selector_type == DLG_IMPORTER_SOURCE_TYPE_DEVICE)
 		g_signal_connect (data->device_chooser,
 				  "changed",
