@@ -647,10 +647,6 @@ dialog_response_cb (GtkDialog *dialog,
 			destroy_dialog (data);
 		break;
 
-	case GTK_RESPONSE_HELP:
-		show_help_dialog (GTK_WINDOW (dialog), "gthumb-rename-series");
-		break;
-
 	case GTK_RESPONSE_OK:
 		ok_button_clicked (data);
 		break;
@@ -685,10 +681,10 @@ update_preview__step2 (GError   *error,
 
 		d = _gtk_message_dialog_new (GTK_WINDOW (data->dialog),
 					     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-					     GTK_STOCK_DIALOG_ERROR,
+					     _GTK_ICON_NAME_DIALOG_ERROR,
 					     _("Could not rename the files"),
 					     error->message,
-					     GTK_STOCK_OK, GTK_RESPONSE_OK,
+					     _GTK_LABEL_OK, GTK_RESPONSE_OK,
 					     NULL);
 		g_signal_connect (d, "response", G_CALLBACK (error_dialog_response_cb), data);
 		gtk_window_present (GTK_WINDOW (d));
@@ -880,6 +876,7 @@ dlg_rename_series (GthBrowser *browser,
 	gtk_tree_view_append_column (GTK_TREE_VIEW (data->list_view), column);
 
 	gtk_widget_show (data->list_view);
+	gtk_widget_set_vexpand (data->list_view, TRUE);
 	gtk_container_add (GTK_CONTAINER (GET_WIDGET ("preview_scrolledwindow")), data->list_view);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (GET_WIDGET ("preview_label")), data->list_view);
