@@ -96,7 +96,7 @@ exec_resize (GthAsyncTask *task,
 	int              max_w, max_h;
 	GthImage        *destination_image;
 
-	source = gth_image_get_cairo_surface (gth_image_task_get_source (GTH_IMAGE_TASK (task)));
+	source = gth_image_task_get_source_surface (GTH_IMAGE_TASK (task));
 	w = cairo_image_surface_get_width (source);
 	h = cairo_image_surface_get_height (source);
 
@@ -141,14 +141,6 @@ exec_resize (GthAsyncTask *task,
 }
 
 
-static void
-help_clicked_cb (GtkWidget  *widget,
-		 DialogData *data)
-{
-	show_help_dialog (GTK_WINDOW (data->dialog), "gthumb-batch-resize");
-}
-
-
 static void
 ok_clicked_cb (GtkWidget  *widget,
 	       DialogData *data)
@@ -355,9 +347,9 @@ dlg_resize_images (GthBrowser *browser,
 	/* Set widgets data. */
 
 	first_file_data = (GthFileData *) data->file_list->data;
-	gtk_file_chooser_set_file (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")),
-				   first_file_data->file,
-				   NULL);
+	_gtk_file_chooser_set_file_parent (GTK_FILE_CHOOSER (GET_WIDGET ("destination_filechooserbutton")),
+				   	   first_file_data->file,
+				   	   NULL);
 
 	unit = g_settings_get_enum (data->settings, PREF_RESIZE_IMAGES_UNIT);
 	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("unit_combobox")), unit);
@@ -465,10 +457,6 @@ dlg_resize_images (GthBrowser *browser,
 			  "clicked",
 			  G_CALLBACK (ok_clicked_cb),
 			  data);
-        g_signal_connect (GET_WIDGET ("help_button"),
-                          "clicked",
-                          G_CALLBACK (help_clicked_cb),
-                          data);
 	g_signal_connect_swapped (GET_WIDGET ("cancel_button"),
 				  "clicked",
 				  G_CALLBACK (gtk_widget_destroy),
