@@ -33,39 +33,15 @@
 
 
 #define BROWSER_DATA_KEY "search-browser-data"
-#define _RESPONSE_REFRESH 2
 
 
-static const char *find_ui_info =
-"<ui>"
-"  <menubar name='MenuBar'>"
-"    <menu name='Edit' action='EditMenu'>"
-"      <placeholder name='Edit_Actions'>"
-"        <menuitem action='Edit_Find'/>"
-"      </placeholder>"
-"    </menu>"
-"  </menubar>"
-"  <toolbar name='ToolBar'>"
-"    <placeholder name='SourceCommands'>"
-"      <toolitem action='Edit_Find'/>"
-"    </placeholder>"
-"  </toolbar>"
-"</ui>";
-
-
-static GtkActionEntry find_action_entries[] = {
-	{ "Edit_Find", GTK_STOCK_FIND,
-	  NULL, NULL,
-	  N_("Find files"),
-	  G_CALLBACK (gth_browser_activate_action_edit_find) }
+static const GActionEntry actions[] = {
+	{ "find", gth_browser_activate_find }
 };
-static guint find_action_entries_size = G_N_ELEMENTS (find_action_entries);
 
 
 typedef struct {
-	GtkActionGroup *find_action;
-	guint           find_merge_id;
-	GtkWidget      *refresh_button;
+	GtkWidget *refresh_button;
 } BrowserData;
 
 
@@ -80,27 +56,21 @@ void
 search__gth_browser_construct_cb (GthBrowser *browser)
 {
 	BrowserData *data;
-	GError      *error = NULL;
 
 	g_return_if_fail (GTH_IS_BROWSER (browser));
 
+	g_action_map_add_action_entries (G_ACTION_MAP (browser),
+					 actions,
+					 G_N_ELEMENTS (actions),
+					 browser);
+	gth_browser_add_header_bar_button (browser,
+					   GTH_BROWSER_HEADER_SECTION_BROWSER_COMMANDS,
+					   "edit-find-symbolic",
+					   _("Find files"),
+					   "win.find",
+					   "<Control>F");
+
 	data = g_new0 (BrowserData, 1);
-
-	data->find_action = gtk_action_group_new ("Find Action");
-	gtk_action_group_set_translation_domain (data->find_action, NULL);
-	gtk_action_group_add_actions (data->find_action,
-				      find_action_entries,
-				      find_action_entries_size,
-				      browser);
-	gtk_ui_manager_insert_action_group (gth_browser_get_ui_manager (browser), data->find_action, 0);
-
-	data->find_merge_id = gtk_ui_manager_add_ui_from_string (gth_browser_get_ui_manager (browser), find_ui_info, -1, &error);
-	if (data->find_merge_id == 0) {
-		g_warning ("building menus failed: %s", error->message);
-		g_error_free (error);
-	}
-	/*gtk_tool_item_set_is_important (GTK_TOOL_ITEM (gtk_ui_manager_get_widget (gth_browser_get_ui_manager (browser), "/ToolBar/SourceCommands/Edit_Find")), TRUE);*/
-
 	g_object_set_data_full (G_OBJECT (browser), BROWSER_DATA_KEY, data, (GDestroyNotify) browser_data_free);
 }
 
@@ -109,7 +79,7 @@ static void
 refresh_button_clicked_cb (GtkButton  *button,
 			   GthBrowser *browser)
 {
-	gth_browser_activate_action_edit_search_update (NULL, browser);
+	gth_browser_update_search (browser);
 }
 
 
@@ -127,14 +97,16 @@ search__gth_browser_update_extra_widget_cb (GthBrowser *browser)
 
 	if (data->refresh_button == NULL) {
 		data->refresh_button = gtk_button_new ();
-		gtk_container_add (GTK_CONTAINER (data->refresh_button), gtk_image_new_from_stock (GTK_STOCK_REFRESH, GTK_ICON_SIZE_BUTTON));
+		gtk_container_add (GTK_CONTAINER (data->refresh_button), gtk_image_new_from_icon_name ("view-refresh-symbolic", GTK_ICON_SIZE_MENU));
 		g_object_add_weak_pointer (G_OBJECT (data->refresh_button), (gpointer *)&data->refresh_button);
 		gtk_button_set_relief (GTK_BUTTON (data->refresh_button), GTK_RELIEF_NONE);
 		gtk_widget_set_tooltip_text (data->refresh_button, _("Search again"));
 		gtk_widget_show_all (data->refresh_button);
-		gedit_message_area_add_action_widget (GEDIT_MESSAGE_AREA (gth_browser_get_list_extra_widget (browser)),
-					              data->refresh_button,
-					              _RESPONSE_REFRESH);
+		gtk_box_pack_start (GTK_BOX (gth_location_bar_get_action_area (GTH_LOCATION_BAR (gth_browser_get_location_bar (browser)))),
+				    data->refresh_button,
+				    FALSE,
+				    FALSE,
+				    0);
 		g_signal_connect (data->refresh_button,
 				  "clicked",
 				  G_CALLBACK (refresh_button_clicked_cb),
@@ -167,7 +139,7 @@ search__dlg_catalog_properties (GtkBuilder  *builder,
 	if (! _g_content_type_is_a (g_file_info_get_content_type (file_data->info), "gthumb/search"))
 		return;
 
-	vbox = gtk_vbox_new (FALSE, 6);
+	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (_gtk_builder_get_widget (builder, "general_vbox")), vbox, FALSE, FALSE, 0);
 
