@@ -80,15 +80,15 @@ typedef struct {
 	GthBrowser    *browser;
 	GthSearchTask *task;
 	gulong         response_id;
-} EmbeddedDialogData;
+} InfoBarData;
 
 
 static void
-embedded_dialog_response_cb (GthEmbeddedDialog *dialog,
-			     int                response_id,
-			     gpointer           user_data)
+info_bar_response_cb (GtkInfoBar *info_bar,
+		      int         response_id,
+		      gpointer    user_data)
 {
-	EmbeddedDialogData *data = user_data;
+	InfoBarData *data = user_data;
 
 	switch (response_id) {
 	case GTK_RESPONSE_CANCEL:
@@ -99,7 +99,7 @@ embedded_dialog_response_cb (GthEmbeddedDialog *dialog,
 		break;
 	}
 
-	g_signal_handler_disconnect (dialog, data->response_id);
+	g_signal_handler_disconnect (info_bar, data->response_id);
 	g_free (data);
 }
 
@@ -134,7 +134,7 @@ done_func (GObject  *object,
 	gsize          size;
 	GFile         *search_result_real_file;
 
-	gth_embedded_dialog_set_secondary_text (GTH_EMBEDDED_DIALOG (task->priv->dialog), NULL);
+	gth_info_bar_set_secondary_text (GTH_INFO_BAR (task->priv->dialog), NULL);
 
 	task->priv->error = NULL;
 	if (error != NULL) {
@@ -157,14 +157,14 @@ done_func (GObject  *object,
 	data = dom_document_dump (doc, &size);
 
 	search_result_real_file = gth_catalog_file_to_gio_file (task->priv->search_catalog);
-	g_write_file_async (search_result_real_file,
-			    data,
-			    size,
-			    TRUE,
-			    G_PRIORITY_DEFAULT,
-			    gth_task_get_cancellable (GTH_TASK (task)),
-			    save_search_result_copy_done_cb,
-			    task);
+	_g_file_write_async (search_result_real_file,
+			     data,
+			     size,
+			     TRUE,
+			     G_PRIORITY_DEFAULT,
+			     gth_task_get_cancellable (GTH_TASK (task)),
+			     save_search_result_copy_done_cb,
+			     task);
 
 	g_object_unref (search_result_real_file);
 	g_object_unref (doc);
@@ -179,7 +179,7 @@ update_secondary_text (GthSearchTask *task)
 
 	format_str = g_strdup_printf ("%"G_GSIZE_FORMAT, task->priv->n_files);
 	msg = g_strdup_printf (_("Files found until now: %s"), format_str);
-	gth_embedded_dialog_set_secondary_text (GTH_EMBEDDED_DIALOG (task->priv->dialog), msg);
+	gth_info_bar_set_secondary_text (GTH_INFO_BAR (task->priv->dialog), msg);
 
 	g_free (format_str);
 	g_free (msg);
@@ -236,7 +236,7 @@ start_dir_func (GFile      *directory,
 
 	uri = g_file_get_parse_name (directory);
 	text = g_strdup_printf ("Searching in %s", uri);
-	gth_embedded_dialog_set_primary_text (GTH_EMBEDDED_DIALOG (task->priv->dialog), text);
+	gth_info_bar_set_primary_text (GTH_INFO_BAR (task->priv->dialog), text);
 
 	g_free (text);
 	g_free (uri);
@@ -252,7 +252,7 @@ browser_location_ready_cb (GthBrowser    *browser,
 			   GthSearchTask *task)
 {
 	GtkWidget          *button;
-	EmbeddedDialogData *dialog_data;
+	InfoBarData *dialog_data;
 	GSettings          *settings;
 	GString            *attributes;
 	const char         *test_attributes;
@@ -266,26 +266,27 @@ browser_location_ready_cb (GthBrowser    *browser,
 
 	task->priv->n_files = 0;
 
-	task->priv->dialog = gth_browser_get_list_extra_widget (browser);
-	gth_embedded_dialog_set_icon (GTH_EMBEDDED_DIALOG (task->priv->dialog), GTK_STOCK_FIND, GTK_ICON_SIZE_DIALOG);
-	gth_embedded_dialog_set_primary_text (GTH_EMBEDDED_DIALOG (task->priv->dialog), _("Searching..."));
+	task->priv->dialog = gth_browser_get_list_info_bar (browser);
+	gth_info_bar_set_icon_name (GTH_INFO_BAR (task->priv->dialog), "edit-find-symbolic", GTK_ICON_SIZE_BUTTON);
+	gth_info_bar_set_primary_text (GTH_INFO_BAR (task->priv->dialog), _("Searching…"));
 	update_secondary_text (task);
-	gedit_message_area_clear_action_area (GEDIT_MESSAGE_AREA (task->priv->dialog));
+	_gtk_info_bar_clear_action_area (GTK_INFO_BAR (task->priv->dialog));
+	gtk_widget_show (task->priv->dialog);
+
 	button = gtk_button_new ();
-	gtk_container_add (GTK_CONTAINER (button), gtk_image_new_from_stock (GTK_STOCK_STOP, GTK_ICON_SIZE_BUTTON));
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
+	gtk_container_add (GTK_CONTAINER (button), gtk_image_new_from_icon_name ("process-stop-symbolic", GTK_ICON_SIZE_BUTTON));
 	gtk_widget_set_tooltip_text (button, _("Cancel the operation"));
 	gtk_widget_show_all (button);
-	gedit_message_area_add_action_widget (GEDIT_MESSAGE_AREA (task->priv->dialog),
-					      button,
-					      GTK_RESPONSE_CANCEL);
+	gtk_info_bar_add_action_widget (GTK_INFO_BAR (task->priv->dialog),
+					button,
+					GTK_RESPONSE_CANCEL);
 
-	dialog_data = g_new0 (EmbeddedDialogData, 1);
+	dialog_data = g_new0 (InfoBarData, 1);
 	dialog_data->browser = task->priv->browser;
 	dialog_data->task = task;
 	dialog_data->response_id = g_signal_connect (task->priv->dialog,
 						     "response",
-						     G_CALLBACK (embedded_dialog_response_cb),
+						     G_CALLBACK (info_bar_response_cb),
 						     dialog_data);
 
 	/**/
@@ -395,14 +396,14 @@ gth_search_task_exec (GthTask *base)
 	data = dom_document_dump (doc, &size);
 
 	search_result_real_file = gth_catalog_file_to_gio_file (task->priv->search_catalog);
-	g_write_file_async (search_result_real_file,
-			    data,
-			    size,
-			    TRUE,
-			    G_PRIORITY_DEFAULT,
-			    gth_task_get_cancellable (GTH_TASK (task)),
-			    clear_search_result_copy_done_cb,
-			    task);
+	_g_file_write_async (search_result_real_file,
+			     data,
+			     size,
+			     TRUE,
+			     G_PRIORITY_DEFAULT,
+			     gth_task_get_cancellable (GTH_TASK (task)),
+			     clear_search_result_copy_done_cb,
+			     task);
 
 	g_object_unref (search_result_real_file);
 	g_object_unref (doc);
