@@ -62,7 +62,6 @@ struct _GthSlideshowPrivate {
 	GthSlideshowDirection  direction;
 #if HAVE_CLUTTER
 	ClutterTimeline       *timeline;
-	ClutterAlpha          *alpha;
 	ClutterActor          *image1;
 	ClutterActor          *image2;
 	ClutterActor          *paused_actor;
@@ -100,6 +99,7 @@ _gth_slideshow_close (GthSlideshow *self)
 
 	browser = self->priv->browser;
 	close_browser = ! gtk_widget_get_visible (GTK_WIDGET (browser));
+	self->priv->projector->show_cursor (self);
 	self->priv->projector->finalize (self);
 	gtk_widget_destroy (GTK_WIDGET (self));
 
@@ -130,6 +130,52 @@ _gth_slideshow_reset_current (GthSlideshow *self)
 }
 
 
+static void _gth_slideshow_load_next_image (GthSlideshow *self);
+
+
+static void
+preloader_load_ready_cb (GObject	*source_object,
+			 GAsyncResult	*result,
+			 gpointer	 user_data)
+{
+	GthSlideshow *self = user_data;
+	GthFileData  *requested;
+	GthImage     *image;
+	int	      requested_size;
+	int	      original_width;
+	int	      original_height;
+	GError	     *error = NULL;
+
+	if (! gth_image_preloader_load_finish (GTH_IMAGE_PRELOADER (source_object),
+					       result,
+					       &requested,
+					       &image,
+					       &requested_size,
+					       &original_width,
+					       &original_height,
+					       &error))
+	{
+		g_clear_error (&error);
+		_gth_slideshow_load_next_image (self);
+		return;
+	}
+
+	_g_object_unref (self->priv->current_pixbuf);
+	self->priv->current_pixbuf = gth_image_get_pixbuf (image);
+
+	if (self->priv->current_pixbuf == NULL) {
+		_gth_slideshow_load_next_image (self);
+		return;
+	}
+
+	self->priv->one_loaded = TRUE;
+	self->priv->projector->image_ready (self, self->priv->current_pixbuf);
+
+	_g_object_unref (requested);
+	_g_object_unref (image);
+}
+
+
 static void
 _gth_slideshow_load_current_image (GthSlideshow *self)
 {
@@ -174,9 +220,12 @@ _gth_slideshow_load_current_image (GthSlideshow *self)
 	gth_image_preloader_load (self->priv->preloader,
 				  requested_file,
 				  MAX (screen_width, screen_height),
+				  NULL,
+				  preloader_load_ready_cb,
+				  self,
+				  2,
 				  next_file,
-				  prev_file,
-				  NULL);
+				  prev_file);
 }
 
 
@@ -241,36 +290,6 @@ view_next_image_automatically (GthSlideshow *self)
 }
 
 
-static void
-image_preloader_requested_ready_cb (GthImagePreloader  *preloader,
-				    GthFileData        *requested,
-				    GthImage           *image,
-				    int                 original_width,
-				    int                 original_height,
-				    GError             *error,
-				    gpointer            user_data)
-{
-	GthSlideshow *self = user_data;
-
-	if (error != NULL) {
-		g_clear_error (&error);
-		_gth_slideshow_load_next_image (self);
-		return;
-	}
-
-	_g_object_unref (self->priv->current_pixbuf);
-	self->priv->current_pixbuf = gth_image_get_pixbuf (image);
-
-	if (self->priv->current_pixbuf == NULL) {
-		_gth_slideshow_load_next_image (self);
-		return;
-	}
-
-	self->priv->one_loaded = TRUE;
-	self->priv->projector->image_ready (self, self->priv->current_pixbuf);
-}
-
-
 static void
 gth_slideshow_finalize (GObject *object)
 {
@@ -447,12 +466,7 @@ gth_slideshow_init (GthSlideshow *self)
 	self->priv->random_order = FALSE;
 	self->priv->current_pixbuf = NULL;
 	self->priv->screensaver = gth_screensaver_new (NULL);
-
-	self->priv->preloader = gth_image_preloader_new (GTH_LOAD_POLICY_ONE_STEP, 3);
-	g_signal_connect (self->priv->preloader,
-			  "requested_ready",
-			  G_CALLBACK (image_preloader_requested_ready_cb),
-			  self);
+	self->priv->preloader = gth_image_preloader_new ();
 }
 
 
@@ -473,7 +487,7 @@ _gth_slideshow_construct (GthSlideshow *self,
 							     NULL);
 	if (self->priv->pause_pixbuf == NULL)
 		self->priv->pause_pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-								     GTK_STOCK_MEDIA_PAUSE,
+								     "media-playback-pause-symbolic",
 								     100,
 								     0,
 								     NULL);
@@ -587,6 +601,13 @@ default_projector_finalize (GthSlideshow *self)
 }
 
 
+static void
+default_projector_show_cursor (GthSlideshow *self)
+{
+	gth_image_viewer_show_cursor (GTH_IMAGE_VIEWER (self->priv->viewer));
+}
+
+
 static void
 default_projector_hide_cursor (GthSlideshow *self)
 {
@@ -718,11 +739,9 @@ default_projector_construct (GthSlideshow *self)
 	self->priv->paint_paused = FALSE;
 
 	self->priv->viewer = gth_image_viewer_new ();
-	gth_image_viewer_set_black_background (GTH_IMAGE_VIEWER (self->priv->viewer), TRUE);
 	gth_image_viewer_hide_frame (GTH_IMAGE_VIEWER (self->priv->viewer));
 	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_FIT_SIZE);
 	gth_image_viewer_set_zoom_change (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_CHANGE_FIT_SIZE);
-	gth_image_viewer_set_transp_type (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_TRANSP_TYPE_BLACK);
 	gth_image_viewer_set_zoom_quality (GTH_IMAGE_VIEWER (self->priv->viewer), GTH_ZOOM_QUALITY_LOW);
 	gth_image_viewer_add_painter (GTH_IMAGE_VIEWER (self->priv->viewer), default_projector_pause_painter, self);
 
@@ -739,6 +758,7 @@ default_projector_construct (GthSlideshow *self)
 GthProjector default_projector = {
 	default_projector_construct,
 	default_projector_paused,
+	default_projector_show_cursor,
 	default_projector_hide_cursor,
 	default_projector_finalize,
 	default_projector_image_ready,
@@ -781,26 +801,11 @@ reset_texture_transformation (GthSlideshow *self,
 
 	clutter_actor_get_size (self->stage, &stage_w, &stage_h);
 
-	clutter_actor_set_anchor_point (texture, 0.0, 0.0);
+	clutter_actor_set_pivot_point (texture, stage_w / 2.0, stage_h / 2.0);
 	clutter_actor_set_opacity (texture, 255);
-	clutter_actor_set_rotation (texture,
-				    CLUTTER_X_AXIS,
-				    0.0,
-				    stage_w / 2.0,
-				    stage_h / 2.0,
-				    0.0);
-	clutter_actor_set_rotation (texture,
-				    CLUTTER_Y_AXIS,
-				    0.0,
-				    stage_w / 2.0,
-				    stage_h / 2.0,
-				    0.0);
-	clutter_actor_set_rotation (texture,
-				    CLUTTER_Z_AXIS,
-				    0.0,
-				    stage_w / 2.0,
-				    stage_h / 2.0,
-				    0.0);
+	clutter_actor_set_rotation_angle (texture, CLUTTER_X_AXIS, 0.0);
+	clutter_actor_set_rotation_angle (texture, CLUTTER_Y_AXIS, 0.0);
+	clutter_actor_set_rotation_angle (texture, CLUTTER_Z_AXIS, 0.0);
 	clutter_actor_set_scale (texture, 1.0, 1.0);
 }
 
@@ -819,7 +824,7 @@ _gth_slideshow_reset_textures_position (GthSlideshow *self)
 	}
 
 	if ((self->current_image != NULL) && (self->next_image != NULL)) {
-		clutter_actor_raise (self->current_image, self->next_image);
+		clutter_actor_set_child_above_sibling (CLUTTER_ACTOR (self->stage), self->current_image, self->next_image);
 		clutter_actor_hide (self->next_image);
 	}
 
@@ -880,7 +885,7 @@ animation_frame_cb (ClutterTimeline *timeline,
 	if (self->priv->transition != NULL)
 		gth_transition_frame (self->priv->transition,
 				      self,
-				      clutter_alpha_get_alpha (self->priv->alpha));
+				      clutter_timeline_get_progress (self->priv->timeline));
 
 	if (self->first_frame)
 		self->first_frame = FALSE;
@@ -978,7 +983,13 @@ static void
 clutter_projector_finalize (GthSlideshow *self)
 {
 	_g_object_unref (self->priv->timeline);
-	_g_object_unref (self->priv->alpha);
+}
+
+
+static void
+clutter_projector_show_cursor (GthSlideshow *self)
+{
+	clutter_stage_show_cursor (CLUTTER_STAGE (self->stage));
 }
 
 
@@ -1002,18 +1013,18 @@ clutter_projector_paused (GthSlideshow *self)
 
 	clutter_actor_get_size (self->stage, &stage_w, &stage_h);
 	clutter_actor_set_position (self->priv->paused_actor, stage_w / 2.0, stage_h / 2.0);
-	clutter_actor_set_anchor_point_from_gravity (self->priv->paused_actor, CLUTTER_GRAVITY_CENTER);
+	clutter_actor_set_pivot_point (self->priv->paused_actor, 0.5, 0.5);
 	clutter_actor_set_scale (self->priv->paused_actor, 1.0, 1.0);
 	clutter_actor_set_opacity (self->priv->paused_actor, 255);
-	clutter_actor_raise_top (self->priv->paused_actor);
+	clutter_actor_set_child_above_sibling (CLUTTER_ACTOR (self->stage), self->priv->paused_actor, NULL);
 	clutter_actor_show (self->priv->paused_actor);
 
-	clutter_actor_animate (self->priv->paused_actor,
-			       CLUTTER_LINEAR, 500,
-	                       "opacity", 0,
-	                       "scale-x", 3.0,
-	                       "scale-y", 3.0,
-	                       NULL);
+	clutter_actor_save_easing_state (self->priv->paused_actor);
+	clutter_actor_set_easing_mode (self->priv->paused_actor, CLUTTER_LINEAR);
+	clutter_actor_set_easing_duration (self->priv->paused_actor, 500);
+	clutter_actor_set_scale (self->priv->paused_actor, 3.0, 3.0);
+	clutter_actor_set_opacity (self->priv->paused_actor, 0);
+	clutter_actor_restore_easing_state (self->priv->paused_actor);
 }
 
 
@@ -1166,7 +1177,7 @@ clutter_projector_construct (GthSlideshow *self)
 	embed = gtk_clutter_embed_new ();
 	self->stage = gtk_clutter_embed_get_stage (GTK_CLUTTER_EMBED (embed));
 	clutter_stage_hide_cursor (CLUTTER_STAGE (self->stage));
-	clutter_stage_set_color (CLUTTER_STAGE (self->stage), &background_color);
+	clutter_actor_set_background_color (CLUTTER_ACTOR (self->stage), &background_color);
 
 	self->priv->last_button_event_time = 0;
 	g_signal_connect (self->stage, "button-press-event", G_CALLBACK (stage_input_cb), self);
@@ -1176,35 +1187,34 @@ clutter_projector_construct (GthSlideshow *self)
 
 	self->priv->image1 = gtk_clutter_texture_new ();
 	clutter_actor_hide (self->priv->image1);
-	clutter_container_add_actor (CLUTTER_CONTAINER (self->stage), self->priv->image1);
+	clutter_actor_add_child (CLUTTER_ACTOR (self->stage), self->priv->image1);
 
 	self->priv->image2 = gtk_clutter_texture_new ();
 	clutter_actor_hide (self->priv->image2);
-	clutter_container_add_actor (CLUTTER_CONTAINER (self->stage), self->priv->image2);
+	clutter_actor_add_child (CLUTTER_ACTOR (self->stage), self->priv->image2);
 
 	self->current_image = NULL;
 	self->next_image = self->priv->image1;
 
 	self->priv->timeline = clutter_timeline_new (GTH_TRANSITION_DURATION);
+	clutter_timeline_set_progress_mode (self->priv->timeline, CLUTTER_EASE_IN_OUT_SINE);
 	g_signal_connect (self->priv->timeline, "completed", G_CALLBACK (animation_completed_cb), self);
 	g_signal_connect (self->priv->timeline, "new-frame", G_CALLBACK (animation_frame_cb), self);
 	g_signal_connect (self->priv->timeline, "started", G_CALLBACK (animation_started_cb), self);
 
-	self->priv->alpha = clutter_alpha_new_full (self->priv->timeline, CLUTTER_EASE_IN_OUT_SINE);
-
 	self->priv->paused_actor = gtk_clutter_texture_new ();
 	if (self->priv->pause_pixbuf != NULL)
 		gtk_clutter_texture_set_from_pixbuf (GTK_CLUTTER_TEXTURE (self->priv->paused_actor),
 				 	 	     self->priv->pause_pixbuf,
 				 	 	     NULL);
 	else
-		gtk_clutter_texture_set_from_stock (GTK_CLUTTER_TEXTURE (self->priv->paused_actor),
-						    GTK_WIDGET (self),
-						    GTK_STOCK_MEDIA_PAUSE,
-						    GTK_ICON_SIZE_DIALOG,
-						    NULL);
+		gtk_clutter_texture_set_from_icon_name (GTK_CLUTTER_TEXTURE (self->priv->paused_actor),
+						    	GTK_WIDGET (self),
+						    	"media-playback-pause-symbolic",
+						    	GTK_ICON_SIZE_DIALOG,
+						    	NULL);
 	clutter_actor_hide (self->priv->paused_actor);
-	clutter_container_add_actor (CLUTTER_CONTAINER (self->stage), self->priv->paused_actor);
+	clutter_actor_add_child (CLUTTER_ACTOR (self->stage), self->priv->paused_actor);
 
 	g_signal_connect (self, "size-allocate", G_CALLBACK (gth_slideshow_size_allocate_cb), self);
 
@@ -1216,6 +1226,7 @@ clutter_projector_construct (GthSlideshow *self)
 GthProjector clutter_projector = {
 	clutter_projector_construct,
 	clutter_projector_paused,
+	clutter_projector_show_cursor,
 	clutter_projector_hide_cursor,
 	clutter_projector_finalize,
 	clutter_projector_image_ready,
