@@ -742,7 +742,7 @@ get_image_attribute (GthWebExporter    *self,
 
 		truncated = g_strndup (value, max_length);
 		if (strlen (truncated) < strlen (value))
-			line = g_strconcat (truncated, "...", NULL);
+			line = g_strconcat (truncated, "…", NULL);
 		else
 			line = g_strdup (truncated);
 
@@ -1832,10 +1832,10 @@ delete_temp_dir_ready_cb (GError   *error,
 
 	dialog = _gtk_message_dialog_new (GTK_WINDOW (self->priv->browser),
 					  GTK_DIALOG_MODAL,
-					  GTK_MESSAGE_INFO,
+					  _GTK_ICON_NAME_DIALOG_INFO,
 					  _("The album has been created successfully."),
 					  NULL,
-					  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+					  _GTK_LABEL_CLOSE, GTK_RESPONSE_CLOSE,
 					  _("_Open in the Browser"), _OPEN_IN_BROWSER_RESPONSE,
 					  _("_View the destination"), _OPEN_FOLDER_RESPONSE,
 					  NULL);
@@ -2112,6 +2112,7 @@ save_thumbnail (gpointer data)
 					"image/jpeg",
 					file_data,
 					TRUE,
+					gth_task_get_cancellable (GTH_TASK (self)),
 					save_thumbnail_ready_cb,
 					self);
 
@@ -2345,6 +2346,7 @@ save_image_preview (gpointer data)
 					"image/jpeg",
 					file_data,
 					TRUE,
+					gth_task_get_cancellable (GTH_TASK (self)),
 					save_image_preview_ready_cb,
 					self);
 
@@ -2452,6 +2454,7 @@ save_resized_image (gpointer data)
 					"image/jpeg",
 					file_data,
 					TRUE,
+					gth_task_get_cancellable (GTH_TASK (self)),
 					save_resized_image_ready_cd,
 					self);
 
@@ -2566,6 +2569,7 @@ image_loader_ready_cb (GObject      *source_object,
 					    &image,
 					    NULL,
 					    NULL,
+					    NULL,
 					    NULL))
 	{
 		load_next_file (self);
