@@ -26,9 +26,28 @@
 #include "cairo-utils.h"
 #include "cairo-scale.h"
 #include "gfixed.h"
+#include "glib-utils.h"
 
 
-#if 1
+#define CLAMP_PIXEL(v) (((v) <= 0) ? 0  : ((v) >= 255) ? 255 : (v));
+#define EPSILON ((ScaleReal) 1.0e-16)
+
+
+typedef double ScaleReal;
+typedef ScaleReal (*weight_func_t) (ScaleReal distance);
+
+
+#ifdef HAVE_VECTOR_OPERATIONS
+
+
+typedef ScaleReal v4r __attribute__ ((vector_size(sizeof(ScaleReal)*4)));
+typedef union {
+	v4r       v;
+	ScaleReal r[4];
+} r4vector;
+
+
+#endif /* HAVE_VECTOR_OPERATIONS */
 
 
 /* -- _cairo_image_surface_scale_nearest -- */
@@ -39,6 +58,7 @@ _cairo_image_surface_scale_nearest (cairo_surface_t *image,
 				    int              new_width,
 				    int              new_height)
 {
+#if 0
 	cairo_surface_t *scaled;
 	int              src_width;
 	int              src_height;
@@ -63,8 +83,8 @@ _cairo_image_surface_scale_nearest (cairo_surface_t *image,
 
 	src_width = cairo_image_surface_get_width  (image);
 	src_height = cairo_image_surface_get_height (image);
-	p_src = cairo_image_surface_get_data (image);
-	p_dest = cairo_image_surface_get_data (scaled);
+	p_src = _cairo_image_surface_flush_and_get_data (image);
+	p_dest = _cairo_image_surface_flush_and_get_data (scaled);
 	src_rowstride = cairo_image_surface_get_stride (image);
 	dest_rowstride = cairo_image_surface_get_stride (scaled);
 
@@ -76,7 +96,7 @@ _cairo_image_surface_scale_nearest (cairo_surface_t *image,
 	p_dest_row = p_dest;
 	p_src_row = p_src;
 	max_row = GINT_TO_FIXED (src_height - 1);
-	max_col= GINT_TO_FIXED (src_width - 1);
+	max_col = GINT_TO_FIXED (src_width - 1);
 	/* pick the pixel in the middle to avoid the shift effect. */
 	y_src = gfixed_div (step_y, GFIXED_2);
 	for (y = 0; y < new_height; y++) {
@@ -100,10 +120,30 @@ _cairo_image_surface_scale_nearest (cairo_surface_t *image,
 	cairo_surface_mark_dirty (scaled);
 
 	return scaled;
-}
+#else
 
+	cairo_surface_t *output;
+	cairo_t         *cr;
+
+	output = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					     new_width,
+					     new_height);
+	cr = cairo_create (output);
+	cairo_scale (cr,
+		     (double) new_width / cairo_image_surface_get_width (image),
+		     (double) new_height / cairo_image_surface_get_height (image));
+	cairo_set_source_surface (cr, image, 0.0, 0.0);
+	cairo_pattern_set_filter (cairo_get_source (cr), CAIRO_FILTER_NEAREST);
+	cairo_rectangle (cr, 0.0, 0.0, cairo_image_surface_get_width (image), cairo_image_surface_get_height (image));
+	cairo_fill (cr);
+	cairo_surface_flush (output);
+
+	cairo_destroy (cr);
+
+	return output;
 
 #endif
+}
 
 
 /* -- _cairo_image_surface_scale_filter --
@@ -113,32 +153,57 @@ _cairo_image_surface_scale_nearest (cairo_surface_t *image,
  * */
 
 
-#define EPSILON ((double) 1.0e-8)
-
-
-typedef double (*weight_func_t) (double distance);
-
-
-static double
-box (double x)
+inline static ScaleReal
+box (ScaleReal x)
 {
 	return 1.0;
 }
 
 
-static double
-triangle (double x)
+inline static ScaleReal
+triangle (ScaleReal x)
 {
 	return (x < 1.0) ? 1.0 - x : 0.0;
 }
 
 
-static double
-sinc_fast (double x)
+static ScaleReal Cubic_P0, Cubic_P2, Cubic_P3;
+static ScaleReal Cubic_Q0, Cubic_Q1, Cubic_Q2, Cubic_Q3;
+static gsize coefficients_initialization = 0;
+
+
+static void
+initialize_coefficients (ScaleReal B,
+			 ScaleReal C)
+{
+	Cubic_P0 = (6.0 - 2.0 * B) / 6.0;
+	/*Cubic_P1 = 0.0;*/
+	Cubic_P2 = (-18.0 + 12.0 * B + 6.0 * C) / 6.0;
+	Cubic_P3 = ( 12.0 - 9.0 * B - 6.0 * C) / 6.0;
+	Cubic_Q0 = (8.0 * B + 24.0 * C) / 6.0;
+	Cubic_Q1 = (-12.0 * B - 48.0 * C) / 6.0;
+	Cubic_Q2 = (6.0 * B + 30.0 * C) / 6.0;
+	Cubic_Q3 = (-1.0 * B - 6.0 * C) / 6.0;
+}
+
+
+inline static ScaleReal
+cubic (ScaleReal x)
+{
+	if (x < 1.0)
+		return Cubic_P0 + x * (x * (Cubic_P2 + x * Cubic_P3));
+	if (x < 2.0)
+		return Cubic_Q0 + x * (Cubic_Q1 + x * (Cubic_Q2 + x * Cubic_Q3));
+	return 0.0;
+}
+
+
+inline static ScaleReal
+sinc_fast (ScaleReal x)
 {
 	if (x > 4.0) {
-		const double alpha = G_PI * x;
-		return sin (alpha) / alpha;
+		const ScaleReal alpha = G_PI * x;
+		return sin ((double) alpha) / alpha;
 	}
 
 	{
@@ -146,35 +211,63 @@ sinc_fast (double x)
 		 * The approximations only depend on x^2 (sinc is an even function).
 		 */
 
-		const double xx = x*x;
+		const ScaleReal xx = x*x;
 
 		/*
-		 * Maximum absolute relative error 6.3e-6 < 1/2^17.
+		 * Maximum absolute relative error 1.2e-12 < 1/2^39.
 		 */
-		const double c0 = 0.173610016489197553621906385078711564924e-2L;
-		const double c1 = -0.384186115075660162081071290162149315834e-3L;
-		const double c2 = 0.393684603287860108352720146121813443561e-4L;
-		const double c3 = -0.248947210682259168029030370205389323899e-5L;
-		const double c4 = 0.107791837839662283066379987646635416692e-6L;
-		const double c5 = -0.324874073895735800961260474028013982211e-8L;
-		const double c6 = 0.628155216606695311524920882748052490116e-10L;
-		const double c7 = -0.586110644039348333520104379959307242711e-12L;
-		const double p = c0+xx*(c1+xx*(c2+xx*(c3+xx*(c4+xx*(c5+xx*(c6+xx*c7))))));
 
-		return (xx-1.0)*(xx-4.0)*(xx-9.0)*(xx-16.0)*p;
+		const ScaleReal c0 = 0.173611111110910715186413700076827593074e-2L;
+		const ScaleReal c1 = -0.289105544717893415815859968653611245425e-3L;
+		const ScaleReal c2 = 0.206952161241815727624413291940849294025e-4L;
+		const ScaleReal c3 = -0.834446180169727178193268528095341741698e-6L;
+		const ScaleReal c4 = 0.207010104171026718629622453275917944941e-7L;
+		const ScaleReal c5 = -0.319724784938507108101517564300855542655e-9L;
+		const ScaleReal c6 = 0.288101675249103266147006509214934493930e-11L;
+		const ScaleReal c7 = -0.118218971804934245819960233886876537953e-13L;
+		const ScaleReal p = c0+xx*(c1+xx*(c2+xx*(c3+xx*(c4+xx*(c5+xx*(c6+xx*c7))))));
+		const ScaleReal d0 = 1.0L;
+		const ScaleReal d1 = 0.547981619622284827495856984100563583948e-1L;
+		const ScaleReal d2 = 0.134226268835357312626304688047086921806e-2L;
+		const ScaleReal d3 = 0.178994697503371051002463656833597608689e-4L;
+		const ScaleReal d4 = 0.114633394140438168641246022557689759090e-6L;
+		const ScaleReal q = d0+xx*(d1+xx*(d2+xx*(d3+xx*d4)));
+
+		return ((xx-1.0)*(xx-4.0)*(xx-9.0)*(xx-16.0)/q*p);
 	}
 }
 
 
+inline static ScaleReal
+mitchell_netravali (ScaleReal x)
+{
+	ScaleReal xx;
+
+	if (x >= 2.0)
+		return 0.0;
+
+	xx = x * x;
+	if ((x >= 0.0) && (x < 1.0))
+		x  = (21 * xx * x) - (36 * xx) + 16;
+	else /* (x >= 1.0) && (x < 2.0) */
+		x  = (-7 * xx * x) + (36 * xx) - (60 * x) + 32;
+
+	return x / 18.0;
+}
+
+
 static struct {
 	weight_func_t weight_func;
-	double        support;
+	ScaleReal     support;
 }
 const filters[N_SCALE_FILTERS] = {
-	{ box,		 .0 },
-	{ box,		 .5 },
-	{ triangle,	1.0 },
-	{ sinc_fast,	3.0 }
+	{ box,			.0 },
+	{ box,			.5 },
+	{ triangle,		1.0 },
+	{ cubic,		2.0 },
+	{ sinc_fast,		2.0 },
+	{ sinc_fast,		3.0 },
+	{ mitchell_netravali,	2.0 }
 };
 
 
@@ -183,7 +276,7 @@ const filters[N_SCALE_FILTERS] = {
 
 typedef struct {
 	weight_func_t  weight_func;
-	double         support;
+	ScaleReal      support;
 	GthAsyncTask  *task;
 	gulong         total_lines;
 	gulong         processed_lines;
@@ -191,221 +284,216 @@ typedef struct {
 } resize_filter_t;
 
 
+static void
+resize_filter_set_type (resize_filter_t *resize_filter,
+			scale_filter_t   filter_type)
+{
+	resize_filter->weight_func = filters[filter_type].weight_func;
+	resize_filter->support = filters[filter_type].support;
+}
+
+
 static resize_filter_t *
-resize_filter_create (scale_filter_t  filter_type,
-		      GthAsyncTask   *task)
+resize_filter_create (GthAsyncTask *task)
 {
 	resize_filter_t *resize_filter;
 
-	resize_filter = g_slice_new (resize_filter_t);
-	resize_filter->weight_func = filters[filter_type].weight_func;
-	resize_filter->support = filters[filter_type].support;
+	resize_filter = g_new (resize_filter_t, 1);
 	resize_filter->task = task;
 	resize_filter->total_lines = 0;
 	resize_filter->processed_lines = 0;
 	resize_filter->cancelled = FALSE;
+	resize_filter_set_type (resize_filter, SCALE_FILTER_TRIANGLE);
 
 	return resize_filter;
 }
 
 
-static double inline
+inline static ScaleReal
 resize_filter_get_support (resize_filter_t *resize_filter)
 {
 	return resize_filter->support;
 }
 
 
-static double inline
+inline static ScaleReal
 resize_filter_get_weight (resize_filter_t *resize_filter,
-			  double           distance)
+			  ScaleReal        distance)
 {
-	double scale = 1.0;
+	ScaleReal window = 1.0;
 
-	if (resize_filter->weight_func == sinc_fast)
-		scale = resize_filter->weight_func (fabs (distance));
+	if (resize_filter->weight_func == sinc_fast) {
+		ScaleReal x = fabs (distance);
 
-	return scale * resize_filter->weight_func (fabs (distance));
+		if (x == 0)
+			window = 1.0;
+		else if ((x > 0) && (x < resize_filter->support))
+			window = resize_filter->weight_func (x / resize_filter->support);
+		else
+			window = 0.0;
+	}
+
+	return window * resize_filter->weight_func (fabs (distance));
 }
 
 
 static void
 resize_filter_destroy (resize_filter_t *resize_filter)
 {
-	g_slice_free (resize_filter_t, resize_filter);
+	g_free (resize_filter);
 }
 
 
-static inline double
-reciprocal (double x)
+static inline ScaleReal
+reciprocal (ScaleReal x)
 {
-	double sign = x < 0.0 ? -1.0 : 1.0;
+	ScaleReal sign = x < 0.0 ? -1.0 : 1.0;
 	return (sign * x) >= EPSILON ? 1.0 / x : sign * (1.0 / EPSILON);
 }
 
 
-#define CLAMP_PIXEL(v) (((v) <= 0) ? 0  : ((v) >= 255) ? 255 : (v));
-
-
 static void
 horizontal_scale_transpose (cairo_surface_t *image,
 			    cairo_surface_t *scaled,
-			    double           scale_factor,
+			    ScaleReal        scale_factor,
 			    resize_filter_t *resize_filter)
 {
-	double    scale;
-	double    support;
-	int       y;
-	guchar   *p_src;
-        guchar   *p_dest;
-        int       src_rowstride;
-        int       dest_rowstride;
-        double   *weights;
-        gfixed   *fixed_weights;
+	ScaleReal  scale;
+	ScaleReal  support;
+	int        y;
+	int        image_width;
+	int        scaled_width;
+	int        scaled_height;
+	guchar    *p_src;
+        guchar    *p_dest;
+        int        src_rowstride;
+        int        dest_rowstride;
+        ScaleReal *weights;
 
 	if (resize_filter->cancelled)
 		return;
 
-        cairo_surface_flush (scaled);
-
-	scale = MAX (1.0 / scale_factor + EPSILON, 1.0);
+	scale = MAX ((ScaleReal) 1.0 / scale_factor + EPSILON, 1.0);
 	support = scale * resize_filter_get_support (resize_filter);
 	if (support < 0.5) {
 		support = 0.5;
 		scale = 1.0;
 	}
 
-	p_src = cairo_image_surface_get_data (image);
-	p_dest = cairo_image_surface_get_data (scaled);
+	image_width = cairo_image_surface_get_width (image);
+	scaled_width = cairo_image_surface_get_width (scaled);
+	scaled_height = cairo_image_surface_get_height (scaled);
+	p_src = _cairo_image_surface_flush_and_get_data (image);
+	p_dest = _cairo_image_surface_flush_and_get_data (scaled);
 	src_rowstride = cairo_image_surface_get_stride (image);
 	dest_rowstride = cairo_image_surface_get_stride (scaled);
-	weights = g_new (double, 2.0 * support + 3.0);
-	fixed_weights = g_new (gfixed, 2.0 * support + 3.0);
+	weights = g_new (ScaleReal, 2.0 * support + 3.0);
 
 	scale = reciprocal (scale);
-	for (y = 0; ! resize_filter->cancelled && (y < cairo_image_surface_get_height (scaled)); y++) {
-	        guchar *p_src_row;
-	        guchar *p_dest_pixel;
-		double  bisect;
-		int     start;
-		int     stop;
-		double  density;
-		int     n;
-		int     x;
-		int     i;
+	for (y = 0; y < scaled_height; y++) {
+	        guchar    *p_src_row;
+	        guchar    *p_dest_pixel;
+		ScaleReal  bisect;
+		int        start;
+		int        stop;
+		ScaleReal  density;
+		int        n;
+		int        x;
+		int        i;
+#ifdef HAVE_VECTOR_OPERATIONS
+		r4vector   v_pixel, v_rgba;
+#endif /* HAVE_VECTOR_OPERATIONS */
 
 		if (resize_filter->task != NULL) {
-			double progress = (double) resize_filter->processed_lines++ / resize_filter->total_lines;
+			double progress;
+
+			gth_async_task_get_data (resize_filter->task, NULL, &resize_filter->cancelled, NULL);
+			if (resize_filter->cancelled)
+				goto out;
+
+			progress = (double) resize_filter->processed_lines++ / resize_filter->total_lines;
 			gth_async_task_set_data (resize_filter->task, NULL, NULL, &progress);
 		}
 
-		bisect = (y + 0.5) / scale_factor + EPSILON;
-		start = MAX (bisect - support + 0.5, 0.0);
-		stop = MIN (bisect + support + 0.5, cairo_image_surface_get_width (image));
+		bisect = ((ScaleReal) y + 0.5) / scale_factor + EPSILON;
+		start = MAX (bisect - support + 0.5, 0);
+		stop = MIN (bisect + support + 0.5, (ScaleReal) image_width);
 
 		density = 0.0;
 		for (n = 0; n < stop - start; n++) {
-			weights[n] = resize_filter_get_weight (resize_filter, scale * ((double) (start + n) - bisect + 0.5));
+			weights[n] = resize_filter_get_weight (resize_filter, scale * ((ScaleReal) (start + n) - bisect + 0.5));
 			density += weights[n];
 		}
 
-		density = reciprocal (density);
-		for (i = 0; i < n; i++) {
-			double w = weights[i] * density;
-			fixed_weights[i] = GDOUBLE_TO_FIXED (w);
+		/*
+		g_assert (n == stop - start);
+		g_assert (stop - start <= (2.0 * support) + 3);
+		*/
+
+		if ((density != 0.0) && (density != 1.0)) {
+			density = reciprocal (density);
+			for (i = 0; i < n; i++)
+				weights[i] *= density;
 		}
 
-		p_src_row = p_src;
-		p_dest_pixel = p_dest + (y * dest_rowstride);
-		for (x = 0; x < cairo_image_surface_get_width (scaled); x++) {
+		p_src_row = p_src + (start * 4);
+		p_dest_pixel = p_dest;
+		for (x = 0; x < scaled_width; x++) {
 			guchar *p_src_pixel;
-			gfixed  r, g, b, a;
-			gfixed  w;
 
-			if (resize_filter->task != NULL) {
-				gth_async_task_get_data (resize_filter->task, NULL, &resize_filter->cancelled, NULL);
-				if (resize_filter->cancelled)
-					break;
-			}
+			p_src_pixel = p_src_row;
 
-			p_src_pixel = p_src_row + (start * 4);
-			r = g = b = a = GFIXED_0;
+#ifdef HAVE_VECTOR_OPERATIONS
 
+			v_rgba.v = (v4r) { 0.0, 0.0, 0.0, 0.0 };
 			for (i = 0; i < n; i++) {
-				w = fixed_weights[i];
-				r += gfixed_mul (w, GINT_TO_FIXED (p_src_pixel[CAIRO_RED]));
-				g += gfixed_mul (w, GINT_TO_FIXED (p_src_pixel[CAIRO_GREEN]));
-				b += gfixed_mul (w, GINT_TO_FIXED (p_src_pixel[CAIRO_BLUE]));
-				a += gfixed_mul (w, GINT_TO_FIXED (p_src_pixel[CAIRO_ALPHA]));
+				v_pixel.v = (v4r) { p_src_pixel[0], p_src_pixel[1], p_src_pixel[2], p_src_pixel[3] };
+				v_rgba.v = v_rgba.v + (v_pixel.v * weights[i]);
+
+				p_src_pixel += 4;
+			}
+			v_rgba.v = v_rgba.v + 0.5;
+
+			p_dest_pixel[0] = CLAMP_PIXEL (v_rgba.r[0]);
+			p_dest_pixel[1] = CLAMP_PIXEL (v_rgba.r[1]);
+			p_dest_pixel[2] = CLAMP_PIXEL (v_rgba.r[2]);
+			p_dest_pixel[3] = CLAMP_PIXEL (v_rgba.r[3]);
+
+#else /* ! HAVE_VECTOR_OPERATIONS */
+
+			ScaleReal r, g, b, a, w;
+
+			r = g = b = a = 0.0;
+			for (i = 0; i < n; i++) {
+				w = weights[i];
+
+				r += w * p_src_pixel[CAIRO_RED];
+				g += w * p_src_pixel[CAIRO_GREEN];
+				b += w * p_src_pixel[CAIRO_BLUE];
+				a += w * p_src_pixel[CAIRO_ALPHA];
 
 				p_src_pixel += 4;
 			}
 
-			r = GFIXED_TO_INT (r);
-			g = GFIXED_TO_INT (g);
-			b = GFIXED_TO_INT (b);
-			a = GFIXED_TO_INT (a);
+			p_dest_pixel[CAIRO_RED] = CLAMP_PIXEL (r + 0.5);
+			p_dest_pixel[CAIRO_GREEN] = CLAMP_PIXEL (g + 0.5);
+			p_dest_pixel[CAIRO_BLUE] = CLAMP_PIXEL (b + 0.5);
+			p_dest_pixel[CAIRO_ALPHA] = CLAMP_PIXEL (a + 0.5);
 
-			p_dest_pixel[CAIRO_RED] = CLAMP_PIXEL (r);
-			p_dest_pixel[CAIRO_GREEN] = CLAMP_PIXEL (g);
-			p_dest_pixel[CAIRO_BLUE] = CLAMP_PIXEL (b);
-			p_dest_pixel[CAIRO_ALPHA] = CLAMP_PIXEL (a);
+#endif /* HAVE_VECTOR_OPERATIONS */
 
 			p_dest_pixel += 4;
 			p_src_row += src_rowstride;
 		}
+
+		p_dest += dest_rowstride;
 	}
 
+	out:
+
 	cairo_surface_mark_dirty (scaled);
 
 	g_free (weights);
-	g_free (fixed_weights);
-}
-
-
-static cairo_surface_t *
-_cairo_image_surface_scale_filter (cairo_surface_t *image,
-				   int              new_width,
-				   int              new_height,
-				   scale_filter_t   filter,
-				   GthAsyncTask    *task)
-{
-	int              src_width;
-	int              src_height;
-	cairo_surface_t *scaled;
-	resize_filter_t *resize_filter;
-	double           x_factor;
-	double           y_factor;
-	cairo_surface_t *tmp;
-
-	src_width = cairo_image_surface_get_width  (image);
-	src_height = cairo_image_surface_get_height (image);
-
-	if ((src_width == new_width) && (src_height == new_height))
-		return _cairo_image_surface_copy (image);
-
-	scaled = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-					     new_width,
-					     new_height);
-	if (scaled == NULL)
-		return NULL;
-
-	resize_filter = resize_filter_create (filter, task);
-	resize_filter->total_lines = new_width + new_height;
-	resize_filter->processed_lines = 0;
-
-	x_factor = (double) new_width / src_width;
-	y_factor = (double) new_height / src_height;
-	tmp = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-					  src_height,
-					  new_width);
-	horizontal_scale_transpose (image, tmp, x_factor, resize_filter);
-	horizontal_scale_transpose (tmp, scaled, y_factor, resize_filter);
-
-	resize_filter_destroy (resize_filter);
-	cairo_surface_destroy (tmp);
-
-	return scaled;
 }
 
 
@@ -416,11 +504,57 @@ _cairo_image_surface_scale (cairo_surface_t  *image,
 			    scale_filter_t    filter,
 			    GthAsyncTask     *task)
 {
-	return _cairo_image_surface_scale_filter (image,
-						  scaled_width,
-						  scaled_height,
-						  filter,
-						  task);
+	int                       src_width;
+	int                       src_height;
+	cairo_surface_t          *scaled;
+	cairo_surface_metadata_t *metadata;
+	resize_filter_t          *resize_filter;
+	ScaleReal                 x_factor;
+	ScaleReal                 y_factor;
+	cairo_surface_t          *tmp;
+
+	src_width = cairo_image_surface_get_width (image);
+	src_height = cairo_image_surface_get_height (image);
+
+	if ((src_width == scaled_width) && (src_height == scaled_height))
+		return _cairo_image_surface_copy (image);
+
+	scaled = _cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					      scaled_width,
+					      scaled_height);
+	_cairo_image_surface_copy_metadata (image, scaled);
+	metadata = _cairo_image_surface_get_metadata (scaled);
+	if (metadata->original_width <= 0) {
+		metadata->original_width = src_width;
+		metadata->original_height = src_height;
+	}
+
+	if (scaled == NULL)
+		return NULL;
+
+	if (g_once_init_enter (&coefficients_initialization)) {
+		initialize_coefficients (1.0, 0.0);
+		g_once_init_leave (&coefficients_initialization, 1);
+	}
+
+	resize_filter = resize_filter_create (task);
+	resize_filter_set_type (resize_filter, filter);
+	resize_filter->total_lines = scaled_width + scaled_height;
+	resize_filter->processed_lines = 0;
+
+	x_factor = (ScaleReal) scaled_width / src_width;
+	y_factor = (ScaleReal) scaled_height / src_height;
+	tmp = _cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
+					   src_height,
+					   scaled_width);
+
+	horizontal_scale_transpose (image, tmp, x_factor, resize_filter);
+	horizontal_scale_transpose (tmp, scaled, y_factor, resize_filter);
+
+	resize_filter_destroy (resize_filter);
+	cairo_surface_destroy (tmp);
+
+	return scaled;
 }
 
 
@@ -496,12 +630,12 @@ _cairo_image_surface_scale_bilinear_2x2 (cairo_surface_t *image,
 	guchar          *p_dest;
 	int              src_rowstride;
 	int              dest_rowstride;
-	double           step_x, step_y;
+	ScaleReal           step_x, step_y;
 	guchar          *p_dest_row;
 	guchar          *p_src_row;
 	guchar          *p_src_col;
 	guchar          *p_dest_col;
-	double           x_src, y_src;
+	ScaleReal           x_src, y_src;
 	int              x, y;
 	guchar           r00, r01, r10, r11;
 	guchar           g00, g01, g10, g11;
@@ -509,8 +643,8 @@ _cairo_image_surface_scale_bilinear_2x2 (cairo_surface_t *image,
 	guchar           a00, a01, a10, a11;
 	guchar           r, g, b, a;
 	guint32          pixel;
-	double           tmp;
-	double           x_fract, y_fract;
+	ScaleReal           tmp;
+	ScaleReal           x_fract, y_fract;
 	int              col, row;
 
 	scaled = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
@@ -519,15 +653,15 @@ _cairo_image_surface_scale_bilinear_2x2 (cairo_surface_t *image,
 
 	src_width = cairo_image_surface_get_width  (image);
 	src_height = cairo_image_surface_get_height (image);
-	p_src = cairo_image_surface_get_data (image);
-	p_dest = cairo_image_surface_get_data (scaled);
+	p_src = _cairo_image_surface_flush_and_get_data (image);
+	p_dest = _cairo_image_surface_flush_and_get_data (scaled);
 	src_rowstride = cairo_image_surface_get_stride (image);
 	dest_rowstride = cairo_image_surface_get_stride (scaled);
 
 	cairo_surface_flush (scaled);
 
-	step_x = (double) src_width / new_width;
-	step_y = (double) src_height / new_height;
+	step_x = (ScaleReal) src_width / new_width;
+	step_y = (ScaleReal) src_height / new_height;
 	p_dest_row = p_dest;
 	p_src_row = p_src;
 	y_src = 0;
@@ -617,7 +751,7 @@ _cairo_surface_reduce_row (guchar *dest_data,
 			   int     src_width)
 {
 	int     x, b;
-	double  sum;
+	ScaleReal  sum;
 	int     col0, col1, col2;
 	guchar  c[4];
 	guint32 pixel;
@@ -684,13 +818,11 @@ _cairo_surface_reduce_by_half (cairo_surface_t *src)
 					   src_width / 2,
 					   src_height / 2);
 
-	cairo_surface_flush (dest);
-
 	dest_rowstride = cairo_image_surface_get_stride (dest);
-	dest_data = cairo_image_surface_get_data (dest);
+	dest_data = _cairo_image_surface_flush_and_get_data (dest);
 
 	src_rowstride = cairo_image_surface_get_stride (src);
-	src_data = cairo_image_surface_get_data (src);
+	src_data = _cairo_image_surface_flush_and_get_data (src);
 
 	for (y = 0; y < src_height - (src_height % 2); y += 2) {
 		row0 = src_data + (MAX (y - 1, 0) * src_rowstride);
@@ -715,7 +847,7 @@ _cairo_surface_reduce_by_half (cairo_surface_t *src)
 #endif
 
 
-cairo_surface_t *
+static cairo_surface_t *
 _cairo_image_surface_scale_bilinear_2x2 (cairo_surface_t *image,
 				         int              new_width,
 				         int              new_height)
@@ -753,6 +885,9 @@ _cairo_image_surface_scale_bilinear (cairo_surface_t *image,
 	cairo_surface_t *tmp2;
 	double           w, h;
 
+	if ((new_width == 0) || (new_height == 0))
+		return NULL;
+
 	scale = (double) new_width / cairo_image_surface_get_width (image);
 	last_scale = 1.0;
 	s = scale;
@@ -782,3 +917,115 @@ _cairo_image_surface_scale_bilinear (cairo_surface_t *image,
 
 	return tmp2;
 }
+
+
+/* -- _cairo_image_surface_scale_async -- */
+
+
+typedef struct {
+	cairo_surface_t *original;
+	int		 new_width;
+	int		 new_height;
+	scale_filter_t   quality;
+	cairo_surface_t *scaled;
+	GthTask         *task;
+} ScaleData;
+
+
+static ScaleData *
+scale_data_new (cairo_surface_t	 *image,
+		int		  new_width,
+		int		  new_height,
+		scale_filter_t    quality,
+		GCancellable     *cancellable)
+{
+	ScaleData *scale_data;
+
+	scale_data = g_new0 (ScaleData, 1);
+	scale_data->original = cairo_surface_reference (image);
+	scale_data->new_width = new_width;
+	scale_data->new_height = new_height;
+	scale_data->quality = quality;
+	scale_data->scaled = NULL;
+	scale_data->task = gth_async_task_new (NULL, NULL, NULL, NULL, NULL);
+	gth_task_set_cancellable (scale_data->task, cancellable);
+
+	return scale_data;
+}
+
+
+static void
+scale_data_free (ScaleData *scale_data)
+{
+	_g_object_unref (scale_data->task);
+	cairo_surface_destroy (scale_data->scaled);
+	cairo_surface_destroy (scale_data->original);
+	g_free (scale_data);
+}
+
+
+static void
+scale_image_thread (GSimpleAsyncResult *result,
+		    GObject            *object,
+		    GCancellable       *cancellable)
+{
+	ScaleData *scale_data;
+
+	scale_data = g_simple_async_result_get_op_res_gpointer (result);
+	scale_data->scaled = _cairo_image_surface_scale (scale_data->original,
+							 scale_data->new_width,
+							 scale_data->new_height,
+							 scale_data->quality,
+							 GTH_ASYNC_TASK (scale_data->task));
+}
+
+
+void
+_cairo_image_surface_scale_async (cairo_surface_t 	 *image,
+				  int		 	  new_width,
+				  int		  	  new_height,
+				  scale_filter_t   	  quality,
+				  GCancellable    	 *cancellable,
+				  GAsyncReadyCallback	  ready_callback,
+				  gpointer		  user_data)
+{
+	GSimpleAsyncResult *result;
+
+	result = g_simple_async_result_new (NULL,
+					    ready_callback,
+					    user_data,
+					    _cairo_image_surface_scale_async);
+	g_simple_async_result_set_op_res_gpointer (result,
+						   scale_data_new (image,
+								   new_width,
+								   new_height,
+								   quality,
+								   cancellable),
+						   (GDestroyNotify) scale_data_free);
+	g_simple_async_result_run_in_thread (result,
+					     scale_image_thread,
+					     G_PRIORITY_DEFAULT,
+					     cancellable);
+
+	g_object_unref (result);
+}
+
+
+cairo_surface_t *
+_cairo_image_surface_scale_finish (GAsyncResult	 *result,
+				   GError	**error)
+{
+	  GSimpleAsyncResult *simple;
+	  ScaleData          *scale_data;
+
+	  g_return_val_if_fail (g_simple_async_result_is_valid (result, NULL, _cairo_image_surface_scale_async), NULL);
+
+	  simple = G_SIMPLE_ASYNC_RESULT (result);
+
+	  if (g_simple_async_result_propagate_error (simple, error))
+		  return NULL;
+
+	  scale_data = g_simple_async_result_get_op_res_gpointer (simple);
+
+	  return cairo_surface_reference (scale_data->scaled);
+}
