@@ -25,11 +25,16 @@
 #include "cairo-utils.h"
 
 
+G_DEFINE_BOXED_TYPE (GthCairoSurface,
+		     gth_cairo_surface,
+		     (GBoxedCopyFunc) cairo_surface_reference,
+		     (GBoxedFreeFunc) cairo_surface_destroy)
+
+
 const unsigned char cairo_channel[4] = { CAIRO_RED, CAIRO_GREEN, CAIRO_BLUE, CAIRO_ALPHA };
 
 
 static cairo_user_data_key_t surface_metadata_key;
-static cairo_user_data_key_t surface_pixels_key;
 
 
 static void
@@ -40,13 +45,6 @@ surface_metadata_free (void *data)
 }
 
 
-static void
-surface_pixels_free (void *data)
-{
-	g_free (data);
-}
-
-
 inline int
 _cairo_multiply_alpha (int color,
 		       int alpha)
@@ -114,6 +112,25 @@ _cairo_clear_surface (cairo_surface_t  **surface)
 }
 
 
+unsigned char *
+_cairo_image_surface_flush_and_get_data (cairo_surface_t *surface)
+{
+	cairo_surface_flush (surface);
+	return cairo_image_surface_get_data (surface);
+}
+
+
+static void
+_cairo_surface_metadata_init (cairo_surface_metadata_t *metadata)
+{
+	metadata->has_alpha = FALSE;
+	metadata->original_width = 0;
+	metadata->original_height = 0;
+	metadata->thumbnail.image_width = 0;
+	metadata->thumbnail.image_height = 0;
+}
+
+
 cairo_surface_metadata_t *
 _cairo_image_surface_get_metadata (cairo_surface_t *surface)
 {
@@ -122,6 +139,7 @@ _cairo_image_surface_get_metadata (cairo_surface_t *surface)
 	metadata = cairo_surface_get_user_data (surface, &surface_metadata_key);
 	if (metadata == NULL) {
 		metadata = g_new0 (cairo_surface_metadata_t, 1);
+		_cairo_surface_metadata_init (metadata);
 		cairo_surface_set_user_data (surface, &surface_metadata_key, metadata, surface_metadata_free);
 	}
 
@@ -129,6 +147,35 @@ _cairo_image_surface_get_metadata (cairo_surface_t *surface)
 }
 
 
+void
+_cairo_image_surface_copy_metadata (cairo_surface_t *src,
+				    cairo_surface_t *dest)
+{
+	cairo_surface_metadata_t *src_metadata;
+	cairo_surface_metadata_t *dest_metadata;
+
+	src_metadata = _cairo_image_surface_get_metadata (src);
+	dest_metadata = _cairo_image_surface_get_metadata (dest);
+
+	dest_metadata->has_alpha = src_metadata->has_alpha;
+	dest_metadata->original_width = src_metadata->original_width;
+	dest_metadata->original_height = src_metadata->original_height;
+	dest_metadata->thumbnail.image_width = src_metadata->thumbnail.image_width;
+	dest_metadata->thumbnail.image_height = src_metadata->thumbnail.image_height;
+
+}
+
+
+void
+_cairo_image_surface_clear_metadata (cairo_surface_t *surface)
+{
+	cairo_surface_metadata_t *metadata;
+
+	metadata = _cairo_image_surface_get_metadata (surface);
+	_cairo_surface_metadata_init (metadata);
+}
+
+
 gboolean
 _cairo_image_surface_get_has_alpha (cairo_surface_t *surface)
 {
@@ -145,62 +192,71 @@ _cairo_image_surface_get_has_alpha (cairo_surface_t *surface)
 }
 
 
+gboolean
+_cairo_image_surface_get_original_size (cairo_surface_t *surface,
+					int             *original_width,
+					int             *original_height)
+{
+	cairo_surface_metadata_t *metadata;
+
+	if (surface == NULL)
+		return FALSE;
+
+	metadata = cairo_surface_get_user_data (surface, &surface_metadata_key);
+	if (metadata == NULL)
+		return FALSE;
+
+	if ((metadata->original_width <= 0) || (metadata->original_height <= 0))
+		return FALSE;
+
+	if (original_width)
+		*original_width = metadata->original_width;
+	if (original_height)
+		*original_height = metadata->original_height;
+
+	return TRUE;
+}
+
+
+cairo_surface_t *
+_cairo_image_surface_create (cairo_format_t format,
+			     int            width,
+			     int            height)
+{
+	cairo_surface_t *result;
+	cairo_status_t   status;
+
+	result = cairo_image_surface_create (format, width, height);
+	status = cairo_surface_status (result);
+	if (status != CAIRO_STATUS_SUCCESS) {
+		g_warning ("_cairo_image_surface_create: could not create the surface: %s", cairo_status_to_string (status));
+		cairo_surface_destroy (result);
+		return NULL;
+	}
+
+	return result;
+}
+
+
 cairo_surface_t *
 _cairo_image_surface_copy (cairo_surface_t *source)
 {
 	cairo_surface_t *result;
-	cairo_format_t   format;
-	int              width;
-	int              height;
-	int              stride;
-	unsigned char   *pixels;
-	cairo_status_t   status;
-	int              source_stride;
-	int              destination_stride;
 	unsigned char   *p_source;
 	unsigned char   *p_destination;
-	int              row_size;
 
 	if (source == NULL)
 		return NULL;
 
-	format = cairo_image_surface_get_format (source);
-	width = cairo_image_surface_get_width (source);
-	height = cairo_image_surface_get_height (source);
-	stride = cairo_format_stride_for_width (format, width);
-	pixels = g_try_malloc (stride * height);
-        if (pixels == NULL)
-                return NULL;
-
-	result = cairo_image_surface_create_for_data (pixels, format, width, height, stride);
-	status = cairo_surface_status (result);
-	if (status != CAIRO_STATUS_SUCCESS) {
-		g_warning ("_cairo_image_surface_copy: could not create the surface: %s", cairo_status_to_string (status));
-		cairo_surface_destroy (result);
+	result = _cairo_image_surface_create (cairo_image_surface_get_format (source),
+					      cairo_image_surface_get_width (source),
+					      cairo_image_surface_get_height (source));
+	if (result == NULL)
 		return NULL;
-	}
-
-	status = cairo_surface_set_user_data (result, &surface_pixels_key, pixels, surface_pixels_free);
-	if (status != CAIRO_STATUS_SUCCESS) {
-		g_warning ("_cairo_image_surface_copy: could not set the user data: %s", cairo_status_to_string (status));
-		cairo_surface_destroy (result);
-		return NULL;
-	}
-
-	cairo_surface_flush (result);
-
-	source_stride = cairo_image_surface_get_stride (source);
-	destination_stride = cairo_image_surface_get_stride (result);
-	p_source = cairo_image_surface_get_data (source);
-	p_destination = cairo_image_surface_get_data (result);
-	row_size = width * 4;
-	while (height-- > 0) {
-		memcpy (p_destination, p_source, row_size);
-
-		p_source += source_stride;
-		p_destination += destination_stride;
-	}
 
+	p_source = _cairo_image_surface_flush_and_get_data (source);
+	p_destination = _cairo_image_surface_flush_and_get_data (result);
+	memcpy (p_destination, p_source, cairo_image_surface_get_stride (source) * cairo_image_surface_get_height (source));
 	cairo_surface_mark_dirty (result);
 
 	return result;
@@ -234,12 +290,10 @@ _cairo_image_surface_copy_subsurface (cairo_surface_t *source,
 		return NULL;
 	}
 
-	cairo_surface_flush (destination);
-
 	source_stride = cairo_image_surface_get_stride (source);
 	destination_stride = cairo_image_surface_get_stride (destination);
-	p_source = cairo_image_surface_get_data (source) + (src_y * source_stride) + (src_x * 4);
-	p_destination = cairo_image_surface_get_data (destination);
+	p_source = _cairo_image_surface_flush_and_get_data (source) + (src_y * source_stride) + (src_x * 4);
+	p_destination = _cairo_image_surface_flush_and_get_data (destination);
 	row_size = width * 4;
 	while (height-- > 0) {
 		memcpy (p_destination, p_source, row_size);
@@ -281,9 +335,8 @@ _cairo_image_surface_create_from_pixbuf (GdkPixbuf *pixbuf)
 		      "pixels", &p_pixels,
 		      NULL );
 	surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, width, height);
-	cairo_surface_flush (surface);
 	s_stride = cairo_image_surface_get_stride (surface);
-	s_pixels = cairo_image_surface_get_data (surface);
+	s_pixels = _cairo_image_surface_flush_and_get_data (surface);
 
 	metadata = _cairo_image_surface_get_metadata (surface);
 	metadata->has_alpha = (p_n_channels == 4);
@@ -501,9 +554,8 @@ _cairo_image_surface_transform (cairo_surface_t *source,
 						  &pixel_step);
 
 	destination = cairo_image_surface_create (format, destination_width, destination_height);
-	cairo_surface_flush (destination);
-	p_source_line = cairo_image_surface_get_data (source);
-	p_destination_line = cairo_image_surface_get_data (destination) + line_start;
+	p_source_line = _cairo_image_surface_flush_and_get_data (source);
+	p_destination_line = _cairo_image_surface_flush_and_get_data (destination) + line_start;
 	while (height-- > 0) {
 		p_source = p_source_line;
 		p_destination = p_destination_line;
@@ -539,9 +591,9 @@ _cairo_image_surface_color_shift (cairo_surface_t *image,
 	width       = cairo_image_surface_get_width (image);
 	height      = cairo_image_surface_get_height (image);
 	src_stride  = cairo_image_surface_get_stride (image);
-	src_pixels  = cairo_image_surface_get_data (image);
+	src_pixels  = _cairo_image_surface_flush_and_get_data (image);
 	dest_stride = cairo_image_surface_get_stride (shifted);
-	dest_pixels = cairo_image_surface_get_data (shifted);
+	dest_pixels = _cairo_image_surface_flush_and_get_data (shifted);
 
 	src_row = src_pixels;
 	dest_row = dest_pixels;
@@ -571,15 +623,17 @@ _cairo_image_surface_color_shift (cairo_surface_t *image,
 		dest_row += dest_stride;
 	}
 
+	cairo_surface_mark_dirty (shifted);
+
 	return shifted;
 }
 
 
 void
-_cairo_copy_line_as_rgba (guchar *dest,
-			  guchar *src,
-			  guint   width,
-			  guint   alpha)
+_cairo_copy_line_as_rgba_big_endian (guchar *dest,
+				     guchar *src,
+				     guint   width,
+				     guint   alpha)
 {
 	guint x;
 
@@ -593,9 +647,41 @@ _cairo_copy_line_as_rgba (guchar *dest,
 	}
 	else {
 		for (x = 0; x < width; x++) {
-			dest[0] = src[CAIRO_RED];
+			CAIRO_GET_RGB (src, dest[0], dest[1], dest[2]);
+
+			src += 4;
+			dest += 3;
+		}
+	}
+}
+
+
+void
+_cairo_copy_line_as_rgba_little_endian (guchar *dest,
+					guchar *src,
+					guint   width,
+					guint   alpha)
+{
+	guint x;
+
+	if (alpha) {
+		int r, g, b, a;
+		for (x = 0; x < width; x++) {
+			CAIRO_GET_RGBA (src, r, g, b, a);
+			dest[0] = b;
+			dest[1] = g;
+			dest[2] = r;
+			dest[3] = a;
+
+			src += 4;
+			dest += 4;
+		}
+	}
+	else {
+		for (x = 0; x < width; x++) {
+			dest[0] = src[CAIRO_BLUE];
 			dest[1] = src[CAIRO_GREEN];
-			dest[2] = src[CAIRO_BLUE];
+			dest[2] = src[CAIRO_RED];
 
 			src += 4;
 			dest += 3;
@@ -627,8 +713,6 @@ _cairo_paint_full_gradient (cairo_surface_t *surface,
 	if (cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS)
 		return;
 
-	cairo_surface_flush (surface);
-
 	_gdk_rgba_to_cairo_color_255 (h_color1, &hcolor1);
 	_gdk_rgba_to_cairo_color_255 (h_color2, &hcolor2);
 	_gdk_rgba_to_cairo_color_255 (v_color1, &vcolor1);
@@ -637,7 +721,7 @@ _cairo_paint_full_gradient (cairo_surface_t *surface,
 	width = cairo_image_surface_get_width (surface);
 	height = cairo_image_surface_get_height (surface);
 	s_stride = cairo_image_surface_get_stride (surface);
-	s_pixels = cairo_image_surface_get_data (surface);
+	s_pixels = _cairo_image_surface_flush_and_get_data (surface);
 
 	for (h = 0; h < height; h++) {
 		guchar *s_iter = s_pixels;
@@ -680,17 +764,13 @@ _cairo_draw_rounded_box (cairo_t *cr,
 	}
 	else {
 		cairo_move_to (cr, x, y + r);
-		if (r > 0)
-			cairo_arc (cr, x + r, y + r, r, 1.0 * M_PI, 1.5 * M_PI);
+		cairo_arc (cr, x + r, y + r, r, 1.0 * M_PI, 1.5 * M_PI);
 		cairo_rel_line_to (cr, w - (r * 2), 0);
-		if (r > 0)
-			cairo_arc (cr, x + w - r, y + r, r, 1.5 * M_PI, 2.0 * M_PI);
+		cairo_arc (cr, x + w - r, y + r, r, 1.5 * M_PI, 2.0 * M_PI);
 		cairo_rel_line_to (cr, 0, h - (r * 2));
-		if (r > 0)
-			cairo_arc (cr, x + w - r, y + h - r, r, 0.0 * M_PI, 0.5 * M_PI);
+		cairo_arc (cr, x + w - r, y + h - r, r, 0.0 * M_PI, 0.5 * M_PI);
 		cairo_rel_line_to (cr, - (w - (r * 2)), 0);
-		if (r > 0)
-			cairo_arc (cr, x + r, y + h - r, r, 0.5 * M_PI, 1.0 * M_PI);
+		cairo_arc (cr, x + r, y + h - r, r, 0.5 * M_PI, 1.0 * M_PI);
 		cairo_rel_line_to (cr, 0, - (h - (r * 2)));
 	}
 }
@@ -795,23 +875,21 @@ _cairo_draw_slide (cairo_t  *cr,
 					 image_height);
 			cairo_fill (cr);
 
-			image_x -= 1;
-			image_y -= 1;
-			image_x2 = image_x + image_width + 2;
-			image_y2 = image_y + image_height + 2;
+			image_x2 = image_x + image_width + 1;
+			image_y2 = image_y + image_height + 1;
 
 			cairo_set_source_rgb (cr, dark_gray, dark_gray, dark_gray);
 			cairo_move_to (cr, image_x, image_y);
-			cairo_line_to (cr, image_x2 - 1, image_y);
+			cairo_line_to (cr, image_x2, image_y);
 			cairo_move_to (cr, image_x, image_y);
-			cairo_line_to (cr, image_x, image_y2 - 1);
+			cairo_line_to (cr, image_x, image_y2);
 			cairo_stroke (cr);
 
 			cairo_set_source_rgb (cr, mid_gray, mid_gray, mid_gray);
-			cairo_move_to (cr, image_x2 - 1, image_y - 1);
-			cairo_line_to (cr, image_x2 - 1, image_y2 - 1);
-			cairo_move_to (cr, image_x, image_y2 - 1);
-			cairo_line_to (cr, image_x2, image_y2 - 1);
+			cairo_move_to (cr, image_x2, image_y);
+			cairo_line_to (cr, image_x2, image_y2);
+			cairo_move_to (cr, image_x, image_y2);
+			cairo_line_to (cr, image_x2, image_y2);
 			cairo_stroke (cr);
 		}
 	}
@@ -819,31 +897,31 @@ _cairo_draw_slide (cairo_t  *cr,
 	/* outer border. */
 
 	cairo_set_source_rgb (cr, mid_gray, mid_gray, mid_gray);
-	cairo_move_to (cr, frame_x - 1, frame_y);
+	cairo_move_to (cr, frame_x, frame_y);
 	cairo_line_to (cr, frame_x2, frame_y);
-	cairo_move_to (cr, frame_x, frame_y - 1);
+	cairo_move_to (cr, frame_x, frame_y);
 	cairo_line_to (cr, frame_x, frame_y2);
 	cairo_stroke (cr);
 
 	cairo_set_source_rgb (cr, darker_gray, darker_gray, darker_gray);
-	cairo_move_to (cr, frame_x2, frame_y - 1);
+	cairo_move_to (cr, frame_x2, frame_y);
 	cairo_line_to (cr, frame_x2, frame_y2);
-	cairo_move_to (cr, frame_x - 1, frame_y2);
+	cairo_move_to (cr, frame_x, frame_y2);
 	cairo_line_to (cr, frame_x2, frame_y2);
 	cairo_stroke (cr);
 
 	cairo_set_source_rgb (cr, light_gray, light_gray, light_gray);
-	cairo_move_to (cr, frame_x, frame_y + 1);
-	cairo_line_to (cr, frame_x2 - 1, frame_y + 1);
-	cairo_move_to (cr, frame_x + 1, frame_y);
-	cairo_line_to (cr, frame_x + 1, frame_y2 - 1);
+	cairo_move_to (cr, frame_x, frame_y);
+	cairo_line_to (cr, frame_x2, frame_y);
+	cairo_move_to (cr, frame_x, frame_y);
+	cairo_line_to (cr, frame_x, frame_y2);
 	cairo_stroke (cr);
 
 	cairo_set_source_rgb (cr, dark_gray, dark_gray, dark_gray);
-	cairo_move_to (cr, frame_x2 - 1, frame_y);
-	cairo_line_to (cr, frame_x2 - 1, frame_y2 - 1);
-	cairo_move_to (cr, frame_x, frame_y2 - 1);
-	cairo_line_to (cr, frame_x2 - 1, frame_y2 - 1);
+	cairo_move_to (cr, frame_x2, frame_y);
+	cairo_line_to (cr, frame_x2, frame_y2);
+	cairo_move_to (cr, frame_x, frame_y2);
+	cairo_line_to (cr, frame_x2, frame_y2);
 	cairo_stroke (cr);
 
 	cairo_restore (cr);
