@@ -118,6 +118,11 @@
 
 /* types */
 
+typedef cairo_surface_t GthCairoSurface;
+
+GType gth_cairo_surface_get_type (void);
+#define GTH_TYPE_CAIRO_SURFACE (gth_cairo_surface_get_type ())
+
 typedef struct {
 	guchar r;
 	guchar g;
@@ -126,7 +131,15 @@ typedef struct {
 } cairo_color_255_t;
 
 typedef struct {
-	gboolean has_alpha;
+	int image_width;
+	int image_height;
+} thumbnail_metadata_t;
+
+typedef struct {
+	gboolean		has_alpha;
+	int			original_width;
+	int			original_height;
+	thumbnail_metadata_t	thumbnail;
 } cairo_surface_metadata_t;
 
 extern const unsigned char cairo_channel[4];
@@ -151,9 +164,19 @@ void               _gdk_rgba_to_cairo_color_255             (GdkRGBA
 /* surface */
 
 void               _cairo_clear_surface                     (cairo_surface_t      **surface);
+unsigned char *    _cairo_image_surface_flush_and_get_data  (cairo_surface_t       *surface);
 cairo_surface_metadata_t *
 		   _cairo_image_surface_get_metadata        (cairo_surface_t       *surface);
+void               _cairo_image_surface_copy_metadata       (cairo_surface_t	   *src,
+							     cairo_surface_t	   *dest);
+void               _cairo_image_surface_clear_metadata      (cairo_surface_t	   *surface);
 gboolean           _cairo_image_surface_get_has_alpha       (cairo_surface_t       *surface);
+gboolean           _cairo_image_surface_get_original_size   (cairo_surface_t       *surface,
+							     int                   *original_width,
+							     int                   *original_height);
+cairo_surface_t *  _cairo_image_surface_create              (cairo_format_t         format,
+							     int                    width,
+							     int                    height);
 cairo_surface_t *  _cairo_image_surface_copy                (cairo_surface_t       *surface);
 cairo_surface_t *  _cairo_image_surface_copy_subsurface     (cairo_surface_t       *surface,
 				      	      	      	     int                    src_x,
@@ -175,7 +198,11 @@ cairo_surface_t *  _cairo_image_surface_transform           (cairo_surface_t
 							     GthTransform           transform);
 cairo_surface_t *  _cairo_image_surface_color_shift         (cairo_surface_t       *image,
 							     int                    shift);
-void               _cairo_copy_line_as_rgba                 (guchar                *dest,
+void               _cairo_copy_line_as_rgba_big_endian      (guchar                *dest,
+							     guchar                *src,
+							     guint                  width,
+							     guint                  alpha);
+void               _cairo_copy_line_as_rgba_little_endian   (guchar                *dest,
 							     guchar                *src,
 							     guint                  width,
 							     guint                  alpha);
