@@ -107,14 +107,6 @@ close_button_clicked_cb (GtkWidget  *widget,
 }
 
 
-static void
-help_button_clicked_cb (GtkWidget  *widget,
-			DialogData *data)
-{
-	show_help_dialog (GTK_WINDOW (data->dialog), NULL);
-}
-
-
 static void
 use_startup_toggled_cb (GtkWidget *widget,
 			DialogData *data)
@@ -137,7 +129,7 @@ set_to_current_cb (GtkWidget  *widget,
 
 		gio_file = gth_file_source_to_gio_file (file_source, gth_browser_get_location (data->browser));
 		uri = g_file_get_uri (gio_file);
-		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (GET_WIDGET ("startup_dir_filechooserbutton")), uri);
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (GET_WIDGET ("startup_dir_filechooserbutton")), uri);
 
 		g_free (uri);
 		g_object_unref (gio_file);
@@ -146,14 +138,6 @@ set_to_current_cb (GtkWidget  *widget,
 }
 
 
-static void
-toolbar_style_changed_cb (GtkWidget  *widget,
-			  DialogData *data)
-{
-	g_settings_set_enum (data->browser_settings, PREF_BROWSER_TOOLBAR_STYLE, gtk_combo_box_get_active (GTK_COMBO_BOX (GET_WIDGET ("toolbar_style_combobox"))));
-}
-
-
 static void
 thumbnails_pane_orientation_changed_cb (GtkWidget  *widget,
 					DialogData *data)
@@ -280,7 +264,7 @@ dlg_preferences (GthBrowser *browser)
 		location = g_file_new_for_uri (startup_location);
 		folder = gth_file_source_to_gio_file (file_source, location);
 		folder_uri = g_file_get_uri (folder);
-		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (GET_WIDGET ("startup_dir_filechooserbutton")), folder_uri);
+		gtk_file_chooser_set_uri (GTK_FILE_CHOOSER (GET_WIDGET ("startup_dir_filechooserbutton")), folder_uri);
 
 		g_free (folder_uri);
 		g_object_unref (folder);
@@ -295,8 +279,6 @@ dlg_preferences (GthBrowser *browser)
 				      g_settings_get_boolean (data->messages_settings, PREF_MSG_CONFIRM_DELETION));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (GET_WIDGET ("ask_to_save_checkbutton")),
 				      g_settings_get_boolean (data->messages_settings, PREF_MSG_SAVE_MODIFIED_IMAGE));
-	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("toolbar_style_combobox")),
-				  g_settings_get_enum (data->browser_settings, PREF_BROWSER_TOOLBAR_STYLE));
 	gtk_combo_box_set_active (GTK_COMBO_BOX (GET_WIDGET ("thumbnails_pane_orient_combobox")),
 				  g_settings_get_enum (data->browser_settings, PREF_BROWSER_VIEWER_THUMBNAILS_ORIENT));
 
@@ -324,17 +306,9 @@ dlg_preferences (GthBrowser *browser)
 			  "clicked",
 			  G_CALLBACK (close_button_clicked_cb),
 			  data);
-	g_signal_connect (G_OBJECT (GET_WIDGET ("help_button")),
-			  "clicked",
-			  G_CALLBACK (help_button_clicked_cb),
-			  data);
 
 	/* general */
 
-	g_signal_connect (G_OBJECT (GET_WIDGET ("toolbar_style_combobox")),
-			  "changed",
-			  G_CALLBACK (toolbar_style_changed_cb),
-			  data);
 	g_signal_connect (GET_WIDGET ("thumbnails_pane_orient_combobox"),
 			  "changed",
 			  G_CALLBACK (thumbnails_pane_orientation_changed_cb),
