@@ -35,11 +35,6 @@ struct _DomDocumentPrivate {
 };
 
 
-static gpointer dom_element_parent_class = NULL;
-static gpointer dom_text_node_parent_class = NULL;
-static gpointer dom_document_parent_class = NULL;
-
-
 GQuark
 dom_error_quark (void)
 {
@@ -103,6 +98,9 @@ _g_xml_tag_is_special (const char *tag_name)
 /* -- DomElement -- */
 
 
+G_DEFINE_TYPE (DomElement, dom_element, G_TYPE_INITIALLY_UNOWNED)
+
+
 static void
 dom_attribute_dump (gpointer key,
 		    gpointer value,
@@ -186,7 +184,7 @@ dom_element_real_dump (DomElement *self,
 
 
 static void
-dom_element_instance_init (DomElement *self)
+dom_element_init (DomElement *self)
 {
 	self->tag_name = NULL;
 	self->prefix = NULL;
@@ -221,38 +219,11 @@ dom_element_finalize (GObject *obj)
 static void
 dom_element_class_init (DomElementClass *klass)
 {
-	dom_element_parent_class = g_type_class_peek_parent (klass);
 	G_OBJECT_CLASS (klass)->finalize = dom_element_finalize;
 	DOM_ELEMENT_CLASS (klass)->dump = dom_element_real_dump;
 }
 
 
-GType
-dom_element_get_type (void)
-{
-	static GType dom_element_type_id = 0;
-	if (dom_element_type_id == 0) {
-		static const GTypeInfo g_define_type_info = {
-			sizeof (DomElementClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) dom_element_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (DomElement),
-			0,
-			(GInstanceInitFunc) dom_element_instance_init,
-			NULL
-		};
-		dom_element_type_id = g_type_register_static (G_TYPE_INITIALLY_UNOWNED,
-							      "DomElement",
-							      &g_define_type_info,
-							      0);
-	}
-	return dom_element_type_id;
-}
-
-
 static DomElement *
 dom_element_new (const char *a_tag_name)
 {
@@ -417,6 +388,9 @@ dom_element_get_inner_text (DomElement *self)
 /* -- DomTextNode -- */
 
 
+G_DEFINE_TYPE (DomTextNode, dom_text_node, DOM_TYPE_ELEMENT)
+
+
 static char *
 dom_text_node_real_dump (DomElement *base,
 			 int         level)
@@ -443,46 +417,19 @@ dom_text_node_finalize (GObject *obj)
 static void
 dom_text_node_class_init (DomTextNodeClass *klass)
 {
-	dom_text_node_parent_class = g_type_class_peek_parent (klass);
 	G_OBJECT_CLASS (klass)->finalize = dom_text_node_finalize;
 	DOM_ELEMENT_CLASS (klass)->dump = dom_text_node_real_dump;
 }
 
 
 static void
-dom_text_node_instance_init (DomTextNode *self)
+dom_text_node_init (DomTextNode *self)
 {
 	DOM_ELEMENT (self)->tag_name = g_strdup (XML_TEXT_NODE_TAG_NAME);
 	self->data = NULL;
 }
 
 
-GType
-dom_text_node_get_type (void)
-{
-	static GType dom_text_node_type_id = 0;
-	if (dom_text_node_type_id == 0) {
-		static const GTypeInfo g_define_type_info = {
-			sizeof (DomTextNodeClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) dom_text_node_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (DomTextNode),
-			0,
-			(GInstanceInitFunc) dom_text_node_instance_init,
-			NULL
-		};
-		dom_text_node_type_id = g_type_register_static (DOM_TYPE_ELEMENT,
-								"DomTextNode",
-								&g_define_type_info,
-								0);
-	}
-	return dom_text_node_type_id;
-}
-
-
 static DomTextNode *
 dom_text_node_new (const char *a_data)
 {
@@ -498,6 +445,9 @@ dom_text_node_new (const char *a_data)
 /* -- DomDocument -- */
 
 
+G_DEFINE_TYPE (DomDocument, dom_document, DOM_TYPE_ELEMENT)
+
+
 static void
 dom_document_finalize (GObject *obj)
 {
@@ -517,13 +467,12 @@ dom_document_finalize (GObject *obj)
 static void
 dom_document_class_init (DomDocumentClass *klass)
 {
-	dom_document_parent_class = g_type_class_peek_parent (klass);
 	G_OBJECT_CLASS (klass)->finalize = dom_document_finalize;
 }
 
 
 static void
-dom_document_instance_init (DomDocument *self)
+dom_document_init (DomDocument *self)
 {
 	DOM_ELEMENT (self)->tag_name = g_strdup (XML_DOCUMENT_TAG_NAME);
 
@@ -532,32 +481,6 @@ dom_document_instance_init (DomDocument *self)
 }
 
 
-GType
-dom_document_get_type (void)
-{
-	static GType dom_document_type_id = 0;
-	if (dom_document_type_id == 0) {
-		static const GTypeInfo g_define_type_info = {
-			sizeof (DomDocumentClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) dom_document_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			sizeof (DomDocument),
-			0,
-			(GInstanceInitFunc) dom_document_instance_init,
-			NULL
-		};
-		dom_document_type_id = g_type_register_static (DOM_TYPE_ELEMENT,
-							       "DomDocument",
-							       &g_define_type_info,
-							       0);
-	}
-	return dom_document_type_id;
-}
-
-
 DomDocument *
 dom_document_new (void)
 {
@@ -571,7 +494,6 @@ dom_document_new (void)
 DomElement *
 dom_document_create_element (DomDocument *self,
 			     const char  *tag_name,
-			     const char  *first_attr,
 			     ...)
 {
 	DomElement *element;
@@ -583,12 +505,10 @@ dom_document_create_element (DomDocument *self,
 	g_return_val_if_fail (tag_name != NULL, NULL);
 
 	element = dom_element_new (tag_name);
-	va_start (var_args, first_attr);
-	attr = first_attr;
-        while (attr != NULL) {
+	va_start (var_args, tag_name);
+        while ((attr = va_arg (var_args, const char *)) != NULL) {
         	value = va_arg (var_args, const char *);
 		dom_element_set_attribute (element, attr, value);
-                attr = va_arg (var_args, const char *);
         }
 	va_end (var_args);
 
@@ -611,7 +531,6 @@ DomElement *
 dom_document_create_element_with_text (DomDocument  *self,
 				       const char   *text_data,
 				       const char   *tag_name,
-				       const char   *first_attr,
 				       ...)
 {
 	DomElement *element;
@@ -623,12 +542,10 @@ dom_document_create_element_with_text (DomDocument  *self,
 	g_return_val_if_fail (tag_name != NULL, NULL);
 
 	element = dom_element_new (tag_name);
-	va_start (var_args, first_attr);
-	attr = first_attr;
-        while (attr != NULL) {
+	va_start (var_args, tag_name);
+        while ((attr = va_arg (var_args, const char *)) != NULL) {
         	value = va_arg (var_args, const char *);
 		dom_element_set_attribute (element, attr, value);
-                attr = va_arg (var_args, const char *);
         }
 	va_end (var_args);
 
@@ -751,6 +668,16 @@ dom_document_load (DomDocument  *self,
 /* -- DomDomizable -- */
 
 
+G_DEFINE_INTERFACE (DomDomizable, dom_domizable, 0)
+
+
+static void
+dom_domizable_default_init (DomDomizableInterface *iface)
+{
+	/* void */
+}
+
+
 DomElement *
 dom_domizable_create_element (DomDomizable *self,
 			      DomDocument  *doc)
@@ -767,34 +694,9 @@ dom_domizable_load_from_element (DomDomizable *self,
 }
 
 
-GType
-dom_domizable_get_type (void)
-{
-	static GType dom_domizable_type_id = 0;
-	if (dom_domizable_type_id == 0) {
-		static const GTypeInfo g_define_type_info = {
-			sizeof (DomDomizableIface),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) NULL,
-			(GClassFinalizeFunc) NULL,
-			NULL,
-			0,
-			0,
-			(GInstanceInitFunc) NULL,
-			NULL
-		};
-		dom_domizable_type_id = g_type_register_static (G_TYPE_INTERFACE,
-								"DomDomizable",
-								&g_define_type_info,
-								0);
-	}
-	return dom_domizable_type_id;
-}
-
-
 /* -- Utilities -- */
 
+
 /* GMarkupParser converts \r into \n, this function compares two strings
  * treating \r characters as they were equal to \n */
 gboolean
