@@ -1056,7 +1056,7 @@ typedef struct {
 } DummyFileCopy;
 
 
-gboolean
+static gboolean
 _g_dummy_file_op_completed (gpointer data)
 {
 	DummyFileCopy *dfd = data;
@@ -1934,32 +1934,22 @@ _g_delete_files_async (GList        *file_list,
 
 
 gboolean
-g_load_file_in_buffer (GFile         *file,
-		       void         **buffer,
-		       gsize         *size,
-		       GCancellable  *cancellable,
-		       GError       **error)
+_g_input_stream_read_all (GInputStream  *istream,
+			  void         **buffer,
+			  gsize         *size,
+			  GCancellable  *cancellable,
+			  GError       **error)
 {
-	GFileInputStream *istream;
-	gboolean          retval;
-	void             *local_buffer;
-	gsize             count;
-	gssize            n;
-	char              tmp_buffer[BUFFER_SIZE];
+	gboolean  retval = FALSE;
+	guchar   *local_buffer = NULL;
+	char     *tmp_buffer;
+	gsize     count;
+	gssize    n;
 
-	istream = g_file_read (file, cancellable, error);
-	if (istream == NULL)
-		return FALSE;
-
-	retval = FALSE;
-	local_buffer = NULL;
+	tmp_buffer = g_new (char, BUFFER_SIZE);
 	count = 0;
 	for (;;) {
-		n = g_input_stream_read (G_INPUT_STREAM (istream),
-					 tmp_buffer,
-					 BUFFER_SIZE,
-					 cancellable,
-					 error);
+		n = g_input_stream_read (istream, tmp_buffer, BUFFER_SIZE, cancellable, error);
 		if (n < 0) {
 			g_free (local_buffer);
 			retval = FALSE;
@@ -1979,7 +1969,27 @@ g_load_file_in_buffer (GFile         *file,
 		count += n;
 	}
 
-	g_object_unref (istream);
+	g_free (tmp_buffer);
+
+	return retval;
+}
+
+
+gboolean
+_g_file_load_in_buffer (GFile         *file,
+		        void         **buffer,
+		        gsize         *size,
+		        GCancellable  *cancellable,
+		        GError       **error)
+{
+	GInputStream *istream;
+	gboolean      retval = FALSE;
+
+	istream = (GInputStream *) g_file_read (file, cancellable, error);
+	if (istream != NULL) {
+		retval = _g_input_stream_read_all (istream, buffer, size, cancellable, error);
+		g_object_unref (istream);
+	}
 
 	return retval;
 }
@@ -1992,7 +2002,7 @@ typedef struct {
 	gpointer             user_data;
 	GInputStream        *stream;
 	guchar               tmp_buffer[BUFFER_SIZE];
-	void                *buffer;
+	guchar              *buffer;
 	gsize                count;
 } LoadData;
 
@@ -2018,14 +2028,14 @@ load_file__stream_read_cb (GObject      *source_object,
 
 	count = g_input_stream_read_finish (load_data->stream, result, &error);
 	if (count < 0) {
-		load_data->callback (&load_data->buffer, -1, error, load_data->user_data);
+		load_data->callback ((gpointer *) &load_data->buffer, -1, error, load_data->user_data);
 		load_data_free (load_data);
 		return;
 	}
 	else if (count == 0) {
 		if (load_data->buffer != NULL)
 			((char *)load_data->buffer)[load_data->count] = 0;
-		load_data->callback (&load_data->buffer, load_data->count, NULL, load_data->user_data);
+		load_data->callback ((gpointer *) &load_data->buffer, load_data->count, NULL, load_data->user_data);
 		load_data_free (load_data);
 		return;
 	}
@@ -2054,7 +2064,7 @@ load_file__file_read_cb (GObject      *source_object,
 
 	load_data->stream = (GInputStream *) g_file_read_finish (G_FILE (source_object), result, &error);
 	if (load_data->stream == NULL) {
-		load_data->callback (&load_data->buffer, -1, error, load_data->user_data);
+		load_data->callback ((gpointer *) &load_data->buffer, -1, error, load_data->user_data);
 		load_data_free (load_data);
 		return;
 	}
@@ -2071,11 +2081,11 @@ load_file__file_read_cb (GObject      *source_object,
 
 
 void
-g_load_file_async (GFile               *file,
-		   int                  io_priority,
-		   GCancellable        *cancellable,
-		   BufferReadyCallback  callback,
-		   gpointer             user_data)
+_g_file_load_async (GFile               *file,
+		    int                  io_priority,
+		    GCancellable        *cancellable,
+		    BufferReadyCallback  callback,
+		    gpointer             user_data)
 {
 	LoadData *load_data;
 
@@ -2089,7 +2099,7 @@ g_load_file_async (GFile               *file,
 }
 
 
-/* -- g_write_file_async -- */
+/* -- _g_file_write_async -- */
 
 
 typedef struct {
@@ -2097,7 +2107,7 @@ typedef struct {
 	GCancellable        *cancellable;
 	BufferReadyCallback  callback;
 	gpointer             user_data;
-	void                *buffer;
+	guchar              *buffer;
 	gsize                count;
 	gsize                written;
 	GError              *error;
@@ -2117,7 +2127,7 @@ write_file__notify (gpointer user_data)
 {
 	WriteData *write_data = user_data;
 
-	write_data->callback (&write_data->buffer, write_data->count, write_data->error, write_data->user_data);
+	write_data->callback ((gpointer *) &write_data->buffer, write_data->count, write_data->error, write_data->user_data);
 	write_data_free (write_data);
 }
 
@@ -2182,7 +2192,7 @@ write_file__replace_ready_cb (GObject      *source_object,
 
 	stream = (GOutputStream*) g_file_replace_finish ((GFile*) source_object, result, &error);
 	if (stream == NULL) {
-		write_data->callback (&write_data->buffer, write_data->count, error, write_data->user_data);
+		write_data->callback ((gpointer *) &write_data->buffer, write_data->count, error, write_data->user_data);
 		write_data_free (write_data);
 		return;
 	}
@@ -2209,7 +2219,7 @@ write_file__create_ready_cb (GObject      *source_object,
 
 	stream = (GOutputStream*) g_file_create_finish ((GFile*) source_object, result, &error);
 	if (stream == NULL) {
-		write_data->callback (&write_data->buffer, write_data->count, error, write_data->user_data);
+		write_data->callback ((gpointer *) &write_data->buffer, write_data->count, error, write_data->user_data);
 		write_data_free (write_data);
 		return;
 	}
@@ -2226,14 +2236,14 @@ write_file__create_ready_cb (GObject      *source_object,
 
 
 void
-g_write_file_async (GFile               *file,
-		    void                *buffer,
-		    gsize                count,
-		    gboolean             replace,
-		    int                  io_priority,
-		    GCancellable        *cancellable,
-		    BufferReadyCallback  callback,
-		    gpointer             user_data)
+_g_file_write_async (GFile               *file,
+		     void                *buffer,
+		     gsize                count,
+		     gboolean             replace,
+		     int                  io_priority,
+		     GCancellable        *cancellable,
+		     BufferReadyCallback  callback,
+		     gpointer             user_data)
 {
 	WriteData *write_data;
 
@@ -2398,17 +2408,17 @@ _g_directory_create_tmp (void)
 }
 
 
-/* -- g_write_file -- */
+/* -- _g_file_write -- */
 
 
 gboolean
-g_write_file (GFile             *file,
-	      gboolean           make_backup,
-	      GFileCreateFlags   flags,
-	      void              *buffer,
-	      gsize              count,
-	      GCancellable      *cancellable,
-	      GError           **error)
+_g_file_write (GFile             *file,
+	       gboolean           make_backup,
+	       GFileCreateFlags   flags,
+	       void              *buffer,
+	       gsize              count,
+	       GCancellable      *cancellable,
+	       GError           **error)
 {
 	gboolean       success;
 	GOutputStream *stream;
