@@ -31,6 +31,7 @@
 #include "glib-utils.h"
 
 #define MAX_PATTERNS 128
+#define BUFFER_SIZE_FOR_SNIFFING 32
 
 
 /* gobject utils*/
@@ -84,7 +85,7 @@ _g_object_list_ref (GList *list)
 void
 _g_object_list_unref (GList *list)
 {
-	g_list_foreach (list, (GFunc) g_object_unref, NULL);
+	g_list_foreach (list, (GFunc) _g_object_unref, NULL);
 	g_list_free (list);
 }
 
@@ -664,7 +665,7 @@ _g_strdup_with_max_size (const char *s,
 		offset = half_max_size + l - max_size;
 		second_half = g_strndup (s + offset, half_max_size);
 
-		result = g_strconcat (first_half, "...", second_half, NULL);
+		result = g_strconcat (first_half, "…", second_half, NULL);
 
 		g_free (first_half);
 		g_free (second_half);
@@ -1164,7 +1165,7 @@ _g_list_insert_list_before (GList *list1,
 
 
 GHashTable *static_strings = NULL;
-static GStaticMutex static_strings_mutex = G_STATIC_MUTEX_INIT;
+static GMutex static_strings_mutex;
 
 
 const char *
@@ -1175,7 +1176,7 @@ get_static_string (const char *s)
 	if (s == NULL)
 		return NULL;
 
-	g_static_mutex_lock (&static_strings_mutex);
+	g_mutex_lock (&static_strings_mutex);
 
 	if (static_strings == NULL)
 		static_strings = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
@@ -1187,7 +1188,7 @@ get_static_string (const char *s)
 				     GINT_TO_POINTER (1));
 	}
 
-	g_static_mutex_unlock (&static_strings_mutex);
+	g_mutex_unlock (&static_strings_mutex);
 
 	return result;
 }
@@ -2183,6 +2184,28 @@ _g_file_get_icon (GFile *file)
 	return icon;
 }
 
+GIcon *
+_g_file_get_symbolic_icon (GFile *file)
+{
+	GIcon     *icon = NULL;
+	GFileInfo *file_info;
+
+	file_info = g_file_query_info (file,
+				       G_FILE_ATTRIBUTE_STANDARD_SYMBOLIC_ICON,
+				       G_FILE_QUERY_INFO_NONE,
+				       NULL,
+				       NULL);
+	if (file_info != NULL) {
+		icon = (GIcon*) g_object_ref (g_file_info_get_attribute_object (file_info, G_FILE_ATTRIBUTE_STANDARD_SYMBOLIC_ICON));
+		g_object_unref (file_info);
+	}
+
+	if (icon == NULL)
+		icon = g_themed_icon_new ("text-x-generic-symbolic");
+
+	return icon;
+}
+
 
 GList *
 _g_file_list_dup (GList *l)
@@ -2798,6 +2821,23 @@ _g_file_info_swap_attributes (GFileInfo  *info,
 }
 
 
+const char *
+_g_content_type_guess_from_name (const char *filename)
+{
+	if (filename == NULL)
+		return NULL;
+
+#if WEBP_IS_UNKNOWN_TO_GLIB
+	const char *ext;
+
+	ext = _g_uri_get_file_extension (filename);
+	if (g_strcmp0 (ext, ".webp") == 0)
+		return "image/webp";
+#endif
+	return g_content_type_guess (filename, NULL, 0, NULL);
+}
+
+
 gboolean
 _g_content_type_is_a (const char *type,
 		      const char *supertype)
@@ -2809,6 +2849,115 @@ _g_content_type_is_a (const char *type,
 }
 
 
+/* -- _g_content_type_get_from_stream -- */
+
+
+static const char *
+get_mime_type_from_magic_numbers (guchar *buffer,
+				  gsize   buffer_size)
+{
+#if ENABLE_MAGIC
+
+	static magic_t magic = NULL;
+
+	if (magic == NULL) {
+		magic = magic_open (MAGIC_MIME_TYPE);
+		if (magic != NULL)
+			magic_load (magic, NULL);
+		else
+			g_warning ("unable to open magic database");
+	}
+
+	if (magic != NULL) {
+		const char * mime_type;
+
+		mime_type = magic_buffer (magic, buffer, buffer_size);
+		if (mime_type)
+			return mime_type;
+
+		g_warning ("unable to detect filetype from magic: %s", magic_error (magic));
+	}
+
+#else
+
+	static const struct magic {
+		const unsigned int off;
+		const unsigned int len;
+		const char * const id;
+		const char * const mime_type;
+	}
+	magic_ids [] = {
+		/* some magic ids taken from magic/Magdir/archive from the file-4.21 tarball */
+		{ 0,  8, "\x89PNG\x0d\x0a\x1a\x0a",		"image/png" },
+		{ 0,  4, "MM\x00\x2a",				"image/tiff" },
+		{ 0,  4, "II\x2a\x00",				"image/tiff" },
+		{ 0,  4, "GIF8",				"image/gif" },
+		{ 0,  3, "\xff\xd8\xff",			"image/jpeg" },
+		{ 8,  7, "WEBPVP8",				"image/webp" }
+	};
+
+	int  i;
+
+	for (i = 0; i < G_N_ELEMENTS (magic_ids); i++) {
+		const struct magic * const magic = &magic_ids[i];
+
+		if ((magic->off + magic->len) > buffer_size)
+			continue;
+
+		if (! memcmp (buffer + magic->off, magic->id, magic->len))
+			return magic->mime_type;
+	}
+
+#endif
+
+	return NULL;
+}
+
+
+const char *
+_g_content_type_get_from_stream (GInputStream  *istream,
+				 GFile         *file,
+				 GCancellable  *cancellable,
+				 GError       **error)
+{
+	guchar      buffer[BUFFER_SIZE_FOR_SNIFFING];
+	gssize      n = 0;
+	gboolean    result_uncertain = FALSE;
+	const char *content_type;
+
+	n = g_input_stream_read (istream,
+				 buffer,
+				 BUFFER_SIZE_FOR_SNIFFING,
+				 cancellable,
+				 error);
+	if (n < 0)
+		return NULL;
+
+	content_type = get_mime_type_from_magic_numbers (buffer, n);
+	if (content_type == NULL)
+		content_type = g_content_type_guess (NULL, buffer, n, &result_uncertain);
+	if (result_uncertain)
+		content_type = NULL;
+
+	if (((content_type == NULL)
+	     || (strcmp (content_type, "application/xml") == 0)
+	     || (strcmp (content_type, "image/tiff") == 0))
+	    && (file != NULL))
+	{
+		char *filename;
+
+		filename = g_file_get_basename (file);
+		content_type = _g_content_type_guess_from_name (filename);
+
+		g_free (filename);
+	}
+
+	g_seekable_seek (G_SEEKABLE (istream), 0, G_SEEK_SET, cancellable, NULL);
+
+	return content_type;
+}
+
+
 gboolean
 _g_mime_type_is_image (const char *mime_type)
 {
@@ -2963,3 +3112,11 @@ _g_list_prepend_link (GList *list,
 	if (list != NULL) list->prev = link;
 	return link;
 }
+
+
+void
+_g_error_free (GError *error)
+{
+	if (error != NULL)
+		g_error_free (error);
+}
