@@ -31,13 +31,15 @@ G_BEGIN_DECLS
 
 /* Math */
 
-#define GDOUBLE_ROUND_TO_INT(x)  ((int) floor ((x) + 0.5))
-#define SQR(x)                   ((x) * (x))
+#define GDOUBLE_ROUND_TO_INT(x)	((int) floor ((x) + 0.5))
+#define SQR(x)			((x) * (x))
+#define MIN3(x,y,z)		((y) <= (z) ? MIN ((x), (y)) : MIN ((x), (z)))
+#define MAX3(x,y,z)		((y) >= (z) ? MAX ((x), (y)) : MAX ((x), (z)))
 
 /* GFile attributes */
 
 #define GFILE_NAME_TYPE_ATTRIBUTES "standard::name,standard::type"
-#define GFILE_DISPLAY_ATTRIBUTES "standard::display-name,standard::icon"
+#define GFILE_DISPLAY_ATTRIBUTES "standard::display-name,standard::icon,standard::symbolic-icon"
 #define GFILE_BASIC_ATTRIBUTES GFILE_DISPLAY_ATTRIBUTES ",standard::name,standard::type"
 
 #define DEFINE_STANDARD_ATTRIBUTES(a) ( \
@@ -48,6 +50,7 @@ G_BEGIN_DECLS
 	"standard::display-name," \
 	"standard::edit-name," \
 	"standard::icon," \
+	"standard::symbolic-icon," \
 	"standard::size," \
 	"thumbnail::path" \
 	"time::created," \
@@ -74,6 +77,7 @@ G_BEGIN_DECLS
 #endif
 
 #define DEFAULT_STRFTIME_FORMAT "%Y-%m-%d--%H.%M.%S"
+#define DEF_ACTION_CALLBACK(x) void x (GSimpleAction *action, GVariant *parameter, gpointer user_data);
 
 /* signals */
 
@@ -248,7 +252,7 @@ gboolean        _g_uri_parent_of_uri             (const char *dir,
 char *          _g_uri_get_parent                (const char *uri);
 char *          _g_uri_remove_extension          (const char *uri);
 char *          _g_build_uri                     (const char *base,
-						  ...);
+						  ...) G_GNUC_NULL_TERMINATED;
 char *          _g_uri_get_scheme                (const char *uri);
 const char *    _g_uri_remove_host               (const char *uri);
 char *          _g_uri_get_host                  (const char *uri);
@@ -270,8 +274,9 @@ GFile *         _g_file_get_destination          (GFile      *source,
 						  GFile      *destination_folder);
 GFile *         _g_file_get_duplicated           (GFile      *file);
 GFile *         _g_file_get_child                (GFile      *file,
-						  ...);
+						  ...) G_GNUC_NULL_TERMINATED;
 GIcon *         _g_file_get_icon                 (GFile      *file);
+GIcon *		_g_file_get_symbolic_icon	 (GFile      *file);
 GList *         _g_file_list_dup                 (GList      *l);
 void            _g_file_list_free                (GList      *l);
 GList *         _g_file_list_new_from_uri_list   (GList      *uris);
