@@ -55,7 +55,8 @@ typedef enum {
 	GTH_FILE_LIST_OP_TYPE_SET_SORT_FUNC,
 	GTH_FILE_LIST_OP_TYPE_ENABLE_THUMBS,
 	GTH_FILE_LIST_OP_TYPE_RENAME_FILE,
-	GTH_FILE_LIST_OP_TYPE_MAKE_FILE_VISIBLE
+	GTH_FILE_LIST_OP_TYPE_MAKE_FILE_VISIBLE,
+	GTH_FILE_LIST_OP_TYPE_RESTORE_STATE
 } GthFileListOpType;
 
 
@@ -72,6 +73,8 @@ typedef struct {
 	GFile               *file;
 	GthFileData         *file_data;
 	int                  position;
+	GList               *selected;
+	double               vscroll;
 } GthFileListOp;
 
 
@@ -83,7 +86,7 @@ typedef struct {
 				  * image. */
 	guint thumb_created : 1; /* Whether a thumb has been
 				  * created for this image. */
-	GdkPixbuf *pixbuf;
+	cairo_surface_t *image;
 } ThumbData;
 
 
@@ -125,7 +128,7 @@ typedef struct {
 struct _GthFileListPrivateData
 {
 	GSettings        *settings;
-	GthFileListType   type;
+	GthFileListMode   type;
 	GtkAdjustment    *vadj;
 	GtkWidget        *notebook;
 	GtkWidget        *view;
@@ -198,6 +201,9 @@ gth_file_list_op_free (GthFileListOp *op)
 	case GTH_FILE_LIST_OP_TYPE_MAKE_FILE_VISIBLE:
 		g_object_unref (op->file);
 		break;
+	case GTH_FILE_LIST_OP_TYPE_RESTORE_STATE:
+		g_list_free_full (op->selected, (GDestroyNotify) gtk_tree_path_free);
+		break;
 	default:
 		break;
 	}
@@ -322,7 +328,7 @@ gth_file_list_get_preferred_width (GtkWidget *widget,
 		*natural_width += border * 2;
 
 	vscrollbar = gtk_scrolled_window_get_vscrollbar (GTK_SCROLLED_WINDOW (file_list->priv->scrolled_window));
-	if (gtk_widget_get_visible (vscrollbar) || (file_list->priv->type == GTH_FILE_LIST_TYPE_V_SIDEBAR)) {
+	if (gtk_widget_get_visible (vscrollbar) || (file_list->priv->type == GTH_FILE_LIST_MODE_V_SIDEBAR)) {
 		int vscrollbar_minimum_width;
 		int vscrollbar_natural_width;
 		int scrollbar_spacing;
@@ -409,7 +415,7 @@ thumb_data_unref (ThumbData *data)
 	data->ref--;
 	if (data->ref > 0)
 		return;
-	_g_object_unref (data->pixbuf);
+	cairo_surface_destroy (data->image);
 	g_free (data);
 }
 
@@ -566,11 +572,11 @@ _gth_file_list_update_orientation (GthFileList *file_list)
 	hscrollbar_policy = GTK_POLICY_AUTOMATIC;
 	vscrollbar_policy = GTK_POLICY_AUTOMATIC;
 
-	if (file_list->priv->type == GTH_FILE_LIST_TYPE_V_SIDEBAR) {
+	if (file_list->priv->type == GTH_FILE_LIST_MODE_V_SIDEBAR) {
 		gtk_orientable_set_orientation (GTK_ORIENTABLE (file_list), GTK_ORIENTATION_VERTICAL);
 		vscrollbar_policy = GTK_POLICY_ALWAYS;
 	}
-	else if (file_list->priv->type == GTH_FILE_LIST_TYPE_H_SIDEBAR)
+	else if (file_list->priv->type == GTH_FILE_LIST_MODE_H_SIDEBAR)
 		gtk_orientable_set_orientation (GTK_ORIENTABLE (file_list), GTK_ORIENTATION_HORIZONTAL);
 
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (file_list->priv->scrolled_window),
@@ -603,7 +609,7 @@ file_store_rows_reordered_cb (GtkTreeModel *tree_model,
 static void
 gth_file_list_construct (GthFileList     *file_list,
 			 GtkWidget       *file_view,
-			 GthFileListType  list_type,
+			 GthFileListMode  list_type,
 			 gboolean         enable_drag_drop)
 {
 	GthFileStore *model;
@@ -672,7 +678,7 @@ gth_file_list_construct (GthFileList     *file_list,
 				  file_list);
 	}
 
-	gth_file_list_set_type (file_list, list_type);
+	gth_file_list_set_mode (file_list, list_type);
 
 	/* pack the widgets together */
 
@@ -694,7 +700,7 @@ gth_file_list_construct (GthFileList     *file_list,
 
 GtkWidget *
 gth_file_list_new (GtkWidget       *file_view,
-		   GthFileListType  list_type,
+		   GthFileListMode  list_type,
 		   gboolean         enable_drag_drop)
 {
 	GtkWidget *widget;
@@ -707,16 +713,16 @@ gth_file_list_new (GtkWidget       *file_view,
 
 
 void
-gth_file_list_set_type (GthFileList     *file_list,
-			GthFileListType  list_type)
+gth_file_list_set_mode (GthFileList     *file_list,
+			GthFileListMode  list_type)
 {
 	g_return_if_fail (GTH_IS_FILE_LIST (file_list));
 
 	file_list->priv->type = list_type;
 
-	if ((file_list->priv->type == GTH_FILE_LIST_TYPE_SELECTOR) || (file_list->priv->type == GTH_FILE_LIST_TYPE_NO_SELECTION))
+	if ((file_list->priv->type == GTH_FILE_LIST_MODE_SELECTOR) || (file_list->priv->type == GTH_FILE_LIST_MODE_NO_SELECTION))
 		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_NONE);
-	else if ((file_list->priv->type == GTH_FILE_LIST_TYPE_H_SIDEBAR) || (file_list->priv->type == GTH_FILE_LIST_TYPE_V_SIDEBAR))
+	else if ((file_list->priv->type == GTH_FILE_LIST_MODE_H_SIDEBAR) || (file_list->priv->type == GTH_FILE_LIST_MODE_V_SIDEBAR))
 		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_SINGLE);
 	else
 		gth_file_selection_set_selection_mode (GTH_FILE_SELECTION (file_list->priv->view), GTK_SELECTION_MULTIPLE);
@@ -725,6 +731,13 @@ gth_file_list_set_type (GthFileList     *file_list,
 }
 
 
+GthFileListMode
+gth_file_list_get_mode (GthFileList *file_list)
+{
+	return file_list->priv->type;
+}
+
+
 typedef struct {
 	GthFileList *file_list;
 	DataFunc     done_func;
@@ -875,11 +888,11 @@ gfl_add_files (GthFileList *file_list,
 
 	cache_base_uri = g_strconcat (get_home_uri (), "/.thumbnails", NULL);
 	for (scan = files; scan; scan = scan->next) {
-		GthFileData *file_data = scan->data;
-		char        *uri;
-		ThumbData   *thumb_data;
-		GIcon       *icon;
-		GdkPixbuf   *pixbuf = NULL;
+		GthFileData     *file_data = scan->data;
+		char            *uri;
+		ThumbData       *thumb_data;
+		GIcon           *icon;
+		cairo_surface_t *image = NULL;
 
 		if (g_file_info_get_file_type (file_data->info) != G_FILE_TYPE_REGULAR)
 			continue;
@@ -898,15 +911,14 @@ gfl_add_files (GthFileList *file_list,
 				     g_object_ref (file_data->file),
 				     thumb_data);
 
-		icon = g_file_info_get_icon (file_data->info);
-		pixbuf = gth_icon_cache_get_pixbuf (file_list->priv->icon_cache, icon);
+		icon = g_file_info_get_symbolic_icon (file_data->info);
+		image = gth_icon_cache_get_surface (file_list->priv->icon_cache, icon);
 		gth_file_store_queue_add (file_store,
 					  file_data,
-					  pixbuf,
+					  image,
 					  TRUE);
 
-		if (pixbuf != NULL)
-			g_object_unref (pixbuf);
+		cairo_surface_destroy (image);
 		g_free (uri);
 	}
 	g_free (cache_base_uri);
@@ -1226,10 +1238,10 @@ gfl_enable_thumbs (GthFileList *file_list,
 	file_store = (GthFileStore*) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
 	if (gth_file_store_get_first (file_store, &iter)) {
 		do {
-			GthFileData *file_data;
-			ThumbData   *thumb_data;
-			GIcon       *icon;
-			GdkPixbuf   *pixbuf;
+			GthFileData     *file_data;
+			ThumbData       *thumb_data;
+			GIcon           *icon;
+			cairo_surface_t *image;
 
 			file_data = gth_file_store_get_file (file_store, &iter);
 
@@ -1237,15 +1249,15 @@ gfl_enable_thumbs (GthFileList *file_list,
 			g_assert (thumb_data != NULL);
 			thumb_data->thumb_loaded = FALSE;
 
-			icon = g_file_info_get_icon (file_data->info);
-			pixbuf = gth_icon_cache_get_pixbuf (file_list->priv->icon_cache, icon);
+			icon = g_file_info_get_symbolic_icon (file_data->info);
+			image = gth_icon_cache_get_surface (file_list->priv->icon_cache, icon);
 			gth_file_store_queue_set (file_store,
 						  &iter,
-						  GTH_FILE_STORE_THUMBNAIL_COLUMN, pixbuf,
+						  GTH_FILE_STORE_THUMBNAIL_COLUMN, image,
 						  GTH_FILE_STORE_IS_ICON_COLUMN, TRUE,
 						  -1);
 
-			_g_object_unref (pixbuf);
+			cairo_surface_destroy (image);
 		}
 		while (gth_file_store_get_next (file_store, &iter));
 
@@ -1290,7 +1302,7 @@ gth_file_list_set_thumb_size (GthFileList *file_list,
 
 	gth_icon_cache_free (file_list->priv->icon_cache);
 	file_list->priv->icon_cache = gth_icon_cache_new (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (file_list))), size / 2);
-	gth_icon_cache_set_fallback (file_list->priv->icon_cache, g_themed_icon_new ("image-x-generic"));
+	gth_icon_cache_set_fallback (file_list->priv->icon_cache, g_themed_icon_new ("text-x-generic-symbolic"));
 
 	gth_file_view_set_thumbnail_size (GTH_FILE_VIEW (file_list->priv->view), file_list->priv->thumb_size);
 
@@ -1314,6 +1326,7 @@ gfl_make_file_visible (GthFileList *file_list,
 
 	pos = gth_file_store_get_pos ((GthFileStore *) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view)), file);
 	if (pos >= 0) {
+		gth_file_selection_unselect_all (GTH_FILE_SELECTION (file_list->priv->view));
 		gth_file_selection_select (GTH_FILE_SELECTION (file_list->priv->view), pos);
 		gth_file_view_set_cursor (GTH_FILE_VIEW (file_list->priv->view), pos);
 	}
@@ -1332,6 +1345,39 @@ gth_file_list_make_file_visible (GthFileList *file_list,
 }
 
 
+static void
+gfl_restore_state (GthFileList *file_list,
+		   GList       *selected,
+		   double       vscroll)
+{
+	GList *scan;
+
+	for (scan = selected; scan; scan = scan->next) {
+		GtkTreePath *path = scan->data;
+		int          pos;
+
+		pos = gtk_tree_path_get_indices (path)[0];
+		gth_file_selection_select (GTH_FILE_SELECTION (file_list->priv->view), pos);
+	}
+
+	gth_file_view_set_vscroll (GTH_FILE_VIEW (file_list->priv->view), vscroll);
+}
+
+
+void
+gth_file_list_restore_state (GthFileList *file_list,
+			     GList       *selected,
+			     double       vscroll)
+{
+	GthFileListOp *op;
+
+	op = gth_file_list_op_new (GTH_FILE_LIST_OP_TYPE_RESTORE_STATE);
+	op->selected = g_list_copy_deep (selected, (GCopyFunc) gtk_tree_path_copy, NULL);
+	op->vscroll = vscroll;
+	_gth_file_list_queue_op (file_list, op);
+}
+
+
 GtkWidget *
 gth_file_list_get_view (GthFileList *file_list)
 {
@@ -1429,12 +1475,12 @@ update_thumb_in_file_view (GthFileList *file_list,
 	if (thumb_data == NULL)
 		return;
 
-	if (thumb_data->pixbuf == NULL)
+	if (thumb_data->image == NULL)
 		return;
 
 	gth_file_store_queue_set (file_store,
 				  &iter,
-				  GTH_FILE_STORE_THUMBNAIL_COLUMN, thumb_data->pixbuf,
+				  GTH_FILE_STORE_THUMBNAIL_COLUMN, thumb_data->image,
 				  GTH_FILE_STORE_IS_ICON_COLUMN, FALSE,
 				  -1);
 	queue_flash_updates (file_list);
@@ -1446,25 +1492,25 @@ set_mime_type_icon (GthFileList *file_list,
 		    GthFileData *file_data,
 		    int          try_pos)
 {
-	GthFileStore *file_store;
-	GtkTreeIter   iter;
-	GIcon        *icon;
-	GdkPixbuf    *pixbuf;
+	GthFileStore    *file_store;
+	GtkTreeIter      iter;
+	GIcon           *icon;
+	cairo_surface_t *image;
 
 	file_store = (GthFileStore *) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
 	if (! get_file_data_iter_with_suggested_pos (file_store, file_data, try_pos, &iter))
 		return;
 
-	icon = g_file_info_get_icon (file_data->info);
-	pixbuf = gth_icon_cache_get_pixbuf (file_list->priv->icon_cache, icon);
+	icon = g_file_info_get_symbolic_icon (file_data->info);
+	image = gth_icon_cache_get_surface (file_list->priv->icon_cache, icon);
 	gth_file_store_queue_set (file_store,
 				  &iter,
-				  GTH_FILE_STORE_THUMBNAIL_COLUMN, pixbuf,
+				  GTH_FILE_STORE_THUMBNAIL_COLUMN, image,
 				  GTH_FILE_STORE_IS_ICON_COLUMN, TRUE,
 				  -1);
 	queue_flash_updates (file_list);
 
-	_g_object_unref (pixbuf);
+	cairo_surface_destroy (image);
 }
 
 
@@ -1473,37 +1519,37 @@ thumbnail_job_ready_cb (GObject      *source_object,
 		        GAsyncResult *result,
 		        gpointer      user_data)
 {
-	ThumbnailJob *job = user_data;
-	GthFileList  *file_list = job->file_list;
-	gboolean      success;
-	GdkPixbuf    *pixbuf = NULL;
-	GError       *error = NULL;
-	ThumbData    *thumb_data;
+	ThumbnailJob    *job = user_data;
+	GthFileList     *file_list = job->file_list;
+	gboolean         success;
+	cairo_surface_t *image = NULL;
+	GError          *error = NULL;
+	ThumbData       *thumb_data;
 
 	success = gth_thumb_loader_load_finish (GTH_THUMB_LOADER (source_object),
 						result,
-						&pixbuf,
+						&image,
 						&error);
 	job->started = FALSE;
 
 	if ((! success && g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 	    || file_list->priv->cancelling)
 	{
-		_g_object_unref (pixbuf);
+		cairo_surface_destroy (image);
 		thumbnail_job_free (job);
 		return;
 	}
 
 	thumb_data = g_hash_table_lookup (file_list->priv->thumb_data, job->file_data->file);
 	if (thumb_data == NULL) {
-		_g_object_unref (pixbuf);
+		cairo_surface_destroy (image);
 		thumbnail_job_free (job);
 		_gth_file_list_update_next_thumb (file_list);
 		return;
 	}
 
-	_g_object_unref (thumb_data->pixbuf);
-	thumb_data->pixbuf = NULL;
+	cairo_surface_destroy (thumb_data->image);
+	thumb_data->image = NULL;
 
 	if (! success) {
 		thumb_data->thumb_created = FALSE;
@@ -1514,7 +1560,7 @@ thumbnail_job_ready_cb (GObject      *source_object,
 		thumb_data->error = TRUE;
 	}
 	else {
-		thumb_data->pixbuf = g_object_ref (pixbuf);
+		thumb_data->image = cairo_surface_reference (image);
 		thumb_data->thumb_created = TRUE;
 		thumb_data->error = FALSE;
 		if (job->update_in_view) {
@@ -1523,7 +1569,7 @@ thumbnail_job_ready_cb (GObject      *source_object,
 		}
 	}
 
-	_g_object_unref (pixbuf);
+	cairo_surface_destroy (image);
 	thumbnail_job_free (job);
 
 	_gth_file_list_update_next_thumb (file_list);
@@ -1535,25 +1581,25 @@ set_loading_icon (GthFileList *file_list,
 		  GthFileData *file_data,
 		  int          try_pos)
 {
-	GthFileStore *file_store;
-	GtkTreeIter   iter;
-	GIcon        *icon;
-	GdkPixbuf    *pixbuf;
+	GthFileStore    *file_store;
+	GtkTreeIter      iter;
+	GIcon           *icon;
+	cairo_surface_t *image;
 
 	file_store = (GthFileStore *) gth_file_view_get_model (GTH_FILE_VIEW (file_list->priv->view));
 	if (! get_file_data_iter_with_suggested_pos (file_store, file_data, try_pos, &iter))
 		return;
 
-	icon = g_themed_icon_new ("image-loading");
-	pixbuf = gth_icon_cache_get_pixbuf (file_list->priv->icon_cache, icon);
+	icon = g_themed_icon_new ("content-loading-symbolic");
+	image = gth_icon_cache_get_surface (file_list->priv->icon_cache, icon);
 	gth_file_store_queue_set (file_store,
 				  &iter,
-				  GTH_FILE_STORE_THUMBNAIL_COLUMN, pixbuf,
+				  GTH_FILE_STORE_THUMBNAIL_COLUMN, image,
 				  GTH_FILE_STORE_IS_ICON_COLUMN, TRUE,
 				  -1);
 	queue_flash_updates (file_list);
 
-	_g_object_unref (pixbuf);
+	cairo_surface_destroy (image);
 	g_object_unref (icon);
 }
 
@@ -1935,6 +1981,9 @@ _gth_file_list_exec_next_op (GthFileList *file_list)
 	case GTH_FILE_LIST_OP_TYPE_MAKE_FILE_VISIBLE:
 		gfl_make_file_visible (file_list, op->file);
 		break;
+	case GTH_FILE_LIST_OP_TYPE_RESTORE_STATE:
+		gfl_restore_state (file_list, op->selected, op->vscroll);
+		break;
 	default:
 		exec_next_op = FALSE;
 		break;
