@@ -25,9 +25,11 @@
 #include "gth-file-properties.h"
 #include "gth-main.h"
 #include "gth-multipage.h"
+#include "gth-preferences.h"
 #include "gth-sidebar.h"
 #include "gth-string-list.h"
 #include "gth-time.h"
+#include "gtk-utils.h"
 
 
 #define GTH_FILE_PROPERTIES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GTH_TYPE_FILE_PROPERTIES, GthFilePropertiesPrivate))
@@ -36,9 +38,11 @@
 #define COMMENT_DEFAULT_HEIGHT 100
 #define CATEGORY_SIZE 1000
 #define MAX_ATTRIBUTE_LENGTH 128
+#define GTH_STYLE_CLASS_COMMENT "comment"
 
 
 enum {
+	SCALE_SET_COLUMN,
 	WEIGHT_COLUMN,
 	ID_COLUMN,
 	DISPLAY_NAME_COLUMN,
@@ -57,6 +61,9 @@ struct _GthFilePropertiesPrivate {
 	GtkWidget     *comment_win;
 	GtkListStore  *tree_model;
 	GtkWidget     *popup_menu;
+	GtkWidget     *details_button;
+	gboolean       show_details;
+	GthFileData   *last_file_data;
 };
 
 
@@ -97,7 +104,7 @@ get_comment (GthFileData *file_data)
 }
 
 
-void
+static void
 gth_file_properties_real_set_file (GthPropertyView *base,
 		 		   GthFileData     *file_data)
 {
@@ -107,8 +114,16 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 	GList             *scan;
 	GtkTextBuffer     *text_buffer;
 	char              *comment;
+	gboolean           details_available;
 
 	self = GTH_FILE_PROPERTIES (base);
+
+	if (file_data != self->priv->last_file_data) {
+		_g_object_unref (self->priv->last_file_data);
+		self->priv->last_file_data = gth_file_data_dup (file_data);
+	}
+
+	gtk_widget_set_sensitive (self->priv->details_button, FALSE);
 	gtk_list_store_clear (self->priv->tree_model);
 
 	if (file_data == NULL) {
@@ -118,6 +133,7 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (self->priv->tree_model), GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID, 0);
 
+	details_available = FALSE;
 	category_hash = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free, NULL);
 	metadata_info = gth_main_get_all_metadata_info ();
 	for (scan = metadata_info; scan; scan = scan->next) {
@@ -134,6 +150,24 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 		if ((value == NULL) || (*value == '\0'))
 			continue;
 
+		if (info->id != NULL) {
+			if (g_str_has_prefix (info->id, "Exif")) {
+				details_available = TRUE;
+				if (! self->priv->show_details)
+					continue;
+			}
+			if (g_str_has_prefix (info->id, "Iptc")) {
+				details_available = TRUE;
+				if (! self->priv->show_details)
+					continue;
+			}
+			if (g_str_has_prefix (info->id, "Xmp")) {
+				details_available = TRUE;
+				if (! self->priv->show_details)
+					continue;
+			}
+		}
+
 		if (value != NULL) {
 			char *tmp_value;
 
@@ -150,9 +184,10 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 			category = gth_main_get_metadata_category (info->category);
 			gtk_list_store_append (self->priv->tree_model, &iter);
 			gtk_list_store_set (self->priv->tree_model, &iter,
+					    SCALE_SET_COLUMN, FALSE,
 					    WEIGHT_COLUMN, PANGO_WEIGHT_BOLD,
 					    ID_COLUMN, category->id,
-					    DISPLAY_NAME_COLUMN, category->display_name,
+					    DISPLAY_NAME_COLUMN, _(category->display_name),
 					    POS_COLUMN, category->sort_order * CATEGORY_SIZE,
 					    -1);
 			g_hash_table_insert (category_hash, g_strdup (info->category), category);
@@ -161,6 +196,7 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 		gtk_list_store_append (self->priv->tree_model, &iter);
 		gtk_list_store_set (self->priv->tree_model,
 				    &iter,
+				    SCALE_SET_COLUMN, TRUE,
 				    WEIGHT_COLUMN, PANGO_WEIGHT_NORMAL,
 				    ID_COLUMN, info->id,
 				    DISPLAY_NAME_COLUMN, _(info->display_name),
@@ -176,6 +212,7 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (self->priv->tree_model), POS_COLUMN, GTK_SORT_ASCENDING);
 	gtk_tree_view_expand_all (GTK_TREE_VIEW (self->priv->tree_view));
+	gtk_widget_set_sensitive (self->priv->details_button, details_available);
 
 	g_hash_table_destroy (category_hash);
 
@@ -203,17 +240,17 @@ gth_file_properties_real_set_file (GthPropertyView *base,
 }
 
 
-const char *
+static const char *
 gth_file_properties_real_get_name (GthMultipageChild *self)
 {
 	return _("Properties");
 }
 
 
-const char *
+static const char *
 gth_file_properties_real_get_icon (GthMultipageChild *self)
 {
-	return GTK_STOCK_PROPERTIES;
+	return "document-properties-symbolic";
 }
 
 
@@ -226,6 +263,7 @@ gth_file_properties_finalize (GObject *base)
 
 	if (self->priv->popup_menu != NULL)
 		gtk_widget_destroy (self->priv->popup_menu);
+	_g_object_unref (self->priv->last_file_data);
 
 	G_OBJECT_CLASS (gth_file_properties_parent_class)->finalize (base);
 }
@@ -317,16 +355,41 @@ tree_view_popup_menu_cb (GtkWidget *widget,
 }
 
 
+static void
+details_button_toggled_cb (GtkToggleButton *button,
+			   gpointer         user_data)
+{
+	GthFileProperties *self = user_data;
+	GSettings         *settings;
+
+	self->priv->show_details = gtk_toggle_button_get_active (button);
+	gth_file_properties_real_set_file (GTH_PROPERTY_VIEW (self), self->priv->last_file_data);
+
+	settings = g_settings_new (GTHUMB_BROWSER_SCHEMA);
+	g_settings_set_boolean (settings, PREF_BROWSER_PROPERTIES_DETAILS, self->priv->show_details);
+	g_object_unref (settings);
+}
+
+
 static void
 gth_file_properties_init (GthFileProperties *self)
 {
+	GSettings         *settings;
 	GtkWidget         *vpaned;
+	GtkWidget         *properties_box;
 	GtkWidget         *scrolled_win;
+	GtkWidget         *button_box;
 	GtkWidget         *menu_item;
 	GtkCellRenderer   *renderer;
 	GtkTreeViewColumn *column;
 
+	settings = g_settings_new (GTHUMB_BROWSER_SCHEMA);
+
 	self->priv = GTH_FILE_PROPERTIES_GET_PRIVATE (self);
+	self->priv->show_details = g_settings_get_boolean (settings, PREF_BROWSER_PROPERTIES_DETAILS);
+	self->priv->last_file_data = NULL;
+
+	g_object_unref (settings);
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (self), GTK_ORIENTATION_VERTICAL);
 	gtk_box_set_spacing (GTK_BOX (self), 6);
@@ -335,18 +398,36 @@ gth_file_properties_init (GthFileProperties *self)
 	gtk_widget_show (vpaned);
 	gtk_box_pack_start (GTK_BOX (self), vpaned, TRUE, TRUE, 0);
 
+	properties_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 5);
+	gtk_widget_show (properties_box);
+	gtk_paned_pack1 (GTK_PANED (vpaned), properties_box, TRUE, FALSE);
+
 	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_win), GTK_SHADOW_ETCHED_IN);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_win), GTK_SHADOW_NONE);
 	gtk_widget_show (scrolled_win);
 	gtk_widget_set_size_request (scrolled_win, -1, MIN_HEIGHT);
-	gtk_paned_pack1 (GTK_PANED (vpaned), scrolled_win, TRUE, FALSE);
+	gtk_box_pack_start (GTK_BOX (properties_box), scrolled_win, TRUE, TRUE, 0);
+
+	button_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
+	gtk_widget_show (button_box);
+	gtk_box_pack_start (GTK_BOX (properties_box), button_box, FALSE, FALSE, 2);
+
+	self->priv->details_button = gtk_toggle_button_new_with_label (_("Details"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->priv->details_button), self->priv->show_details);
+	gtk_widget_show (self->priv->details_button);
+	gtk_box_pack_start (GTK_BOX (button_box), self->priv->details_button, FALSE, FALSE, 0);
+	g_signal_connect (self->priv->details_button,
+			  "toggled",
+			  G_CALLBACK (details_button_toggled_cb),
+			  self);
 
 	self->priv->tree_view = gtk_tree_view_new ();
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (self->priv->tree_view), FALSE);
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (self->priv->tree_view), TRUE);
 	gtk_tree_view_set_tooltip_column (GTK_TREE_VIEW (self->priv->tree_view), TOOLTIP_COLUMN);
 	self->priv->tree_model = gtk_list_store_new (NUM_COLUMNS,
+						     G_TYPE_BOOLEAN,
 						     PANGO_TYPE_WEIGHT,
 						     G_TYPE_STRING,
 						     G_TYPE_STRING,
@@ -364,7 +445,7 @@ gth_file_properties_init (GthFileProperties *self)
 	/* popup menu */
 
 	self->priv->popup_menu = gtk_menu_new ();
-	menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_COPY, NULL);
+	menu_item = gtk_menu_item_new_with_label (_GTK_LABEL_COPY);
 	gtk_widget_show (menu_item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (self->priv->popup_menu),
 			       menu_item);
@@ -387,6 +468,7 @@ gth_file_properties_init (GthFileProperties *self)
 	column = gtk_tree_view_column_new_with_attributes ("",
 							   renderer,
 							   "text", DISPLAY_NAME_COLUMN,
+							   "scale-set", SCALE_SET_COLUMN,
 							   "weight", WEIGHT_COLUMN,
 							   NULL);
 	g_object_set (renderer,
@@ -395,7 +477,7 @@ gth_file_properties_init (GthFileProperties *self)
 		      NULL);
 
 	gtk_tree_view_column_set_expand (column, TRUE);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (self->priv->tree_view),
 				     column);
 
@@ -412,7 +494,7 @@ gth_file_properties_init (GthFileProperties *self)
 		      NULL);
 
 	gtk_tree_view_column_set_expand (column, TRUE);
-	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (self->priv->tree_view),
 				     column);
 
@@ -424,12 +506,15 @@ gth_file_properties_init (GthFileProperties *self)
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (self->priv->comment_win), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (self->priv->comment_win), GTK_SHADOW_ETCHED_IN);
 	gtk_widget_set_size_request (self->priv->comment_win, -1, COMMENT_DEFAULT_HEIGHT);
+	gtk_style_context_add_class (gtk_widget_get_style_context (self->priv->comment_win), GTH_STYLE_CLASS_COMMENT);
 	gtk_paned_pack2 (GTK_PANED (vpaned), self->priv->comment_win, FALSE, FALSE);
 
 	self->priv->comment_view = gtk_text_view_new ();
+	gtk_style_context_add_class (gtk_widget_get_style_context (self->priv->comment_view), GTH_STYLE_CLASS_COMMENT);
 	gtk_text_view_set_editable (GTK_TEXT_VIEW (self->priv->comment_view), FALSE);
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (self->priv->comment_view), GTK_WRAP_WORD);
 	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (self->priv->comment_view), TRUE);
+
 	gtk_widget_show (self->priv->comment_view);
 	gtk_container_add (GTK_CONTAINER (self->priv->comment_win), self->priv->comment_view);
 }
