@@ -61,7 +61,7 @@ G_DEFINE_TYPE (GthFileSourceVfs, gth_file_source_vfs, GTH_TYPE_FILE_SOURCE)
 
 
 static GList *
-gth_file_source_vfs_add_special_dir (GList         *list, 
+gth_file_source_vfs_add_special_dir (GList         *list,
 				     GthFileSource *file_source,
 				     GUserDirectory special_dir)
 {
@@ -89,8 +89,7 @@ gth_file_source_vfs_add_special_dir (GList         *list,
 static GList *
 gth_file_source_vfs_add_uri (GList         *list,
 			     GthFileSource *file_source,
-			     const char    *uri,
-			     char          *display_name)
+			     const char    *uri)
 {
 	GFile *file;
 
@@ -99,8 +98,8 @@ gth_file_source_vfs_add_uri (GList         *list,
 		GFileInfo *info;
 
 		info = gth_file_source_get_file_info (file_source, file, GFILE_BASIC_ATTRIBUTES ",access::*");
-		g_file_info_set_display_name (info, display_name);
 		list = g_list_append (list, gth_file_data_new (file, info));
+
 		g_object_unref (info);
 	}
 	g_object_unref (file);
@@ -112,18 +111,18 @@ gth_file_source_vfs_add_uri (GList         *list,
 static GList *
 gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
 {
-	GList       *list;
-	GList       *mounts;
-	GList       *scan;
+	GList *list;
+	GList *mounts;
+	GList *scan;
 
 	list = NULL;
 
-	list = gth_file_source_vfs_add_uri (list, file_source, get_home_uri (), _("Home Folder"));
+	list = gth_file_source_vfs_add_uri (list, file_source, get_home_uri ());
 	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_PICTURES);
 	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_VIDEOS);
 	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_DESKTOP);
-	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_DOCUMENTS);
-        list = gth_file_source_vfs_add_uri (list, file_source, "file:///", _("File System"));
+	/*list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_DOCUMENTS);*/
+	list = gth_file_source_vfs_add_uri (list, file_source, "file:///");
 
 	mounts = g_volume_monitor_get_mounts (g_volume_monitor_get ());
 	for (scan = mounts; scan; scan = scan->next) {
@@ -146,7 +145,7 @@ gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
 
 		info = g_file_query_info (file, GFILE_BASIC_ATTRIBUTES ",access::*", G_FILE_QUERY_INFO_NONE, NULL, NULL);
 
-		icon = g_mount_get_icon (mount);
+		icon = g_mount_get_symbolic_icon (mount);
 		name = g_mount_get_name (mount);
 
 		drive = g_mount_get_drive (mount);
@@ -163,7 +162,7 @@ gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
 			g_free (drive_name);
 		}
 
-		g_file_info_set_icon (info, icon);
+		g_file_info_set_symbolic_icon (info, icon);
 		g_file_info_set_display_name (info, name);
 
 		list = g_list_append (list, gth_file_data_new (file, info));
@@ -199,6 +198,7 @@ gth_file_source_vfs_get_file_info (GthFileSource *file_source,
 {
 	GFile     *gio_file;
 	GFileInfo *file_info;
+	char      *uri;
 
 	gio_file = gth_file_source_to_gio_file (file_source, file);
 	file_info = g_file_query_info (gio_file,
@@ -207,6 +207,19 @@ gth_file_source_vfs_get_file_info (GthFileSource *file_source,
 				       NULL,
 				       NULL);
 
+	uri = g_file_get_uri (gio_file);
+	if (g_strcmp0 (uri, "file:///") == 0) {
+		GIcon *icon;
+
+		g_file_info_set_display_name (file_info, _("Computer"));
+		icon = g_themed_icon_new ("drive-harddisk-symbolic");
+		g_file_info_set_symbolic_icon (file_info, icon);
+
+		g_object_unref (icon);
+	}
+	else if (g_strcmp0 (uri, get_home_uri ()) == 0)
+		g_file_info_set_display_name (file_info, _("Home Folder"));
+
 	g_object_unref (gio_file);
 
 	return file_info;
@@ -393,7 +406,7 @@ mount_monitor_mountpoints_changed_cb (GVolumeMonitor *volume_monitor,
 				      GMount         *mount,
 				      gpointer        user_data)
 {
-	call_when_idle ((DataFunc) gth_monitor_file_entry_points_changed, gth_main_get_default_monitor ());
+	call_when_idle ((DataFunc) gth_monitor_entry_points_changed, gth_main_get_default_monitor ());
 }
 
 
@@ -737,8 +750,8 @@ trash_files (GtkWindow *window,
 				d = _gtk_yesno_dialog_new (window,
 							   GTK_DIALOG_MODAL,
 							   _("The files cannot be moved to the Trash. Do you want to delete them permanently?"),
-							   GTK_STOCK_CANCEL,
-							   GTK_STOCK_DELETE);
+							   _GTK_LABEL_CANCEL,
+							   _GTK_LABEL_DELETE);
 				g_signal_connect (d,
 						  "response",
 						  G_CALLBACK (delete_permanently_response_cb),
@@ -806,10 +819,10 @@ gth_file_mananger_trash_files (GtkWindow *window,
 
 		d = _gtk_message_dialog_new (window,
 					     GTK_DIALOG_MODAL,
-					     GTK_STOCK_DIALOG_QUESTION,
+					     _GTK_ICON_NAME_DIALOG_QUESTION,
 					     prompt,
 					     NULL,
-					     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					     _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
 					     _("Mo_ve to Trash"), GTK_RESPONSE_YES,
 					     NULL);
 		g_signal_connect (d, "response",
@@ -849,11 +862,11 @@ gth_file_mananger_delete_files (GtkWindow *window,
 
 	d = _gtk_message_dialog_new (window,
 				     GTK_DIALOG_MODAL,
-				     GTK_STOCK_DIALOG_QUESTION,
+				     _GTK_ICON_NAME_DIALOG_QUESTION,
 				     prompt,
 				     _("If you delete a file, it will be permanently lost."),
-				     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				     GTK_STOCK_DELETE, GTK_RESPONSE_YES,
+				     _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL,
+				     _GTK_LABEL_DELETE, GTK_RESPONSE_YES,
 				     NULL);
 	g_signal_connect (d, "response", G_CALLBACK (delete_permanently_response_cb), file_list);
 	gtk_widget_show (d);
