@@ -34,11 +34,11 @@ static guint gth_file_tool_signals[LAST_SIGNAL] = { 0 };
 
 
 struct _GthFileToolPrivate {
-	GtkWidget  *window;
-	const char *icon_name;
-	const char *button_text;
-	const char *options_title;
-	gboolean    separator;
+	GtkWidget		*window;
+	const char		*icon_name;
+	const char		*options_title;
+	GthToolboxSection	 section;
+	gboolean		 cancelled;
 };
 
 
@@ -59,6 +59,13 @@ gth_file_tool_base_activate (GthFileTool *self)
 }
 
 
+static void
+gth_file_tool_base_cancel (GthFileTool *self)
+{
+	/* void*/
+}
+
+
 static GtkWidget *
 gth_file_tool_base_get_options (GthFileTool *self)
 {
@@ -96,6 +103,7 @@ gth_file_tool_class_init (GthFileToolClass *klass)
 
 	klass->update_sensitivity = gth_file_tool_base_update_sensitivity;
 	klass->activate = gth_file_tool_base_activate;
+	klass->cancel = gth_file_tool_base_cancel;
 	klass->get_options = gth_file_tool_base_get_options;
 	klass->destroy_options = gth_file_tool_base_destroy_options;
 
@@ -125,38 +133,35 @@ gth_file_tool_init (GthFileTool *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GTH_TYPE_FILE_TOOL, GthFileToolPrivate);
 	self->priv->icon_name = NULL;
-	self->priv->button_text = NULL;
 	self->priv->options_title = NULL;
+	self->priv->cancelled = FALSE;
+	self->priv->section = GTH_TOOLBOX_SECTION_COLORS;
 
-	gtk_button_set_relief (GTK_BUTTON (self), GTK_RELIEF_NONE);
+	/*gtk_button_set_relief (GTK_BUTTON (self), GTK_RELIEF_NONE);*/
 }
 
 
 void
-gth_file_tool_construct (GthFileTool *self,
-			 const char  *icon_name,
-			 const char  *button_text,
-			 const char  *options_title,
-			 gboolean     separator)
+gth_file_tool_construct (GthFileTool		*self,
+			 const char		*icon_name,
+			 const char		*options_title,
+			 GthToolboxSection       section)
 {
 	GtkWidget *hbox;
 	GtkWidget *icon;
-	GtkWidget *label;
 
 	self->priv->icon_name = icon_name;
-	self->priv->button_text = button_text;
 	self->priv->options_title = options_title;
-	self->priv->separator = separator;
+	self->priv->section = section;
 
-	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 
 	icon = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
 	gtk_widget_show (icon);
 	gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
 
-	label = gtk_label_new (button_text);
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_widget_set_tooltip_text (GTK_WIDGET (self), options_title);
 
 	gtk_widget_show (hbox);
 	gtk_container_add (GTK_CONTAINER (self), hbox);
@@ -185,10 +190,30 @@ gth_file_tool_get_icon_name (GthFileTool *self)
 void
 gth_file_tool_activate (GthFileTool *self)
 {
+	self->priv->cancelled = FALSE;
 	GTH_FILE_TOOL_GET_CLASS (self)->activate (self);
 }
 
 
+void
+gth_file_tool_cancel (GthFileTool *self)
+{
+	if (self->priv->cancelled)
+		return;
+
+	self->priv->cancelled = TRUE;
+	GTH_FILE_TOOL_GET_CLASS (self)->cancel (self);
+	/*gth_file_tool_hide_options (self); FIXME */
+}
+
+
+gboolean
+gth_file_tool_is_cancelled (GthFileTool *self)
+{
+	return self->priv->cancelled;
+}
+
+
 void
 gth_file_tool_update_sensitivity (GthFileTool *self)
 {
@@ -217,6 +242,13 @@ gth_file_tool_get_options (GthFileTool *self)
 }
 
 
+GthToolboxSection
+gth_file_tool_get_section (GthFileTool *self)
+{
+	return self->priv->section;
+}
+
+
 const char *
 gth_file_tool_get_options_title (GthFileTool *self)
 {
@@ -224,13 +256,6 @@ gth_file_tool_get_options_title (GthFileTool *self)
 }
 
 
-gboolean
-gth_file_tool_has_separator (GthFileTool *self)
-{
-	return self->priv->separator;
-}
-
-
 void
 gth_file_tool_destroy_options (GthFileTool *self)
 {
