@@ -36,6 +36,7 @@
 #define EMPTY_URI   "..."
 #define LOADING_URI "."
 #define PARENT_URI  ".."
+#define UPDATE_MONITORED_LOCATIONS_DELAY 500
 
 
 typedef enum {
@@ -71,11 +72,19 @@ enum {
 };
 
 
+typedef struct {
+	GHashTable *locations;
+	GList      *sources;
+	guint       update_id;
+} MonitorData;
+
+
 struct _GthFolderTreePrivate
 {
 	GFile            *root;
+	GHashTable       *entry_points;		/* An entry point is a root child */
+	gboolean          recalc_entry_points;
 	GtkTreeStore     *tree_store;
-	GthIconCache     *icon_cache;
 	GtkCellRenderer  *text_renderer;
 	GtkTreePath      *hover_path;
 
@@ -90,6 +99,10 @@ struct _GthFolderTreePrivate
 	gboolean         drag_started : 1;    /* Whether the drag has started. */
 	int              drag_start_x;        /* The position where the drag started. */
 	int              drag_start_y;
+
+	/* monitored locations  */
+
+	MonitorData      monitor;
 };
 
 
@@ -99,6 +112,9 @@ static guint gth_folder_tree_signals[LAST_SIGNAL] = { 0 };
 G_DEFINE_TYPE (GthFolderTree, gth_folder_tree, GTK_TYPE_TREE_VIEW)
 
 
+static void remove_all_locations_from_the_monitor (GthFolderTree *folder_tree);
+
+
 static void
 gth_folder_tree_finalize (GObject *object)
 {
@@ -111,9 +127,17 @@ gth_folder_tree_finalize (GObject *object)
 			gtk_target_list_unref (folder_tree->priv->drag_target_list);
 			folder_tree->priv->drag_target_list = NULL;
 		}
+		if (folder_tree->priv->monitor.update_id != 0) {
+			g_source_remove (folder_tree->priv->monitor.update_id);
+			folder_tree->priv->monitor.update_id = 0;
+		}
+		g_hash_table_unref (folder_tree->priv->entry_points);
+		remove_all_locations_from_the_monitor (folder_tree);
+		g_hash_table_unref (folder_tree->priv->monitor.locations);
+		_g_object_list_unref (folder_tree->priv->monitor.sources);
 		if (folder_tree->priv->root != NULL)
 			g_object_unref (folder_tree->priv->root);
-		gth_icon_cache_free (folder_tree->priv->icon_cache);
+
 		g_free (folder_tree->priv);
 		folder_tree->priv = NULL;
 	}
@@ -268,6 +292,8 @@ text_renderer_editing_started_cb (GtkCellRenderer *cell,
 
 	if (GTK_IS_ENTRY (editable))
 	      gtk_entry_set_text (GTK_ENTRY (editable), g_file_info_get_edit_name (file_data->info));
+
+	_g_object_unref (file_data);
 }
 
 
@@ -293,9 +319,12 @@ add_columns (GthFolderTree *folder_tree,
 	column = gtk_tree_view_column_new ();
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
+	g_object_set (renderer,
+		      "follow-state", TRUE,
+		      NULL);
 	gtk_tree_view_column_pack_start (column, renderer, FALSE);
 	gtk_tree_view_column_set_attributes (column, renderer,
-					     "pixbuf", COLUMN_ICON,
+					     "gicon", COLUMN_ICON,
 					     NULL);
 
 	folder_tree->priv->text_renderer = renderer = gtk_cell_renderer_text_new ();
@@ -371,6 +400,175 @@ row_activated_cb (GtkTreeView       *tree_view,
 }
 
 
+/* -- update_monitored_locations  -- */
+
+
+static GthFileSource *
+get_monitor_file_source_for_file (GthFolderTree *folder_tree,
+				  GFile         *file)
+{
+	GList *scan;
+	char  *uri;
+
+	uri = g_file_get_uri (file);
+	for (scan = folder_tree->priv->monitor.sources; scan; scan = scan->next) {
+		GthFileSource *file_source = scan->data;
+
+		if (gth_file_source_supports_scheme (file_source, uri)) {
+			g_free (uri);
+			return file_source;
+		}
+	}
+
+	g_free (uri);
+
+	return NULL;
+}
+
+
+static void
+_gth_folder_tree_remove_from_monitor (GthFolderTree *folder_tree,
+				      GFile         *file)
+{
+	GthFileSource *file_source;
+
+	file_source = get_monitor_file_source_for_file (folder_tree, file);
+	if (file_source == NULL)
+		return;
+
+	gth_file_source_monitor_directory (file_source, file, FALSE);
+}
+
+
+static void
+remove_all_locations_from_the_monitor (GthFolderTree *folder_tree)
+{
+	GList *locations;
+	GList *scan;
+
+	locations = g_hash_table_get_keys (folder_tree->priv->monitor.locations);
+	for (scan = locations; scan; scan = scan->next)
+		_gth_folder_tree_remove_from_monitor (folder_tree, G_FILE (scan->data));
+	g_hash_table_remove_all (folder_tree->priv->monitor.locations);
+
+	g_list_free (locations);
+}
+
+
+static void
+_gth_folder_tree_add_to_monitor (GthFolderTree *folder_tree,
+				 GFile         *file)
+{
+	GthFileSource *file_source;
+
+	file_source = get_monitor_file_source_for_file (folder_tree, file);
+	if (file_source == NULL) {
+		file_source = gth_main_get_file_source (file);
+		if (file_source == NULL)
+			return;
+
+		folder_tree->priv->monitor.sources = g_list_prepend (folder_tree->priv->monitor.sources, file_source);
+	}
+
+	gth_file_source_monitor_directory (file_source, file, TRUE);
+	g_hash_table_add (folder_tree->priv->monitor.locations, file);
+}
+
+
+static void
+add_to_open_locations (GtkTreeView *tree_view,
+		       GtkTreePath *path,
+		       gpointer     user_data)
+{
+	GHashTable  *open_locations = user_data;
+	GthFileData *file_data;
+
+	file_data = gth_folder_tree_get_file (GTH_FOLDER_TREE (tree_view), path);
+	if (file_data != NULL) {
+		g_hash_table_add (open_locations, g_object_ref (file_data->file));
+		g_object_unref (file_data);
+	}
+}
+
+
+static gboolean
+update_monitored_locations (gpointer user_data)
+{
+	GthFolderTree *folder_tree = user_data;
+	GHashTable    *open_locations;
+	GList         *locations;
+	GList         *locations_to_remove;
+	GList         *scan;
+
+	if (folder_tree->priv->monitor.update_id != 0) {
+		g_source_remove (folder_tree->priv->monitor.update_id);
+		folder_tree->priv->monitor.update_id = 0;
+	}
+
+	open_locations = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, NULL);
+	gtk_tree_view_map_expanded_rows (GTK_TREE_VIEW (folder_tree),
+					 add_to_open_locations,
+					 open_locations);
+
+#if 0
+	{
+		g_print ("** expanded locations **\n");
+
+		locations = g_hash_table_get_keys (open_locations);
+		for (scan = locations; scan; scan = scan->next) {
+			GFile *file = scan->data;
+
+			g_print ("\t%s\n", g_file_get_uri (file));
+		}
+		g_list_free (locations);
+	}
+#endif
+
+	/* remove the old locations */
+
+	locations_to_remove = NULL;
+	locations = g_hash_table_get_keys (folder_tree->priv->monitor.locations);
+	for (scan = locations; scan; scan = scan->next) {
+		GFile *file = scan->data;
+
+		if (! g_hash_table_contains (open_locations, file)) {
+			_gth_folder_tree_remove_from_monitor (folder_tree, file);
+			locations_to_remove = g_list_prepend (locations_to_remove, g_object_ref (file));
+		}
+	}
+
+	for (scan = locations_to_remove; scan; scan = scan->next)
+		g_hash_table_remove (folder_tree->priv->monitor.locations, G_FILE (scan->data));
+
+	g_list_free (locations);
+	g_list_free (locations_to_remove);
+
+	/* add the new locations */
+
+	locations = g_hash_table_get_keys (open_locations);
+	for (scan = locations; scan; scan = scan->next) {
+		GFile *file = scan->data;
+
+		if (! g_hash_table_contains (folder_tree->priv->monitor.locations, file))
+			_gth_folder_tree_add_to_monitor (folder_tree, file);
+	}
+
+	g_list_free (locations);
+	g_hash_table_unref (open_locations);
+
+	return FALSE;
+}
+
+
+static void
+queue_update_monitored_locations (GthFolderTree *folder_tree)
+{
+	if (folder_tree->priv->monitor.update_id != 0)
+		g_source_remove (folder_tree->priv->monitor.update_id);
+	folder_tree->priv->monitor.update_id = g_timeout_add (UPDATE_MONITORED_LOCATIONS_DELAY, update_monitored_locations, folder_tree);
+}
+
+
 static gboolean
 row_expanded_cb (GtkTreeView  *tree_view,
 		 GtkTreeIter  *expanded_iter,
@@ -392,12 +590,25 @@ row_expanded_cb (GtkTreeView  *tree_view,
 	if ((entry_type == ENTRY_TYPE_FILE) && ! loaded)
 		g_signal_emit (folder_tree, gth_folder_tree_signals[LIST_CHILDREN], 0, file_data->file);
 
+	queue_update_monitored_locations (folder_tree);
+
 	_g_object_unref (file_data);
 
 	return FALSE;
 }
 
 
+static gboolean
+row_collapsed_cb (GtkTreeView *tree_view,
+		  GtkTreeIter *iter,
+		  GtkTreePath *path,
+		  gpointer     user_data)
+{
+	queue_update_monitored_locations (GTH_FOLDER_TREE (user_data));
+	return FALSE;
+}
+
+
 static gboolean
 popup_menu_cb (GtkWidget *widget,
 	       gpointer   user_data)
@@ -571,11 +782,13 @@ motion_notify_event_cb (GtkWidget      *widget,
 
 			/**/
 
-			context = gtk_drag_begin (widget,
-						  folder_tree->priv->drag_target_list,
-						  folder_tree->priv->drag_actions,
-						  1,
-						  (GdkEvent *) event);
+			context = gtk_drag_begin_with_coordinates (widget,
+								   folder_tree->priv->drag_target_list,
+								   folder_tree->priv->drag_actions,
+								   1,
+								   (GdkEvent *) event,
+								   -1,
+								   -1);
 
 			dnd_surface = gtk_tree_view_create_row_drag_icon (GTK_TREE_VIEW (folder_tree), path);
 			gtk_drag_set_icon_surface (context, dnd_surface);
@@ -854,7 +1067,7 @@ _gth_folder_tree_add_loading_item (GthFolderTree *folder_tree,
 	gtk_tree_store_set (folder_tree->priv->tree_store, &iter,
 			    COLUMN_STYLE, PANGO_STYLE_ITALIC,
 			    COLUMN_TYPE, ENTRY_TYPE_LOADING,
-			    COLUMN_NAME, _("Loading..."),
+			    COLUMN_NAME, _("Loading…"),
 			    COLUMN_SORT_KEY, sort_key,
 			    COLUMN_SORT_ORDER, 0,
 			    -1);
@@ -896,19 +1109,15 @@ _gth_folder_tree_set_file_data (GthFolderTree *folder_tree,
 {
 	const char *name;
 	char       *sort_key;
-	GIcon      *icon;
-	GdkPixbuf  *pixbuf;
 
 	name = g_file_info_get_display_name (file_data->info);
 	if (name == NULL)
 		return FALSE;
-	sort_key = g_utf8_collate_key_for_filename (name, -1);
-	icon = g_file_info_get_icon (file_data->info);
-	pixbuf = gth_icon_cache_get_pixbuf (folder_tree->priv->icon_cache, icon);
 
+	sort_key = g_utf8_collate_key_for_filename (name, -1);
 	gtk_tree_store_set (folder_tree->priv->tree_store, iter,
 			    COLUMN_STYLE, PANGO_STYLE_NORMAL,
-			    COLUMN_ICON, pixbuf,
+			    COLUMN_ICON, g_file_info_get_symbolic_icon (file_data->info),
 			    COLUMN_TYPE, ENTRY_TYPE_FILE,
 			    COLUMN_FILE_DATA, file_data,
 			    COLUMN_NAME, name,
@@ -919,7 +1128,6 @@ _gth_folder_tree_set_file_data (GthFolderTree *folder_tree,
 			    -1);
 
 	g_free (sort_key);
-	_g_object_unref (pixbuf);
 
 	return TRUE;
 }
@@ -942,6 +1150,59 @@ _gth_folder_tree_iter_has_no_child (GthFolderTree *folder_tree,
 }
 
 
+static void
+_gth_folder_tree_update_entry_points (GthFolderTree *folder_tree)
+{
+	GtkTreeIter iter;
+
+	if (! folder_tree->priv->recalc_entry_points)
+		return;
+
+	folder_tree->priv->recalc_entry_points = FALSE;
+
+	g_hash_table_remove_all (folder_tree->priv->entry_points);
+
+	if (! gtk_tree_model_iter_children (GTK_TREE_MODEL (folder_tree->priv->tree_store), &iter, NULL))
+		return;
+
+	do {
+		GthFileData *file_data;
+		EntryType    file_entry_type;
+
+		gtk_tree_model_get (GTK_TREE_MODEL (folder_tree->priv->tree_store), &iter,
+				    COLUMN_FILE_DATA, &file_data,
+				    COLUMN_TYPE, &file_entry_type,
+				    -1);
+		if ((file_entry_type == ENTRY_TYPE_FILE) && (file_data != NULL))
+			g_hash_table_add (folder_tree->priv->entry_points, g_object_ref (file_data->file));
+
+		_g_object_unref (file_data);
+	}
+	while (gtk_tree_model_iter_next (GTK_TREE_MODEL (folder_tree->priv->tree_store), &iter));
+}
+
+
+/*
+ * Returns TRUE if file_data points to a folder contained in the entry point
+ * list but it's not real entry point, for example it returns TRUE for
+ * '/home/user/Images' or '/home/user/Documents'.
+ * This entries are duplicates of the entry points and are treated in a special
+ * way to avoid confusion.
+ * */
+static gboolean
+_gth_folder_tree_is_entry_point_dup (GthFolderTree *folder_tree,
+				     GtkTreeIter   *iter,
+				     GthFileData   *file_data)
+{
+	_gth_folder_tree_update_entry_points (folder_tree);
+
+	if (g_hash_table_lookup (folder_tree->priv->entry_points, file_data->file) == NULL)
+		return FALSE;
+
+	return ! g_file_info_get_attribute_boolean (file_data->info, "gthumb::entry-point");
+}
+
+
 static gboolean
 _gth_folder_tree_add_file (GthFolderTree *folder_tree,
 			   GtkTreeIter   *parent,
@@ -969,25 +1230,36 @@ _gth_folder_tree_add_file (GthFolderTree *folder_tree,
 				    COLUMN_WEIGHT, PANGO_WEIGHT_NORMAL,
 				    -1);
 
-	if (! g_file_info_get_attribute_boolean (fd->info, "gthumb::no-child"))
+	if (! g_file_info_get_attribute_boolean (fd->info, "gthumb::no-child")
+	    && ! _gth_folder_tree_is_entry_point_dup (folder_tree, &iter, fd))
+	{
 		_gth_folder_tree_add_loading_item (folder_tree, &iter, TRUE);
+	}
 
 	return TRUE;
 }
 
 
 static void
-gth_folder_tree_construct (GthFolderTree *folder_tree)
+gth_folder_tree_init (GthFolderTree *folder_tree)
 {
 	GtkTreeSelection *selection;
 
-	folder_tree->priv->icon_cache = gth_icon_cache_new (gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (folder_tree))),
-							    _gtk_widget_lookup_for_size (GTK_WIDGET (folder_tree), GTK_ICON_SIZE_MENU));
+	folder_tree->priv = g_new0 (GthFolderTreePrivate, 1);
+	folder_tree->priv->drag_source_enabled = FALSE;
+	folder_tree->priv->dragging = FALSE;
+	folder_tree->priv->drag_started = FALSE;
+	folder_tree->priv->drag_target_list = NULL;
+	folder_tree->priv->entry_points = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, NULL);
+	folder_tree->priv->recalc_entry_points = FALSE;
+	folder_tree->priv->monitor.locations = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, g_object_unref, NULL);
+	folder_tree->priv->monitor.sources = NULL;
+	folder_tree->priv->monitor.update_id = 0;
 
 	folder_tree->priv->tree_store = gtk_tree_store_new (NUM_COLUMNS,
 							    PANGO_TYPE_STYLE,
 							    PANGO_TYPE_WEIGHT,
-							    GDK_TYPE_PIXBUF,
+							    G_TYPE_ICON,
 							    G_TYPE_INT,
 							    G_TYPE_OBJECT,
 							    G_TYPE_STRING,
@@ -1039,18 +1311,10 @@ gth_folder_tree_construct (GthFolderTree *folder_tree)
 			  "row-expanded",
 			  G_CALLBACK (row_expanded_cb),
 			  folder_tree);
-}
-
-
-static void
-gth_folder_tree_init (GthFolderTree *folder_tree)
-{
-	folder_tree->priv = g_new0 (GthFolderTreePrivate, 1);
-	folder_tree->priv->drag_source_enabled = FALSE;
-	folder_tree->priv->dragging = FALSE;
-	folder_tree->priv->drag_started = FALSE;
-	folder_tree->priv->drag_target_list = NULL;
-	gth_folder_tree_construct (folder_tree);
+	g_signal_connect (folder_tree,
+			  "row-collapsed",
+			  G_CALLBACK (row_collapsed_cb),
+			  folder_tree);
 }
 
 
@@ -1086,23 +1350,23 @@ gth_folder_tree_set_list (GthFolderTree *folder_tree,
 
 	if (open_parent) {
 		char        *sort_key;
-		GdkPixbuf   *pixbuf;
+		GIcon       *icon;
 		GtkTreeIter  iter;
 
 		sort_key = g_utf8_collate_key_for_filename (PARENT_URI, -1);
-		pixbuf = gtk_widget_render_icon_pixbuf (GTK_WIDGET (folder_tree), GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU);
+		icon = g_themed_icon_new ("go-up-symbolic");
 
 		gtk_tree_store_append (folder_tree->priv->tree_store, &iter, NULL);
 		gtk_tree_store_set (folder_tree->priv->tree_store, &iter,
 				    COLUMN_STYLE, PANGO_STYLE_ITALIC,
-				    COLUMN_ICON, pixbuf,
+				    COLUMN_ICON, icon,
 				    COLUMN_TYPE, ENTRY_TYPE_PARENT,
 				    COLUMN_NAME, _("(Open Parent)"),
 				    COLUMN_SORT_KEY, sort_key,
 				    COLUMN_SORT_ORDER, 0,
 				    -1);
 
-		g_object_unref (pixbuf);
+		g_object_unref (icon);
 		g_free (sort_key);
 	}
 
@@ -1175,7 +1439,9 @@ _gth_folder_tree_get_children (GthFolderTree *folder_tree,
 				    COLUMN_TYPE, &file_type,
 				    -1);
 		if ((file_type == ENTRY_TYPE_FILE) && (file_data != NULL))
-			list = g_list_prepend (list, file_data);
+			list = g_list_prepend (list, g_object_ref (file_data));
+
+		_g_object_unref (file_data);
 	}
 	while (gtk_tree_model_iter_next (tree_model, &iter));
 
@@ -1318,8 +1584,8 @@ gth_folder_tree_set_children (GthFolderTree *folder_tree,
 		gtk_tree_store_set (folder_tree->priv->tree_store, p_parent_iter,
 				    COLUMN_LOADED, TRUE,
 				    -1);
-
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (folder_tree->priv->tree_store), COLUMN_NAME, GTK_SORT_ASCENDING);
+	folder_tree->priv->recalc_entry_points = TRUE;
 
 	emit_fake_motion_notify_event (folder_tree);
 }
@@ -1422,6 +1688,8 @@ gth_folder_tree_add_children (GthFolderTree *folder_tree,
 
 	if (! is_empty)
 		_gth_folder_tree_remove_child_type (folder_tree, p_parent_iter, ENTRY_TYPE_EMPTY);
+
+	folder_tree->priv->recalc_entry_points = TRUE;
 }
 
 
@@ -1517,6 +1785,8 @@ gth_folder_tree_delete_children (GthFolderTree *folder_tree,
 
 	if (gtk_tree_model_iter_n_children (GTK_TREE_MODEL (folder_tree->priv->tree_store), p_parent_iter) > 1)
 		_gth_folder_tree_remove_child_type (folder_tree, p_parent_iter, ENTRY_TYPE_EMPTY);
+
+	folder_tree->priv->recalc_entry_points = TRUE;
 }
 
 
@@ -1770,4 +2040,3 @@ gth_folder_tree_unset_drag_source (GthFolderTree *self)
 {
 	self->priv->drag_source_enabled = FALSE;
 }
-
