@@ -43,7 +43,6 @@ enum {
 };
 
 enum {
-	CHANNEL_COLUMN_ICON,
 	CHANNEL_COLUMN_NAME,
 	CHANNEL_COLUMN_SENSITIVE
 };
@@ -325,16 +324,16 @@ _cairo_set_source_color_from_channel (cairo_t *cr,
 	switch (channel) {
 	case GTH_HISTOGRAM_CHANNEL_VALUE:
 	default:
-		cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 1.0);
+		cairo_set_source_rgba (cr, 0.8, 0.8, 0.8, 1.0);
 		break;
 	case GTH_HISTOGRAM_CHANNEL_RED:
-		cairo_set_source_rgba (cr, 1.0, 0.0, 0.0, 1.0);
+		cairo_set_source_rgba (cr, 0.68, 0.18, 0.19, 1.0); /* #af2e31 */
 		break;
 	case GTH_HISTOGRAM_CHANNEL_GREEN:
-		cairo_set_source_rgba (cr, 0.0, 1.0, 0.0, 1.0);
+		cairo_set_source_rgba (cr, 0.33, 0.78, 0.30, 1.0); /* #55c74d */
 		break;
 	case GTH_HISTOGRAM_CHANNEL_BLUE:
-		cairo_set_source_rgba (cr, 0.0, 0.0, 1.0, 1.0);
+		cairo_set_source_rgba (cr, 0.13, 0.54, 0.8, 1.0); /* #238acc */
 		break;
 	case GTH_HISTOGRAM_CHANNEL_ALPHA:
 		cairo_set_source_rgba (cr, 0.5, 0.5, 0.5, 1.0);
@@ -484,10 +483,10 @@ gth_histogram_paint_rgb (GthHistogramView *self,
 		if (((self->priv->selection_start > 0) || (self->priv->selection_end < 255))
 		    && (i >= self->priv->selection_start) && (i <= self->priv->selection_end))
 		{
-			cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
+			cairo_set_source_rgb (cr, 0.3, 0.3, 0.3);
 		}
 		else
-			cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+			cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 		value = gth_histogram_get_value (self->priv->histogram, min_c, i);
 		y = CLAMP ((int) (allocation->height * convert_to_scale (self->priv->scale_type, value)) / max, 0, allocation->height);
 		cairo_rectangle (cr,
@@ -511,7 +510,7 @@ gth_histogram_paint_grid (GthHistogramView *self,
 	int     i;
 
 	gtk_style_context_get_border_color (style_context,
-					    gtk_widget_get_state (GTK_WIDGET (self)),
+					    gtk_widget_get_state_flags (GTK_WIDGET (self)),
 					    &color);
 	gdk_cairo_set_source_rgba (cr, &color);
 	cairo_set_line_width (cr, 0.5);
@@ -566,6 +565,7 @@ histogram_view_draw_cb (GtkWidget *widget,
 	style_context = gtk_widget_get_style_context (widget);
 	gtk_style_context_save (style_context);
 	gtk_style_context_add_class (style_context, GTK_STYLE_CLASS_VIEW);
+	gtk_style_context_add_class (style_context, "histogram");
 
 	gtk_widget_get_allocation (widget, &allocation);
 	gtk_render_background (style_context, cr, 0, 0, allocation.width, allocation.height);
@@ -578,7 +578,7 @@ histogram_view_draw_cb (GtkWidget *widget,
 
 		cairo_set_antialias (cr, CAIRO_ANTIALIAS_NONE);
 
-		gtk_style_context_get_padding (style_context, gtk_widget_get_state (widget), &padding);
+		gtk_style_context_get_padding (style_context, gtk_widget_get_state_flags (widget), &padding);
 
 		inner_allocation.x = padding.left;
 		inner_allocation.y = padding.top;
@@ -825,7 +825,7 @@ gth_histogram_view_init (GthHistogramView *self)
 	self->priv->linear_histogram_button = gtk_toggle_button_new ();
 	gtk_widget_set_tooltip_text (self->priv->linear_histogram_button, _("Linear scale"));
 	gtk_button_set_relief (GTK_BUTTON (self->priv->linear_histogram_button), GTK_RELIEF_NONE);
-	gtk_container_add (GTK_CONTAINER (self->priv->linear_histogram_button), gtk_image_new_from_icon_name ("histogram-linear", GTK_ICON_SIZE_MENU));
+	gtk_container_add (GTK_CONTAINER (self->priv->linear_histogram_button), gtk_image_new_from_icon_name ("format-linear-symbolic", GTK_ICON_SIZE_MENU));
 	gtk_widget_show_all (self->priv->linear_histogram_button);
 	gtk_box_pack_start (GTK_BOX (sub_box), self->priv->linear_histogram_button, FALSE, FALSE, 0);
 
@@ -837,7 +837,7 @@ gth_histogram_view_init (GthHistogramView *self)
 	self->priv->logarithmic_histogram_button = gtk_toggle_button_new ();
 	gtk_widget_set_tooltip_text (self->priv->logarithmic_histogram_button, _("Logarithmic scale"));
 	gtk_button_set_relief (GTK_BUTTON (self->priv->logarithmic_histogram_button), GTK_RELIEF_NONE);
-	gtk_container_add (GTK_CONTAINER (self->priv->logarithmic_histogram_button), gtk_image_new_from_icon_name ("histogram-logarithmic", GTK_ICON_SIZE_MENU));
+	gtk_container_add (GTK_CONTAINER (self->priv->logarithmic_histogram_button), gtk_image_new_from_icon_name ("format-logarithmic-symbolic", GTK_ICON_SIZE_MENU));
 	gtk_widget_show_all (self->priv->logarithmic_histogram_button);
 	gtk_box_pack_start (GTK_BOX (sub_box), self->priv->logarithmic_histogram_button, FALSE, FALSE, 0);
 
@@ -859,20 +859,10 @@ gth_histogram_view_init (GthHistogramView *self)
 	gtk_label_set_attributes (GTK_LABEL (label), attr_list);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (sub_box), label, FALSE, FALSE, 0);
-	channel_model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
+	channel_model = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_BOOLEAN);
 	self->priv->channel_combo_box = gtk_combo_box_new_with_model (GTK_TREE_MODEL (channel_model));
 	g_object_unref (channel_model);
 
-	renderer = gtk_cell_renderer_pixbuf_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (self->priv->channel_combo_box),
-				    renderer,
-				    FALSE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (self->priv->channel_combo_box),
-			    	    	renderer,
-			    	    	"icon-name", CHANNEL_COLUMN_ICON,
-			    	    	"sensitive", CHANNEL_COLUMN_SENSITIVE,
-			    	    	NULL);
-
 	renderer = gtk_cell_renderer_text_new ();
 	g_object_set (renderer, "attributes", attr_list, NULL);
 	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (self->priv->channel_combo_box),
@@ -886,38 +876,32 @@ gth_histogram_view_init (GthHistogramView *self)
 
 	gtk_list_store_append (channel_model, &iter);
 	gtk_list_store_set (channel_model, &iter,
-			    CHANNEL_COLUMN_ICON, "channel-value",
 			    CHANNEL_COLUMN_NAME, _("Value"),
 			    CHANNEL_COLUMN_SENSITIVE, TRUE,
 			    -1);
 	gtk_list_store_append (channel_model, &iter);
 	gtk_list_store_set (channel_model, &iter,
-			    CHANNEL_COLUMN_ICON, "channel-red",
 			    CHANNEL_COLUMN_NAME, _("Red"),
 			    CHANNEL_COLUMN_SENSITIVE, TRUE,
 			    -1);
 	gtk_list_store_append (channel_model, &iter);
 	gtk_list_store_set (channel_model, &iter,
-			    CHANNEL_COLUMN_ICON, "channel-green",
 			    CHANNEL_COLUMN_NAME, _("Green"),
 			    CHANNEL_COLUMN_SENSITIVE, TRUE,
 			    -1);
 	gtk_list_store_append (channel_model, &iter);
 	gtk_list_store_set (channel_model, &iter,
-			    CHANNEL_COLUMN_ICON, "channel-blue",
 			    CHANNEL_COLUMN_NAME, _("Blue"),
 			    CHANNEL_COLUMN_SENSITIVE, TRUE,
 			    -1);
 	gtk_list_store_append (channel_model, &iter);
 	gtk_list_store_set (channel_model, &iter,
-			    CHANNEL_COLUMN_ICON, "channel-alpha",
 			    CHANNEL_COLUMN_NAME, _("Alpha"),
 			    CHANNEL_COLUMN_SENSITIVE, FALSE,
 			    -1);
 	gtk_list_store_append (channel_model, &iter);
 	/* Translators: RGB is an acronym for Red Green Blue */
 	gtk_list_store_set (channel_model, &iter,
-			    CHANNEL_COLUMN_ICON, "channel-rgb",
 			    CHANNEL_COLUMN_NAME, _("RGB"),
 			    CHANNEL_COLUMN_SENSITIVE, TRUE,
 			    -1);
@@ -942,7 +926,7 @@ gth_histogram_view_init (GthHistogramView *self)
 	self->priv->histogram_view = gtk_drawing_area_new ();
 	gtk_widget_add_events (self->priv->histogram_view, GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_STRUCTURE_MASK);
 	gtk_widget_show (self->priv->histogram_view);
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (view_container), self->priv->histogram_view);
+	gtk_container_add (GTK_CONTAINER (view_container), self->priv->histogram_view);
 
 	g_signal_connect (self->priv->histogram_view,
 			  "draw",
