@@ -125,72 +125,6 @@ gth_histogram_changed (GthHistogram *self)
 }
 
 
-void
-gth_histogram_calculate_for_pixbuf (GthHistogram    *self,
-				    const GdkPixbuf *pixbuf)
-{
-	int    **values;
-	int     *values_max;
-	int      width, height, n_channels;
-	int      rowstride;
-	guchar  *line, *pixel;
-	int      i, j, max;
-
-	g_return_if_fail (GTH_IS_HISTOGRAM (self));
-
-	values = self->priv->values;
-	values_max = self->priv->values_max;
-
-	if (pixbuf == NULL) {
-		self->priv->n_channels = 0;
-		histogram_reset_values (self);
-		gth_histogram_changed (self);
-		return;
-	}
-
-	n_channels = gdk_pixbuf_get_n_channels (pixbuf);
-	rowstride  = gdk_pixbuf_get_rowstride (pixbuf);
-	line       = gdk_pixbuf_get_pixels (pixbuf);
-	width      = gdk_pixbuf_get_width (pixbuf);
-	height     = gdk_pixbuf_get_height (pixbuf);
-
-	self->priv->n_channels = n_channels + 1;
-	histogram_reset_values (self);
-
-	for (i = 0; i < height; i++) {
-		pixel = line;
-
-		for (j = 0; j < width; j++) {
-			/* count values for each RGB channel */
-			values[1][pixel[CAIRO_RED]] += 1;
-			values[2][pixel[CAIRO_GREEN]] += 1;
-			values[3][pixel[CAIRO_BLUE]] += 1;
-			if (n_channels > 3)
-				values[4][ pixel[CAIRO_ALPHA] ] += 1;
-
-			/* count value for Value channel */
-			max = MAX (pixel[CAIRO_RED], pixel[CAIRO_GREEN]);
-			max = MAX (pixel[CAIRO_BLUE], max);
-			values[0][max] += 1;
-
-			/* track max value for each channel */
-			values_max[0] = MAX (values_max[0], values[0][max]);
-			values_max[1] = MAX (values_max[1], values[1][pixel[0]]);
-			values_max[2] = MAX (values_max[2], values[2][pixel[1]]);
-			values_max[3] = MAX (values_max[3], values[3][pixel[2]]);
-			if (n_channels > 3)
-				values_max[4] = MAX (values_max[4], values[4][pixel[CAIRO_ALPHA]]);
-
-			pixel += n_channels;
-		}
-
-		line += rowstride;
-	}
-
-	gth_histogram_changed (self);
-}
-
-
 void
 gth_histogram_calculate_for_image (GthHistogram    *self,
 				   cairo_surface_t *image)
@@ -217,7 +151,7 @@ gth_histogram_calculate_for_image (GthHistogram    *self,
 
 	has_alpha  = _cairo_image_surface_get_has_alpha (image);
 	rowstride  = cairo_image_surface_get_stride (image);
-	line       = cairo_image_surface_get_data (image);
+	line       = _cairo_image_surface_flush_and_get_data (image);
 	width      = cairo_image_surface_get_width (image);
 	height     = cairo_image_surface_get_height (image);
 
@@ -232,25 +166,25 @@ gth_histogram_calculate_for_image (GthHistogram    *self,
 
 			/* count values for each RGB channel */
 
-			values[1][red] += 1;
-			values[2][green] += 1;
-			values[3][blue] += 1;
+			values[GTH_HISTOGRAM_CHANNEL_RED][red] += 1;
+			values[GTH_HISTOGRAM_CHANNEL_GREEN][green] += 1;
+			values[GTH_HISTOGRAM_CHANNEL_BLUE][blue] += 1;
 			if (has_alpha)
-				values[4][alpha] += 1;
+				values[GTH_HISTOGRAM_CHANNEL_ALPHA][alpha] += 1;
 
 			/* count value for Value channel */
 
 			max = MAX (MAX (red, green), blue);
-			values[0][max] += 1;
+			values[GTH_HISTOGRAM_CHANNEL_VALUE][max] += 1;
 
 			/* track max value for each channel */
 
-			values_max[0] = MAX (values_max[0], values[0][max]);
-			values_max[1] = MAX (values_max[1], values[1][red]);
-			values_max[2] = MAX (values_max[2], values[2][green]);
-			values_max[3] = MAX (values_max[3], values[3][blue]);
+			values_max[GTH_HISTOGRAM_CHANNEL_VALUE] = MAX (values_max[GTH_HISTOGRAM_CHANNEL_VALUE], values[GTH_HISTOGRAM_CHANNEL_VALUE][max]);
+			values_max[GTH_HISTOGRAM_CHANNEL_RED] = MAX (values_max[GTH_HISTOGRAM_CHANNEL_RED], values[GTH_HISTOGRAM_CHANNEL_RED][red]);
+			values_max[GTH_HISTOGRAM_CHANNEL_GREEN] = MAX (values_max[GTH_HISTOGRAM_CHANNEL_GREEN], values[GTH_HISTOGRAM_CHANNEL_GREEN][green]);
+			values_max[GTH_HISTOGRAM_CHANNEL_BLUE] = MAX (values_max[GTH_HISTOGRAM_CHANNEL_BLUE], values[GTH_HISTOGRAM_CHANNEL_BLUE][blue]);
 			if (has_alpha)
-				values_max[4] = MAX (values_max[4], values[4][alpha]);
+				values_max[GTH_HISTOGRAM_CHANNEL_ALPHA] = MAX (values_max[GTH_HISTOGRAM_CHANNEL_ALPHA], values[GTH_HISTOGRAM_CHANNEL_ALPHA][alpha]);
 
 			pixel += 4;
 		}
@@ -341,3 +275,36 @@ gth_histogram_get_nchannels (GthHistogram *self)
 	g_return_val_if_fail (self != NULL, 0.0);
 	return self->priv->n_channels - 1;
 }
+
+
+long **
+gth_histogram_get_cumulative (GthHistogram *self)
+{
+	long **cumulative;
+	int    c, v;
+
+	cumulative = g_new (long *, GTH_HISTOGRAM_N_CHANNELS);
+	for (c = 0; c < GTH_HISTOGRAM_N_CHANNELS; c++) {
+		int s;
+
+		cumulative[c] = g_new (long, 256);
+		s = 0;
+		for (v = 0; v < 256; v++) {
+			s += gth_histogram_get_value (self, c, v);
+			cumulative[c][v] = s;
+		}
+	}
+
+	return cumulative;
+}
+
+
+void
+gth_cumulative_histogram_free (long **cumulative)
+{
+	int c;
+
+	for (c = 0; c < GTH_HISTOGRAM_N_CHANNELS; c++)
+		g_free (cumulative[c]);
+	g_free (cumulative);
+}
