@@ -25,12 +25,27 @@
 #include "cairo-utils.h"
 #include "glib-utils.h"
 #include "gth-image-dragger.h"
+#include "pixbuf-utils.h"
+
+
+#define SIZE_TOO_BIG_FOR_SCALE_BILINEAR (3000 * 3000)
+#define FRAME_BORDER 25
+
+
+/* Properties */
+enum {
+        PROP_0,
+        PROP_SHOW_FRAME
+};
 
 
 struct _GthImageDraggerPrivate {
 	GthImageViewer  *viewer;
 	gboolean         draggable;
+	gboolean         show_frame;
 	cairo_surface_t *scaled;
+	double           scaled_zoom;
+	GthTask         *scale_task;
 };
 
 
@@ -54,12 +69,61 @@ gth_image_dragger_finalize (GObject *object)
 
 	self = GTH_IMAGE_DRAGGER (object);
 	_cairo_clear_surface (&self->priv->scaled);
+	if (self->priv->scale_task != NULL)
+		gth_task_cancel (self->priv->scale_task);
 
 	/* Chain up */
 	G_OBJECT_CLASS (gth_image_dragger_parent_class)->finalize (object);
 }
 
 
+static void
+gth_image_dragger_set_property (GObject      *object,
+				guint         property_id,
+				const GValue *value,
+				GParamSpec   *pspec)
+{
+	GthImageDragger *self;
+
+        self = GTH_IMAGE_DRAGGER (object);
+
+	switch (property_id) {
+	case PROP_SHOW_FRAME:
+		self->priv->show_frame = g_value_get_boolean (value);
+		if (self->priv->viewer != NULL) {
+			if (self->priv->show_frame)
+				gth_image_viewer_show_frame (self->priv->viewer, FRAME_BORDER);
+			else
+				gth_image_viewer_hide_frame (self->priv->viewer);
+		}
+		break;
+	default:
+		break;
+	}
+}
+
+
+static void
+gth_image_dragger_get_property (GObject    *object,
+				guint       property_id,
+				GValue     *value,
+				GParamSpec *pspec)
+{
+	GthImageDragger *self;
+
+        self = GTH_IMAGE_DRAGGER (object);
+
+	switch (property_id) {
+	case PROP_SHOW_FRAME:
+		g_value_set_boolean (value, self->priv->show_frame);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+
 static void
 gth_image_dragger_class_init (GthImageDraggerClass *class)
 {
@@ -69,6 +133,18 @@ gth_image_dragger_class_init (GthImageDraggerClass *class)
 
 	gobject_class = (GObjectClass*) class;
 	gobject_class->finalize = gth_image_dragger_finalize;
+	gobject_class->set_property = gth_image_dragger_set_property;
+	gobject_class->get_property = gth_image_dragger_get_property;
+
+	/* properties */
+
+	g_object_class_install_property (gobject_class,
+					 PROP_SHOW_FRAME,
+					 g_param_spec_boolean ("show-frame",
+							       "Show Frame",
+							       "Whether to show a frame around the image",
+							       FALSE,
+							       G_PARAM_READWRITE));
 }
 
 
@@ -77,6 +153,11 @@ gth_image_dragger_init (GthImageDragger *dragger)
 {
 	dragger->priv = G_TYPE_INSTANCE_GET_PRIVATE (dragger, GTH_TYPE_IMAGE_DRAGGER, GthImageDraggerPrivate);
 	dragger->priv->scaled = NULL;
+	dragger->priv->scaled_zoom = 0;
+	dragger->priv->scale_task = NULL;
+	dragger->priv->viewer = NULL;
+	dragger->priv->draggable = FALSE;
+	dragger->priv->show_frame = FALSE;
 }
 
 
@@ -86,6 +167,8 @@ gth_image_dragger_set_viewer (GthImageViewerTool *base,
 {
 	GthImageDragger *self = GTH_IMAGE_DRAGGER (base);
 	self->priv->viewer = image_viewer;
+	if (self->priv->show_frame)
+		gth_image_viewer_show_frame (self->priv->viewer, FRAME_BORDER);
 }
 
 
@@ -94,6 +177,9 @@ gth_image_dragger_unset_viewer (GthImageViewerTool *base,
 		       	        GthImageViewer     *image_viewer)
 {
 	GthImageDragger *self = GTH_IMAGE_DRAGGER (base);
+
+	if ((self->priv->viewer != NULL) && self->priv->show_frame)
+		gth_image_viewer_hide_frame (self->priv->viewer);
 	self->priv->viewer = NULL;
 }
 
@@ -178,23 +264,23 @@ gth_image_dragger_draw (GthImageViewerTool *self,
 	if (gth_image_viewer_get_current_image (viewer) == NULL)
 		return;
 
+	gth_image_viewer_paint_frame (viewer, cr);
+
 	if (dragger->priv->scaled != NULL)
 		gth_image_viewer_paint_region (viewer,
 					       cr,
 					       dragger->priv->scaled,
-					       viewer->x_offset - viewer->image_area.x,
-					       viewer->y_offset - viewer->image_area.y,
-					       &viewer->image_area,
-					       NULL,
+					       viewer->image_area.x,
+					       viewer->image_area.y,
+					       &viewer->visible_area,
 					       CAIRO_FILTER_FAST);
 	else
 		gth_image_viewer_paint_region (viewer,
 					       cr,
 					       gth_image_viewer_get_current_image (viewer),
-					       viewer->x_offset - viewer->image_area.x,
-					       viewer->y_offset - viewer->image_area.y,
-					       &viewer->image_area,
-					       NULL,
+					       viewer->image_area.x,
+					       viewer->image_area.y,
+					       &viewer->visible_area,
 					       gth_image_viewer_get_zoom_quality_filter (viewer));
 
 	gth_image_viewer_apply_painters (viewer, cr);
@@ -216,7 +302,8 @@ gth_image_dragger_button_press (GthImageViewerTool *self,
 	if (! dragger->priv->draggable)
 		return FALSE;
 
-	if ((event->button == 1) && ! viewer->dragging) {
+	if (((event->button == 1) || (event->button == 2)) &&
+			! viewer->dragging) {
 		GdkCursor     *cursor;
 		GdkGrabStatus  retval;
 
@@ -254,7 +341,7 @@ gth_image_dragger_button_release (GthImageViewerTool *self,
 	GthImageDragger *dragger;
 	GthImageViewer  *viewer;
 
-	if (event->button != 1)
+	if ((event->button != 1) && (event->button != 2))
 		return FALSE;
 
 	dragger = (GthImageDragger *) self;
@@ -293,55 +380,178 @@ gth_image_dragger_motion_notify (GthImageViewerTool *self,
 }
 
 
+/* -- _gth_image_dragger_scale -- */
+
+
+typedef struct {
+	GthImageDragger *dragger;
+	cairo_surface_t *image;
+	int              new_width;
+	int              new_height;
+	cairo_surface_t *scaled;
+	scale_filter_t   filter;
+} ScaleData;
+
+
+static void
+scale_data_free (ScaleData *scale_data)
+{
+	if (scale_data == NULL)
+		return;
+	cairo_surface_destroy (scale_data->image);
+	cairo_surface_destroy (scale_data->scaled);
+	g_free (scale_data);
+}
+
+
+static gpointer
+_gth_image_dragger_scale_exec (GthAsyncTask *task,
+			       gpointer      user_data)
+{
+	ScaleData *scale_data = user_data;
+
+	scale_data->scaled = _cairo_image_surface_scale (scale_data->image,
+							 scale_data->new_width,
+							 scale_data->new_height,
+							 scale_data->filter,
+							 task);
+
+	return NULL;
+}
+
+
+static void
+_gth_image_dragger_scale_after (GthAsyncTask *task,
+				GError       *error,
+				gpointer      user_data)
+{
+	ScaleData *scale_data = user_data;
+
+	if (error == NULL) {
+		GthImageDragger *dragger = scale_data->dragger;
+
+		_cairo_clear_surface (&dragger->priv->scaled);
+		dragger->priv->scaled = cairo_surface_reference (scale_data->scaled);
+		if (dragger->priv->viewer != NULL)
+			gtk_widget_queue_draw (GTK_WIDGET (dragger->priv->viewer));
+
+		if (GTH_TASK (task) == dragger->priv->scale_task)
+			dragger->priv->scale_task = NULL;
+	}
+
+	g_object_unref (task);
+}
+
+
+static void
+_gth_image_dragger_create_scaled_high_quality (GthImageDragger *self,
+					       cairo_surface_t *image,
+					       int              new_width,
+					       int              new_height,
+					       scale_filter_t   filter)
+{
+	ScaleData *scale_data;
+
+	if (self->priv->scale_task != NULL)
+		gth_task_cancel (self->priv->scale_task);
+
+	scale_data = g_new0 (ScaleData, 1);
+	scale_data->dragger = self;
+	scale_data->image = cairo_surface_reference (image);
+	scale_data->new_width = new_width;
+	scale_data->new_height = new_height;
+	scale_data->scaled = NULL;
+	scale_data->filter = filter;
+
+	self->priv->scale_task = gth_async_task_new (NULL,
+						     _gth_image_dragger_scale_exec,
+						     _gth_image_dragger_scale_after,
+						     scale_data,
+						     (GDestroyNotify) scale_data_free);
+
+	gth_task_exec (self->priv->scale_task, NULL);
+}
+
+
 static void
 _gth_image_dragger_update_scaled_image (GthImageDragger *self)
 {
 	cairo_surface_t *image;
+	double           zoom_from_original;
+	double           zoom_from_image;
 	int              image_width, image_height;
 	int              original_width, original_height;
-	double           zoom;
+	int              new_width, new_height;
+	scale_filter_t   filter;
 
+	self->priv->scaled_zoom = 0;
 	_cairo_clear_surface (&self->priv->scaled);
-	self->priv->scaled = NULL;
 
-	if (gth_image_viewer_is_animation (self->priv->viewer))
+	if (self->priv->scale_task != NULL) {
+		gth_task_cancel (self->priv->scale_task);
+		self->priv->scale_task = NULL;
+	}
+
+	if (self->priv->viewer == NULL)
 		return;
 
+	if (gth_image_viewer_is_animation (self->priv->viewer)) {
+		gtk_widget_queue_draw (GTK_WIDGET (self->priv->viewer));
+		return;
+	}
+
 	image = gth_image_viewer_get_current_image (self->priv->viewer);
-	if (image == NULL)
+	if ((image == NULL) || (cairo_surface_status (image) != CAIRO_STATUS_SUCCESS)) {
+		gtk_widget_queue_draw (GTK_WIDGET (self->priv->viewer));
 		return;
+	}
 
+	if (gth_image_viewer_get_zoom_quality (self->priv->viewer) == GTH_ZOOM_QUALITY_LOW) {
+		gtk_widget_queue_draw (GTK_WIDGET (self->priv->viewer));
+		return;
+	}
+
+	zoom_from_original = gth_image_viewer_get_zoom (self->priv->viewer);
+	gth_image_viewer_get_original_size (self->priv->viewer, &original_width, &original_height);
+	new_width = zoom_from_original * original_width;
+	new_height = zoom_from_original * original_height;
 	image_width = cairo_image_surface_get_width (image);
 	image_height = cairo_image_surface_get_height (image);
-	gth_image_viewer_get_original_size (self->priv->viewer, &original_width, &original_height);
+	zoom_from_image = (double) new_width / image_width;
 
-	if ((original_width != image_width) || (original_height != image_height))
+	if (zoom_from_image >= 1.0) {
+		gtk_widget_queue_draw (GTK_WIDGET (self->priv->viewer));
 		return;
+	}
 
-	if (gth_image_viewer_get_zoom_quality (self->priv->viewer) == GTH_ZOOM_QUALITY_LOW)
-		return;
+	if (image_width * image_height <= SIZE_TOO_BIG_FOR_SCALE_BILINEAR)
+		filter = SCALE_FILTER_TRIANGLE;
+	else
+		filter = SCALE_FILTER_BOX;
 
-	zoom = gth_image_viewer_get_zoom (self->priv->viewer);
-	if (zoom >= 1.0 / _CAIRO_MAX_SCALE_FACTOR)
-		return;
-
-	self->priv->scaled = _cairo_image_surface_scale_bilinear (image,
-								  zoom * image_width,
-								  zoom * image_height);
+	self->priv->scaled_zoom = zoom_from_original;
+	_gth_image_dragger_create_scaled_high_quality (self, image, new_width, new_height, filter);
 }
 
 
 static void
-gth_image_dragger_image_changed (GthImageViewerTool *self)
+gth_image_dragger_image_changed (GthImageViewerTool *base)
 {
-	_gth_image_dragger_update_scaled_image (GTH_IMAGE_DRAGGER (self));
+	_gth_image_dragger_update_scaled_image (GTH_IMAGE_DRAGGER (base));
 }
 
 
 static void
-gth_image_dragger_zoom_changed (GthImageViewerTool *self)
+gth_image_dragger_zoom_changed (GthImageViewerTool *base)
 {
-	_gth_image_dragger_update_scaled_image (GTH_IMAGE_DRAGGER (self));
+	GthImageDragger *self = GTH_IMAGE_DRAGGER (base);
+
+	if ((self->priv->viewer == NULL)
+		|| (gth_image_viewer_get_zoom_quality (self->priv->viewer) == GTH_ZOOM_QUALITY_LOW)
+		|| (self->priv->scaled_zoom != gth_image_viewer_get_zoom (self->priv->viewer)))
+	{
+		_gth_image_dragger_update_scaled_image (self);
+	}
 }
 
 
@@ -365,7 +575,9 @@ gth_image_dragger_gth_image_tool_interface_init (GthImageViewerToolInterface *if
 
 
 GthImageViewerTool *
-gth_image_dragger_new (void)
+gth_image_dragger_new (gboolean show_frame)
 {
-	return (GthImageViewerTool *) g_object_new (GTH_TYPE_IMAGE_DRAGGER, NULL);
+	return (GthImageViewerTool *) g_object_new (GTH_TYPE_IMAGE_DRAGGER,
+						    "show-frame", show_frame,
+						    NULL);
 }
