@@ -128,6 +128,18 @@ gth_image_new (void)
 }
 
 
+GthImage *
+gth_image_new_for_surface (cairo_surface_t *surface)
+{
+	GthImage *image;
+
+	image = gth_image_new ();
+	gth_image_set_cairo_surface (image, surface);
+
+	return image;
+}
+
+
 GthImage *
 gth_image_new_for_pixbuf (GdkPixbuf *value)
 {
@@ -140,6 +152,37 @@ gth_image_new_for_pixbuf (GdkPixbuf *value)
 }
 
 
+GthImage *
+gth_image_copy (GthImage *image)
+{
+	GthImage *new_image;
+
+	new_image = gth_image_new ();
+
+	switch (image->priv->format) {
+	case GTH_IMAGE_FORMAT_CAIRO_SURFACE:
+		new_image->priv->format = GTH_IMAGE_FORMAT_CAIRO_SURFACE;
+		new_image->priv->data.surface = _cairo_image_surface_copy (image->priv->data.surface);
+		break;
+
+	case GTH_IMAGE_FORMAT_GDK_PIXBUF:
+		new_image->priv->format = GTH_IMAGE_FORMAT_GDK_PIXBUF;
+		new_image->priv->data.pixbuf = gdk_pixbuf_copy (image->priv->data.pixbuf);
+		break;
+
+	case GTH_IMAGE_FORMAT_GDK_PIXBUF_ANIMATION:
+		new_image->priv->format = GTH_IMAGE_FORMAT_GDK_PIXBUF;
+		new_image->priv->data.pixbuf = gdk_pixbuf_copy (gdk_pixbuf_animation_get_static_image (image->priv->data.pixbuf_animation));
+		break;
+
+	default:
+		break;
+	}
+
+	return new_image;
+}
+
+
 void
 gth_image_set_cairo_surface (GthImage        *image,
 			     cairo_surface_t *value)
@@ -300,8 +343,11 @@ gth_image_get_pixbuf_animation (GthImage *image)
 
 
 gboolean
-gth_image_is_animation (GthImage *image)
+gth_image_get_is_animation (GthImage *image)
 {
+	if (image == NULL)
+		return FALSE;
+
 	return ((image->priv->format == GTH_IMAGE_FORMAT_GDK_PIXBUF_ANIMATION)
 	        && (! gdk_pixbuf_animation_is_static_image (image->priv->data.pixbuf_animation)));
 }
