@@ -25,6 +25,7 @@
 #include <glib.h>
 #include <glib-object.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
+#include <cairo.h>
 #include "gth-file-data.h"
 
 G_BEGIN_DECLS
@@ -70,6 +71,7 @@ typedef GthImage * (*GthImageLoaderFunc) (GInputStream  *istream,
 					  int            requested_size,
 					  int           *original_width,
 					  int           *original_height,
+					  gboolean      *loaded_original,
 					  gpointer       user_data,
 					  GCancellable  *cancellable,
 					  GError       **error);
@@ -94,7 +96,7 @@ GdkPixbuf *           gth_image_get_pixbuf                  (GthImage
 void                  gth_image_set_pixbuf_animation        (GthImage           *image,
 						             GdkPixbufAnimation *value);
 GdkPixbufAnimation *  gth_image_get_pixbuf_animation        (GthImage           *image);
-gboolean              gth_image_is_animation                (GthImage           *image);
+gboolean              gth_image_get_is_animation            (GthImage           *image);
 
 G_END_DECLS
 
