@@ -36,7 +36,7 @@
 struct _GthOverwriteDialogPrivate {
 	GtkBuilder     *builder;
 	GFile          *source;
-	GdkPixbuf      *source_pixbuf;
+	GthImage       *source_image;
 	GFile          *destination;
 	GtkWidget      *old_image_viewer;
 	GtkWidget      *new_image_viewer;
@@ -61,7 +61,7 @@ gth_overwrite_dialog_finalize (GObject *object)
 	_g_object_unref (dialog->priv->destination_data);
 	g_object_unref (dialog->priv->builder);
 	_g_object_unref (dialog->priv->source);
-	_g_object_unref (dialog->priv->source_pixbuf);
+	_g_object_unref (dialog->priv->source_image);
 	g_object_unref (dialog->priv->destination);
 	_g_object_unref (dialog->priv->old_image_loader);
 	_g_object_unref (dialog->priv->new_image_loader);
@@ -103,6 +103,7 @@ image_loader_ready_cb (GObject      *source_object,
 					    &image,
 					    NULL,
 					    NULL,
+					    NULL,
 					    &error))
 	{
 		return;
@@ -156,7 +157,7 @@ info_ready_cb (GList    *files,
 		icon = (GIcon*) g_file_info_get_attribute_object (self->priv->source_data->info, "preview::icon");
 		if (icon == NULL)
 			icon = g_content_type_get_icon (g_file_info_get_content_type (self->priv->source_data->info));
-		pixbuf = _g_icon_get_pixbuf (icon, ICON_SIZE, gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (self))));
+		pixbuf = _g_icon_get_pixbuf (icon, ICON_SIZE, _gtk_widget_get_icon_theme (GTK_WIDGET (self)));
 		if (pixbuf != NULL) {
 			gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->new_image_viewer), pixbuf, -1, -1);
 			g_object_unref (pixbuf);
@@ -174,11 +175,11 @@ info_ready_cb (GList    *files,
 				       image_loader_ready_cb,
 				       self);
 	}
-	else if (self->priv->source_pixbuf != NULL) {
+	else if (self->priv->source_image != NULL) {
 		gtk_widget_hide (_gtk_builder_get_widget (self->priv->builder, "new_filename_label"));
 		gtk_widget_hide (_gtk_builder_get_widget (self->priv->builder, "new_size_label"));
 		gtk_widget_hide (_gtk_builder_get_widget (self->priv->builder, "new_modified_label"));
-		gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->new_image_viewer), self->priv->source_pixbuf, -1, -1);
+		gth_image_viewer_set_image (GTH_IMAGE_VIEWER (self->priv->new_image_viewer), self->priv->source_image, -1, -1);
 	}
 
 	/* old image  */
@@ -204,7 +205,7 @@ info_ready_cb (GList    *files,
 	icon = (GIcon*) g_file_info_get_attribute_object (self->priv->destination_data->info, "preview::icon");
 	if (icon == NULL)
 		icon = g_content_type_get_icon (g_file_info_get_content_type (self->priv->destination_data->info));
-	pixbuf = _g_icon_get_pixbuf (icon, ICON_SIZE, gtk_icon_theme_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (self))));
+	pixbuf = _g_icon_get_pixbuf (icon, ICON_SIZE, _gtk_widget_get_icon_theme (GTK_WIDGET (self)));
 	if (pixbuf != NULL) {
 		gth_image_viewer_set_pixbuf (GTH_IMAGE_VIEWER (self->priv->old_image_viewer), pixbuf, -1, -1);
 		g_object_unref (pixbuf);
@@ -245,8 +246,8 @@ gth_overwrite_dialog_construct (GthOverwriteDialog   *self,
 	gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (self))), 5);
 	gtk_container_set_border_width (GTK_CONTAINER (self), 5);
 
-	gtk_dialog_add_button (GTK_DIALOG (self), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
-	gtk_dialog_add_button (GTK_DIALOG (self), GTK_STOCK_OK, GTK_RESPONSE_OK);
+	gtk_dialog_add_button (GTK_DIALOG (self), _GTK_LABEL_CANCEL, GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (self), _GTK_LABEL_OK, GTK_RESPONSE_OK);
 	gtk_dialog_set_default_response (GTK_DIALOG (self), GTK_RESPONSE_OK);
 
 	self->priv->builder = _gtk_builder_new_from_file ("overwrite-dialog.ui", NULL);
@@ -288,14 +289,12 @@ gth_overwrite_dialog_construct (GthOverwriteDialog   *self,
 	gtk_widget_set_size_request (_gtk_builder_get_widget (self->priv->builder, "new_image_frame"), PREVIEW_SIZE, PREVIEW_SIZE);
 
 	self->priv->old_image_viewer = gth_image_viewer_new ();
-	gth_image_viewer_set_transp_type (GTH_IMAGE_VIEWER (self->priv->old_image_viewer), GTH_TRANSP_TYPE_NONE);
 	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->old_image_viewer), GTH_FIT_SIZE_IF_LARGER);
 	gth_image_viewer_hide_frame (GTH_IMAGE_VIEWER (self->priv->old_image_viewer));
 	gtk_widget_show (self->priv->old_image_viewer);
 	gtk_container_add (GTK_CONTAINER (_gtk_builder_get_widget (self->priv->builder, "old_image_frame")), self->priv->old_image_viewer);
 
 	self->priv->new_image_viewer = gth_image_viewer_new ();
-	gth_image_viewer_set_transp_type (GTH_IMAGE_VIEWER (self->priv->new_image_viewer), GTH_TRANSP_TYPE_NONE);
 	gth_image_viewer_set_fit_mode (GTH_IMAGE_VIEWER (self->priv->new_image_viewer), GTH_FIT_SIZE_IF_LARGER);
 	gth_image_viewer_hide_frame (GTH_IMAGE_VIEWER (self->priv->new_image_viewer));
 	gtk_widget_show (self->priv->new_image_viewer);
@@ -326,7 +325,7 @@ gth_overwrite_dialog_construct (GthOverwriteDialog   *self,
 
 GtkWidget *
 gth_overwrite_dialog_new (GFile                *source,
-			  GdkPixbuf            *source_pixbuf,
+			  GthImage             *source_image,
 			  GFile                *destination,
 			  GthOverwriteResponse  default_respose,
 			  gboolean              single_file)
@@ -335,7 +334,7 @@ gth_overwrite_dialog_new (GFile                *source,
 
 	self = g_object_new (GTH_TYPE_OVERWRITE_DIALOG, NULL);
 	self->priv->source = _g_object_ref (source);
-	self->priv->source_pixbuf = _g_object_ref (source_pixbuf);
+	self->priv->source_image = _g_object_ref (source_image);
 	self->priv->destination = g_object_ref (destination);
 	gth_overwrite_dialog_construct (self, default_respose, single_file);
 
