@@ -31,23 +31,28 @@ G_DEFINE_TYPE (GthWindow, gth_window, GTK_TYPE_APPLICATION_WINDOW)
 
 enum  {
 	PROP_0,
-	PROP_N_PAGES
+	PROP_N_PAGES,
+	PROP_USE_HEADER_BAR
 };
 
 
 struct _GthWindowPrivate {
 	int              n_pages;
+	gboolean         use_header_bar;
 	int              current_page;
 	GtkWidget       *grid;
-	GtkWidget       *notebook;
+	GtkWidget       *stack;
+	GtkWidget       *headerbar;
 	GtkWidget       *menubar;
 	GtkWidget       *toolbar;
 	GtkWidget       *infobar;
 	GtkWidget       *statusbar;
 	GtkWidget      **toolbars;
 	GtkWidget      **contents;
+	GtkWidget      **pages;
 	GthWindowSize   *window_size;
 	GtkWindowGroup  *window_group;
+	GtkAccelGroup   *accel_group;
 };
 
 
@@ -68,27 +73,24 @@ gth_window_set_n_pages (GthWindow *self,
 	gtk_widget_show (self->priv->grid);
 	gtk_container_add (GTK_CONTAINER (self), self->priv->grid);
 
-	self->priv->notebook = gtk_notebook_new ();
-	gtk_style_context_remove_class (gtk_widget_get_style_context (self->priv->notebook), GTK_STYLE_CLASS_NOTEBOOK);
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (self->priv->notebook), FALSE);
-	gtk_notebook_set_show_border (GTK_NOTEBOOK (self->priv->notebook), FALSE);
-	gtk_widget_show (self->priv->notebook);
+	self->priv->stack = gtk_stack_new ();
+	gtk_stack_set_transition_type (GTK_STACK (self->priv->stack), GTK_STACK_TRANSITION_TYPE_CROSSFADE);
+	gtk_widget_show (self->priv->stack);
 	gtk_grid_attach (GTK_GRID (self->priv->grid),
-			 self->priv->notebook,
+			 self->priv->stack,
 			 0, 2,
 			 1, 1);
 
 	self->priv->toolbars = g_new0 (GtkWidget *, n_pages);
 	self->priv->contents = g_new0 (GtkWidget *, n_pages);
+	self->priv->pages = g_new0 (GtkWidget *, n_pages);
 
 	for (i = 0; i < n_pages; i++) {
 		GtkWidget *page;
 
-		page = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+		self->priv->pages[i] = page = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 		gtk_widget_show (page);
-		gtk_notebook_append_page (GTK_NOTEBOOK (self->priv->notebook),
-					  page,
-					  NULL);
+		gtk_container_add (GTK_CONTAINER (self->priv->stack), page);
 
 		self->priv->toolbars[i] = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 		gtk_widget_show (self->priv->toolbars[i]);
@@ -105,6 +107,16 @@ gth_window_set_n_pages (GthWindow *self,
 }
 
 
+static void
+_gth_window_add_header_bar (GthWindow *self)
+{
+	self->priv->headerbar = gtk_header_bar_new ();
+	gtk_widget_show (self->priv->headerbar);
+	gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->priv->headerbar), TRUE);
+	gtk_window_set_titlebar (GTK_WINDOW (self), self->priv->headerbar);
+}
+
+
 static void
 gth_window_set_property (GObject      *object,
 			 guint         property_id,
@@ -119,6 +131,11 @@ gth_window_set_property (GObject      *object,
 	case PROP_N_PAGES:
 		gth_window_set_n_pages (self, g_value_get_int (value));
 		break;
+	case PROP_USE_HEADER_BAR:
+		self->priv->use_header_bar = g_value_get_boolean (value);
+		if (self->priv->use_header_bar)
+			_gth_window_add_header_bar (self);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -140,6 +157,9 @@ gth_window_get_property (GObject    *object,
 	case PROP_N_PAGES:
 		g_value_set_int (value, self->priv->n_pages);
 		break;
+	case PROP_USE_HEADER_BAR:
+		g_value_set_boolean (value, self->priv->use_header_bar);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -154,8 +174,10 @@ gth_window_finalize (GObject *object)
 
 	g_free (window->priv->toolbars);
 	g_free (window->priv->contents);
+	g_free (window->priv->pages);
 	g_free (window->priv->window_size);
 	g_object_unref (window->priv->window_group);
+	g_object_unref (window->priv->accel_group);
 
 	G_OBJECT_CLASS (gth_window_parent_class)->finalize (object);
 }
@@ -187,7 +209,7 @@ gth_window_real_set_current_page (GthWindow *window,
 		return;
 
 	window->priv->current_page = page;
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook), page);
+	gtk_stack_set_visible_child (GTK_STACK (window->priv->stack), window->priv->pages[page]);
 
 	for (i = 0; i < window->priv->n_pages; i++)
 		if (i == page)
@@ -227,7 +249,7 @@ gth_window_realize (GtkWidget *widget)
 	}
 	gtk_style_context_add_provider_for_screen (screen,
 						   GTK_STYLE_PROVIDER (css_provider),
-						   GTK_STYLE_PROVIDER_PRIORITY_THEME);
+						   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
 
 	g_object_unref (css_provider);
 }
@@ -262,6 +284,13 @@ gth_window_class_init (GthWindowClass *klass)
 							   G_MAXINT,
 							   1,
 							   G_PARAM_READWRITE));
+	g_object_class_install_property (G_OBJECT_CLASS (klass),
+					 PROP_USE_HEADER_BAR,
+					 g_param_spec_boolean ("use-header-bar",
+							       "use-header-bar",
+							       "use-header-bar",
+							       FALSE,
+							       G_PARAM_READWRITE));
 }
 
 
@@ -271,15 +300,22 @@ gth_window_init (GthWindow *window)
 	window->priv = G_TYPE_INSTANCE_GET_PRIVATE (window, GTH_TYPE_WINDOW, GthWindowPrivate);
 	window->priv->grid = NULL;
 	window->priv->contents = NULL;
+	window->priv->pages = NULL;
 	window->priv->n_pages = 0;
 	window->priv->current_page = GTH_WINDOW_PAGE_UNDEFINED;
 	window->priv->menubar = NULL;
 	window->priv->toolbar = NULL;
 	window->priv->infobar = NULL;
 	window->priv->statusbar = NULL;
+	window->priv->headerbar = NULL;
+	window->priv->use_header_bar = FALSE;
+
 	window->priv->window_group = gtk_window_group_new ();
 	gtk_window_group_add_window (window->priv->window_group, GTK_WINDOW (window));
 
+	window->priv->accel_group = gtk_accel_group_new ();
+	gtk_window_add_accel_group (GTK_WINDOW (window), window->priv->accel_group);
+
 	gtk_window_set_application (GTK_WINDOW (window), Main_Application);
 }
 
@@ -324,6 +360,7 @@ gth_window_attach (GthWindow     *window,
 		return;
 	}
 
+	gtk_widget_set_vexpand (child, FALSE);
 	gtk_grid_attach (GTK_GRID (window->priv->grid),
 			  child,
 			  0, position,
@@ -362,6 +399,7 @@ gth_window_attach_content (GthWindow *window,
 
 	_gtk_container_remove_children (GTK_CONTAINER (window->priv->contents[page]), NULL, NULL);
 	gtk_widget_set_hexpand (child, TRUE);
+	gtk_widget_set_vexpand (child, TRUE);
 	gtk_container_add (GTK_CONTAINER (window->priv->contents[page]), child);
 }
 
@@ -445,6 +483,13 @@ gth_window_get_area (GthWindow     *window,
 }
 
 
+GtkWidget *
+gth_window_get_header_bar (GthWindow *window)
+{
+	return window->priv->headerbar;
+}
+
+
 void
 gth_window_save_page_size (GthWindow *window,
 			   int        page,
@@ -510,3 +555,113 @@ gth_window_get_page_size (GthWindow *window,
 
 	return TRUE;
 }
+
+
+void
+gth_window_set_title (GthWindow  *window,
+		      const char *title,
+		      const char *subtitle)
+{
+	if (window->priv->use_header_bar) {
+		gtk_header_bar_set_title (GTK_HEADER_BAR (window->priv->headerbar), title);
+		gtk_header_bar_set_subtitle (GTK_HEADER_BAR (window->priv->headerbar), subtitle);
+	}
+	else {
+		GString *complete_title;
+
+		complete_title = g_string_new (title);
+		if (subtitle != NULL) {
+			g_string_append (complete_title, " - ");
+			g_string_append (complete_title, subtitle);
+		}
+		gtk_window_set_title (GTK_WINDOW (window), complete_title->str);
+
+		g_string_free (complete_title, TRUE);
+	}
+}
+
+
+GtkAccelGroup *
+gth_window_get_accel_group (GthWindow *window)
+{
+	if (window->priv->accel_group == NULL) {
+		window->priv->accel_group = gtk_accel_group_new ();
+		gtk_window_add_accel_group (GTK_WINDOW (window), window->priv->accel_group);
+	}
+
+	return window->priv->accel_group;
+}
+
+
+void
+gth_window_add_accelerators (GthWindow			*window,
+			     const GthAccelerator	*accelerators,
+			     int		 	 n_accelerators)
+{
+	GtkAccelGroup *accel_group;
+	int            i;
+
+	accel_group = gth_window_get_accel_group (window);
+	for (i = 0; i < n_accelerators; i++) {
+		const GthAccelerator *acc = accelerators + i;
+
+		_gtk_window_add_accelerator_for_action (GTK_WINDOW (window),
+							accel_group,
+							acc->action_name,
+							acc->accelerator,
+							NULL);
+	}
+}
+
+
+void
+gth_window_enable_action (GthWindow  *window,
+			  const char *action_name,
+			  gboolean    enabled)
+{
+	GAction *action;
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (window), action_name);
+	g_object_set (action, "enabled", enabled, NULL);
+}
+
+
+gboolean
+gth_window_get_action_state (GthWindow  *window,
+			     const char *action_name)
+{
+	GAction  *action;
+	GVariant *state;
+	gboolean  value;
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (window), action_name);
+	g_return_val_if_fail (action != NULL, FALSE);
+	state = g_action_get_state (action);
+	value = g_variant_get_boolean (state);
+
+	g_variant_unref (state);
+
+	return value;
+}
+
+
+void
+gth_window_change_action_state (GthWindow  *window,
+			        const char *action_name,
+			        gboolean    value)
+{
+	GAction  *action;
+	GVariant *old_state;
+	GVariant *new_state;
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (window), action_name);
+	g_return_if_fail (action != NULL);
+
+	old_state = g_action_get_state (action);
+	new_state = g_variant_new_boolean (value);
+	if ((old_state == NULL) || ! g_variant_equal (old_state, new_state))
+		g_action_change_state (action, new_state);
+
+	if (old_state != NULL)
+		g_variant_unref (old_state);
+}
