@@ -23,6 +23,7 @@
 #define GTH_WINDOW_H
 
 #include <gtk/gtk.h>
+#include "gtk-utils.h"
 
 G_BEGIN_DECLS
 
@@ -87,6 +88,7 @@ void           gth_window_show_only_content  (GthWindow     *window,
 					      gboolean       only_content);
 GtkWidget *    gth_window_get_area           (GthWindow     *window,
 					      GthWindowArea  area);
+GtkWidget *    gth_window_get_header_bar     (GthWindow     *window);
 void           gth_window_save_page_size     (GthWindow     *window,
 					      int            page,
 					      int            width,
@@ -99,6 +101,21 @@ gboolean       gth_window_get_page_size      (GthWindow     *window,
 		      	      	      	      int           *height);
 void           gth_window_clear_saved_size   (GthWindow     *window,
       	      	      	      	      	      int            page);
+void           gth_window_set_title          (GthWindow     *window,
+					      const char    *title,
+					      const char    *subtitle);
+GtkAccelGroup *gth_window_get_accel_group	(GthWindow		*window);
+void           gth_window_add_accelerators	(GthWindow		*window,
+						 const GthAccelerator	*accelerators,
+						 int			 n_accelerators);
+void		gth_window_enable_action	(GthWindow		*window,
+						 const char		*action_name,
+						 gboolean		 enabled);
+gboolean	gth_window_get_action_state	(GthWindow		*window,
+						 const char		*action_name);
+void		gth_window_change_action_state	(GthWindow		*window,
+						 const char		*action_name,
+						 gboolean		 value);
 
 G_END_DECLS
 
