@@ -21,6 +21,7 @@
 
 #include <config.h>
 #include <string.h>
+#include "color-utils.h"
 #include "gth-image-utils.h"
 #include "gtk-utils.h"
 
@@ -29,95 +30,10 @@
 #define GTHUMB_RESOURCE_BASE_PATH "/org/gnome/gThumb/resources/"
 
 
-void
-_gtk_action_group_add_actions_with_flags (GtkActionGroup          *action_group,
-					  const GthActionEntryExt *entries,
-					  guint                    n_entries,
-					  gpointer                 user_data)
-{
-	guint i;
-
-	g_return_if_fail (GTK_IS_ACTION_GROUP (action_group));
-
-	for (i = 0; i < n_entries; i++) {
-		GtkAction  *action;
-		const char *label;
-		const char *tooltip;
-
-		label = gtk_action_group_translate_string (action_group, entries[i].label);
-		tooltip = gtk_action_group_translate_string (action_group, entries[i].tooltip);
-
-		action = gtk_action_new (entries[i].name,
-					 label,
-					 tooltip,
-					 NULL);
-
-		if (entries[i].stock_id) {
-			g_object_set (action, "stock-id", entries[i].stock_id, NULL);
-			if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), entries[i].stock_id))
-				g_object_set (action, "icon-name", entries[i].stock_id, NULL);
-		}
-
-		if (entries[i].callback) {
-			GClosure *closure;
-
-			closure = g_cclosure_new (entries[i].callback, user_data, NULL);
-			g_signal_connect_closure (action, "activate", closure, FALSE);
-		}
-
-		gtk_action_group_add_action_with_accel (action_group,
-							action,
-							entries[i].accelerator);
-
-		if (entries[i].flags & GTH_ACTION_FLAG_ALWAYS_SHOW_IMAGE)
-			gtk_action_set_always_show_image (action, TRUE);
-
-		if (entries[i].flags & GTH_ACTION_FLAG_IS_IMPORTANT)
-			gtk_action_set_is_important (action, TRUE);
-
-		g_object_unref (action);
-	}
-}
-
-
-GtkWidget *
-_gtk_button_new_from_stock_with_text (const char *stock_id,
-				      const char *text)
-{
-	GtkWidget    *button;
-	GtkWidget    *image;
-	const char   *label_text;
-	gboolean      text_is_stock;
-	GtkStockItem  stock_item;
-
-	button = gtk_button_new ();
-
-	if (gtk_stock_lookup (text, &stock_item)) {
-		label_text = stock_item.label;
-		text_is_stock = TRUE;
-	}
-	else {
-		label_text = text;
-		text_is_stock = FALSE;
-	}
-
-	image = gtk_image_new_from_stock (text_is_stock ? text : stock_id, GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image (GTK_BUTTON (button), image);
-
-	gtk_button_set_use_underline (GTK_BUTTON (button), TRUE);
-	gtk_button_set_label (GTK_BUTTON (button), label_text);
-
-	gtk_widget_set_can_default (button, TRUE);
-	gtk_widget_show (button);
-
-	return button;
-}
-
-
 GtkWidget*
 _gtk_message_dialog_new (GtkWindow        *parent,
 			 GtkDialogFlags    flags,
-			 const char       *stock_id,
+			 const char       *icon_name,
 			 const char       *message,
 			 const char       *secondary_message,
 			 const char       *first_button_text,
@@ -144,9 +60,9 @@ _gtk_message_dialog_new (GtkWindow        *parent,
 
 	/* set the icon */
 
-	gtk_image_set_from_stock (GTK_IMAGE (_gtk_builder_get_widget (builder, "icon_image")),
-				  stock_id,
-				  GTK_ICON_SIZE_DIALOG);
+	gtk_image_set_from_icon_name (GTK_IMAGE (_gtk_builder_get_widget (builder, "icon_image")),
+				      icon_name,
+				      GTK_ICON_SIZE_DIALOG);
 
 	/* set the message */
 
@@ -214,7 +130,6 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 	GtkWidget    *image;
 	GtkWidget    *hbox;
 	GtkWidget    *button;
-	char         *stock_id = GTK_STOCK_DIALOG_QUESTION;
 
 	d = gtk_dialog_new_with_buttons ("", parent, flags, NULL, NULL);
 	gtk_window_set_resizable (GTK_WINDOW (d), FALSE);
@@ -225,7 +140,7 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 
 	/* Add label and image */
 
-	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_DIALOG);
+	image = gtk_image_new_from_icon_name (_GTK_ICON_NAME_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
 	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
 
 	label = gtk_label_new (message);
@@ -249,14 +164,17 @@ _gtk_yesno_dialog_new (GtkWindow        *parent,
 
 	/* Add buttons */
 
-	button = _gtk_button_new_from_stock_with_text (GTK_STOCK_CANCEL, no_button_text);
+	button = gtk_button_new_with_mnemonic (no_button_text);
+	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
 				      button,
 				      GTK_RESPONSE_CANCEL);
 
 	/**/
 
-	button = _gtk_button_new_from_stock_with_text (GTK_STOCK_OK, yes_button_text);
+	button = gtk_button_new_with_mnemonic (yes_button_text);
+	gtk_widget_set_can_default (button, TRUE);
+	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (d),
 				      button,
 				      GTK_RESPONSE_YES);
@@ -280,10 +198,10 @@ _gtk_error_dialog_from_gerror_run (GtkWindow   *parent,
 
 	d = _gtk_message_dialog_new (parent,
 				     GTK_DIALOG_DESTROY_WITH_PARENT,
-				     GTK_STOCK_DIALOG_ERROR,
+				     _GTK_ICON_NAME_DIALOG_ERROR,
 				     title,
 				     gerror->message,
-				     GTK_STOCK_OK, GTK_RESPONSE_OK,
+				     _GTK_LABEL_OK, GTK_RESPONSE_OK,
 				     NULL);
 	gtk_dialog_run (GTK_DIALOG (d));
 
@@ -309,10 +227,10 @@ _gtk_error_dialog_from_gerror_show (GtkWindow   *parent,
 
 	d = _gtk_message_dialog_new (parent,
 				     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				     GTK_STOCK_DIALOG_ERROR,
+				     _GTK_ICON_NAME_DIALOG_ERROR,
 				     title,
 				     (gerror != NULL) ? gerror->message : NULL,
-				     GTK_STOCK_OK, GTK_RESPONSE_OK,
+				     _GTK_LABEL_OK, GTK_RESPONSE_OK,
 				     NULL);
 	g_signal_connect (d, "response", G_CALLBACK (error_dialog_response_cb), NULL);
 
@@ -335,10 +253,10 @@ _gtk_error_dialog_run (GtkWindow        *parent,
 
 	d =  _gtk_message_dialog_new (parent,
 				      GTK_DIALOG_MODAL,
-				      GTK_STOCK_DIALOG_ERROR,
+				      _GTK_ICON_NAME_DIALOG_ERROR,
 				      message,
 				      NULL,
-				      GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
+				      _GTK_LABEL_CLOSE, GTK_RESPONSE_CANCEL,
 				      NULL);
 	g_free (message);
 
@@ -366,10 +284,10 @@ _gtk_error_dialog_show (GtkWindow  *parent,
 
 	d = _gtk_message_dialog_new (parent,
 				     GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-				     GTK_STOCK_DIALOG_ERROR,
+				     _GTK_ICON_NAME_DIALOG_ERROR,
 				     title,
 				     message,
-				     GTK_STOCK_OK, GTK_RESPONSE_OK,
+				     _GTK_LABEL_OK, GTK_RESPONSE_OK,
 				     NULL);
 	g_signal_connect (d, "response", G_CALLBACK (error_dialog_response_cb), NULL);
 
@@ -394,10 +312,10 @@ _gtk_info_dialog_run (GtkWindow        *parent,
 
 	d =  _gtk_message_dialog_new (parent,
 				      GTK_DIALOG_MODAL,
-				      GTK_STOCK_DIALOG_INFO,
+				      _GTK_ICON_NAME_DIALOG_INFO,
 				      message,
 				      NULL,
-				      GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
+				      _GTK_LABEL_CLOSE, GTK_RESPONSE_CANCEL,
 				      NULL);
 	g_free (message);
 
@@ -447,7 +365,7 @@ _g_icon_get_pixbuf (GIcon        *icon,
 			g_error_free (error);
 		}
 
-		gtk_icon_info_free (icon_info);
+		g_object_unref (icon_info);
 	}
 
 	return pixbuf;
@@ -480,16 +398,16 @@ show_help_dialog (GtkWindow  *parent,
 	char   *uri;
 	GError *error = NULL;
 
-	uri = g_strconcat ("ghelp:gthumb", section ? "?" : NULL, section, NULL);
+	uri = g_strconcat ("help:gthumb", section ? "/" : NULL, section, NULL);
 	if (! gtk_show_uri (gtk_window_get_screen (parent), uri, GDK_CURRENT_TIME, &error)) {
   		GtkWidget *dialog;
 
 		dialog = _gtk_message_dialog_new (parent,
 						  GTK_DIALOG_DESTROY_WITH_PARENT,
-						  GTK_STOCK_DIALOG_ERROR,
+						  _GTK_ICON_NAME_DIALOG_ERROR,
 						  _("Could not display help"),
 						  error->message,
-						  GTK_STOCK_OK, GTK_RESPONSE_OK,
+						  _GTK_LABEL_OK, GTK_RESPONSE_OK,
 						  NULL);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
@@ -753,9 +671,9 @@ _gtk_widget_lookup_for_size (GtkWidget   *widget,
 			     GtkIconSize  icon_size)
 {
 	int w, h;
-	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
-					   icon_size,
-					   &w, &h);
+
+	gtk_icon_size_lookup (icon_size, &w, &h);
+
 	return MAX (w, h);
 }
 
@@ -852,7 +770,7 @@ _gtk_info_bar_clear_action_area (GtkInfoBar *info_bar)
 
 typedef struct {
 	GMainLoop     *loop;
-	GdkDragAction  action;
+	GdkDragAction  action_name;
 } DropActionData;
 
 
@@ -873,7 +791,7 @@ ask_drag_drop_action_item_activate_cb (GtkMenuItem *menuitem,
 {
 	DropActionData *drop_data = user_data;
 
-	drop_data->action = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menuitem), "drop-action"));
+	drop_data->action_name = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menuitem), "drop-action"));
 	if (g_main_loop_is_running (drop_data->loop))
 		g_main_loop_quit (drop_data->loop);
 }
@@ -910,7 +828,7 @@ _gtk_menu_ask_drag_drop_action (GtkWidget     *widget,
 	GtkWidget      *menu;
 	GtkWidget      *item;
 
-	drop_data.action = 0;
+	drop_data.action_name = 0;
 	drop_data.loop = g_main_loop_new (NULL, FALSE);
 
 	menu = gtk_menu_new ();
@@ -959,7 +877,7 @@ _gtk_menu_ask_drag_drop_action (GtkWidget     *widget,
 	gtk_widget_destroy (menu);
 	g_main_loop_unref (drop_data.loop);
 
-	return drop_data.action;
+	return drop_data.action_name;
 }
 
 
@@ -968,19 +886,31 @@ _gdk_rgba_shade (GdkRGBA *color,
 		 GdkRGBA *result,
 		 gdouble  factor)
 {
-	GtkSymbolicColor *sym_color;
-	GtkSymbolicColor *sym_result;
-	gboolean          retval;
+	guchar hue, sat, lum;
+	guchar red, green, blue;
+	double tmp;
 
-	sym_color = gtk_symbolic_color_new_literal (color);
-	sym_result = gtk_symbolic_color_new_shade (sym_color, factor);
+	gimp_rgb_to_hsl (color->red * 255,
+			 color->green * 255,
+			 color->blue * 255,
+			 &hue,
+			 &sat,
+			 &lum);
 
-	retval = gtk_symbolic_color_resolve (sym_result, NULL, result);
+	tmp = factor * (double) sat;
+	sat = (guchar) CLAMP (tmp, 0, 255);
 
-	gtk_symbolic_color_unref (sym_result);
-	gtk_symbolic_color_unref (sym_color);
+	tmp = factor * (double) lum;
+	lum = (guchar) CLAMP (tmp, 0, 255);
 
-	return retval;
+	gimp_hsl_to_rgb (hue, sat, lum, &red, &green, &blue);
+
+	result->red = (double) red / 255.0;
+	result->green = (double) green / 255.0;
+	result->blue = (double) blue / 255.0;
+	result->alpha = color->alpha;
+
+	return TRUE;
 }
 
 
@@ -1062,3 +992,222 @@ _gtk_file_chooser_set_file_parent (GtkFileChooser   *chooser,
 	return result;
 }
 
+
+static void
+_gtk_menu_button_set_style_for_header_bar (GtkWidget *button)
+{
+	GtkStyleContext *context;
+
+	gtk_widget_set_valign (button, GTK_ALIGN_CENTER);
+	context = gtk_widget_get_style_context (button);
+	gtk_style_context_add_class (context, "image-button");
+	gtk_style_context_remove_class (context, "text-button");
+}
+
+
+GtkWidget *
+_gtk_menu_button_new_for_header_bar (void)
+{
+	GtkWidget *button;
+
+	button = gtk_menu_button_new ();
+	_gtk_menu_button_set_style_for_header_bar (button);
+
+	return button;
+}
+
+
+GtkWidget *
+_gtk_image_button_new_for_header_bar (const char *icon_name)
+{
+	GtkWidget *button;
+
+	button = gtk_button_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+	_gtk_menu_button_set_style_for_header_bar (button);
+
+	return button;
+}
+
+
+GtkWidget *
+_gtk_toggle_image_button_new_for_header_bar (const char *icon_name)
+{
+	GtkWidget *button;
+	GtkWidget *image;
+
+	button = gtk_toggle_button_new ();
+	image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+	gtk_widget_show (image);
+	gtk_container_add (GTK_CONTAINER (button), image);
+	_gtk_menu_button_set_style_for_header_bar (button);
+
+	return button;
+}
+
+
+/* -- _gtk_window_add_accelerator_for_action -- */
+
+
+typedef struct {
+	GtkWindow *window;
+	char      *action_name;
+	GVariant  *target;
+} AccelData;
+
+
+static void
+accel_data_free (gpointer  user_data,
+                 GClosure *closure)
+{
+	AccelData *accel_data = user_data;
+
+	g_return_if_fail (accel_data != NULL);
+
+	if (accel_data->target != NULL)
+		g_variant_unref (accel_data->target);
+	g_free (accel_data->action_name);
+	g_free (accel_data);
+}
+
+
+static void
+window_accelerator_activated_cb (GtkAccelGroup	*accel_group,
+				 GObject		*object,
+				 guint		 key,
+				 GdkModifierType	 mod,
+				 gpointer		 user_data)
+{
+	AccelData *accel_data = user_data;
+	GAction   *action;
+
+	action = g_action_map_lookup_action (G_ACTION_MAP (accel_data->window), accel_data->action_name);
+	if (action != NULL)
+		g_action_activate (action, accel_data->target);
+}
+
+
+void
+_gtk_window_add_accelerator_for_action (GtkWindow	*window,
+					GtkAccelGroup	*accel_group,
+					const char	*action_name,
+					const char	*accel,
+					GVariant	*target)
+{
+	AccelData	*accel_data;
+	guint		 key;
+	GdkModifierType  mods;
+	GClosure	*closure;
+
+	if ((action_name == NULL) || (accel == NULL))
+		return;
+
+	if (g_str_has_prefix (action_name, "app."))
+		return;
+
+	accel_data = g_new0 (AccelData, 1);
+	accel_data->window = window;
+	/* remove the win. prefix from the action name */
+	if (g_str_has_prefix (action_name, "win."))
+		accel_data->action_name = g_strdup (action_name + strlen ("win."));
+	else
+		accel_data->action_name = g_strdup (action_name);
+	if (target != NULL)
+		accel_data->target = g_variant_ref (target);
+
+	gtk_accelerator_parse (accel, &key, &mods);
+	closure = g_cclosure_new (G_CALLBACK (window_accelerator_activated_cb),
+				  accel_data,
+				  accel_data_free);
+	gtk_accel_group_connect (accel_group,
+				 key,
+				 mods,
+				 0,
+				 closure);
+}
+
+
+/* -- _gtk_window_add_accelerators_from_menu --  */
+
+
+static void
+add_accelerators_from_menu_item (GtkWindow      *window,
+				 GtkAccelGroup  *accel_group,
+				 GMenuModel     *model,
+				 int             item)
+{
+	GMenuAttributeIter	*iter;
+	const char		*key;
+	GVariant		*value;
+	const char		*accel = NULL;
+	const char		*action = NULL;
+	GVariant		*target = NULL;
+
+	iter = g_menu_model_iterate_item_attributes (model, item);
+	while (g_menu_attribute_iter_get_next (iter, &key, &value)) {
+		if (g_str_equal (key, "action") && g_variant_is_of_type (value, G_VARIANT_TYPE_STRING))
+			action = g_variant_get_string (value, NULL);
+		else if (g_str_equal (key, "accel") && g_variant_is_of_type (value, G_VARIANT_TYPE_STRING))
+			accel = g_variant_get_string (value, NULL);
+		else if (g_str_equal (key, "target"))
+			target = g_variant_ref (value);
+		g_variant_unref (value);
+	}
+	g_object_unref (iter);
+
+	_gtk_window_add_accelerator_for_action (window,
+						accel_group,
+						action,
+						accel,
+						target);
+
+	if (target != NULL)
+		g_variant_unref (target);
+}
+
+
+static void
+add_accelerators_from_menu (GtkWindow      *window,
+			    GtkAccelGroup  *accel_group,
+			    GMenuModel     *model)
+{
+	int		 i;
+	GMenuLinkIter	*iter;
+	const char	*key;
+	GMenuModel	*m;
+
+	for (i = 0; i < g_menu_model_get_n_items (model); i++) {
+		add_accelerators_from_menu_item (window, accel_group, model, i);
+
+		iter = g_menu_model_iterate_item_links (model, i);
+		while (g_menu_link_iter_get_next (iter, &key, &m)) {
+			add_accelerators_from_menu (window, accel_group, m);
+			g_object_unref (m);
+		}
+		g_object_unref (iter);
+	}
+}
+
+
+void
+_gtk_window_add_accelerators_from_menu (GtkWindow  *window,
+					GMenuModel *menu)
+{
+	GtkAccelGroup *accel_group;
+
+	accel_group = gtk_accel_group_new ();
+	add_accelerators_from_menu (window, accel_group, menu);
+	gtk_window_add_accel_group (window, accel_group);
+}
+
+
+gboolean
+_gtk_window_get_is_maximized (GtkWindow *window)
+{
+	GdkWindow *gdk_win;
+
+	gdk_win = gtk_widget_get_window (GTK_WIDGET (window));
+	if (gdk_win == NULL)
+		return FALSE;
+
+	return (gdk_window_get_state (gdk_win) & GDK_WINDOW_STATE_MAXIMIZED) != 0;
+}
