@@ -28,32 +28,30 @@
 
 G_BEGIN_DECLS
 
-typedef enum {
-	GTH_ACTION_FLAG_NONE              = 0,
-	GTH_ACTION_FLAG_ALWAYS_SHOW_IMAGE = 1 << 0,
-	GTH_ACTION_FLAG_IS_IMPORTANT      = 1 << 1
-} GthActionFlags;
+#define _GTK_ICON_NAME_DIALOG_ERROR "dialog-error-symbolic"
+#define _GTK_ICON_NAME_DIALOG_INFO "dialog-information-symbolic"
+#define _GTK_ICON_NAME_DIALOG_QUESTION "dialog-question-symbolic"
+#define _GTK_ICON_NAME_DIALOG_WARNING "dialog-warning-symbolic"
+
+#define _GTK_LABEL_CANCEL _("_Cancel")
+#define _GTK_LABEL_CLOSE _("_Close")
+#define _GTK_LABEL_COPY _("Copy")
+#define _GTK_LABEL_DELETE _("_Delete")
+#define _GTK_LABEL_HELP _("_Help")
+#define _GTK_LABEL_NEW _("_New")
+#define _GTK_LABEL_OK _("_Ok")
+#define _GTK_LABEL_REMOVE _("_Remove")
+#define _GTK_LABEL_SAVE _("_Save")
+
 
 typedef struct {
-	const char     *name;
-	const char     *stock_id;
-	const char     *label;
-	const char     *accelerator;
-	const char     *tooltip;
-	GthActionFlags  flags;
-	GCallback       callback;
-} GthActionEntryExt;
+	const char *action_name;
+	const char *accelerator;
+} GthAccelerator;
 
-void            _gtk_action_group_add_actions_with_flags   (GtkActionGroup   *action_group,
-							    const GthActionEntryExt
-									     *entries,
-							    guint             n_entries,
-							    gpointer          user_data);
-GtkWidget *     _gtk_button_new_from_stock_with_text       (const char       *stock_id,
-							    const char       *text);
 GtkWidget *     _gtk_message_dialog_new                    (GtkWindow        *parent,
 							    GtkDialogFlags    flags,
-							    const char       *stock_id,
+							    const char       *icon_name,
 							    const char       *message,
 							    const char       *secondary_message,
 							    const char       *first_button_text,
@@ -133,6 +131,20 @@ GtkIconTheme *  _gtk_widget_get_icon_theme                 (GtkWidget        *wi
 void            _gtk_combo_box_add_image_sizes             (GtkComboBox      *combo_box,
 							    int               active_width,
 							    int               active_height);
+gboolean        _gtk_file_chooser_set_file_parent          (GtkFileChooser   *chooser,
+							    GFile            *file,
+							    GError          **error);
+GtkWidget *     _gtk_menu_button_new_for_header_bar        (void);
+GtkWidget *     _gtk_image_button_new_for_header_bar       (const char       *icon_name);
+GtkWidget *     _gtk_toggle_image_button_new_for_header_bar(const char       *icon_name);
+void		_gtk_window_add_accelerator_for_action     (GtkWindow		*window,
+							    GtkAccelGroup	*accel_group,
+							    const char		*action_name,
+							    const char		*accel,
+							    GVariant		*target);
+void		_gtk_window_add_accelerators_from_menu	   (GtkWindow		*window,
+							    GMenuModel		*menu);
+gboolean	_gtk_window_get_is_maximized 		   (GtkWindow		*window);
 
 G_END_DECLS
 
