@@ -27,6 +27,7 @@
 #include "gth-time.h"
 #include "gth-preferences.h"
 #include "gth-user-dir.h"
+#include "main-migrate.h"
 
 
 #define MAX_LINE_LENGTH 4096
@@ -82,6 +83,7 @@ migration_for_each_file (GFile     *file,
 	char          *extension;
 	char          *new_buffer;
 	gsize          new_buffer_size;
+	GFile         *catalogs_dir;
 	char          *catalogs_path;
 	char          *relative_path;
 	char          *tmp_path;
@@ -94,11 +96,11 @@ migration_for_each_file (GFile     *file,
 	if (g_file_info_get_file_type (info) != G_FILE_TYPE_REGULAR)
 		return;
 
-	if (! g_load_file_in_buffer (file,
-				     (void **) &buffer,
-				     &buffer_size,
-				     NULL,
-				     &error))
+	if (! _g_file_load_in_buffer (file,
+				      (void **) &buffer,
+				      &buffer_size,
+				      NULL,
+				      &error))
 	{
 		g_warning ("%s", error->message);
 		return;
@@ -428,7 +430,8 @@ migration_for_each_file (GFile     *file,
 
 	new_buffer = dom_document_dump (document, &new_buffer_size);
 
-	catalogs_path = gth_user_dir_get_file (GTH_DIR_DATA, "gthumb", "catalogs", NULL);
+	catalogs_dir = gth_user_dir_get_file_for_write (GTH_DIR_DATA, GTHUMB_DIR, "catalogs", NULL);
+	catalogs_path = g_file_get_path (catalogs_dir);
 	relative_path = g_file_get_relative_path (data->collections_dir, file);
 	tmp_path = g_strconcat (catalogs_path, G_DIR_SEPARATOR_S, relative_path, NULL);
 	full_path_no_ext = _g_uri_remove_extension (tmp_path);
@@ -444,13 +447,13 @@ migration_for_each_file (GFile     *file,
 	*/
 
 	if (! g_file_query_exists (catalog_file, NULL)) {
-		if (! g_write_file (catalog_file,
-				    FALSE,
-				    G_FILE_CREATE_PRIVATE,
-				    new_buffer,
-				    new_buffer_size,
-				    NULL,
-				    &error))
+		if (! _g_file_write (catalog_file,
+				     FALSE,
+				     G_FILE_CREATE_PRIVATE,
+				     new_buffer,
+				     new_buffer_size,
+				     NULL,
+				     &error))
 		{
 			g_warning ("%s", error->message);
 		}
@@ -463,6 +466,7 @@ migration_for_each_file (GFile     *file,
 	g_free (tmp_path);
 	g_free (relative_path);
 	g_free (catalogs_path);
+	g_object_unref (catalogs_dir);
 	g_free (new_buffer);
 	g_object_unref (document);
 	g_free (buffer);
