@@ -41,7 +41,8 @@
 #include "gth-file-source-vfs.h"
 #include "gth-main.h"
 #include "gth-preferences.h"
-#include "gth-window-actions-callbacks.h"
+#include "gtk-utils.h"
+#include "main.h"
 #include "main-migrate.h"
 
 
@@ -128,6 +129,7 @@ import_photos_from_location (GFile *location)
 	GtkWidget *window;
 
 	window = gth_browser_new (NULL, NULL);
+	gth_browser_set_close_with_task (GTH_BROWSER (window), TRUE);
 	gth_hook_invoke ("import-photos", window, location, NULL);
 }
 
@@ -295,6 +297,9 @@ typedef GtkApplicationClass GthApplicationClass;
 static gpointer gth_application_parent_class;
 
 
+GType gth_application_get_type (void);
+
+
 G_DEFINE_TYPE (GthApplication, gth_application, GTK_TYPE_APPLICATION)
 
 
@@ -318,82 +323,13 @@ gth_application_init (GthApplication *app)
 }
 
 
-static void
-activate_new_window (GSimpleAction *action,
-		     GVariant      *parameter,
-		     gpointer       user_data)
-{
-        GApplication *application = user_data;
-        GList        *windows;
-
-        windows = gtk_application_get_windows (GTK_APPLICATION (application));
-        if (windows != NULL)
-        	gth_browser_activate_action_file_new_window (NULL, windows->data);
-}
-
-
-static void
-activate_preferences (GSimpleAction *action,
-		      GVariant      *parameter,
-		      gpointer       user_data)
-{
-        GApplication *application = user_data;
-        GList        *windows;
-
-        windows = gtk_application_get_windows (GTK_APPLICATION (application));
-        if (windows != NULL)
-        	gth_browser_activate_action_edit_preferences (NULL, windows->data);
-}
-
-
-static void
-activate_help (GSimpleAction *action,
-               GVariant      *parameter,
-               gpointer       user_data)
-{
-        GApplication *application = user_data;
-        GList        *windows;
-
-        windows = gtk_application_get_windows (GTK_APPLICATION (application));
-        if (windows != NULL)
-        	gth_browser_activate_action_help_help (NULL, windows->data);
-}
-
-
-static void
-activate_about (GSimpleAction *action,
-		GVariant      *parameter,
-		gpointer       user_data)
-{
-        GApplication *application = user_data;
-        GList        *windows;
-
-        windows = gtk_application_get_windows (GTK_APPLICATION (application));
-        if (windows != NULL)
-        	gth_browser_activate_action_help_about (NULL, windows->data);
-}
-
-
-static void
-activate_quit (GSimpleAction *action,
-               GVariant      *parameter,
-               gpointer       user_data)
-{
-        GApplication *application = user_data;
-        GList        *windows;
-
-        windows = gtk_application_get_windows (GTK_APPLICATION (application));
-        if (windows != NULL)
-        	gth_window_activate_action_file_quit_application (NULL, windows->data);
-}
-
-
 static const GActionEntry app_menu_entries[] = {
-	{ "new-window",  activate_new_window },
-	{ "preferences",  activate_preferences },
-	{ "help",  activate_help },
-	{ "about", activate_about },
-	{ "quit",  activate_quit }
+	{ "new-window", gth_application_activate_new_window },
+	{ "preferences", gth_application_activate_preferences },
+	{ "shortcuts", gth_application_activate_show_shortcuts },
+	{ "help", gth_application_activate_show_help },
+	{ "about", gth_application_activate_about },
+	{ "quit", gth_application_activate_quit }
 };
 
 
@@ -402,7 +338,6 @@ _gth_application_initialize_app_menu (GApplication *application)
 {
 	gboolean    show_app_menu;
 	GtkBuilder *builder;
-	GError     *error = NULL;
 
 	g_object_get (gtk_settings_get_default (),
 		      "gtk-shell-shows-app-menu", &show_app_menu,
@@ -415,38 +350,7 @@ _gth_application_initialize_app_menu (GApplication *application)
 					 G_N_ELEMENTS (app_menu_entries),
 					 application);
 
-	builder = gtk_builder_new ();
-	gtk_builder_add_from_string (builder,
-			"<interface>"
-			"  <menu id='app-menu'>"
-			"    <section>"
-			"      <item>"
-			"        <attribute name='label' translatable='yes'>New _Window</attribute>"
-			"        <attribute name='action'>app.new-window</attribute>"
-			"      </item>"
-			"    </section>"
-			"    <section>"
-			"      <item>"
-			"        <attribute name='label' translatable='yes'>_Preferences</attribute>"
-			"        <attribute name='action'>app.preferences</attribute>"
-			"      </item>"
-			"    </section>"
-			"    <section>"
-			"      <item>"
-			"        <attribute name='label' translatable='yes'>_Help</attribute>"
-			"        <attribute name='action'>app.help</attribute>"
-			"      </item>"
-			"      <item>"
-			"        <attribute name='label' translatable='yes'>_About gThumb</attribute>"
-			"        <attribute name='action'>app.about</attribute>"
-			"      </item>"
-			"      <item>"
-			"        <attribute name='label' translatable='yes'>_Quit</attribute>"
-			"        <attribute name='action'>app.quit</attribute>"
-			"      </item>"
-			"    </section>"
-			"  </menu>"
-			"</interface>", -1, &error);
+	builder = _gtk_builder_new_from_resource ("app-menu.ui");
 	gtk_application_set_app_menu (GTK_APPLICATION (application),
 				      G_MENU_MODEL (gtk_builder_get_object (builder, "app-menu")));
 
@@ -459,6 +363,8 @@ gth_application_startup (GApplication *application)
 {
 	G_APPLICATION_CLASS (gth_application_parent_class)->startup (application);
 
+	 g_object_set (gtk_settings_get_default (), "gtk-application-prefer-dark-theme", TRUE, NULL);
+
 	_gth_application_initialize_app_menu (application);
 	gth_pref_initialize ();
 	migrate_data ();
@@ -491,7 +397,6 @@ gth_application_create_option_context (void)
 		g_option_context_add_group (context, egg_sm_client_get_option_group ());
 #endif
 #ifdef HAVE_CLUTTER
-		g_option_context_add_group (context, cogl_get_option_group ());
 		g_option_context_add_group (context, clutter_get_option_group_without_init ());
 		g_option_context_add_group (context, gtk_clutter_get_option_group ());
 #endif
@@ -631,9 +536,8 @@ gth_application_command_line (GApplication            *application,
 
 	/* Open each dir in a new window */
 
-	singleton = (dirs != NULL) && (dirs->next == NULL);
 	for (scan = dirs; scan; scan = scan->next)
-		open_browser_window ((GFile *) scan->data, NULL, ! singleton);
+		open_browser_window ((GFile *) scan->data, NULL, TRUE);
 
 	gdk_notify_startup_complete ();
 
@@ -655,11 +559,10 @@ gth_application_local_command_line (GApplication   *application,
         GError          *error = NULL;
         gboolean         handled_locally = FALSE;
 
-        local_argv = g_strdupv (*arguments);
-        local_argc = g_strv_length (local_argv);
-
         *exit_status = 0;
 
+        local_argv = g_strdupv (*arguments);
+        local_argc = g_strv_length (local_argv);
         context = gth_application_create_option_context ();
 	if (! g_option_context_parse (context, &local_argc, &local_argv, &error)) {
 		*exit_status = EXIT_FAILURE;
@@ -667,6 +570,23 @@ gth_application_local_command_line (GApplication   *application,
                 g_clear_error (&error);
                 handled_locally = TRUE;
 	}
+	g_strfreev (local_argv);
+
+	/* substitute the dot with the local current folder */
+
+	local_argv = *arguments;
+	if ((local_argv != NULL) && (local_argv[0] != NULL)) {
+		int i;
+
+		for (i = 1; local_argv[i] != NULL; i++) {
+			if (strstr (local_argv[i], ".") != NULL) {
+				GFile *location = g_file_new_for_commandline_arg (local_argv[i]);
+				g_free (local_argv[i]);
+				local_argv[i] = g_file_get_uri (location);
+				g_object_unref (location);
+			}
+		}
+	}
 
 	if (version) {
 		g_printf ("%s %s, Copyright © 2001-2010 Free Software Foundation, Inc.\n", PACKAGE_NAME, PACKAGE_VERSION);
@@ -674,7 +594,6 @@ gth_application_local_command_line (GApplication   *application,
 	}
 
 	g_option_context_free (context);
-        g_strfreev (local_argv);
 
         return handled_locally;
 }
@@ -717,8 +636,6 @@ main (int argc, char *argv[])
 
 	program_argv0 = argv[0];
 
-	g_type_init ();
-
 	/* text domain */
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
